package de.bsvrz.buv.plugin.engstvw;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.engstvw.Engstelle.EnumEngstellenTypen;
import de.bsvrz.buv.plugin.engstvw.Engstelle.EnumStatusBaustelle;
import de.bsvrz.buv.rw.basislib.kalender.DatumZeit;
import de.bsvrz.buv.rw.basislib.kalender.DatumZeit.Eingabetyp;
import de.bsvrz.buv.rw.basislib.login.UrlasserDialog;
import de.bsvrz.buv.rw.basislib.ort.FilterAsbOrtsangabe;
import de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer;
import de.bsvrz.buv.rw.basislib.ort.FilterKnoten;
import de.bsvrz.buv.rw.basislib.ort.FilterStrassenSegmentOffset;
import de.bsvrz.buv.rw.basislib.ort.OrtsangabeComposite;
import de.bsvrz.buv.rw.basislib.ort.OrtsangabeEnumeratorAuswahl;
import de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.EnumAsbVerkehrsRichtung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Definitionsdialog zum Anlegen und Bearbeiten von Engstellen.
 * 
 * @author HofmannJ
 * 
 */
public class Definitionsdialog extends Dialog
{
	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	private Shell _shell = null;

	private Composite _bereichOben = null;

	private Group _gruppeZeitlicheGueltigkeit = null;

	private Group _gruppeLageVon = null;

	private Group _gruppeLageBis = null;

	private Composite _bereichUnten = null;

	private Composite _bereichUntenRechts = null;

	private Button _schaltflaecheOk = null;

	private Button _schaltflaecheAbbrechen = null;

	private Button _schaltflaecheHilfe = null;

	private Label _labelName = null;

	private Label _labelTypEngstelle = null;

	private Label _labelStatusBaustelle = null;

	private Label _labelRestkapazitaet = null;

	private Text _textName = null;

	private Combo _comboTypEngstelle = null;

	private Combo _comboStatusBaustelle = null;

	private Composite _bereichRestkapazitaet = null;

	private Text _textRestkapazitaet = null;

	private Label _labelFzh = null;

	private Label _labelVon = null;

	private Label _labelBis = null;

	private ComboViewer _comboViewerTypEngstelle = null;

	private ComboViewer _comboViewerStatusBaustelle = null;

	/**
	 * Verbindung zum DaV
	 */
	private ClientDavInterface _davVerbindung = null;

	/**
	 * Verbindung zum DaV
	 */
	private DaVKommunikation _daVKommunikation = DaVKommunikation.getInstanz();

	/**
	 * Datenstruktur der Engstelle.
	 */
	private Engstelle _engstelleDatensatz = null;

	private OrtsangabeComposite _ocLageVon;

	private OrtsangabeComposite _ocLageBis;

	private DatumZeit _zeitVon;

	private DatumZeit _zeitBis;

	/**
	 * Flag, zeigt an, ob der Dialog geschlossen werden darf. Dies ist z.B. dann
	 * nicht der Fall (==False), wenn vor dem Klicken auf OK nicht alle Felder
	 * selektiert wurden. Bei Abbrechen ist dieser Wert immer true.
	 */
	private boolean _schlieenOk = false;

	/**
	 * 
	 */
	private boolean _bearbeiten = false;

	int hoehe = 750;

	int breite = 560;

	private boolean _kopieren;

	/**
	 * Konstruktor fr neue Instanzen der Klasse {@link DefinitionsdialogAlt}.
	 * 
	 * @param parent
	 *          Eltern-Shell, die fr die Darstellung benutzt werden soll.
	 */
	public Definitionsdialog(Shell parent)
	{
		super(parent, 0);
	}

	/**
	 * ffnet den Dialog {@link DefinitionsdialogAlt} zum Anlegen einer einer
	 * neuen Engstelle (Datensatz vom Typ {@link Engstelle}).
	 * 
	 * @param davVerbindung
	 *          Verbindung zum DaV.
	 * @return Liefert den erfassten Datensatz vom Typ {@link Engstelle} ode
	 *         <code>null</code>, wenn keine Engstelle angelegt wurde und auf
	 *         Abbrechen geklickt wurde.
	 */
	public Engstelle oeffne(ClientDavInterface davVerbindung)
	{

		_bearbeiten = false;

		this._davVerbindung = davVerbindung;

		Shell parent = getParent();
		_shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL
				| SWT.RESIZE);
		_shell.setText("Definition Engstellen");

		// _shell.addDisposeListener(new DisposeListener(){
		//
		// @Override
		// public void widgetDisposed(DisposeEvent e)
		// {
		// _schlieenOk = true;
		// _engstelleDatensatz = null;
		// //schliesseDialog();
		//         
		// }
		//      
		// });

		erzeugeBereichOben();
		erzeugeGruppeZeitlicheGueltigkeit();
		erzeugeGruppeLageVon();
		erzeugeGruppeLageBis();
		erzeugeBereichUnten();

		_shell.setLayout(new GridLayout());
		_shell.setSize(new Point(breite, hoehe));
		_shell.open();
		Display display = parent.getDisplay();
		while (!_shell.isDisposed())
		{
			if (!display.readAndDispatch())
				display.sleep();
		}
		return _engstelleDatensatz;
	}

	/**
	 * ffnet den Dialog {@link DefinitionsdialogAlt} zum Bearbeiten eine
	 * Datensatzes vom Typ {@link Engstelle}.
	 * 
	 * @param davVerbindung
	 *          Verbindung zum DaV
	 * @param engstelleDatensatz
	 *          Datensatz, der bearbeitet werden soll. Es wird auf einem clone
	 *          dieses Datensatzes gearbeitet.
	 * @return Liefert eine Engstelle zurck, wenn der Dialog korrekt ausgefllt
	 *         wurde. Wird auf Abbrechen gedrckt, so wird <code>null</code>
	 *         zurckgeliefert. Der alte Datensatz ist durch diesen zu ersetzen
	 */
	public Engstelle oeffne(ClientDavInterface davVerbindung,
			Engstelle engstelleDatensatz, boolean kopieren)
	{
		_bearbeiten = true;
		_kopieren = kopieren;
		this._davVerbindung = davVerbindung;
		if (engstelleDatensatz != null)
		{
			this._engstelleDatensatz = engstelleDatensatz;
		}

		Shell parent = getParent();
		_shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		_shell.setText("Definition Engstellen");

		// _shell.addDisposeListener(new DisposeListener(){
		//
		// @Override
		// public void widgetDisposed(DisposeEvent e)
		// {
		// _schlieenOk = true;
		// _engstelleDatensatz = null;
		// }
		//      
		// });

		erzeugeBereichOben();
		erzeugeGruppeZeitlicheGueltigkeit();
		erzeugeGruppeLageVon();
		erzeugeGruppeLageBis();
		erzeugeBereichUnten();

		_shell.setLayout(new GridLayout());
		_shell.setSize(new Point(breite, hoehe));
		_shell.open();

		if (engstelleDatensatz != null)
		{
			initialisiereMitEngstelle(_engstelleDatensatz);
		}

		Display display = parent.getDisplay();
		while (!_shell.isDisposed())
		{
			if (!display.readAndDispatch())
				display.sleep();
		}
		return _engstelleDatensatz;
	}

	/**
	 * Fhrt eine Initialisierung der Oberflche mit einer anzugebenden
	 * {@link Engstelle} durch, d.h. die jeweiligen Felder werden mit den
	 * vorgegebenen Werten initialisiert.
	 * 
	 * @param engstelle
	 *          {@link Engstelle}, mit der die Oberflche initialisiert werden
	 *          soll. Darf <b>nicht</b> <code>null</code> sein.
	 */
	protected void initialisiereMitEngstelle(Engstelle engstelle)
	{
		if (engstelle != null)
		{
			/*
			 * Name
			 */
			if (engstelle.getName() != null)
			{
				_textName.setText(engstelle.getName());
			}

			/*
			 * Engstellentyp
			 */
			if (engstelle.getGewaehlterEngstellenTyp() != null)
			{
				_comboViewerTypEngstelle.setSelection(new StructuredSelection(engstelle
						.getGewaehlterEngstellenTyp()));
			}

			/*
			 * Status Baustelle
			 */
			if ((engstelle.getGewaehlterEngstellenTyp() != null)
					&& (engstelle.getGewaehlterEngstellenTyp()
							.equals(EnumEngstellenTypen.Baustelle)))
			{
				if (engstelle.getGewaehlterStatusBaustelle() != null)
				{
					_comboViewerStatusBaustelle.setSelection(new StructuredSelection(
							engstelle.getGewaehlterStatusBaustelle()));
				}
			}

			/*
			 * Restkapazitt
			 */
			_textRestkapazitaet.setText("" + engstelle.getRestkapazitaet());

			/*
			 * Zeitliche Gltigkeit von
			 */
			if (engstelle.getZeitlicheGueltigkeitVon() > 0)
			{
				_zeitVon.setDatum(new Date(engstelle.getZeitlicheGueltigkeitVon()));
			} else
			{
				_zeitVon.setDatum(new Date());
			}

			/*
			 * Zeitliche Gltigkeit bis
			 */
			if (engstelle.getZeitlicheGueltigkeitBis() > 0)
			{
				_zeitBis.setDatum(new Date(engstelle.getZeitlicheGueltigkeitBis()));
			} else
			{
				_zeitBis.setDatum(new Date());
			}

			/*
			 * Lage von
			 */
			OrtsangabeEnumeratorAuswahl typLageVon = engstelle
					.getLageVonEngstellenAnfangOrtsangabeTyp();
			if (typLageVon != null)
			{
				switch (typLageVon)
				{
				case asborstangabe:
					if (engstelle.getVonDatenOrtsangabe() != null)
					{
						initLage(_ocLageVon, engstelle.getVonDatenOrtsangabe());
					}
					break;

				case betriebskilometer:
					if (engstelle.getVonDatenBetriebskilometer() != null)
					{
						initLage(_ocLageVon, engstelle.getVonDatenBetriebskilometer());
					}
					break;

				case knotenoffset:
					if (engstelle.getVonDatenKnotenOffset() != null)
					{
						initLage(_ocLageVon, engstelle.getVonDatenKnotenOffset());
					}
					break;

				case strassensegment:
					if (engstelle.getVonDatenStrassenSegmentOffset() != null)
					{
						initLage(_ocLageVon, engstelle.getVonDatenStrassenSegmentOffset());
					}
					break;
				}
			}

			/*
			 * Lage bis
			 */
			OrtsangabeEnumeratorAuswahl typLageBis = engstelle
					.getLageBisEngstellenEndeOrtsangabeTyp();
			if (typLageBis != null)
			{
				switch (typLageBis)
				{
				case asborstangabe:
					if (engstelle.getBisDatenAsbOrtsangabe() != null)
					{
						initLage(_ocLageBis, engstelle.getBisDatenAsbOrtsangabe());
					}
					break;

				case betriebskilometer:
					if (engstelle.getBisDatenBetriebskilometer() != null)
					{
						initLage(_ocLageBis, engstelle.getBisDatenBetriebskilometer());
					}
					break;

				case knotenoffset:
					if (engstelle.getBisDatenKnotenOffset() != null)
					{
						initLage(_ocLageBis, engstelle.getBisDatenKnotenOffset());
					}
					break;

				case strassensegment:
					if (engstelle.getBisDatenStrassenSegmentOffset() != null)
					{
						initLage(_ocLageBis, engstelle.getBisDatenStrassenSegmentOffset());
					}
					break;
				}
			}

		}
	}

	/**
	 * Initialisiert die Lage fr ein {@link OrtsangabeComposite}.
	 * 
	 * @param oc
	 *          {@link OrtsangabeComposite}, das zu Initialisieren ist.
	 * @param filter
	 *          Filter, mit den das {@link OrtsangabeComposite} initialisiert
	 *          werden soll. Folgende Typen werden untersttzt:
	 *          {@link FilterAsbOrtsangabe}, {@link FilterBetriebskilometer},
	 *          {@link FilterKnoten} sowie {@link FilterStrassenSegmentOffset}.
	 */
	private void initLage(OrtsangabeComposite oc, Object filter)
	{
		/*
		 * ASB
		 */
		if (filter instanceof FilterAsbOrtsangabe)
		{
			FilterAsbOrtsangabe filterASB = (FilterAsbOrtsangabe) filter;

			String pidSelektierteStrasse = filterASB.getPidSelektierteStrasse();
			String anfangsKnoten = filterASB.getAnfangsKnoten();
			String endKnoten = filterASB.getEndKnoten();
			long stationierung = filterASB.getStationierung();
			EnumAsbVerkehrsRichtung asbVerkehrsRichtung = filterASB
					.getAsbVerkehrsRichtung();

			oc.setAsb(pidSelektierteStrasse, anfangsKnoten, endKnoten, stationierung,
					asbVerkehrsRichtung);
		}

		/*
		 * Betriebskilometer
		 */
		if (filter instanceof FilterBetriebskilometer)
		{
			FilterBetriebskilometer filterBK = (FilterBetriebskilometer) filter;

			String pidSelektierteStrasse = filterBK.getPidSelektierteStrasse();
			Richtung himmelsRichtung = filterBK.getSelektierteRichtung();
			long betriebsKilometer = filterBK.getBetriebskilometer();
			String selektierterBlock = filterBK.getSelektierteBlockNummern();

			oc.setBetriebskilometer(pidSelektierteStrasse, himmelsRichtung,
					betriebsKilometer, selektierterBlock);
		}

		/*
		 * Knoten
		 */
		if (filter instanceof FilterKnoten)
		{
			FilterKnoten filterKnoten = (FilterKnoten) filter;

			String pidSelektierteStrasse = filterKnoten.getPidStrasse();
			String pidSelektierterKnoten = filterKnoten.getPidKnoten();
			float km = filterKnoten.getKm();
			de.bsvrz.buv.rw.basislib.ort.FilterKnoten.Richtung selektierteRichtung = filterKnoten
					.getRichtung();

			oc.setKnotenOffset(pidSelektierteStrasse, pidSelektierterKnoten, km,
					selektierteRichtung);
		}

		/*
		 * Straensegment Offset
		 */
		if (filter instanceof FilterStrassenSegmentOffset)
		{
			FilterStrassenSegmentOffset filterSS = (FilterStrassenSegmentOffset) filter;

			String pidSelektierteStrasse = filterSS.getPidSelektierteStrasse();
			String pidStrassenSegment = filterSS.getPidSelektiertesStrassenSegment();
			long km = filterSS.getKm();

			oc.setStrassenSegment(pidSelektierteStrasse, pidStrassenSegment, km);
		}
	}

	/**
	 * Sichert die im Dialog erfassten Daten
	 */
	protected void sichereErfassteDaten()
	{
		boolean bNameEngstelle = false;
		boolean bTypEngstelle = false;
		boolean bStatusBaustelle = true;
		boolean bRestKapazitaet = false;
		boolean bZeitlicheGueltigkeit = false;

		/*
		 * Wenn kein Speicher fr eine Engstelle initialisiert wurde, dann wird
		 * dieser frisch angelegt.
		 */
		if (_engstelleDatensatz == null)
		{
			_engstelleDatensatz = new Engstelle();
		}

		/*
		 * Name der Engstelle
		 */
		if ((_textName.getText() != null) && (_textName.getText().length() > 0))
		{
			_engstelleDatensatz.setName(_textName.getText());

			if (!_bearbeiten || _kopieren)
				_engstelleDatensatz.setPid("situation."
						+ _textName.getText().replace(" ", ""));
			bNameEngstelle = true;
		}

		/*
		 * Typ der Engstelle
		 */
		ISelection selectionTypEngstelle = _comboViewerTypEngstelle.getSelection();
		if (selectionTypEngstelle != null)
		{
			if (selectionTypEngstelle instanceof StructuredSelection)
			{
				StructuredSelection ssTypEngstelle = (StructuredSelection) selectionTypEngstelle;
				Object firstEngstellenTyp = ssTypEngstelle.getFirstElement();
				if (firstEngstellenTyp != null)
				{
					if (firstEngstellenTyp instanceof EnumEngstellenTypen)
					{
						EnumEngstellenTypen typ = (EnumEngstellenTypen) firstEngstellenTyp;
						_engstelleDatensatz.setGewaehlterEngstellenTyp(typ);
						bTypEngstelle = true;
					}
				}
			}
		}

		/*
		 * Status Baustelle
		 */
		if (_engstelleDatensatz.getGewaehlterEngstellenTyp() != null)
		{
			if (_engstelleDatensatz.getGewaehlterEngstellenTyp().equals(
					EnumEngstellenTypen.Baustelle))
			{
				bStatusBaustelle = false;

				ISelection selectionStatusBaustelle = _comboViewerStatusBaustelle
						.getSelection();
				if (selectionStatusBaustelle != null)
				{
					if (selectionStatusBaustelle instanceof StructuredSelection)
					{
						StructuredSelection ssStatusBaustelle = (StructuredSelection) selectionStatusBaustelle;
						Object firstElement = ssStatusBaustelle.getFirstElement();
						if (firstElement != null)
						{
							if (firstElement instanceof EnumStatusBaustelle)
							{
								EnumStatusBaustelle statusBaustelle = (EnumStatusBaustelle) firstElement;
								_engstelleDatensatz
										.setGewaehlterStatusBaustelle(statusBaustelle);
								bStatusBaustelle = true;
							}
						}
					}
				}
			}
		}

		/*
		 * Restkapazitt
		 */
		if ((_textRestkapazitaet.getText() != null)
				&& (_textRestkapazitaet.getText().length() > 0))
		{
			try
			{
				long restKap = Long.valueOf(_textRestkapazitaet.getText());
				_engstelleDatensatz.setRestkapazitaet(restKap);
				bRestKapazitaet = true;

			} catch (NumberFormatException nfe)
			{
				nfe.printStackTrace();
			}
		}

		/*
		 * Zeitliche Gueltigkeit von/bis
		 */
		long von = _zeitVon.getDatum().getTime();
		long bis = _zeitBis.getDatum().getTime();

		if (von <= bis)
		{
			bZeitlicheGueltigkeit = true;
			_engstelleDatensatz.setZeitlicheGueltigkeitVon(von);
			_engstelleDatensatz.setZeitlicheGueltigkeitBis(bis);
		}

		/*
		 * Lage Von Ortsangabe
		 */
		OrtsangabeEnumeratorAuswahl auswahlArtVon = _ocLageVon.getAuswahlArt();
		_engstelleDatensatz.setLageVonEngstellenAnfangOrtsangabeTyp(auswahlArtVon);
		switch (auswahlArtVon)
		{
		case asborstangabe:
			_engstelleDatensatz.setVonDatenAsbOrtsangabe(_ocLageVon
					.getFilterAsbOrtsangabe());
			break;
		case betriebskilometer:
			_engstelleDatensatz.setVonDatenBetriebskilometer(_ocLageVon
					.getFilterBetriebskilometer());
			break;
		case knotenoffset:
			_engstelleDatensatz.setVonDatenKnotenOffset(_ocLageVon.getFilterKnoten());
			break;
		case strassensegment:
			_engstelleDatensatz.setVonDatenStrassenSegmentOffset(_ocLageVon
					.getFilterStrassenSegmentOffset());
			break;
		}

		/*
		 * Lage Bis Ortsangabe
		 */
		OrtsangabeEnumeratorAuswahl auswahlArtBis = _ocLageBis.getAuswahlArt();
		_engstelleDatensatz.setLageBisEngstellenEndeOrtsangabeTyp(auswahlArtBis);
		switch (auswahlArtBis)
		{
		case asborstangabe:
			_engstelleDatensatz.setBisDatenAsbOrtsangabe(_ocLageBis
					.getFilterAsbOrtsangabe());
			break;
		case betriebskilometer:
			_engstelleDatensatz.setBisDatenBetriebskilometer(_ocLageBis
					.getFilterBetriebskilometer());
			break;
		case knotenoffset:
			_engstelleDatensatz.setBisDatenKnotenOffset(_ocLageBis.getFilterKnoten());
			break;
		case strassensegment:
			_engstelleDatensatz.setBisDatenStrassenSegmentOffset(_ocLageBis
					.getFilterStrassenSegmentOffset());
			break;
		}

		_schlieenOk = true;

		if (!bNameEngstelle)
		{
			fehlerDialog("Name der Engstelle fehlt",
					"Sie mssen einen Namen fr die Engstelle eingeben!");
			_schlieenOk = false;
		}

		if (!bTypEngstelle)
		{
			fehlerDialog("Typ der Engstelle wurde nicht ausgewhlt",
					"Sie mssen einen Typ fr die Engstelle auswhlen!");
			_schlieenOk = false;
		}

		if (!bStatusBaustelle)
		{
			fehlerDialog("Status der Baustelle wurde nicht ausgewhlt",
					"Sie mssen einen Status fr die Baustelle auswhlen!");
			_schlieenOk = false;
		}

		if (!bRestKapazitaet)
		{
			fehlerDialog("Keine gltige Restkapazitt angegeben",
					"Sie mssen eine gltige Restkapazitt angeben!");
			_schlieenOk = false;
		}

		if (!bZeitlicheGueltigkeit)
		{
			fehlerDialog("Zeitliche Gltigkeit ungltig",
					"Der Von-Zeitpunkt darf nicht nach dem Bis-Zeitpunkt liegen!");
			_schlieenOk = false;
		}

	}

	private void erzeugeUndParametriere()
	{

		if (!_bearbeiten || _kopieren)
		{

			if (!_daVKommunikation.isObjektVorhanden(_engstelleDatensatz.getPid()))
			{
				_daVKommunikation.erzeugeEngstelle(_engstelleDatensatz);
			} else
			{
				MessageBox mb = new MessageBox(new Shell(Display.getCurrent()),
						SWT.ICON_INFORMATION | SWT.OK);
				mb.setText("Fehler Definition");
				mb.setMessage("Engstelle schon vorhanden: "
						+ _engstelleDatensatz.getPid());
				mb.open();
				_schlieenOk = false;
				return;
			}

		}

		DataModel dataModel = _daVKommunikation.holeDaVVerbindung().getDataModel();

		List<ResultData> list = new ArrayList<ResultData>();
		if (_engstelleDatensatz.getGewaehlterEngstellenTyp().equals(
				Engstelle.EnumEngstellenTypen.Unfall))
		{

			AttributeGroup atg1 = dataModel
					.getAttributeGroup("atg.unfallEigenschaften");
			Aspect asp1 = dataModel.getAspect("asp.parameterVorgabe");
			Data data1 = _daVKommunikation.holeDaVVerbindung().createData(atg1);
			data1.getItem("RestKapazitt").asScaledValue().set(
					_engstelleDatensatz.getRestkapazitaet());

			ResultData resultData1 = new ResultData(_daVKommunikation
					.holeDaVVerbindung().getDataModel().getObject(
							_engstelleDatensatz.getPid()), new DataDescription(atg1, asp1),
					new Date().getTime(), data1);
			list.add(resultData1);

		} else if (_engstelleDatensatz.getGewaehlterEngstellenTyp().equals(
				Engstelle.EnumEngstellenTypen.Baustelle))
		{

			AttributeGroup atg1 = dataModel
					.getAttributeGroup("atg.baustellenEigenschaften");
			Aspect asp1 = dataModel.getAspect("asp.parameterVorgabe");
			Data data1 = _daVKommunikation.holeDaVVerbindung().createData(atg1);

			data1.getItem("Status").asTextValue().setText(
					_engstelleDatensatz.getGewaehlterStatusBaustelle().toString());
			data1.getItem("RestKapazitt").asScaledValue().set(
					_engstelleDatensatz.getRestkapazitaet());
			data1.getItem("Veranlasser").asTextValue().setText("VRZ-intern");

			ResultData resultData1 = new ResultData(_daVKommunikation
					.holeDaVVerbindung().getDataModel().getObject(
							_engstelleDatensatz.getPid()), new DataDescription(atg1, asp1),
					new Date().getTime(), data1);
			list.add(resultData1);

		}

		AttributeGroup atg = dataModel
				.getAttributeGroup("atg.situationsEigenschaften");
		Aspect asp = dataModel.getAspect("asp.parameterVorgabe");
		Data data = _daVKommunikation.holeDaVVerbindung().createData(atg);

		data.getItem("StartZeit").asTimeValue().setMillis(
				_engstelleDatensatz.getZeitlicheGueltigkeitVon());

		long dauer = _engstelleDatensatz.getZeitlicheGueltigkeitBis()
				- _engstelleDatensatz.getZeitlicheGueltigkeitVon();
		data.getItem("Dauer").asTimeValue().setMillis(dauer);

		FilterStrassenSegmentOffset filter1 = _engstelleDatensatz
				.getVonDatenStrassenSegmentOffset();
		FilterStrassenSegmentOffset filter2 = _engstelleDatensatz
				.getBisDatenStrassenSegmentOffset();

		int loc = 0;
		if (filter1.getPidSelektiertesStrassenSegment() != null)
		{
			loc++;
		} else
			data.getItem("StartOffset").asScaledValue().set(0L);

		if (filter1.getPidSelektiertesStrassenSegment() != null)
		{
			loc++;
		}

		Array array = data.getItem("StraenSegment").asArray();
		array.setLength(loc);

		if (filter1.getPidSelektiertesStrassenSegment() != null)
		{
			String pid1 = filter1.getPidSelektiertesStrassenSegment();
			SystemObject so1 = _daVKommunikation.holeDaVVerbindung().getDataModel()
					.getObject(pid1);
			array.getItem(0).asReferenceValue().setSystemObject(so1);
			data.getItem("StartOffset").asScaledValue().set(filter1.getKm());
		}

		if (filter2.getPidSelektiertesStrassenSegment() != null)
		{
			String pid2 = filter2.getPidSelektiertesStrassenSegment();
			SystemObject so2 = _daVKommunikation.holeDaVVerbindung().getDataModel()
					.getObject(pid2);
			if (loc == 2)
				array.getItem(1).asReferenceValue().setSystemObject(so2);
			else
				array.getItem(0).asReferenceValue().setSystemObject(so2);

			data.getItem("EndOffset").asScaledValue().set(filter2.getKm());
		} else
			data.getItem("EndOffset").asScaledValue().set(0L);

		ResultData resultData = new ResultData(_daVKommunikation
				.holeDaVVerbindung().getDataModel().getObject(
						_engstelleDatensatz.getPid()), new DataDescription(atg, asp),
				new Date().getTime(), data);
		list.add(resultData);

		UrlasserDialog urlasserDialog = new UrlasserDialog(_shell);
		_schlieenOk = (urlasserDialog.anzeigen(list)).getEingabeOk();

	}

	private void fehlerDialog(String titel, String meldung)
	{
		MessageBox mb = new MessageBox(_shell, SWT.ICON_ERROR | SWT.OK);
		mb.setText(titel);
		mb.setMessage(meldung);
		mb.open();
	}

	/**
	 * Schliet den Dialog.
	 */
	private void schliesseDialog()
	{
		if (_schlieenOk)
		{
			_shell.dispose();
		}
	}

	/**
	 * bernimmt die aktuellen Einstellungen. Dabei wird geprft: <br>
	 * <ul>
	 * <li>dass ein Name eingegeben und ein Engstellentyp ausgewhlt wurde,</li>
	 * <li>ob ein Status ausgewhlt wurde, wenn "Baustelle" ausgewhlt wurde,</li>
	 * <li>ob eine Restkapazitt eingegeben wurde.</li>
	 * <li>ob "Lage von" und "Lage bis" ausgewhlt wurden.</li> *
	 * <li>ob die Zeitangaben gltig sind.</li>
	 * </ul>
	 * <br>
	 * Falls nicht, wird eine entsprechende Fehlermeldung ausgegeben.
	 * 
	 */
	private void aktionOk()
	{
		sichereErfassteDaten();

		if (_schlieenOk)
		{
			erzeugeUndParametriere();
			schliesseDialog();
		}
	}

	/**
	 * Schliet den Dialog ohne bernahme der getroffenen Eingstellungen.
	 * 
	 */
	private void aktionAbbrechen()
	{
		_schlieenOk = true;
		_engstelleDatensatz = null;
		schliesseDialog();
	}

	/**
	 * Zeigt die Hilfe zum Dialog {@link DefinitionsdialogAlt} an.
	 * 
	 */
	private void aktionHilfe()
	{
		String contextId = Aktivator.PLUGIN_ID + ".engstellenverwaltung";
		_debug.fine("ContextID: " + contextId);
		PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
	}

	/**
	 * Erzeugt bereichOben
	 * 
	 */
	private void erzeugeBereichOben()
	{
		GridData gridDataTextBane = new GridData();
		gridDataTextBane.horizontalAlignment = GridData.FILL;
		gridDataTextBane.grabExcessHorizontalSpace = true;
		gridDataTextBane.verticalAlignment = GridData.CENTER;
		GridLayout gridLayoutOben = new GridLayout();
		gridLayoutOben.numColumns = 2;
		GridData gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;
		gridData.horizontalAlignment = GridData.FILL;
		_bereichOben = new Composite(_shell, SWT.NONE);
		_bereichOben.setLayoutData(gridData);
		_bereichOben.setLayout(gridLayoutOben);
		_labelName = new Label(_bereichOben, SWT.NONE);
		_labelName.setText("Name:");
		_textName = new Text(_bereichOben, SWT.BORDER);
		_textName.setLayoutData(gridDataTextBane);
		_labelTypEngstelle = new Label(_bereichOben, SWT.NONE);
		_labelTypEngstelle.setText("Typ Engstelle:");
		erzeugeComboTypEngstelle();
		_labelStatusBaustelle = new Label(_bereichOben, SWT.NONE);
		_labelStatusBaustelle.setText("Status Baustelle:");
		erzeugeComboStatusBaustelle();
		_labelRestkapazitaet = new Label(_bereichOben, SWT.NONE);
		_labelRestkapazitaet.setText("Restkapazitt:");
		erzeugeBereichRestkapazitaet();
	}

	/**
	 * Initialisiert gruppeZeitlicheGueltigkeit
	 * 
	 */
	private void erzeugeGruppeZeitlicheGueltigkeit()
	{
		GridData gridDataBis = new GridData();
		gridDataBis.horizontalAlignment = GridData.FILL;
		gridDataBis.grabExcessHorizontalSpace = true;
		gridDataBis.verticalAlignment = GridData.CENTER;
		GridData gridDataVon = new GridData();
		gridDataVon.horizontalAlignment = GridData.FILL;
		gridDataVon.grabExcessHorizontalSpace = true;
		gridDataVon.verticalAlignment = GridData.CENTER;
		GridLayout gridLayoutZeitlicheGueltigkeit = new GridLayout();
		gridLayoutZeitlicheGueltigkeit.numColumns = 2;
		GridData gridDataZeitlicheGueltigkeit = new GridData();
		gridDataZeitlicheGueltigkeit.grabExcessHorizontalSpace = true;
		gridDataZeitlicheGueltigkeit.verticalAlignment = GridData.CENTER;
		gridDataZeitlicheGueltigkeit.horizontalAlignment = GridData.FILL;
		_gruppeZeitlicheGueltigkeit = new Group(_shell, SWT.NONE);
		_gruppeZeitlicheGueltigkeit.setLayoutData(gridDataZeitlicheGueltigkeit);
		_gruppeZeitlicheGueltigkeit.setLayout(gridLayoutZeitlicheGueltigkeit);
		_gruppeZeitlicheGueltigkeit.setText("Zeitliche Gltigkeit:");

		_labelVon = new Label(_gruppeZeitlicheGueltigkeit, SWT.NONE);
		_labelVon.setText("von:");

		_zeitVon = new DatumZeit(_gruppeZeitlicheGueltigkeit, SWT.NONE,
				Eingabetyp.datumuhrEinfach, false, false);
		_zeitVon.setLayoutData(gridDataVon);

		_labelBis = new Label(_gruppeZeitlicheGueltigkeit, SWT.NONE);
		_labelBis.setText("bis:");

		_zeitBis = new DatumZeit(_gruppeZeitlicheGueltigkeit, SWT.NONE,
				Eingabetyp.datumuhrEinfach, false, false);
		_zeitBis.setLayoutData(gridDataBis);
	}

	/**
	 * Initialisiert gruppeLageVon
	 * 
	 */
	private void erzeugeGruppeLageVon()
	{
		GridData gridDataVon = new GridData();
		gridDataVon.horizontalAlignment = GridData.FILL;
		gridDataVon.grabExcessHorizontalSpace = true;
		gridDataVon.grabExcessVerticalSpace = true;
		gridDataVon.verticalAlignment = GridData.FILL;
		GridData gridDataLageVon = new GridData();
		gridDataLageVon.grabExcessHorizontalSpace = true;
		gridDataLageVon.verticalAlignment = GridData.FILL;
		gridDataLageVon.grabExcessVerticalSpace = true;
		gridDataLageVon.horizontalAlignment = GridData.FILL;
		_gruppeLageVon = new Group(_shell, SWT.NONE);
		_gruppeLageVon.setLayout(new GridLayout());
		_gruppeLageVon.setLayoutData(gridDataLageVon);
		_gruppeLageVon.setText("Lage von (Engstellenanfang)");
		// _textOrtVonPlatzhalter = new Text(_gruppeLageVon, SWT.BORDER);
		// _textOrtVonPlatzhalter.setLayoutData(gridDataVon);

		_ocLageVon = new OrtsangabeComposite(_gruppeLageVon, SWT.NONE,
				_davVerbindung);
		_ocLageVon.setLayoutData(gridDataVon);
	}

	/**
	 * Initialisiert gruppeLageBis
	 * 
	 */
	private void erzeugeGruppeLageBis()
	{
		GridData gridDataBis = new GridData();
		gridDataBis.horizontalAlignment = GridData.FILL;
		gridDataBis.grabExcessHorizontalSpace = true;
		gridDataBis.grabExcessVerticalSpace = true;
		gridDataBis.verticalAlignment = GridData.FILL;
		GridData gridDataLageBis = new GridData();
		gridDataLageBis.grabExcessHorizontalSpace = true;
		gridDataLageBis.verticalAlignment = GridData.FILL;
		gridDataLageBis.grabExcessVerticalSpace = true;
		gridDataLageBis.horizontalAlignment = GridData.FILL;
		_gruppeLageBis = new Group(_shell, SWT.NONE);
		_gruppeLageBis.setLayout(new GridLayout());
		_gruppeLageBis.setLayoutData(gridDataLageBis);
		_gruppeLageBis.setText("Lage bis (Engstellenende)");
		// _textOrtBisPlatzhalter = new Text(_gruppeLageBis, SWT.BORDER);
		// _textOrtBisPlatzhalter.setLayoutData(gridDataBis);

		_ocLageBis = new OrtsangabeComposite(_gruppeLageBis, SWT.NONE,
				_davVerbindung);
		_ocLageBis.setLayoutData(gridDataBis);
	}

	/**
	 * Initialisiert bereichUnten
	 * 
	 */
	private void erzeugeBereichUnten()
	{
		GridData gridDataBereichUnten = new GridData();
		gridDataBereichUnten.grabExcessHorizontalSpace = true;
		gridDataBereichUnten.verticalAlignment = GridData.CENTER;
		gridDataBereichUnten.horizontalAlignment = GridData.FILL;
		_bereichUnten = new Composite(_shell, SWT.NONE);
		_bereichUnten.setLayout(new GridLayout());
		erzeugeBereichUntenRechts();
		_bereichUnten.setLayoutData(gridDataBereichUnten);
	}

	/**
	 * Initialisiert bereichUntenRechts
	 * 
	 */
	private void erzeugeBereichUntenRechts()
	{
		GridData gridDataBereichUntenRechts = new GridData();
		gridDataBereichUntenRechts.horizontalAlignment = GridData.END;
		gridDataBereichUntenRechts.grabExcessHorizontalSpace = true;
		gridDataBereichUntenRechts.verticalAlignment = GridData.CENTER;
		RowLayout rowLayout = new RowLayout();
		rowLayout.pack = false;
		_bereichUntenRechts = new Composite(_bereichUnten, SWT.NONE);
		_bereichUntenRechts.setLayout(rowLayout);
		_bereichUntenRechts.setLayoutData(gridDataBereichUntenRechts);
		_schaltflaecheOk = new Button(_bereichUntenRechts, SWT.NONE);
	  _schaltflaecheOk.setText("OK");
		_schaltflaecheOk.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionOk();
			}			
		});
		
		_schaltflaecheAbbrechen = new Button(_bereichUntenRechts, SWT.NONE);
		_schaltflaecheAbbrechen.setText("Abbrechen");
		_schaltflaecheAbbrechen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionAbbrechen();
			}			
		});
		
		_schaltflaecheHilfe = new Button(_bereichUntenRechts, SWT.NONE);
		_schaltflaecheHilfe.setText("Hilfe");		
		_schaltflaecheHilfe.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionHilfe();
			}			
		});
	}

	/**
	 * Initialisiert comboTypEngstelle
	 * 
	 */
	private void erzeugeComboTypEngstelle()
	{
		GridData gridDataTypEngstelle = new GridData();
		gridDataTypEngstelle.horizontalAlignment = GridData.FILL;
		gridDataTypEngstelle.grabExcessHorizontalSpace = true;
		gridDataTypEngstelle.verticalAlignment = GridData.CENTER;
		_comboTypEngstelle = new Combo(_bereichOben, SWT.READ_ONLY);
		_comboTypEngstelle.setLayoutData(gridDataTypEngstelle);
		_comboViewerTypEngstelle = new ComboViewer(_comboTypEngstelle);

		/*
		 * Verfgbare Werte fr den Engstellentyp
		 */
		Object[] engstellenTypen = new Object[] { EnumEngstellenTypen.Baustelle,
				EnumEngstellenTypen.Unfall };

		_comboViewerTypEngstelle.add(engstellenTypen);

		_comboViewerTypEngstelle
				.addSelectionChangedListener(new ISelectionChangedListener()
				{

					@Override
					public void selectionChanged(SelectionChangedEvent event)
					{
						if (event instanceof SelectionChangedEvent)
						{
							SelectionChangedEvent sce = (SelectionChangedEvent) event;
							ISelection selection = sce.getSelection();
							if (selection instanceof StructuredSelection)
							{
								StructuredSelection ss = (StructuredSelection) selection;
								Object firstElement = ss.getFirstElement();
								if (firstElement instanceof EnumEngstellenTypen)
								{
									/*
									 * Die ComboBox zur Auswahl des Baustellenstatus darf nur dann
									 * aktiv sein, wenn es sich um eine Baustelle handelt.
									 */
									if (firstElement.equals(EnumEngstellenTypen.Baustelle))
									{
										_comboStatusBaustelle.setEnabled(true);
									} else
									{
										_comboStatusBaustelle.setEnabled(false);
									}
								}
							}

						}

					}

				});

	}

	/**
	 * Initialisiert comboStatusBaustelle
	 * 
	 */
	private void erzeugeComboStatusBaustelle()
	{
		GridData gridDataStatusBaustelle = new GridData();
		gridDataStatusBaustelle.horizontalAlignment = GridData.FILL;
		gridDataStatusBaustelle.grabExcessHorizontalSpace = true;
		gridDataStatusBaustelle.verticalAlignment = GridData.CENTER;
		_comboStatusBaustelle = new Combo(_bereichOben, SWT.READ_ONLY);
		_comboStatusBaustelle.setLayoutData(gridDataStatusBaustelle);
		_comboViewerStatusBaustelle = new ComboViewer(_comboStatusBaustelle);

		/*
		 * Verfgbare Werte fr den Status der Baustelle
		 */
		Object[] statuseBaustelle = new Object[] { EnumStatusBaustelle.entworfen,
				EnumStatusBaustelle.geplant, EnumStatusBaustelle.gltig,
				EnumStatusBaustelle.storniert };

		_comboViewerStatusBaustelle.add(statuseBaustelle);

		_comboStatusBaustelle.setEnabled(false);
	}

	/**
	 * Initialisiert bereichRestkapazitaet
	 * 
	 */
	private void erzeugeBereichRestkapazitaet()
	{
		GridData gridDataFzh = new GridData();
		GridData gridDataTextRestkapazitaet = new GridData();
		gridDataTextRestkapazitaet.grabExcessHorizontalSpace = true;
		gridDataTextRestkapazitaet.horizontalAlignment = GridData.FILL;
		gridDataTextRestkapazitaet.verticalAlignment = GridData.CENTER;
		gridDataTextRestkapazitaet.grabExcessVerticalSpace = false;
		GridLayout gridLayoutBereichRestkapazitaet = new GridLayout();
		gridLayoutBereichRestkapazitaet.numColumns = 2;
		gridLayoutBereichRestkapazitaet.marginWidth = 0;
		gridLayoutBereichRestkapazitaet.horizontalSpacing = 5;
		GridData gridDataBereichRestkapazitaet = new GridData();
		gridDataBereichRestkapazitaet.horizontalAlignment = GridData.FILL;
		gridDataBereichRestkapazitaet.grabExcessHorizontalSpace = true;
		gridDataBereichRestkapazitaet.verticalAlignment = GridData.CENTER;
		_bereichRestkapazitaet = new Composite(_bereichOben, SWT.NONE);
		_bereichRestkapazitaet.setLayoutData(gridDataBereichRestkapazitaet);
		_bereichRestkapazitaet.setLayout(gridLayoutBereichRestkapazitaet);
		_textRestkapazitaet = new Text(_bereichRestkapazitaet, SWT.BORDER);
		_textRestkapazitaet.setLayoutData(gridDataTextRestkapazitaet);
		_labelFzh = new Label(_bereichRestkapazitaet, SWT.NONE);
		_labelFzh.setText("Fz/h");
		_labelFzh.setLayoutData(gridDataFzh);
	}

}
