package de.bsvrz.buv.plugin.engstvw;

import de.bsvrz.buv.rw.basislib.ort.FilterAsbOrtsangabe;
import de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer;
import de.bsvrz.buv.rw.basislib.ort.FilterKnoten;
import de.bsvrz.buv.rw.basislib.ort.FilterStrassenSegmentOffset;
import de.bsvrz.buv.rw.basislib.ort.OrtsangabeEnumeratorAuswahl;

/**
 * Liefert die Datenstruktur einer Engstelle. <br>
 * Implementiert das Interface {@link Cloneable}.
 * 
 * @version $Revision: 1.1 $ / $Date: 2010/02/02 15:21:25 $ / ($Author: HofmannJ $)
 * @author HofmannJ
 * 
 */
public class Engstelle implements Cloneable
{
  /**
   * Pid dieses Objekts im DaV
   */
  private String _pid = null;

  /**
   * Name der Engstelle
   */
  private String _name = null;

  /**
   * Mgliche Engstelltentypen
   * 
   * @author HofmannJ
   */
  public enum EnumEngstellenTypen {
    /**
     * Engstellentyp Baustelle
     */
    Baustelle,

    /**
     * Engstellentyp Unfall
     */
    Unfall
  };

  /**
   * Gewhlter Engstellentyp
   */
  private EnumEngstellenTypen _gewaehlterEngstellenTyp = null;

  /**
   * Enumerator fr den Baustellenstatus.
   * 
   * @author HofmannJ
   */
  public enum EnumStatusBaustelle {
    /**
     * Baustellenstatus entworfen
     */
    entworfen,

    /**
     * Baustellenstatus geplant
     */
    geplant,

    /**
     * Baustellenstatus gltig
     */
    gltig,

    /**
     * Baustellenstatus storiniert
     */
    storniert
  }

  /**
   * Gewhlter Baustellenstatus. Ist nur mglich, wenn der Engstellentyp auf "Baustelle" steht.
   */
  private EnumStatusBaustelle _gewaehlterStatusBaustelle = null;

  /**
   * Definition der Restkapazitt in Fz/h.
   */
  private long _restkapazitaet = 0;

  /**
   * Definiert Anfangsdatum und Anfangszeit in Millisekunden seit dem 1.1.1970 Format fr die Darstellung: TT.MM.JJJJ
   * hh:mm
   */
  private long _zeitlicheGueltigkeitVon = 0;

  /**
   * Definiert Enddatum und Enzeit in Millisekunden seit dem 1.1.1970 Format fr die Darstellung: TT.MM.JJJJ hh:mm
   */
  private long _zeitlicheGueltigkeitBis = 0;

  /**
   * Definiert, welcher Ortsangabetyp fr Lage von Engstellenanfang gewhlt wurde.
   */
  private OrtsangabeEnumeratorAuswahl _lageVonEngstellenAnfangOrtsangabeTyp = null;

  /*
   * Daten fr Lage von Engstellenanfang
   */
  private FilterAsbOrtsangabe _vonDatenAsbOrtsangabe = null;

  private FilterBetriebskilometer _vonDatenBetriebskilometer = null;

  private FilterKnoten _vonDatenKnotenOffset = null;

  private FilterStrassenSegmentOffset _vonDatenStrassenSegmentOffset = null;

  /**
   * Definiert, welcher Ortsangabetyp fr Lage bis Engstellenende gewhlt wurde.
   */
  private OrtsangabeEnumeratorAuswahl _lageBisEngstellenEndeOrtsangabeTyp = null;

  /*
   * Daten fr Lage bis Engstellenende
   */
  private FilterAsbOrtsangabe _bisDatenAsbOrtsangabe = null;

  private FilterBetriebskilometer _bisDatenBetriebskilometer = null;

  private FilterKnoten _bisDatenKnotenOffset = null;

  private FilterStrassenSegmentOffset _bisDatenStrassenSegmentOffset = null;

  /**
   * Parameterloser Konstruktor fr eine neue Instanz der Klasse {@link Engstelle}.
   * 
   */
  public Engstelle()
  {
  }

  /**
   * Erzeugt einen Clone dieser Klasse und setzt den Prfix "Kopie von " vor den Namen.
   * 
   * @return Clone dieser Klasse.
   */
  public Engstelle clone()
  {
    Engstelle engstelleDatensatz = new Engstelle();
    String name = "Kopie von " + _name;
    engstelleDatensatz._name = name;
    engstelleDatensatz._gewaehlterEngstellenTyp = _gewaehlterEngstellenTyp;
    engstelleDatensatz._gewaehlterStatusBaustelle = _gewaehlterStatusBaustelle;
    engstelleDatensatz._restkapazitaet = _restkapazitaet;
    engstelleDatensatz._zeitlicheGueltigkeitVon = _zeitlicheGueltigkeitVon;
    engstelleDatensatz._zeitlicheGueltigkeitBis = _zeitlicheGueltigkeitBis;
    engstelleDatensatz._lageVonEngstellenAnfangOrtsangabeTyp = _lageVonEngstellenAnfangOrtsangabeTyp;
    engstelleDatensatz._vonDatenAsbOrtsangabe = _vonDatenAsbOrtsangabe;
    engstelleDatensatz._vonDatenBetriebskilometer = _vonDatenBetriebskilometer;
    engstelleDatensatz._vonDatenKnotenOffset = _vonDatenKnotenOffset;
    engstelleDatensatz._vonDatenStrassenSegmentOffset = _vonDatenStrassenSegmentOffset;
    engstelleDatensatz._lageBisEngstellenEndeOrtsangabeTyp = _lageBisEngstellenEndeOrtsangabeTyp;
    engstelleDatensatz._bisDatenAsbOrtsangabe = _bisDatenAsbOrtsangabe;
    engstelleDatensatz._bisDatenBetriebskilometer = _bisDatenBetriebskilometer;
    engstelleDatensatz._bisDatenKnotenOffset = _bisDatenKnotenOffset;
    engstelleDatensatz._bisDatenStrassenSegmentOffset = _bisDatenStrassenSegmentOffset;
    return engstelleDatensatz;
  }

  /**
   * Liefert die Pid dieses Objekts im DaV
   * 
   */
  public void setPid(String pid)
  {
    _pid = pid;
  }

  /**
   * Setzt die Pid dieses Objekts im DaV
   * 
   */
  public String getPid()
  {
    return _pid;
  }

  /**
   * Liefert den Namen der Engstelle.
   * 
   * @return Name der Engstelle.
   */
  public String getName()
  {
    return _name;
  }

  /**
   * Setzt den Namen der Engstelle.
   * 
   * @param _name
   *          Name der Engstelle.
   */
  public void setName(String _name)
  {
    this._name = _name;
  }

  /**
   * Liefert den gewhlten Engstellentyp.
   * 
   * @return Gewhlter Engstellentyp.
   */
  public EnumEngstellenTypen getGewaehlterEngstellenTyp()
  {
    return _gewaehlterEngstellenTyp;
  }

  /**
   * Setzt den EngstellenTyp.
   * 
   * @param engstellenTyp
   *          EngstellenTyp.
   */
  public void setGewaehlterEngstellenTyp(EnumEngstellenTypen engstellenTyp)
  {
    _gewaehlterEngstellenTyp = engstellenTyp;
  }

  /**
   * Liefert den Status der Baustelle ({@link Engstelle#_gewaehlterStatusBaustelle}), wenn die Engstelle ({@link Engstelle#_gewaehlterEngstellenTyp})
   * vom Typ Baustelle ist, sonst wird <code>null</code> geliefert.
   * 
   * @return Status der Baustelle.
   */
  public EnumStatusBaustelle getGewaehlterStatusBaustelle()
  {
    return _gewaehlterStatusBaustelle;
  }

  /**
   * Setzt den Status der Baustelle ({@link Engstelle#_gewaehlterStatusBaustelle}), wenn die Engstelle ({@link Engstelle#_gewaehlterEngstellenTyp})
   * vom Typ Baustelle ist.
   * 
   * @param statusBaustelle
   *          Status der Baustelle.
   */
  public void setGewaehlterStatusBaustelle(EnumStatusBaustelle statusBaustelle)
  {
    _gewaehlterStatusBaustelle = statusBaustelle;
  }

  /**
   * Liefert die Restkapazitt der Engstelle in Fz/h.
   * 
   * @return Restkapazitt der Engstelle in Fz/h.
   */
  public long getRestkapazitaet()
  {
    return _restkapazitaet;
  }

  /**
   * Setzt die Restkapazitt der Engstelle in Fz/h.
   * 
   * @param _restkapazitaet
   *          Restkapazitt der Engstelle in Fz/h.
   */
  public void setRestkapazitaet(long _restkapazitaet)
  {
    this._restkapazitaet = _restkapazitaet;
  }

  /**
   * Liefert die zeitliche Gltigkeit, von der an die Engstelle gltig ist.
   * 
   * @return Zeitliche Gltigkeit, von der an die Engstelle gltig ist.
   */
  public long getZeitlicheGueltigkeitVon()
  {
    return _zeitlicheGueltigkeitVon;
  }

  /**
   * Setzt die zeitliche Gltigkeit, von der an die Engstelle gltig ist.
   * 
   * @param gueltigkeitVon
   *          Zeitliche Gltigkeit, von der an die Engstelle gltig ist.
   */
  public void setZeitlicheGueltigkeitVon(long gueltigkeitVon)
  {
    _zeitlicheGueltigkeitVon = gueltigkeitVon;
  }

  /**
   * Liefert die zeitliche Gltigkeit, bis zu der die Engstelle gltig ist.
   * 
   * @return Zeitliche Gltigkeit, bis zu der die Engstelle gltig ist.
   */
  public long getZeitlicheGueltigkeitBis()
  {
    return _zeitlicheGueltigkeitBis;
  }

  /**
   * Setzt die zeitliche Gltigkeit, bis zu der die Engstelle gltig ist.
   * 
   * @param gueltigkeitBis
   *          Zeitliche Gltigkeit, bis zu der die Engstelle gltig ist.
   */
  public void setZeitlicheGueltigkeitBis(long gueltigkeitBis)
  {
    _zeitlicheGueltigkeitBis = gueltigkeitBis;
  }

  /**
   * Liefert den OrtsangabenTyp ({@link OrtsangabeEnumeratorAuswahl}), der fr Lage von Engstellenanfang gewhlt
   * wurde.
   * 
   * @return OrtsangabenTyp ({@link OrtsangabeEnumeratorAuswahl}), der fr Lage von Engstellenanfang gewhlt wurde.
   */
  public OrtsangabeEnumeratorAuswahl getLageVonEngstellenAnfangOrtsangabeTyp()
  {
    return _lageVonEngstellenAnfangOrtsangabeTyp;
  }

  /**
   * 
   * @param vonEngstellenAnfangOrtsangabeTyp
   */
  public void setLageVonEngstellenAnfangOrtsangabeTyp(OrtsangabeEnumeratorAuswahl vonEngstellenAnfangOrtsangabeTyp)
  {
    _lageVonEngstellenAnfangOrtsangabeTyp = vonEngstellenAnfangOrtsangabeTyp;
  }

  /**
   * Wenn es sich bei der Lage von Ortsangabe um eine ASB-Ortsangabe handelt, wird hier der erzeuge
   * {@link FilterAsbOrtsangabe} zurckgeliefert.
   * 
   * @return {@link FilterAsbOrtsangabe}.
   */
  public FilterAsbOrtsangabe getVonDatenOrtsangabe()
  {
    return _vonDatenAsbOrtsangabe;
  }

  /**
   * Wenn bei Lage von Ortsangabe eine ASB-Ortsangabe gesetzt werden soll, wird hier ein {@link FilterAsbOrtsangabe}
   * bergeben.
   * 
   * @param datenAsbOrtsangabe
   *          {@link FilterAsbOrtsangabe}
   */
  public void setVonDatenAsbOrtsangabe(FilterAsbOrtsangabe datenAsbOrtsangabe)
  {
    _vonDatenAsbOrtsangabe = datenAsbOrtsangabe;
  }

  /**
   * Liefert fr den {@link FilterBetriebskilometer} fr die Von-Ortsangabe.
   * 
   * @return {@link FilterBetriebskilometer}
   */
  public FilterBetriebskilometer getVonDatenBetriebskilometer()
  {
    return _vonDatenBetriebskilometer;
  }

  /**
   * Setzt den {@link FilterBetriebskilometer} fr die Von-Ortsangabe.
   * 
   * @param datenBetriebskilometer
   *          {@link FilterBetriebskilometer}
   */
  public void setVonDatenBetriebskilometer(FilterBetriebskilometer datenBetriebskilometer)
  {
    _vonDatenBetriebskilometer = datenBetriebskilometer;
  }

  /**
   * Liefert den {@link FilterKnoten} fr die Von-Ortsangabe.
   * 
   * @return {@link FilterKnoten}
   */
  public FilterKnoten getVonDatenKnotenOffset()
  {
    return _vonDatenKnotenOffset;
  }

  /**
   * Setzt den {@link FilterKnoten} fr die Von-Ortsangabe.
   * 
   * @param datenKnotenOffset
   *          {@link FilterKnoten}.
   */
  public void setVonDatenKnotenOffset(FilterKnoten datenKnotenOffset)
  {
    _vonDatenKnotenOffset = datenKnotenOffset;
  }

  /**
   * Liefert den {@link FilterStrassenSegmentOffset} fr die Von-Ortsangabe.
   * 
   * @return {@link FilterStrassenSegmentOffset}
   */
  public FilterStrassenSegmentOffset getVonDatenStrassenSegmentOffset()
  {
    return _vonDatenStrassenSegmentOffset;
  }

  /**
   * Setzt den {@link FilterStrassenSegmentOffset} fr die Von-Ortsangabe
   * 
   * @param datenStrassenSegmentOffset
   *          {@link FilterStrassenSegmentOffset}
   */
  public void setVonDatenStrassenSegmentOffset(FilterStrassenSegmentOffset datenStrassenSegmentOffset)
  {
    _vonDatenStrassenSegmentOffset = datenStrassenSegmentOffset;
  }

  /*
   * ===BIS=================================================
   */

  /**
   * Liefert den OrtsangabenTyp ({@link OrtsangabeEnumeratorAuswahl}), der fr Lage bis Engstelle gewhlt wurde.
   * 
   * @return OrtsangabenTyp ({@link OrtsangabeEnumeratorAuswahl}), der fr Lage bis Engstelle gewhlt wurde.
   */
  public OrtsangabeEnumeratorAuswahl getLageBisEngstellenEndeOrtsangabeTyp()
  {
    return _lageBisEngstellenEndeOrtsangabeTyp;
  }

  /**
   * Setzt den OrtsangabenTyp {@link OrtsangabeEnumeratorAuswahl}, der fr Lage bis Engstelle gesetzt werden soll.
   * 
   * @param bisEngstellenEndeOrtsangabeTyp
   *          OrtsangabenTyp {@link OrtsangabeEnumeratorAuswahl}, der fr Lage bis Engstelle gesetzt werden soll.
   */
  public void setLageBisEngstellenEndeOrtsangabeTyp(OrtsangabeEnumeratorAuswahl bisEngstellenEndeOrtsangabeTyp)
  {
    _lageBisEngstellenEndeOrtsangabeTyp = bisEngstellenEndeOrtsangabeTyp;
  }

  /**
   * Liefert den {@link FilterAsbOrtsangabe} fr die Bis-Ortsangabe.
   * 
   * @return {@link FilterAsbOrtsangabe}
   */
  public FilterAsbOrtsangabe getBisDatenAsbOrtsangabe()
  {
    return _bisDatenAsbOrtsangabe;
  }

  /**
   * Setzt den {@link FilterAsbOrtsangabe} fr die Bis-Ortsangabe.
   * 
   * @param datenAsbOrtsangabe
   *          {@link FilterAsbOrtsangabe}
   */
  public void setBisDatenAsbOrtsangabe(FilterAsbOrtsangabe datenAsbOrtsangabe)
  {
    _bisDatenAsbOrtsangabe = datenAsbOrtsangabe;
  }

  /**
   * Liefert den {@link FilterBetriebskilometer} fr die Bis-Ortsangabe.
   * 
   * @return {@link FilterBetriebskilometer}
   */
  public FilterBetriebskilometer getBisDatenBetriebskilometer()
  {
    return _bisDatenBetriebskilometer;
  }

  /**
   * Setzt den {@link FilterBetriebskilometer} fr die Bis-Ortsangabe.
   * 
   * @param datenBetriebskilometer
   *          {@link FilterBetriebskilometer}
   */
  public void setBisDatenBetriebskilometer(FilterBetriebskilometer datenBetriebskilometer)
  {
    _bisDatenBetriebskilometer = datenBetriebskilometer;
  }

  /**
   * Liefert den {@link FilterKnoten} fr die Bis-Ortsangabe.
   * 
   * @return {@link FilterKnoten}
   */
  public FilterKnoten getBisDatenKnotenOffset()
  {
    return _bisDatenKnotenOffset;
  }

  /**
   * Setzt den {@link FilterKnoten} fr die Bis-Ortsangabe.
   * 
   * @param datenKnotenOffset
   *          {@link FilterKnoten}
   */
  public void setBisDatenKnotenOffset(FilterKnoten datenKnotenOffset)
  {
    _bisDatenKnotenOffset = datenKnotenOffset;
  }

  /**
   * Liefert den {@link FilterStrassenSegmentOffset} fr die Bis-Ortsangabe.
   * 
   * @return {@link FilterStrassenSegmentOffset}
   */
  public FilterStrassenSegmentOffset getBisDatenStrassenSegmentOffset()
  {
    return _bisDatenStrassenSegmentOffset;
  }

  /**
   * Setzt den {@link FilterStrassenSegmentOffset} fr die Bis-Ortsangabe.
   * 
   * @param datenStrassenSegmentOffset
   *          {@link FilterStrassenSegmentOffset}
   */
  public void setBisDatenStrassenSegmentOffset(FilterStrassenSegmentOffset datenStrassenSegmentOffset)
  {
    _bisDatenStrassenSegmentOffset = datenStrassenSegmentOffset;
  }

}
