package de.bsvrz.buv.plugin.ereigniskal;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.swt.widgets.Display;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IBenutzerZuhoerer;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ClientApplication;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationAuthority;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.MutableSetChangeListener;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.operatingMessage.MessageCauser;
import de.bsvrz.sys.funclib.operatingMessage.MessageGrade;
import de.bsvrz.sys.funclib.operatingMessage.MessageSender;
import de.bsvrz.sys.funclib.operatingMessage.MessageState;
import de.bsvrz.sys.funclib.operatingMessage.MessageType;
import de.bsvrz.sys.funclib.vewdynobj.VerwaltungDynObj;

/**
 * Dient der Kommunikation mit dem DaV bzgl. der Ereigniskalenderverwaltung.
 * 
 * @author HofmannJ
 * 
 */
public class DaVKommunikation implements ClientReceiverInterface, MutableSetChangeListener
{

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Das Verbindungsobjekt
   */
  private ClientDavInterface _connection = null;

  /**
   * Singleton-Instanz
   */
  private static DaVKommunikation _instanz = null;
  
  /**
   * SynchronisationsObjekt fr den Singleton
   */
  private static Object _syncObject = new Object();

  /**
   * Liste der {@link IBenutzerZuhoerer}, die ber einen neue angemeldeten Benuter informiert werden mchten.
   */
  private LinkedList<IEreignisKalenderListener> _ereignisKalenderListener = new LinkedList<IEreignisKalenderListener>();

  /**
   * Liste der Ereigniskalendereintraege
   */
  private List<Eintrag> _eintraege = new LinkedList<Eintrag>();

  /**
   * Liefert die Singletoninstanz.
   * 
   * @return Instanz dieser Klasse.
   */
  public static DaVKommunikation getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new DaVKommunikation();
    }
//    if (_instanz == null) { 
//      synchronized(_syncObject) { 
//          if (_instanz == null) { 
//              _instanz = new DaVKommunikation();
//          }
//      }
//    }
    return _instanz;
  }

  /**
   * Konstruktor, setzt die Verbindung zum DaV.
   * 
   * @param connection
   *          Verbindung zum DaV.
   */
  private DaVKommunikation()
  {

    this._connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    /*
     * Hier muss auf die DaVVerbindung mit den entsprechenden Listenern gehrt werden.
     */
    IDaVVerbindungZuhoerer zuhoerer = new IDaVVerbindungZuhoerer() {
      public void verbindungAbgebaut(DaVVerbindungEreignis e)
      {
        _connection = null;
      }

      public void verbindungAufgebaut(DaVVerbindungEreignis e)
      {
        // Anmelden, wenn die Verbindung aufgebaut wurde
        _connection = e.getDaVVerbindung();
      }

    };

    if (_connection != null)
    {
      // Anmelden, wenn die Verbindung vorhanden ist.
      // Ansonsten wird dies dem Listener berlassen.
    }
    else
      _debug.error("Plugin Ereigniskalender : Connection ist null!");

  }

  /**
   * Setzt die Verbindung zum DaV.
   * 
   * @param connnection
   *          Verbindung zum DaV.
   */
  public void setzeDaVVerbindung(ClientDavInterface connnection)
  {
    this._connection = connnection;
  }

  /**
   * Holt die Verbindung zum DaV.
   * 
   * @param connnection
   *          Verbindung zum DaV.
   */
  public ClientDavInterface holeDaVVerbindung()
  {
    return _connection;
  }

  /**
   * Fgt einen Listener zur Liste der Ereigniskalenderlistener hinzu.
   * 
   * @param listener
   */
  public void hinzufuegenEreignisKalenderListener(IEreignisKalenderListener listener)
  {
    if (!_ereignisKalenderListener.contains(listener))
    {
      _ereignisKalenderListener.add(listener);
    }
    else
    {
      _debug
          .error("Der EreignisKalenderZuhrer ist bereits in der Zuhrerliste vorhanden und wird nicht erneut hinzugefgt");
    }
  }

  /**
   * Entfernt einen {@link IEreignisKalenderListener} von der Liste der Zuhrer.
   * 
   * @param benutzerZuhoerer
   *          Zu entfernender {@link IEreignisKalenderListener}.
   */
  public void entferneEreignisKalenderListener(IEreignisKalenderListener listener)
  {
    if (_ereignisKalenderListener.contains(listener))
    {
      _ereignisKalenderListener.remove(listener);
    }
    else
    {
      _debug
          .error("Der EreignisKalenderZuhrer ist in der Zuhrerliste nicht vorhanden und wird somit nicht entfernt.");
    }
  }

  /**
   * Benachrichtigt alle registrierten {@link IEreignisKalenderListener} darber, dass ein Eintrag verndert wurde.
   * 
   */
  public void benachrichtigeEreignisKalenderListener(List<Eintrag> eintraege)
  {
    EreignisKalenderEreignis ereignis = new EreignisKalenderEreignis(this, eintraege);

    Iterator<IEreignisKalenderListener> it = _ereignisKalenderListener.iterator();
    while (it.hasNext())
    {
      IEreignisKalenderListener listener = it.next();
      listener.ereignisKalenderEingang(ereignis);
    }
  }

  /**
   * Methode die das Objekt objId beim Datenverteiler abmeldet zum Lesen der Attibutgruppe atgPid unter dem Aspekt
   * aspPid.
   * 
   * @param objId
   *          Id des Objekts
   * @param atgPid
   *          Attributgruppe die abgemeldet werden soll
   * @param aspPid
   *          Apekt der abgemeldet werden soll
   */
  private void abmeldeLeseDaV(Long objId, String atgPid, String aspPid)
  {
    if (objId == null || atgPid == null || aspPid == null)
      throw new IllegalArgumentException();

    SystemObject obj = _connection.getDataModel().getObject(objId);
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.unsubscribeReceiver(this, obj, dd);
  }

  /**
   * Methode die das Objekt objId beim Datenverteiler abmeldet zum Lesen der Attibutgruppe atgPid unter dem Aspekt
   * aspPid.
   * 
   * @param objId
   *          Id des Objekts
   * @param atgPid
   *          Attributgruppe die abgemeldet werden soll
   * @param aspPid
   *          Apekt der abgemeldet werden soll
   */
  private void abmeldeLeseDaV(List objs, String atgPid, String aspPid)
  {
    if (objs == null || atgPid == null || aspPid == null)
      throw new IllegalArgumentException();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.unsubscribeReceiver(this, objs, dd);
  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet zum Lesen der Attibutgruppe atgPid unter dem
   * Aspekt aspPid.
   * 
   * @param objPid
   *          Pid des Objekts
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   * @param role
   *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   */
  private void anmeldeLeseDaV(String objPid, String atgPid, String aspPid, ReceiverRole role)
  {
    if (objPid == null || atgPid == null || aspPid == null || role == null)
      throw new IllegalArgumentException();

    SystemObject obj = _connection.getDataModel().getObject(objPid);
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, obj, dd, ReceiveOptions.normal(), role);

  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet zum Lesen der Attibutgruppe atgPid unter dem
   * Aspekt aspPid.
   * 
   * @param objPid
   *          Pid des Objekts
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   * @param role
   *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   */
  private void anmeldeLeseDaV(List objs, String atgPid, String aspPid, ReceiverRole role)
  {

    if (objs == null || atgPid == null || aspPid == null || role == null)
      throw new IllegalArgumentException();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, objs, dd, ReceiveOptions.normal(), role);

    ConfigurationObject konfigurationsObjekt = (ConfigurationObject)_connection.getLocalConfigurationAuthority();

    MutableSet ms = konfigurationsObjekt.getMutableSet("Ereignisse");

    if (ms != null)
    {
      ms.addChangeListener(this);
      _debug.fine("Listener Menge Ereignisse angemeldet.");
    }
    else{
      _debug.error("Menge Ereignisse ist null");
      ClientApplication app = _connection.getLocalApplicationObject();

      SystemObject user = (SystemObject)_connection.getLocalUser();
      MessageCauser causer = new MessageCauser(user, "Fehler: Anmelden auf Menge", app.getName());
      MessageSender instance = MessageSender.getInstance();
      instance.init(_connection, "", app.getName());
      instance.setApplicationLabel(app.getName());

      Long id = app.getId();        
      
      instance.sendMessage(id.toString(), 
          MessageType.SYSTEM_DOMAIN, 
          "", 
          MessageGrade.ERROR,
          (SystemObject)_connection.getLocalConfigurationAuthority(), 
          MessageState.NEW_MESSAGE, 
          causer,
          "Menge Ereignisse ist null!");         
    }

  }

  /**
   * Verarbeitet smtliche Updates
   */
  public void update(ResultData[] results)
  {
    List<Eintrag> listLoesch = new ArrayList<Eintrag>();
    List<Eintrag> listHinzu = new ArrayList<Eintrag>();

    for (int i = 0; i < results.length; i++)
    {

      final ResultData result = results[i];
      final Data data = result.getData();
      final SystemObject object = result.getObject();

      if (data != null)
      {

        Eintrag eintrag = new Eintrag();
        eintrag.setPid(object.getPid());
        eintrag.setName(object.getName());
        AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.ereignisEigenschaften");
        Data configData = object.getConfigurationData(atg);
        String beschreibung = configData.getItem("Ereignisbeschreibung").asTextValue().getText();
        eintrag.setBeschreibung(beschreibung);
        SystemObject typObject = configData.getItem("EreignisTypReferenz").asReferenceValue().getSystemObject();
        eintrag.setEreignisTyp(typObject);
        eintrag.setQuelle(data.getItem("Quelle").asTextValue().getText());
        SystemObject systemObject = data.getItem("SystemKalenderEintragReferenz").asReferenceValue().getSystemObject();

        long anfang = data.getItem("BeginnZeitlicheGltigkeit").asTimeValue().getMillis();
        long ende = data.getItem("EndeZeitlicheGltigkeit").asTimeValue().getMillis();

        if (anfang != ende)
        {

          eintrag.setArtDerZeitspezifikation(Eintrag._artDerZeitspezifikation.Zeitbereichsangabe);
        }
        else
        {

          eintrag.setArtDerZeitspezifikation(Eintrag._artDerZeitspezifikation.Systemkalendereintrag);
        }

        eintrag.setAnfangsZeitpunkt(anfang);
        eintrag.setEndZeitpunkt(ende);
        eintrag.setSystemKalenderEintrag(systemObject);

        List<EintragRaeumlicheGueltigkeit> list1 = new ArrayList<EintragRaeumlicheGueltigkeit>();
        ReferenceArray array1 = data.getItem("RumlicheGltigkeit").asReferenceArray();
        for (int j = 0; j < array1.getLength(); j++)
        {
          SystemObject so = array1.getReferenceValue(j).getSystemObject();
          ConfigurationObject co = (ConfigurationObject)so;
          EintragRaeumlicheGueltigkeit e = new EintragRaeumlicheGueltigkeit(so.getName(), co.getInfo().getShortInfo(),
              so);
          list1.add(e);

        }
        eintrag.setListeEintragRaeumlicheGueltigkeit(list1);

        List<EintragVerkehrlicheGueltigkeit> list2 = new ArrayList<EintragVerkehrlicheGueltigkeit>();
        Array array2 = data.getItem("VerkehrlicheGltigkeit").asArray();
        for (int j = 0; j < array2.getLength(); j++)
        {
          EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt anfBezug = null;
          EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt endBezug = null;

          long anfZeit = array2.getItem(j).getItem("ZeitDauerAnfangIntervall").asTimeValue().getMillis();
          String s1 = array2.getItem(j).getItem("ZeitBezugAnfangIntervall").asTextValue().getText();
          long endZeit = array2.getItem(j).getItem("ZeitDauerEndeIntervall").asTimeValue().getMillis();
          String s2 = array2.getItem(j).getItem("ZeitBezugEndeIntervall").asTextValue().getText();

          if (s1.equals(EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.vorAnfang.toString()))
            anfBezug = EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.vorAnfang;
          else if (s1.equals(EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.nachAnfang.toString()))
            anfBezug = EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.nachAnfang;
          else if (s1.equals(EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.vorEnde.toString()))
            anfBezug = EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.vorEnde;
          else if (s1.equals(EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.nachEnde.toString()))
            anfBezug = EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.nachEnde;

          if (s2.equals(EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.vorAnfang.toString()))
            endBezug = EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.vorAnfang;
          else if (s2.equals(EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.nachAnfang.toString()))
            endBezug = EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.nachAnfang;
          else if (s2.equals(EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.vorEnde.toString()))
            endBezug = EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.vorEnde;
          else if (s2.equals(EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.nachEnde.toString()))
            endBezug = EintragVerkehrlicheGueltigkeit._Bezugszeitpunkt.nachEnde;

          EintragVerkehrlicheGueltigkeit e = new EintragVerkehrlicheGueltigkeit(anfZeit, anfBezug, endZeit, endBezug);
          list2.add(e);

        }
        eintrag.setListeEintragVerkehrlicheGueltigkeit(list2);

        String vorhanden = null;

        for (de.bsvrz.buv.plugin.ereigniskal.Eintrag e : _eintraege)
        {
          if (e.getPid().equals(eintrag.getPid()))
            vorhanden = e.getPid();
        }

        if (vorhanden == null)
          _eintraege.add(eintrag);
        else
        {
          for (Eintrag e : _eintraege)
          {
            if (e.getPid().equals(eintrag.getPid()))
            {
              listLoesch.add(e);
              listHinzu.add(eintrag);

            }
          }
        }
      }
    }

    _eintraege.removeAll(listLoesch);
    _eintraege.addAll(listHinzu);

    Display.getDefault().asyncExec(new Runnable() {
      public void run()
      {
        benachrichtigeEreignisKalenderListener(_eintraege);
      }
    });

  }

  public List<Eintrag> getEreignisKalenderEintraege()
  {

    return _eintraege;

  }

  @Override
  public void update(MutableSet set, SystemObject[] addedObjects, SystemObject[] removedObjects)
  {
    _debug.fine("update Menge: " + set.getName());

    if (set.getName().equals("Ereignisse"))
    {

      if (addedObjects != null)
      {
        for (int i = 0; i < addedObjects.length; i++)
        {

          SystemObject so = addedObjects[i];

          _debug.fine("added - " + addedObjects[i]);

          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.ereignisParameter");
          Aspect asp = _connection.getDataModel().getAspect("asp.parameterSoll");
          DataDescription dd = new DataDescription(atg, asp);

          _connection.subscribeReceiver(this, so, dd, ReceiveOptions.normal(), ReceiverRole.receiver());

        }

      }

      List<Eintrag> list = new ArrayList<Eintrag>();

      if (removedObjects != null)
      {
        for (int i = 0; i < removedObjects.length; i++)
        {
          SystemObject so = removedObjects[i];

          _debug.fine("removed - " + removedObjects[i]);

          for (Eintrag e : _eintraege)
          {
            if (e.getPid().equals(so.getPid()))
            {

              list.add(e);
              break;
            }
          }

        }

      }

      _eintraege.removeAll(list);

      Display.getDefault().asyncExec(new Runnable() {
        public void run()
        {
          benachrichtigeEreignisKalenderListener(_eintraege);
        }
      });

    }

  }

  /**
   * Erzeugt ein Ereignis anhand des bergegeben eintags
   * 
   * @param eintrag
   */
  public void erzeugeEreignisKalenderEintrag(Eintrag eintrag)
  {

    DataModel dataModel = _connection.getDataModel();

    // ConfigurationArea configArea = dataModel.getConfigurationArea("kb.default.testKonfiguration");
    // ConfigurationObject configObj = (ConfigurationObject)dataModel.getObject("kv.testKonfiguration");

    ConfigurationArea configArea = bestimmeDefaultBereich(_connection);
    ConfigurationAuthority configObj = _connection.getLocalConfigurationAuthority();

    DynamicObjectType dynObjTyp = (DynamicObjectType)dataModel.getType("typ.ereignis");
    AttributeGroup attributgruppe = dataModel.getAttributeGroup("atg.ereignisEigenschaften");
    Aspect aspekt = dataModel.getAspect("asp.eigenschaften");
    VerwaltungDynObj vewEigenschaften = new VerwaltungDynObj(_connection, dataModel, configArea, dynObjTyp, configObj,
        attributgruppe, aspekt);

    //if (dataModel.getObject(eintrag.getPid()) == null)
    {

      Data data = _connection.createData(attributgruppe);
      data.getItem("Ereignisbeschreibung").asTextValue().setText(eintrag.getBeschreibung());
      data.getItem("EreignisTypReferenz").asReferenceValue().setSystemObject(eintrag.getEreignisTyp());
      Data[] datas = new Data[1];
      datas[0] = data;
      vewEigenschaften.erzeuge(eintrag.getPid(), eintrag.getName(), "Ereignisse", datas);

    }

  }

  /**
   * Loescht das Ereignis mit der angegebenen Pid
   * 
   * @param pid
   */
  public void loescheEreignisKalenderEintrag(String pid)
  {

    DataModel dataModel = _connection.getDataModel();

    AttributeGroup atg = dataModel.getAttributeGroup("atg.ereignisParameter");
    Aspect asp = dataModel.getAspect("asp.parameterSoll");
    DataDescription dd = new DataDescription(atg, asp);

    _connection.unsubscribeReceiver(this, dataModel.getObject(pid), dd);

    // ConfigurationArea configArea = dataModel.getConfigurationArea("kb.default.testKonfiguration");
    // ConfigurationObject confObject = (ConfigurationObject)dataModel.getObject("kv.testKonfiguration");

    ConfigurationArea configArea = bestimmeDefaultBereich(_connection);
    ConfigurationAuthority confObject = _connection.getLocalConfigurationAuthority();

    DynamicObjectType dynObjType = (DynamicObjectType)dataModel.getType("typ.ereignis");

    AttributeGroup attributgruppe = dataModel.getAttributeGroup("atg.ereignisEigenschaften");
    Aspect aspekt = dataModel.getAspect("asp.eigenschaften");
    VerwaltungDynObj vewEigenschaften = new VerwaltungDynObj(_connection, dataModel, configArea, dynObjType,
        confObject, attributgruppe, aspekt);
    Data data = _connection.createData(attributgruppe);
    Data[] datas = new Data[1];
    datas[0] = data;
    vewEigenschaften.setDynamicObject((DynamicObject)dataModel.getObject(pid));
    vewEigenschaften.loesche(pid, "Ereignisse");

  }

  /**
   * Fhrt eine Neuanmldung der Ereignisse durch
   */
  public void anmeldenNeu()
  {

    if (_connection != null)
    {

      ConfigurationObject konfigurationsObjekt = (ConfigurationObject)_connection.getLocalConfigurationAuthority();

      ObjectSet objekte = konfigurationsObjekt.getObjectSet("Ereignisse");

      List<SystemObject> list = objekte.getElements();

      List<SystemObject> listTmp = new ArrayList<SystemObject>();
      for (SystemObject systemObject : list)
      {
        if (systemObject.isValid())
        {
          listTmp.add(systemObject);

        }
      }

      anmeldeLeseDaV(listTmp, "atg.ereignisParameter", "asp.parameterSoll", ReceiverRole.receiver());

    }

  }

  /**
   * Bestimmt den Default-Konfigurationsbereich des DaV
   * 
   * @param connection
   * @return
   */
  public ConfigurationArea bestimmeDefaultBereich(ClientDavInterface connection)
  {
    Data data = connection.getDataModel().getConfigurationAuthority().getConfigurationData(
        connection.getDataModel().getAttributeGroup("atg.konfigurationsVerantwortlicherEigenschaften"));

    if (data != null)
    {
      TextArray array = data.getTextArray("defaultBereich");

      if (array.getLength() == 1)
      {
        return (ConfigurationArea)connection.getDataModel().getObject(array.getText(0));
      }
    }

    return null;
  }
  
  public boolean isObjektVorhanden(String pid){
    
    SystemObject object = _connection.getDataModel().getObject(pid);
    
    if (object != null)
      return true;
    
    return false;
  }
  

}
