/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.plugin.pruefrw.objektzusammenstellung;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.basislib.objektzusammenstellung.Objektzusammenstellung;
import de.bsvrz.buv.rw.basislib.objektzusammenstellung.OeffneSichtObjektzusammenstellung;
import de.bsvrz.buv.rw.basislib.objektzusammenstellung.SichtObjektzusammenstellung;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Diese Klasse realisiert einen {@link ViewPart}, mit dessen Hilfe die
 * {@link Objektzusammenstellung} geffnet und die dort erfassten Daten
 * ausgelesen werden. <br>
 * <br>
 * 
 * <ul>
 * <li> Die Aktion hinter dem Button <code>buttonOeffnen</code> zeigt, wie die
 * Objektzusammenstellung geffnet wird. </li>
 * <li> Die Aktion hinter dem Button
 * <code>buttonInhalteObjektzusammentellungDarstellen</code> zeigt, wie die
 * Objekte aus der Objektzusammenstellung ausgelesen werden.</li>
 * </ul>
 * 
 * @author HofmannJ
 * 
 */
public class ViewObjektzusammenstellungTest extends ViewPart
{
	/**
	 * ID der View zur Objektzusammenstellung
	 */
	public static final String ID = "de.bsvrz.buv.plugin.beispiele.objektzusammenstellung.viewObjektzusammenstellungTest";

	private Composite top = null;

	private Group groupOben = null;

	private Group groupUnten = null;

	/**
	 * Button zum ffnen der Objektzusammenstellung
	 */
	private Button buttonOeffnen = null;

	/**
	 * Button zum Auslesen der zusammengestellten Objekte
	 */
	private Button buttonInhalteObjektzusammentellungDarstellen = null;

	/**
	 * Textfeld fr die Ausgabe
	 */
	private Text textAreaAusgabe = null;

	/**
	 * Feld zur Anzeige der SecondaryID der aktuellen View
	 */
	private Text _textSecID = null;

	@Override
	public void createPartControl(Composite parent)
	{
		top = new Composite(parent, SWT.NONE);
		top.setLayout(new GridLayout());
		createGroupOben();
		createGroupUnten();
	}

	@Override
	public void setFocus()
	{
	}

	/**
	 * This method initializes groupOben
	 * 
	 */
	private void createGroupOben()
	{
		GridData gridData11 = new GridData();
		gridData11.grabExcessHorizontalSpace = true;
		gridData11.horizontalAlignment = GridData.FILL;
		gridData11.verticalAlignment = GridData.FILL;
		gridData11.grabExcessVerticalSpace = true;
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		groupOben = new Group(top, SWT.NONE);
		groupOben.setLayout(new GridLayout());
		groupOben.setLayoutData(gridData);
		groupOben.setText("Oben");
		textAreaAusgabe = new Text(groupOben, SWT.MULTI | SWT.WRAP | SWT.V_SCROLL);
		textAreaAusgabe.setEditable(false);
		textAreaAusgabe.setLayoutData(gridData11);
	}

	/**
	 * Diese Mehtode ffnet die Objektzusammenstellung.
	 */
	private void aktionObjektZusammenstellungOeffnen()
	{
		// try
		// {
		// PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage()
		// .showView(_idObjektzusammenstellung);
		//
		// } catch (PartInitException e)
		// {
		//
		// e.printStackTrace();
		// }

		String secID = OeffneSichtObjektzusammenstellung.erzeugeNeueSicht();
		if (_textSecID != null)
		{
			_textSecID.setText(secID);
		}

		System.out.println("SecID:=" + secID);
		Iterator<String> it = OeffneSichtObjektzusammenstellung.sucheAlleSichtInstanzen().iterator();
		while (it.hasNext())
		{
			System.out.println("\t=> " + it.next());
		}

	}

	/**
	 * Diese Methode liest die zusammengestellten Objekte aus einer geffneten
	 * Objektzusammenstellung
	 */
	private void aktionObjektZusammenstellungGetDaten()
	{
		// SichtObjektzusammenstellung sichtObjektzusammenstellung =
		// (SichtObjektzusammenstellung) PlatformUI
		// .getWorkbench().getActiveWorkbenchWindow().getActivePage().findView(
		// _idObjektzusammenstellung);

		String secID = _textSecID.getText();
		SichtObjektzusammenstellung sichtObjektzusammenstellung = OeffneSichtObjektzusammenstellung
				.sucheSicht(secID);

		if (sichtObjektzusammenstellung != null)
		{
			List<SystemObject> daten = sichtObjektzusammenstellung.getListeDaten();
			Iterator<SystemObject> it = daten.iterator();
			String ausgabe = "";
			while (it.hasNext())
			{
				SystemObject so = it.next();
				// System.out.println("S: " + so.toString());
				ausgabe = ausgabe + so.toString() + "\n";

			}
			textAreaAusgabe.setText(ausgabe);
		}
	}

	/**
	 * This method initializes groupUnten
	 * 
	 */
	private void createGroupUnten()
	{

		GridData gridDataGroupUnten = new GridData();
		gridDataGroupUnten.horizontalAlignment = GridData.FILL;
		gridDataGroupUnten.grabExcessHorizontalSpace = true;
		gridDataGroupUnten.verticalAlignment = GridData.CENTER;

		RowLayout rl = new RowLayout();
		rl.pack = false;

		groupUnten = new Group(top, SWT.NONE);
		groupUnten.setLayoutData(gridDataGroupUnten);
		groupUnten.setLayout(rl);
		groupUnten.setText("Unten");

		/*
		 * Button ffnen
		 */
		buttonOeffnen = new Button(groupUnten, SWT.NONE);
		buttonOeffnen.setText("Objektzusammenstellung ffnen");
		buttonOeffnen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionObjektZusammenstellungOeffnen();
			}			
		});

		/*
		 * Inhalt Objektzusammenstellung darstellen
		 */
		buttonInhalteObjektzusammentellungDarstellen = new Button(groupUnten,
				SWT.NONE);
		buttonInhalteObjektzusammentellungDarstellen
				.setText("Inhalte einer Offenen Objektzusammenstellung darstellen");
		buttonInhalteObjektzusammentellungDarstellen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionObjektZusammenstellungGetDaten();
			}			
		});
		
		
		String ttDarstellen = "";
		ttDarstellen = ttDarstellen
				+ "Stellt den Inhalt der Objektzusammenstellung dar.\n";
		ttDarstellen = ttDarstellen
				+ "Hierzu wird die Sicht der Objektzusammenstellung\n";
		ttDarstellen = ttDarstellen
				+ "mit der im Textfeld SecondaryID angegebenen\n";
		ttDarstellen = ttDarstellen + "SecondaryID verwendet.";
		buttonInhalteObjektzusammentellungDarstellen.setToolTipText(ttDarstellen);

		/*
		 * Secondary ID Feld
		 */
		Label labelSecID = new Label(groupUnten, SWT.NONE);
		labelSecID.setText("SecondaryID:");

		_textSecID = new Text(groupUnten, SWT.BORDER);
		String ttText = "";
		ttText = ttText
				+ "In diesem Feld tragen Sie die SecondaryID der Objektzusammenstellung\n"
				+ " ein, auf der Sie arbeiten mchten. Der ToolTip des Reiters der\n"
				+ "jeweiligen Sicht Objektzusammenstellung liefert Ihnen die SecondaryID,\n"
				+ "jedoch nur wenn die betreffende Sicht den Focus hat.";
		_textSecID.setToolTipText(ttText);

		LinkedList<String> listeSecIDs = OeffneSichtObjektzusammenstellung.sucheAlleSichtInstanzen();
		if ((listeSecIDs != null) && (listeSecIDs.size() > 0))
		{
			String secID = listeSecIDs.getFirst();
			// OeffneSichtObjektzusammenstellung.oeffneSicht(secID);
			_textSecID.setText(secID);
		}
	}

}
