package de.bsvrz.buv.plugin.sim;

/**
 * Liefert die Daten, die fr den Dialog {@link Simulationssteuerung} notwendig sind.
 * 
 * 
 * @version $Revision: 1.1 $ / $Date: 2010/02/02 15:22:01 $ / ($Author: HofmannJ $)
 * @author HofmannJ
 * 
 */
public class SimulationssteuerungDaten
{
  /**
   * Verweis auf die zugehrige Simulation
   */
  private Simulation _simulation = null;

  /**
   * Reprsentiert die Zustnde einer Simulation
   * 
   * @author HofmannJ
   */
  public enum SimulationsZustand {
    /**
     * Neu
     */
    neu,
    /**
     * Vorstart
     */
    Vorstart,

    /**
     * Start
     */
    Start,

    /**
     * Pause
     */
    Pause,

    /**
     * Stop
     */
    Stop,
    /**
     * Geloescht
     */
    gelscht

  }

  private SimulationsZustand _aktuellerZustand = null;

  /**
   * Speichert die gesetzte Triggergeschwindigkeit.
   */
  private float _ablaufgeschwindigkeit = 0.0f;
  //Sans 20091124
  
  /**
   * Simulationszeitpunkt in Millisekunden seit dem 1.1.1970. Wird bei Online-Simulation nicht dargestellt.
   */
  private long _simulationszeitpunkt = 0;

  /**
   * Konstruktor fr eine Instanz der Klasse {@link SimulationssteuerungDaten}.
   * 
   * @param simulation
   *          Simulation, die gesteuert werden soll.
   */
  public SimulationssteuerungDaten(Simulation simulation)
  {
    this._simulation = simulation;
  }

  /**
   * Liefert den Namen der Simulation.
   * 
   * @return Namen der Simulation.
   */
  public String getName()
  {
    return _simulation.getName();
  }

  /**
   * Liefert die Ablaufgeschwindigkeit.
   * 
   * @return Ablaufgeschwindigkeit.
   */
  public float getAblaufgeschwindigkeit()
  {
    return _ablaufgeschwindigkeit;
  }

  /**
   * Setzt die Ablaufgeschwindigkeit.
   * 
   * @param _ablaufgeschwindigkeit
   *          Ablaufgeschwindigkeit.
   */
  public void setAblaufgeschwindigkeit(float _ablaufgeschwindigkeit)
  {
    this._ablaufgeschwindigkeit = _ablaufgeschwindigkeit;
  }

  /**
   * Liefert den Simulationszeitpunkt
   * 
   * @return Simulationszeitpunkt.
   */
  public long getSimulationszeitpunkt()
  {
    return _simulationszeitpunkt;
  }

  /**
   * Setzt den Simulationszeitpunkt.
   * 
   * @param _simulationszeitpunkt
   *          Simulationszeitpunkt.
   */
  public void setSimulationszeitpunkt(long _simulationszeitpunkt)
  {
    this._simulationszeitpunkt = _simulationszeitpunkt;
  }

  /**
   * Liefert den aktuellen Zustand der Simulation.
   * 
   * @return Aktueller Zustand der Simulation.
   */
  public SimulationsZustand getAktuellerZustand()
  {
    return _aktuellerZustand;
  }

  /**
   * Setzt den aktuellen Zustand der Simulation.
   * 
   * @param zustand
   *          Aktueller Zustand der Simulation.
   */
  public void setAktuellerZustand(SimulationsZustand zustand)
  {
    _aktuellerZustand = zustand;
  }

}
