package de.bsvrz.buv.plugin.syskal;

/**
 * Liefert die Reprsentation eines Systemkalender Eintrages.
 * 
 * @version $Revision: 1.1 $ / $Date: 2010/02/02 15:22:19 $ / ($Author: HofmannJ $)
 * @author HofmannJ
 * 
 */
public class Eintrag implements Cloneable
{
  /**
   * Pid des Systemkalendereintrags.
   */
  private String _pid = null;

  /**
   * Name des Systemkalendereintrags.
   */
  private String _name = null;

  /**
   * Enumerator fr die Art der Zeitbereichsangabe.
   * 
   * @author HofmannJ
   */
  public enum _artDerZeitspezifikation {
    Datumseingabe, Zeitbereichsangabe, FreieDefinition, VordefinierterEintrag
  };

  /**
   * Legt die Art der Zeitspezifikation ber den Enumerator <code>enumArtDerZeitspezifikation</code> fest.
   */
  private _artDerZeitspezifikation _gewaehlteArtDerZeitspezifikation = null;

  /**
   * Speichert das Datum in Millisekunden seit dem 1.1.1970 bei der Datumseingabe
   */
  private String _datum = null;

  /**
   * Speichert den Anfangszeitpunkt in Millisekunden seit dem 1.1.1970 bei der Zeitbereichsangabe.
   */
  private long _anfangsZeitpunkt = 0L;

  /**
   * Speichert den Endzeitpunkt in Millisekunden seit dem 1.1.1970 bei der Zeitbereichsangabe.
   */
  private long _endZeitpunkt = 0L;

  /**
   * Speichert die Zeitspezifikation bei der freien Definition
   */
  private String _freieDefinition = null;

  /**
   * Speichert die Zeitspezifikation bei der freien Definition
   */
  private String _vordefinierterEintrag = null;

  /**
   * Setzt die Art der Zeitspezifikation auf Datumseingabe und setzt Name des Systemkalendereintrages sowie das Datum.
   * 
   * @param name
   *          Name des Systemkalendereintrags.
   * @param datum
   *          Datum des Systemkalendereintrags.
   */
  public void setDatum(String name, String datum)
  {
    this._name = name;
    this._gewaehlteArtDerZeitspezifikation = _gewaehlteArtDerZeitspezifikation.Datumseingabe;
    this._datum = datum;
  }

  /**
   * Setzt die Art der Zeitspezifikation auf Zeitbereichsangabe
   * 
   * @param name
   *          Name des Systemkalendereintrags.
   * @param anfangsZeitpunkt
   *          Anfangszeitpunkt des Systemkalendereintrags in Millisekunden seit dem 1.1.1970.
   * @param endZeitpunkt
   *          Endzeitpunkt des Systemkalendereintrags in Millisekunden seit dem 1.1.1970.
   */
  public void setZeitbereich(String name, long anfangsZeitpunkt, long endZeitpunkt)
  {
    this._name = name;
    this._gewaehlteArtDerZeitspezifikation = _gewaehlteArtDerZeitspezifikation.Zeitbereichsangabe;
    this._anfangsZeitpunkt = anfangsZeitpunkt;
    this._endZeitpunkt = endZeitpunkt;
  }

  /**
   * Setzt die art der Zeitspezifikation auf freie Definition.
   * 
   * @param name
   *          Name des Systemkalendereintrags.
   * @param freieDefinition
   *          Freie Definition der Zeitspezifikation.
   */
  public void setFreieDefinition(String name, String freieDefinition)
  {
    this._name = name;
    this._gewaehlteArtDerZeitspezifikation = _gewaehlteArtDerZeitspezifikation.FreieDefinition;
    this._freieDefinition = freieDefinition;
  }

  /**
   * Liefert einen Clone dieser Klasse und setzt den Prfix "Kopie von " vor den Namen.
   * 
   * @return Clone dieser Klasse.
   */
  public Eintrag clone()
  {
    Eintrag systemkalenderDatensatz = new Eintrag();
    String neuerName = "Kopie von " + _name;
    systemkalenderDatensatz.setName(neuerName);

    systemkalenderDatensatz.setArtDerZeitspezifikation(_gewaehlteArtDerZeitspezifikation);

    switch (_gewaehlteArtDerZeitspezifikation)
    {
      case Datumseingabe:
        systemkalenderDatensatz.setDatum(neuerName, _datum);
        break;
      case Zeitbereichsangabe:
        systemkalenderDatensatz.setZeitbereich(neuerName, _anfangsZeitpunkt, _endZeitpunkt);
        break;
      case FreieDefinition:
        systemkalenderDatensatz.setFreieDefinition(neuerName, _freieDefinition);
        break;
      case VordefinierterEintrag:
        systemkalenderDatensatz.setArtDerZeitspezifikation(_artDerZeitspezifikation.FreieDefinition);
        systemkalenderDatensatz.setFreieDefinition(neuerName, _vordefinierterEintrag);       
        break;
    }

    return systemkalenderDatensatz;
  }

  /**
   * Liefert den Namen des Systemkalender Datensatzes.
   * 
   * @return Name des SystemkalenderDatensatzes.
   */
  public String getName()
  {
    return _name;
  }

  /**
   * Setzt den Namen des Systemkalender Datensatzes.
   * 
   * @param name
   *          Name des Systemkalender Datensatzes.
   */
  public void setName(String name)
  {
    this._name = name;
  }

  /**
   * Liefert die Art der Zeitspezifikation
   * 
   * @return Art der Zeitspezifikation.
   */
  public _artDerZeitspezifikation getArtDerZeitspezifikation()
  {
    return _gewaehlteArtDerZeitspezifikation;
  }

  /**
   * Setzt die Art der Zeitspezifikation.
   * 
   * @param artDerZeitspezifikation
   *          Art der Zeitspezifikation.
   */
  public void setArtDerZeitspezifikation(_artDerZeitspezifikation artDerZeitspezifikation)
  {
    _gewaehlteArtDerZeitspezifikation = artDerZeitspezifikation;
  }

  /**
   * Liefert das Datum bei Datumseingabe. <br>
   * Ist {@link Eintrag#_gewaehlteArtDerZeitspezifikation} nicht auf Datumseingabe gesetzt, wird -1 geliefert.
   * 
   * @return Datum in Millisekunden seit 1.1.1970.
   */
  public String getDatum()
  {
    if (_gewaehlteArtDerZeitspezifikation.equals(_gewaehlteArtDerZeitspezifikation.Datumseingabe))
    {
      return _datum;
    }
    else
      return "";

  }

  /**
   * Setzt das Datum bei Datumseingabe. <br>
   * Ist {@link Eintrag#_gewaehlteArtDerZeitspezifikation} nicht auf Datumseingabe gesetzt, wird das Datum nicht
   * gesetzt.
   * 
   * @param datum
   *          Datum in Millisekunden seit 1.1.1970.
   */
  public void setDatum(String datum)
  {
    if (_gewaehlteArtDerZeitspezifikation.equals(_gewaehlteArtDerZeitspezifikation.Datumseingabe))
    {
      this._datum = datum;
    }
  }

  /**
   * Liefert den Anfangszeitpunkt bei Zeitbereichseingabe. <br>
   * Ist {@link Eintrag#_gewaehlteArtDerZeitspezifikation} nicht auf Zeitbereichseingabe gesetzt, wird -1 geliefert.
   * 
   * @return Anfangszeitpunkt in Millisekunden seit dem 1.1.1970.
   */
  public long getAnfangsZeitpunkt()
  {
    if (_gewaehlteArtDerZeitspezifikation.equals(_gewaehlteArtDerZeitspezifikation.Zeitbereichsangabe))
    {
      return _anfangsZeitpunkt;
    }
    else
    {
      return -1;
    }

  }

  /**
   * Setzt den Anfangszeitpunkt bei Zeitbereichseingabe. Ist {@link Eintrag#_gewaehlteArtDerZeitspezifikation} nicht auf
   * Zeitbereichseingabe gesetzt, wird der Anfangszeitpunkt nicht gesetzt.
   * 
   * @param zeitpunkt
   *          Anfangszeitpunkt in Millisekunden seit dem 1.1.1970.
   */
  public void setAnfangsZeitpunkt(long zeitpunkt)
  {
    if (_gewaehlteArtDerZeitspezifikation.equals(_gewaehlteArtDerZeitspezifikation.Zeitbereichsangabe))
    {
      _anfangsZeitpunkt = zeitpunkt;
    }

  }

  /**
   * Liefert den Endzeitpunkt bei Zeitbereichseingabe. <br>
   * Ist {@link Eintrag#_gewaehlteArtDerZeitspezifikation} nicht auf Zeitbereichseingabe gesetzt, wird -1 geliefert.
   * 
   * @return Endzeitpunkt in Millisekunden seit dem 1.1.1970.
   */
  public long getEndZeitpunkt()
  {
    if (_gewaehlteArtDerZeitspezifikation.equals(_gewaehlteArtDerZeitspezifikation.Zeitbereichsangabe))
    {
      return _endZeitpunkt;
    }
    else
    {
      return -1;
    }
  }

  /**
   * Setzt den Endzeitpunkt bei Zeitbereichseingabe. Ist {@link Eintrag#_gewaehlteArtDerZeitspezifikation} nicht auf
   * Zeitbereichseingabe gesetzt, wird der Endzeitpunkt nicht gesetzt.
   * 
   * @param zeitpunkt
   *          Endzeitpunkt in Millisekunden seit dem 1.1.1970.
   */
  public void setEndZeitpunkt(long zeitpunkt)
  {
    if (_gewaehlteArtDerZeitspezifikation.equals(_gewaehlteArtDerZeitspezifikation.Zeitbereichsangabe))
    {
      _endZeitpunkt = zeitpunkt;
    }
  }

  /**
   * Liefert die freie Definition der Zeitspezifikation. <br>
   * Ist {@link Eintrag#_gewaehlteArtDerZeitspezifikation} nicht auf freie Definition gesetzt, wird <code>null</code>
   * geliefert.
   * 
   * @return Freie Definition der Zeitspezifikation.
   */
  public String getFreieDefinition()
  {
    //if (_gewaehlteArtDerZeitspezifikation.equals(_gewaehlteArtDerZeitspezifikation.FreieDefinition))
    {
      return _freieDefinition;
    }
//    else
//    {
//      return null;
//    }
  }

  /**
   * Setzt die freie Definition der Zeitspezifikation. <br>
   * Ist {@link Eintrag#_gewaehlteArtDerZeitspezifikation} nicht auf freie Definition gesetzt, wird die freie Definition
   * der Zeitspezifikation nicht gesetzt.
   * 
   * @param definition
   *          Freie Definition der Zeitspezifikation.
   */
  public void setFreieDefinition(String definition)
  {
    if (_gewaehlteArtDerZeitspezifikation.equals(_gewaehlteArtDerZeitspezifikation.FreieDefinition))
    {
      _freieDefinition = definition;
    }
  }

  /**
   * Liefert die Pid des Systemkalendereintrags.
   * 
   * @return Pid des Systemkalendereintrags.
   */
  public String getPid()
  {
    return _pid;
  }

  /**
   * Setzt die Pid des Systemkalendereintrags.
   * 
   * @param pid
   *          Pid des Systemkalendereintrags.
   */
  public void setPid(String pid)
  {
    this._pid = pid;
  }

  public String getVordefinierterEintrag()
  {
    if (_gewaehlteArtDerZeitspezifikation.equals(_gewaehlteArtDerZeitspezifikation.VordefinierterEintrag))
    {
      return _vordefinierterEintrag;
    }
    else
    {
      return null;
    }
  }

  public void setVordefinierterEintrag(String definiton)
  {
    if (_gewaehlteArtDerZeitspezifikation.equals(_gewaehlteArtDerZeitspezifikation.VordefinierterEintrag))
    {
      this._vordefinierterEintrag = definiton;
    }
  }

}
