/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender;

import static org.junit.Assert.assertEquals;

import java.util.Date;
import java.util.Iterator;
import java.util.TreeMap;
import java.util.Map.Entry;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.buv.rw.basislib.kalender.ZeitDauer.SelektierInfo;
import de.bsvrz.buv.rw.basislib.kalender.ZeitDauer.ValidierungsInfo;

/**
 * @author HofmannJ
 * 
 */
public class ZeitDauerTest
{
	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception
	{
	}

	/**
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception
	{
	}

	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception
	{
	}

	/**
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception
	{
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.kalender.ZeitDauer#bestimmeZeitklassenZugehoerigkeit(java.lang.String)}.
	 */
	@Test
	public void testBestimmeZeitklassenZugehoerigkeit()
	{
		ZeitDauer zeitDauer = new ZeitDauer(new Date().getTime());

		boolean korrekt = true;

		/*
		 * Jahre
		 */
		if (korrekt)
		{
			if (!((zeitDauer.bestimmeZeitklassenZugehoerigkeit("Jahr")
					.equals(zeitDauer.getZeitKlasseJahr()))
					&& (zeitDauer.bestimmeZeitklassenZugehoerigkeit("Jahre")
							.equals(zeitDauer.getZeitKlasseJahr())) && (zeitDauer
					.bestimmeZeitklassenZugehoerigkeit("J").equals(zeitDauer
					.getZeitKlasseJahr()))))
			{
				korrekt = false;
			}
		}

		/*
		 * Monate
		 */
		if (korrekt)
		{
			if (!((zeitDauer.bestimmeZeitklassenZugehoerigkeit("Monat")
					.equals(zeitDauer.getZeitKlasseMonat()))
					&& (zeitDauer.bestimmeZeitklassenZugehoerigkeit("Monate")
							.equals(zeitDauer.getZeitKlasseMonat())) && (zeitDauer
					.bestimmeZeitklassenZugehoerigkeit("M").equals(zeitDauer
					.getZeitKlasseMonat()))))
			{
				korrekt = false;
			}
		}

		/*
		 * Tage
		 */
		if (korrekt)
		{
			if (!((zeitDauer.bestimmeZeitklassenZugehoerigkeit("Tag")
					.equals(zeitDauer.getZeitKlasseTag()))
					&& (zeitDauer.bestimmeZeitklassenZugehoerigkeit("Tage")
							.equals(zeitDauer.getZeitKlasseTag())) && (zeitDauer
					.bestimmeZeitklassenZugehoerigkeit("T").equals(zeitDauer
					.getZeitKlasseTag()))))
			{
				korrekt = false;
			}
		}

		/*
		 * Stunden
		 */
		if (korrekt)
		{
			if (!((zeitDauer.bestimmeZeitklassenZugehoerigkeit("Stunde")
					.equals(zeitDauer.getZeitKlasseStunde()))
					&& (zeitDauer.bestimmeZeitklassenZugehoerigkeit("Stunden")
							.equals(zeitDauer.getZeitKlasseStunde())) && (zeitDauer
					.bestimmeZeitklassenZugehoerigkeit("h").equals(zeitDauer
					.getZeitKlasseStunde()))))
			{
				korrekt = false;
			}
		}

		/*
		 * Minuten
		 */
		if (korrekt)
		{
			if (!((zeitDauer.bestimmeZeitklassenZugehoerigkeit("Minute")
					.equals(zeitDauer.getZeitKlasseMinute()))
					&& (zeitDauer.bestimmeZeitklassenZugehoerigkeit("Minuten")
							.equals(zeitDauer.getZeitKlasseMinute())) && (zeitDauer
					.bestimmeZeitklassenZugehoerigkeit("m").equals(zeitDauer
					.getZeitKlasseMinute()))))
			{
				korrekt = false;
			}
		}

		/*
		 * Sekunden
		 */
		if (korrekt)
		{
			if (!((zeitDauer.bestimmeZeitklassenZugehoerigkeit("Sekunde")
					.equals(zeitDauer.getZeitKlasseSekunde()))
					&& (zeitDauer.bestimmeZeitklassenZugehoerigkeit("Sekunden")
							.equals(zeitDauer.getZeitKlasseSekunde())) && (zeitDauer
					.bestimmeZeitklassenZugehoerigkeit("s").equals(zeitDauer
					.getZeitKlasseSekunde()))))
			{
				korrekt = false;
			}
		}

		/*
		 * Tausendstel
		 */
		if (korrekt)
		{
			if (!((zeitDauer.bestimmeZeitklassenZugehoerigkeit("Tausendstel")
					.equals(zeitDauer.getZeitKlasseTausendstel()))
					&& (zeitDauer.bestimmeZeitklassenZugehoerigkeit("Tausendstel")
							.equals(zeitDauer.getZeitKlasseTausendstel())) && (zeitDauer
					.bestimmeZeitklassenZugehoerigkeit("t").equals(zeitDauer
					.getZeitKlasseTausendstel()))))
			{
				korrekt = false;
			}
		}

		assertEquals(true, korrekt);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.kalender.ZeitDauer#verifyEingabe(java.lang.String, int, java.lang.String, boolean)}.
	 */
	@Test
	public void testVerifyEingabeStringIntStringBoolean()
	{
		ZeitDauer zeitDauer = new ZeitDauer(0);

		boolean korrekt = true;

		/*
		 * Keine Zahl am Anfang
		 */
		if (zeitDauer.verifyEingabe("A", 0, "", true))
		{
			korrekt = false;
		}

		/*
		 * 2 * Leerzeichen
		 */
		if (zeitDauer.verifyEingabe(" ", 3, "12 ", true))
		{
			korrekt = false;
		}

		/*
		 * Zahl Leerzeichen Zahl
		 */
		if (zeitDauer.verifyEingabe("1", 3, "12 ", true))
		{
			korrekt = false;
		}

		/*
		 * Ungltiges Zeichen
		 */
		if (zeitDauer.verifyEingabe("b", 3, "12 ", true))
		{
			korrekt = false;
		}

		/*
		 * Ungltige Zeitklasse
		 */
		if (zeitDauer.verifyEingabe("g", 6, "12 Tag", true))
		{
			korrekt = false;
		}

		assertEquals(true, korrekt);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.kalender.ZeitDauer#bestimmeNaechstesZahlenfeld(java.lang.String, int, int, int)}.
	 */
	@Test
	public void testBestimmeNaechstesZahlenfeld()
	{
		ZeitDauer zeitDauer = new ZeitDauer(new Date().getTime());

		String text = "12 Stunden 13 Minuten 15 Sekunden";

		SelektierInfo sel1 = zeitDauer
				.bestimmeNaechstesZahlenfeld(text, 14, -1, -1);

		SelektierInfo sel2 = zeitDauer.bestimmeNaechstesZahlenfeld(text, 3, 0, 1);

		SelektierInfo sel3 = zeitDauer.bestimmeNaechstesZahlenfeld(text, 5, 4, 5);

		assertEquals(true, ((sel1.getStartPos() == 22) && (sel1.getEndPos() == 23))
				&& ((sel2.getStartPos() == 11) && (sel2.getEndPos() == 12))
				&& ((sel3.getStartPos() == 11) && (sel3.getEndPos() == 12)));
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.kalender.ZeitDauer#bestimmeVorhergehendesZahlenfeld(java.lang.String, int, int, int)}.
	 */
	@Test
	public void testBestimmeVorhergehendesZahlenfeld()
	{
		ZeitDauer zeitDauer = new ZeitDauer(new Date().getTime());

		String text = "12 Stunden 13 Minuten 15 Sekunden";

		SelektierInfo sel1 = zeitDauer.bestimmeVorhergehendesZahlenfeld(text, 14,
				-1, -1);

		SelektierInfo sel2 = zeitDauer.bestimmeVorhergehendesZahlenfeld(text, 1, 0,
				2);

		SelektierInfo sel3 = zeitDauer.bestimmeVorhergehendesZahlenfeld(text, 5, 4,
				5);

		assertEquals(true, ((sel1.getStartPos() == 11) && (sel1.getEndPos() == 12))
				&& ((sel2.getStartPos() == 22) && sel2.getEndPos() == 23)
				&& ((sel3.getStartPos() == 0) && (sel3.getEndPos() == 1)));
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.kalender.ZeitDauer#bestimmeMarkierung(java.lang.String, int)}.
	 */
	@Test
	public void testBestimmeMarkierung()
	{
		ZeitDauer zeitDauer = new ZeitDauer(new Date().getTime());

		String text = "12 Stunden 13 Minuten 15 Sekunden";

		int[] mark1 = zeitDauer.bestimmeMarkierung(text, 0);
		int[] mark2 = zeitDauer.bestimmeMarkierung(text, 2);
		int[] mark3 = zeitDauer.bestimmeMarkierung(text, 1);
		int[] mark4 = zeitDauer.bestimmeMarkierung(text, 4);

		assertEquals(true, ((mark1[0] == 0) && (mark1[1] == 1))
				&& ((mark2[0] == 0) && (mark2[1] == 1))
				&& ((mark3[0] == 0) && (mark3[1] == 1)) && (mark4 == null));
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.kalender.ZeitDauer#wertAendern(java.lang.String, java.lang.String, int, int)}.
	 */
	@Test
	public void testWertAendern()
	{
		boolean korrekt = true;

		ZeitDauer zeitDauer = new ZeitDauer(new Date().getTime());

		String text = "12 Stunden 13 Minuten 15 Sekunden";

		SelektierInfo wertAendern = zeitDauer.wertAendern(text, "12", 2, 10);
		if (!("22 Stunden 13 Minuten 15 Sekunden"
				.equals(wertAendern.getNeuerText())))
		{
			korrekt = false;
		}

		if (korrekt)
		{
			if (!((wertAendern.getStartPos() == 0) && (wertAendern.getEndPos() == 1)))
			{
				korrekt = false;
			}
		}

		/*
		 * Minus
		 */
		text = "12 Stunden 13 Minuten 15 Sekunden";
		SelektierInfo wertAendernMinus = zeitDauer.wertAendern(text, "13", 13, -13);

		if (!("12 Stunden 0 Minuten 15 Sekunden".equals(wertAendernMinus
				.getNeuerText())))
		{
			korrekt = false;
		}

		if (korrekt)
		{
			if (!((wertAendernMinus.getStartPos() == 11) && (wertAendernMinus
					.getEndPos() == 11)))
			{
				korrekt = false;
			}
		}

		assertEquals(true, korrekt);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.kalender.ZeitDauer#getListeGueltigerStrings()}.
	 */
	@Test
	public void testGetListeGueltigerStrings()
	{
		ZeitDauer zeitDauer = new ZeitDauer(new Date().getTime());

		TreeMap<String, String> tm = zeitDauer.getListeGueltigerStrings();

		String aktuell = "";
		Iterator<Entry<String, String>> it = tm.entrySet().iterator();
		while (it.hasNext())
		{
			String value = it.next().getValue();
			aktuell = aktuell + value;
		}

		assertEquals(" 0123456789adeghijklmnorstu", aktuell);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.kalender.ZeitDauer#validiereFelder(java.lang.String)}.
	 */
	@Test
	public void testValidiereFelder()
	{
		ZeitDauer zeitDauer = new ZeitDauer(new Date().getTime());

		boolean erfolgreich = true;

		String text = "12 Stunden 13 Minuten 15 Sekunden";

		ValidierungsInfo info = zeitDauer.validiereFelder(text);
		if (!(info.getFeldInfos().size() == 3
				&& info.getNichtBestimmbareFelder().size() == 0 && info
				.getSonstigeInfos().length() == 0))
		{
			erfolgreich = false;
		}

		text = "12 Stunden 13 Minuten 15 Sek1den";
		info = zeitDauer.validiereFelder(text);
		if (!(info.getFeldInfos().size() == 2 && info.getNichtBestimmbareFelder()
				.size() == 1))
		{
			erfolgreich = false;
		}

		assertEquals(true, erfolgreich);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.kalender.ZeitDauer#getZeitdauerInBezugAufAnfangszeitpunkt(java.lang.String)}.
	 */
	@Test
	public void testGetZeitdauerInBezugAufAnfangszeitpunkt()
	{
		ZeitDauer zeitDauer = new ZeitDauer(new Date().getTime());

		long dauer = zeitDauer
				.getZeitdauerInBezugAufAnfangszeitpunkt("12 Stunden 12 Tausendstel");

		assertEquals("43200012", "" + dauer);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.kalender.ZeitDauer#setZeitdauerInBezugAufAnfangszeitpunkt(long, long)}.
	 */
	@Test
	public void testSetZeitdauerInBezugAufAnfangszeitpunkt()
	{
		ZeitDauer zeitDauer = new ZeitDauer(0);

		String string = zeitDauer.setZeitdauerInBezugAufAnfangszeitpunkt(0,
				412221412321l);

		assertEquals(
				"13 Jahre 0 Monate 23 Tage 1 Stunde 56 Minuten 52 Sekunden 321 Tausendstel",
				string);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.kalender.ZeitDauer#setZeidauerInBezugAufEndpunkt(long, long)}.
	 */
	@Test
	public void testSetZeidauerInBezugAufEndpunkt()
	{
		ZeitDauer zeitDauer = new ZeitDauer(0);

		String string = zeitDauer.setZeidauerInBezugAufEndpunkt(5000, 10000);

		assertEquals(
				"0 Jahre 0 Monate 0 Tage 0 Stunden 0 Minuten 5 Sekunden 0 Tausendstel",
				string);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.kalender.ZeitDauer#normalisieren(java.lang.String)}.
	 */
	@Test
	public void testNormalisieren()
	{
		ZeitDauer zeitDauer = new ZeitDauer(0);

		String normalisieren = zeitDauer
				.normalisieren("12 Stunden 18 Monate 33 Tage 62 Minuten 67 Sekunden 1200 Tausendstel");

		assertEquals(
				"1 Jahr 7 Monate 2 Tage 13 Stunden 3 Minuten 8 Sekunden 200 Tausendstel",
				normalisieren);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.kalender.ZeitDauer#baueZeitString(int, int, int, int, int, int, int)}.
	 */
	@Test
	public void testBaueZeitString()
	{
		ZeitDauer zeitDauer = new ZeitDauer(0);
		String string1 = zeitDauer.baueZeitString(1, 0, 1, 2, 0, 3, 0);

		boolean korrekt = true;
		if (!string1
				.equals("1 Jahr 0 Monate 1 Tag 2 Stunden 0 Minuten 3 Sekunden 0 Tausendstel"))
		{
			korrekt = false;
		}

		zeitDauer.setKurzdarstellung(true);
		String string2 = zeitDauer.baueZeitString(1, 0, 1, 2, 0, 3, 0);

		if (!string2.equals("1 J 0 M 1 T 2 h 0 m 3 s 0 t"))
		{
			korrekt = false;
		}

		assertEquals(true, korrekt);
	}

}
