/**
 * 
 */
package de.bsvrz.buv.rw.basislib.listendarstellung;

import static org.junit.Assert.assertEquals;

import java.util.Iterator;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.buv.rw.basislib.Defs;
import de.bsvrz.buv.rw.basislib.login.StrukturEinlesen;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavConnection;

/**
 * Dieser Test muss nach dem Urlasser-Test durchgefhrt werden, da auf die dort
 * angelegten Daten zurckgegriffen wird.<br>
 * Es wird die Listendarstellung getestet. Dazu wird geprft ob
 * <ul>
 * <li>die Kopfzeilen korrekt erstellt wurden,</li>
 * <li>die Datenspalten mit den richtigen Daten gefllt wurden.</li>
 * </ul>
 * 
 * @author HofmannJ
 * 
 */
public class TabellenModellTest
{
	/**
	 * Benutzer fr die Anmeldung am DaV.
	 */
	private static final String _benutzer = "Tester";

	/**
	 * Passwort fr die Anmeldung am DaV.
	 */
	private static final String _passwort = "geheim";

	/**
	 * Ursache fr die Anmeldung am DaV.
	 */
	private static final String _ursache = "Eine Ursache fr den JUnit-Test";

	/**
	 * Veranlasser fr die Anmeldung am DaV.
	 */
	private static final String _veranlasser = "Ein Veranlasser fr den JUnit-Test";

	/**
	 * Adresse des Datenverteilers
	 */
	private static final String _adresseDaV = "127.0.0.1";

	/**
	 * Port des DaV
	 */
	private static final int _portDaV = 8083;

	// hk.open(pidSystemObject, typ, atg, asp, darstellungsArt)
	// private static final String pidSystemObject = "kv.testKonfiguration";
	private static final String pidSystemObject = Defs.PID_AOE;
	private static final String typ = "typ.autarkeOrganisationsEinheit";
	private static final String atg = "atg.globaleEinstellungenOberflcheNetzweit";
	private static final String asp = "asp.parameterSoll";

	/**
	 * Speichert die Verbindung zum Datenverteiler
	 */
	private ClientDavConnection _verbindung = null;

	private TabellenModell _tabellenModell = null;

	private Hilfsklasse _hilfKlasse;

	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception
	{
	}

	/**
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception
	{
	}

	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception
	{
	}

	/**
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception
	{
	}

	@Test
	public void testTabellenModell2()
	{
		/*
		 * Prfung auf gleiche Kopfdaten
		 */
		String kopfString = "GlobaleEinstellungenOberflcheNetzweit=>BenutzereinstellungenGlobal=>Voreinstellungen [..]=>Urlasser=>BenutzerReferenz=>Ursache=>Veranlasser";
		boolean kopfGleich = false;

		/*
		 * Prfung auf gleiche Nutzdaten
		 */
		String inhaltNutzdaten = new StrukturEinlesen().einlesen();
		boolean inhaltGleich = false;

		/*
		 * Prfung auf gleichen Veranlasser
		 */
		boolean ursacheGleich = false;

		/*
		 * Verbindung zum DaV aufbauen
		 */
		if (!DaVVerbindung.getInstanz().istDaVVerbindungAktiv())
		{
			DaVVerbindung.getInstanz().setDaVBenutzer(_benutzer);
			DaVVerbindung.getInstanz().setDaVPasswort(_passwort);
			DaVVerbindung.getInstanz().setDaVIp(_adresseDaV);
			DaVVerbindung.getInstanz().setDaVPort(_portDaV);

			DaVVerbindung.getInstanz().baueVerbindungMitDaVAuf();
		}

		_verbindung = (ClientDavConnection) DaVVerbindung.getInstanz()
				.getDaVVerbindung();

		_tabellenModell = new TabellenModell(_verbindung, typ, atg, asp, null);

		DavEmpfangenSendenAenderungsDarstellung darstellung = new DavEmpfangenSendenAenderungsDarstellung(
				_verbindung);
		darstellung.setTabellenModell(_tabellenModell);

		Warten warten = new Warten();
		darstellung.anmeldenEmpfang(pidSystemObject, atg, asp);
		warten.warte(5000);
		darstellung.abmeldenEmpfang(pidSystemObject, atg, asp);

		String knotenDFS = KnotenDFS(_tabellenModell.getWurzelKopfzeile());
		knotenDFS = knotenDFS.substring(0, knotenDFS.length() - 2);

		/*
		 * Trennen der Verbindung zum DaV
		 */
		DaVVerbindung.getInstanz().trenneDaVVerbindung();

		/*
		 * Prfung: Stimmt die DFS-Traversierung der gelieferten Kopfzeilen,
		 * ausgehende vom Wurzelknoten, mit der erwarteten DFS-Traversierung
		 * berein?
		 */
		if (kopfString.equals(knotenDFS))
		{
			kopfGleich = true;
		}

		StringBuffer sb = new StringBuffer();
		for (int i = 3; i < _tabellenModell.getRowCount(); i++)
		{
			Object object = _tabellenModell.getContentAt(3, i);
			System.out.println("OBJ(" + object.getClass().getName() + "): \""
					+ object + "\"");
			if (object instanceof String)
			{
				sb.append((String) object);
			}
		}

		System.out.println("SB=>\n" + sb.toString());

		inhaltNutzdaten = inhaltNutzdaten.replaceAll("\n", "");
		inhaltNutzdaten = inhaltNutzdaten.replaceAll("\r", "");

		String buffer = sb.toString();
		buffer = buffer.replaceAll("\n", "");
		buffer = buffer.replaceAll("\r", "");

		if (inhaltNutzdaten.equals(buffer.toString()))
		{
			inhaltGleich = true;
		}

		/*
		 * Ursache + Veranlasser prfen
		 */
		Object potUrsache = _tabellenModell.getContentAt(_tabellenModell
				.getColumnCount() - 2, _tabellenModell.getFixedHeaderRowCount());
		Object potVeranlasser = _tabellenModell.getContentAt(_tabellenModell
				.getColumnCount() - 1, _tabellenModell.getFixedHeaderRowCount());

		System.out.println(potUrsache);
		System.out.println(potVeranlasser);

		if ((potUrsache instanceof String) && (potVeranlasser instanceof String))
		{
			String aktUrsache = (String) potUrsache;
			String aktVeranlasser = (String) potVeranlasser;

			System.out.println("Ursache: " + aktUrsache);
			System.out.println("Veranlasser: " + aktVeranlasser);

			if ((_ursache.equals(aktUrsache))
					&& (_veranlasser.equals(aktVeranlasser)))
			{
				ursacheGleich = true;
			}
		}

		System.out.println("\n======Tabellenmodell Test======");
		System.out.println("kopfGleich: " + kopfGleich);
		System.out.println("inhaltGleich: " + inhaltGleich);
		System.out.println("ursacheGleich: " + ursacheGleich);
		System.out.println("===============================\n");

		System.out.println("Anzahl Zeichen Nutzdaten: " + inhaltNutzdaten.length()
				+ " StringBuffer: " + buffer.toString().length());

		assertEquals(true, kopfGleich && inhaltGleich && ursacheGleich);
	}

	private String KnotenDFS(Knoten knoten)
	{
		String re = "";
		re = re + knoten.getSpaltenBezeichnung() + "=>";
		if (knoten.hatNachfolger())
		{
			Iterator<Knoten> it = knoten.getNachfolger().iterator();
			while (it.hasNext())
			{
				Knoten nachfolger = it.next();
				re = re + KnotenDFS(nachfolger);
			}
		}
		return re;
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.basislib.listendarstellung.TabellenModell#getInitialColumnWidth(int)}
	 * .
	 */

	public void testTabellenModell()
	{
		Warten warten = new Warten();
		warten.warte(3000);

		/*
		 * Verbindung zum DaV aufbauen
		 */
		DaVVerbindung.getInstanz().setDaVBenutzer(_benutzer);
		DaVVerbindung.getInstanz().setDaVPasswort(_passwort);
		DaVVerbindung.getInstanz().setDaVIp(_adresseDaV);
		DaVVerbindung.getInstanz().setDaVPort(_portDaV);

		DaVVerbindung.getInstanz().baueVerbindungMitDaVAuf();

		_verbindung = (ClientDavConnection) DaVVerbindung.getInstanz()
				.getDaVVerbindung();

		_hilfKlasse = new Hilfsklasse(_verbindung);

		/*
		 * Mock
		 */
		Mock m = new Mock();

		m.durchfuehren();

		synchronized (m)
		{
			try
			{
				m.wait(20000);
			} catch (InterruptedException e)
			{
				e.printStackTrace();
			}
		}

		/*
		 * Trennen der Verbindung
		 */
		DaVVerbindung.getInstanz().trenneDaVVerbindung();
	}

	private class Mock implements Runnable
	{
		private Boolean _threadEnde = new Boolean(false);

		public Mock()
		{
		}

		private void durchfuehren()
		{
			startThread();

			_hilfKlasse.open(pidSystemObject, typ, atg, asp,
					EnumDarstellungsArt.zustandsDarstellung);

			synchronized (_threadEnde)
			{
				try
				{
					System.out.println("VOR WARTEN AUF ENDE");
					_threadEnde.wait(12000);
					System.out.println("NACH WARTEN AUF ENDE");

				} catch (InterruptedException e)
				{

					e.printStackTrace();
				} finally
				{
					synchronized (this)
					{
						this.notifyAll();
					}
				}
			}
		}

		private void startThread()
		{
			Thread th = new Thread(this);
			th.start();
		}

		@Override
		public void run()
		{
			try
			{
				Thread.sleep(5000);
			} catch (InterruptedException e)
			{
				e.printStackTrace();
			} finally
			{
				// Runnable runner = new Runnable()
				// {
				// @Override
				// public void run()
				// {
				// _tabellenModell = _hilfKlasse.aktionDialogSchliessen();
				// }
				// };
				//
				// System.out.println("VOR SCHLIESSEN");
				// PlatformUI.getWorkbench().getDisplay().syncExec(runner);
				// System.out.println("NACH SCHLIESSEN");

				_tabellenModell = _hilfKlasse.getTabellenModell();

				System.out.println("===>"
						+ _tabellenModell.getWurzelKopfzeile().getSpaltenBezeichnung()
						+ "<===");

			}

			synchronized (_threadEnde)
			{
				_threadEnde.notifyAll();
			}
		}

	}

	/**
	 * Thread-Klasse, liefert einenb Befehl <code>warten</code>, mit dessen Hilfe
	 * die Ausfhrung des JUnit-Testes pausiert werden kann.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class Warten implements Runnable
	{
		private boolean _laufeThread = false;
		private Long _warteZeit = new Long(1000);

		public void warte(long warteZeit)
		{
			synchronized (_warteZeit)
			{
				_warteZeit = warteZeit;
			}

			startThread();

			synchronized (this)
			{
				try
				{
					wait();
				} catch (InterruptedException e)
				{
					e.printStackTrace();
				}
			}
		}

		private void startThread()
		{
			if (!_laufeThread)
			{
				Thread th = new Thread(this);
				th.setName("Warten...");
				_laufeThread = true;
				th.start();
			}
		}

		@Override
		public void run()
		{
			if (_laufeThread)
			{
				try
				{
					Thread.sleep(_warteZeit);
				} catch (InterruptedException e)
				{
					e.printStackTrace();
				} finally
				{
					synchronized (this)
					{
						notifyAll();
					}
				}
			}

			_laufeThread = false;
		}

	}

}
