/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.mehrfachzwischenablage;

import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

/**
 * Diese Klasse implementiert eine Testview fr das Kopieren von Elementen ber
 * die Mehrfachzwischenablage. Mittels der Methoden
 * {@link MehrfachZwischenablageDatenTestView#setInput(Collection)} bzw.
 * {@link MehrfachZwischenablageDatenTestView#setInput(Object[])} knnen Daten
 * in die Sicht dieser Tabelle eingebracht werden. Der Test
 * {@link MehrfachZwischenablageDatenTest} kann ber die Methode
 * {@link MehrfachZwischenablageDatenTestView#allesSelektieren()} smtliche in
 * der Tabelle dargestellten Objekte selektieren um somit das Kopieren in die
 * Mehrfachzwischenablage zu ermglichen.
 * 
 * @author HofmannJ
 * 
 */
public class MehrfachZwischenablageDatenTestView extends ViewPart implements
		IMehrfachZwischenablageEinfuegen
{

	public static final String ID = "de.bsvrz.buv.rw.basislib.mehrfachzwischenablage.MehrfachZwischenablageDatenTestView";
	private Composite top = null;
	private Group groupDaten = null;
	private Composite compositeKontrolle = null;
	private Button buttonOk = null;
	private Button buttonAbbrechen = null;
	private Table tableData = null;
	private TableViewer tableViewer = null;

	@Override
	public void createPartControl(Composite parent)
	{
		top = new Composite(parent, SWT.NONE);
		top.setLayout(new GridLayout());
		createGroupDaten();
		createCompositeKontrolle();
	}

	@Override
	public void setFocus()
	{
	}

	/**
	 * This method initializes groupDaten
	 * 
	 */
	private void createGroupDaten()
	{
		GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.grabExcessVerticalSpace = true;
		gridData2.verticalAlignment = GridData.FILL;
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.grabExcessVerticalSpace = true;
		gridData1.verticalAlignment = GridData.FILL;
		groupDaten = new Group(top, SWT.NONE);
		groupDaten.setLayout(new GridLayout());
		groupDaten.setLayoutData(gridData1);
		groupDaten.setText("Daten:");
		tableData = new Table(groupDaten, SWT.FULL_SELECTION | SWT.MULTI
				| SWT.BORDER);
		tableData.setHeaderVisible(true);
		tableData.setLayoutData(gridData2);
		tableData.setLinesVisible(true);
		tableViewer = new TableViewer(tableData);
		getSite().setSelectionProvider(tableViewer);

		tableViewer.setContentProvider(new ContentProvider());
		tableViewer.setLabelProvider(new LabelProvider());

		TableColumn tc = new TableColumn(tableData, SWT.LEFT);
		tc.setText("Eintrge:");
		tc.setWidth(350);
	}

	/**
	 * This method initializes compositeKontrolle
	 * 
	 */
	private void createCompositeKontrolle()
	{
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.END;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;
		RowLayout rowLayout = new RowLayout();
		rowLayout.pack = false;
		compositeKontrolle = new Composite(top, SWT.NONE);
		compositeKontrolle.setLayout(rowLayout);
		compositeKontrolle.setLayoutData(gridData);
		buttonOk = new Button(compositeKontrolle, SWT.NONE);
		buttonOk.setText("OK");
		buttonOk.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				aktionOk();
			}			
		});
		
		
		
		buttonAbbrechen = new Button(compositeKontrolle, SWT.NONE);
		buttonAbbrechen.setText("Abbrechen");
		buttonAbbrechen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionAbbrechen();
			}			
		});
		
	}

	private void aktionOk()
	{
		allesSelektieren();
		MehrfachZwischenablageDaten.getInstanz().selektionHinzufuegen();
		try
		{
			PlatformUI
					.getWorkbench()
					.getActiveWorkbenchWindow()
					.getActivePage()
					.showView(
							MehrfachZwischenablageParameter.viewIdSichtMehrfachZwischenablage);
		} catch (PartInitException e)
		{
			e.printStackTrace();
		}
	}

	private void aktionAbbrechen()
	{
		String[] werte = new String[] { "Wert1", "Wert2", "Wert3" };
		setInput(werte);
	}

	@SuppressWarnings("unchecked")
	public void setInput(Collection col)
	{
		tableViewer.setInput(col);
	}

	public void setInput(Object[] objArray)
	{
		tableViewer.setInput(objArray);
	}

	public void allesSelektieren()
	{
		TableItem[] items = tableData.getItems();

		LinkedList<Object> selektionsListe = new LinkedList<Object>();

		for (int i = 0; i < items.length; i++)
		{
			selektionsListe.add(items[i].getData());
		}

		tableViewer.setSelection(new StructuredSelection(selektionsListe), true);
	}
	
	public Object[] getAlleTabellenElemente()
	{
		Object[] re = null;
		
		TableItem[] items = tableData.getItems();
		
		if((items != null) && (items.length > 0))
		{
			re = new Object[items.length];
			
			for(int i = 0; i < items.length; i++)
			{
				re[i] = items[i].getData();
			}
		}
		
		
		return re;
	}
	

	/**
	 * ContentProvider fr den {@link TableViewer}.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class ContentProvider implements IStructuredContentProvider
	{

		@SuppressWarnings("unchecked")
		@Override
		public Object[] getElements(Object inputElement)
		{
			Object[] re = null;

			if (inputElement instanceof Collection)
			{
				Collection col = (Collection) inputElement;
				re = col.toArray();
			} else if (inputElement instanceof Object[])
			{
				re = (Object[]) inputElement;
			}

			return re;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}

	}

	/**
	 * {@link ITableLabelProvider} fr den {@link TableViewer}.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class LabelProvider implements ITableLabelProvider
	{

		@Override
		public Image getColumnImage(Object element, int columnIndex)
		{
			return null;
		}

		@Override
		public String getColumnText(Object element, int columnIndex)
		{
			if (element != null)
			{
				return element.toString();
			} else
			{
				return "Element ist NULL!";
			}
		}

		@Override
		public void addListener(ILabelProviderListener listener)
		{
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		@Override
		public void removeListener(ILabelProviderListener listener)
		{
		}

	}

	@Override
	public void einfuegen(List<Object> objekt)
	{
		setInput(objekt);

	}

}
