/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.berechtigung;

public class ArrayQueue implements Queue
{
  private Object[] elements; // Elemente

  private int l = 0, u = 0; // unterer und oberer Zeiger

  // Queue mit vorgegebener Kapazitt erzeugen
  public ArrayQueue(int size)
  {
    elements = new Object[size];
  }

  // Queue mit Standardkapazitt erzeugen
  public ArrayQueue()
  {
    elements = new Object[100];
  }

  public void enter(Object obj) throws QueueException
  {
    if ((elements.length - l + u) % elements.length == elements.length - 1)
      // Kapazitt ist erschpft
      throw new QueueException();
    elements[u] = obj;
    // oberen Zeiger aktualisieren
    u = (u + 1) % elements.length;
  }

  public Object leave() throws QueueException
  {
    if (isEmpty())
      throw new QueueException();
    Object obj = elements[l];
    elements[l] = null;
    // unteren Zeiger aktualisieren
    l = (l + 1) % elements.length;
    return obj;
  }

  public Object front() throws QueueException
  {
    if (isEmpty())
      throw new QueueException();
    return elements[l];
  }

  public boolean isEmpty()
  {
    return l == u;
  }
}
