/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.berechtigung;

/**
 * Interface fr die Listener der Oberflaechenberechtigung.
 * 
 * Durch die Implementierung dieses Listeners knnen sich Klassen ber die Freigabe oder Sperrung des entsprechenden
 * Oberflchenelementes informieren.
 * 
 * @version $Revision: 1.2 $ / $Date: 2009/12/08 08:50:11 $ / ($Author: HofmannJ $)
 * 
 * @author HofmannJ
 * 
 */
public interface IOberflaechenBerechtigungZuhoerer
{
  /**
   * Dieser Listener wird aktiv, sobald die Sperrung erfolgt ist.
   * 
   * Fr die Implementierung des Listeners muss die Funktion <code>e.hasBerechtigung</code> == false liefern.
   * 
   * Dann kann mittels <code>e.getSystemObjekte</code> die Liste der Systemobjekte abgerufen werden (z.B. fr eine
   * Ausgabe) abgerufen werden, wegen denen die Sperrung erfolgt.
   * 
   * @param e
   *          OberflaechenBerechtigungEvent, der ausgelst wurde.
   */
  public void sperrung(OberflaechenBerechtigungEreignis e);

  /**
   * Dieser Listener wird aktiv, sobald die Freigabe erfolgt ist.
   * 
   * Fr die Implementierung des Listeners muss die Funktion <code>e.hasBerechtigung</code> == true liefern.
   * 
   * @param e
   *          OberflaechenBerechtigungEvent, der ausgelst wurde.
   */
  public void freigabe(OberflaechenBerechtigungEreignis e);
}
