/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.berechtigung;

import java.util.EventObject;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Die Klasse stellt den passenden Event zum SystemObjekteListener zur Verfgung. Mit ihm knnen Zuhoerer darber
 * informiert werden, ob Objekte hinzugekommen bzw. Objekte aus der Konfiguration entfernt wurden.
 * 
 * @version $Revision: 1.2 $ / $Date: 2009/12/08 08:50:11 $ / ($Author: HofmannJ $)
 * 
 * @author HofmannJ
 * 
 */
public class SystemObjekteEreignis extends EventObject
{
  /**
   * Bei <code>true</code> erfolgt die Freigabe, bei <code>false</code> nicht.
   */
  private boolean _aenderung = true;

  /**
   * Liste von Systemobjekten, auf die sich dieser Event bezieht und fr die eine Freigabe erfolgt.
   */
  private SystemObject _systemObjektHinzu = null;

  /**
   * Liste von Systemobjekten, auf die sich dieser Event bezieht und fr die eine Sperrung erfolgt.
   */
  private SystemObject _systemObjektWeg = null;

  /**
   * Konstruktor des Events. Im Objekt <code>source</code> wird das Objekt, auf welches sich der Event bezieht,
   * bergeben.
   * 
   * @param source
   *          Objekt, auf dass sich der Event bezieht. Es kann sich hierbei z.B. um eine Liste von Systemobjekten
   *          handeln.
   * @param freigabe
   *          Freigabe erfolgt wenn <code>true</code>, bei <code>false</code> erfolgt die Sperrung.
   */
  public SystemObjekteEreignis(Object source, boolean aenderung, SystemObject systemObjektHinzu,
      SystemObject systemObjektWeg)
  {
    super(source);
    this._aenderung = aenderung;

    this._systemObjektHinzu = systemObjektHinzu;
    this._systemObjektWeg = systemObjektWeg;
  }

  /**
   * Diese Methode prft, ob eine nderung der Liste von SystemObjekten erfolgte.
   * 
   * @return Liefert <code>true</code>, wenn eine nderung erfolgte, sonst <code>false</code>.
   */
  public boolean getAenderung()
  {
    return _aenderung;
  }

  /**
   * Liefert eine Liste von Systemobjekten zurck, auf die sich dieser Event bezieht. <br>
   * Es wird die Liste der SystemObjekte zurckgeliefert, fr die eine Freigabe erfolgen kann.
   * 
   * @return Liste der Systemobjekte, auf die sich die Freigabe bezieht.
   */
  public SystemObject getSystemObjektHinzu()
  {
    return _systemObjektHinzu;
  }

  /**
   * Liefert eine Liste von Systemobjekten zurck, auf die sich dieser Event bezieht. <br>
   * Es wird die Liste der SystemObjekte zurckgeliefert, fr die eine Sperrung erfolgt.
   * 
   * @return Liste der SystemObjekte, auf die sich die Sperrung bezieht.
   */
  public SystemObject getSystemObjektWeg()
  {
    return _systemObjektWeg;
  }

  /**
   * Generierte serielle Versions UID dieses Events.
   */
  private static final long serialVersionUID = -247745193727961181L;

}
