/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.einstellungen.hilfsklassen;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.Activator;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenSpeicher;
import de.bsvrz.buv.rw.basislib.login.UrlasserDialog;
import de.bsvrz.buv.rw.basislib.login.UrlasserInfo;
import de.bsvrz.buv.rw.basislib.login.UrlasserObjekt;
import de.bsvrz.buv.rw.rw.dav.BenutzerAngemeldet;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Klasse zum Speichern und Lesen der benutzerspezifischen, netzweiten
 * Einstellungen.
 * 
 * @author HofmannJ
 * 
 */
public class EinstellungenBenutzerSpezifischNetzweit implements
		ClientSenderInterface
{
	/**
	 * Verbindung zum DaV
	 */
	private ClientDavConnection _verbindung = null;

	/**
	 * IP
	 */
	private static final String _ip = "localhost";

	/**
	 * Port
	 */
	private static final int _port = 8083;

	/**
	 * Login
	 */
	private static final String _benutzer = "Tester";

	/**
	 * Passwort
	 */
	private static final String _passwort = "geheim";

	/**
	 * Datenmodell des DaV
	 */
	private DataModel _dataModell = null;

	/**
	 * SystemObjekt
	 */
	private SystemObject _systemObject = null;

	/**
	 * Bezeichnung der Attributgruppe
	 */
	private static final String _atgBezeichnung = "atg.benutzerEinstellungenOberflcheNetzweit";

	/**
	 * Item Benutzereinstellungen
	 */
	private final String _itemBenutzereinstellungen = "Benutzereinstellungen";

	/**
	 * Item Voreinstellungen
	 */
	private final String _itemVoreinstellungen = "Voreinstellungen";

	/**
	 * Item Benutzer
	 */
	private final String _itemBenutzer = "benutzer";

	/**
	 * Bezeichnung des Aspektes soll (zum Empfangen)
	 */
	private static final String _aspSoll = "asp.parameterSoll";

	/**
	 * Bezeichnung des Aspektes vorgabe (zum Senden)
	 */
	private static final String _aspVorgabe = "asp.parameterVorgabe";

	/**
	 * Attributgruppe
	 */
	private AttributeGroup _atg = null;

	/**
	 * Aspekt
	 */
	private Aspect _asp = null;

	/**
	 * Datenbeschreibung
	 */
	private DataDescription _dataDescription = null;

	/**
	 * Synchronisation der Sende Steuerung
	 */
	private Boolean _sendeSteuerung = new Boolean(true);

	private Boolean _warten = new Boolean(true);

	/**
	 * Max. Gre der Queue
	 */
	private static final int MAXQUEUE = 10;

	/**
	 * Queue
	 */
	private final Queue<Data> produkte = new LinkedList<Data>();

	/**
	 * Lock der Queue
	 */
	private final Lock lock = new ReentrantLock();

	/**
	 * Bedingung fr Queue nicht voll
	 */
	private final Condition nichtVoll = lock.newCondition();

	/**
	 * Bedingung fr Queue nicht leer
	 */
	private final Condition nichtLeer = lock.newCondition();

	/**
	 * Flag, zeigt an ob schon mal eine Meldung zu der Angabe eines falschen
	 * Oberflchenobjekte ber den Parameter <code>-oberflaechenObjekt</code>
	 * ausgegeben wurde.
	 */
	private boolean _meldungFalschesOberflaechenObjektSchonGezeigt = false;

	/**
	 * Wenn <code>true</code>, erfolgt das senden ohne den {@link UrlasserDialog}.
	 */
	private boolean _sendenOhneUrlasserDlg = false;

	// public EinstellungenBenutzerSpezifischNetzweit()
	// {
	// baueDaVVerbindungAuf();
	//
	// _dataModell = _verbindung.getDataModel();
	//
	// // testObjekteBestimmen();
	// String pid = bestimmePidEinesBenutzers("Tester");
	// System.out.println("PID: " + pid);
	// // senden();
	//
	// // Data empfangeDaten = empfangeDatenBenutzereinstellungenNetzweit();
	// // System.out.println("EMPFANGEN: " + empfangeDaten.toString());
	// // sendenBearbeitetTest();
	//
	// SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
	// String d = sdf.format(new Date());
	//
	// editEintrag("TestDatenverteilerBenutzer", "XML\nIST\nToll\n!\n" + d);
	//
	// Warten w = new Warten();
	// synchronized (_warten)
	// {
	// try
	// {
	// _warten.wait();
	// } catch (InterruptedException e)
	// {
	// e.printStackTrace();
	// }
	// }
	//
	// String eintrag = getEintrag("TestDatenverteilerBenutzer");
	// System.out.println("Eintrag:\n" + eintrag);
	//
	// // editEintrag("Tester", "Noch\nEin\nPaar\nZeilen\nMehr!!!");
	// // String eintragTester = getEintrag("Tester");
	// // System.out.println("Eintrag Tester: " + eintragTester);
	//
	// schliesseDaVVerbindung();
	// }

	/**
	 * Konstruktor, bernimmt die Verbindung zum DaV.
	 * 
	 * @param verbindung
	 *          DaV-Verbindung
	 */
	public EinstellungenBenutzerSpezifischNetzweit(ClientDavConnection verbindung)
	{
		this._verbindung = verbindung;
		this._dataModell = _verbindung.getDataModel();
	}

	/**
	 * Anmelden zum Senden von Daten
	 */
	private void anmeldenSenden()
	{
		_dataModell = _verbindung.getDataModel();

		List<SystemObject> objekte = bestimmeObjekte("typ.autarkeOrganisationsEinheit");
		if (objekte.size() == 1)
		{
			_systemObject = objekte.get(0);
			_atg = _dataModell.getAttributeGroup(_atgBezeichnung);

			_asp = _dataModell.getAspect(_aspVorgabe);
			_dataDescription = new DataDescription(_atg, _asp);

			SenderRole senderRole = SenderRole.sender();

			try
			{
				_verbindung.subscribeSender(this, _systemObject, _dataDescription,
						senderRole);
			} catch (OneSubscriptionPerSendData e)
			{
				e.printStackTrace();
			}

		}

	}

	/**
	 * @deprecated Nicht mehr verwenden!
	 */
	protected void senden()
	{
		anmeldenSenden();
		_atg = _dataModell.getAttributeGroup(_atgBezeichnung);
		Data data = _verbindung.createData(_atg);
		Array array = data.getArray("Benutzereinstellungen");
		array.setLength(2);

		Data item = null;

		for (int i = 0; i < 2; i++)
		{
			item = array.getItem(i);
			// TextArray textArray = item.getTextArray(_itemVoreinstellungen);

			Array arrayText = item.getArray(_itemVoreinstellungen);

			ReferenceValue referenceValue = item.getReferenceValue(_itemBenutzer);

			// textArray.setLength(2);
			arrayText.setLength(2);

			// textArray.getTextValue(0).setText("Hallo Welt! <= Zeile 1");
			// textArray.getTextValue(1).setText("Hallo Welt! <= Zeile 2");

			arrayText.getTextValue(0).setText("Hallo Welt! <= Zeile 1");
			arrayText.getTextValue(1).setText("Hallo Welt! <= Zeile 2");

			referenceValue.setSystemObject(bestimmeSOEinesBenutzers("Tester"));
		}

		ResultData datenTelegramm = new ResultData(_systemObject, _dataDescription,
				new Date().getTime(), data);

		try
		{
			_verbindung.sendData(datenTelegramm);
		} catch (DataNotSubscribedException e)
		{
			e.printStackTrace();
		} catch (SendSubscriptionNotConfirmed e)
		{
			e.printStackTrace();
		}

		abmeldenSenden();
	}

	/**
	 * Senden von Testdaten
	 * 
	 * @deprecated Nicht mehr Verwenden
	 */
	protected void sendenBearbeitetTest()
	{

		/*
		 * Daten Holen
		 */
		Data data = empfangeDatenBenutzereinstellungenNetzweit();

		if (data != null)
		{
			/*
			 * Benutzereintrag suchen und Werte ndern (TEST)
			 */
			Data benEintrag = sucheBenutzerspezifischenEintrag("Tester", data);
			setBenutzersepzifischeEinstellungen(benEintrag,
					"123\nTest\nUndNochMal\n111\nfjfjfj\nelffl\n12i123i");

			/*
			 * Suchen nach TestDatenverteilerBenutzer
			 */
			Data test = sucheBenutzerspezifischenEintrag(
					"TestDatenverteilerBenutzer", data);
			if (test == null)
			{
				// Nicht gefunden
			}

			/*
			 * Anlegen des Benutzers
			 */
			anlegenBenutzerspezifischenEintrag(data, "TestDatenverteilerBenutzer");

			// System.out.println("DATEN: " + data.toString());
			//
			// Array array = data.getArray("Benutzereinstellungen");
			//
			// System.out.println("Lnge: " + array.getLength());
			//
			// System.out.println("ARRAY: " + array.toString());
			//
			// array.setLength(6);

			// System.out.println("ARRAY: " + array.toString());

			anmeldenSenden();
			ResultData datenTelegramm = new ResultData(_systemObject,
					_dataDescription, new Date().getTime(), data);

			try
			{
				_verbindung.sendData(datenTelegramm);
			} catch (DataNotSubscribedException e)
			{
				e.printStackTrace();
			} catch (SendSubscriptionNotConfirmed e)
			{
				e.printStackTrace();
			}
			abmeldenSenden();
		}
	}

	/**
	 * Berarbeitet einen benutzerspezifischen Eintrag.
	 * 
	 * @param benutzerName
	 *          Benutzer, fr den ein Eintrag angepasst werden soll.
	 * @param xmlStruktur
	 *          String der XML-Struktur, die gespeichert werden soll.
	 * @return Liefert <code>true</code>, wenn erfolgreich gespeichert werden
	 *         konnte.
	 */
	public boolean editEintrag(String benutzerName, String xmlStruktur)
	{
		boolean erfolgreich = false;

		/*
		 * Daten Holen
		 */
		Data data = empfangeDatenBenutzereinstellungenNetzweit();

		if (data != null)
		{
			/*
			 * Benutzerdaten suchen
			 */
			Data benutzerDaten = sucheBenutzerspezifischenEintrag(benutzerName, data);

			/*
			 * Existiert Benutzer bzw. dessen Daten?
			 */
			if (benutzerDaten != null)
			{
				setBenutzersepzifischeEinstellungen(benutzerDaten, xmlStruktur);
				erfolgreich = true;
			} else
			{
				System.out
						.println("Benutzerspezifische Einstellungen mssen angelegt werden");
				if (anlegenBenutzerspezifischenEintrag(data, benutzerName))
				{
					Data neuangelegterBenutzereintrag = sucheBenutzerspezifischenEintrag(
							benutzerName, data);

					setBenutzersepzifischeEinstellungen(neuangelegterBenutzereintrag,
							xmlStruktur);

					erfolgreich = true;

					System.out.println("Benutzerspezifische Einstellungen angelegt");
				} else
				{
					System.out.println("Anlegen gescheitert!");
				}
			}

			if (erfolgreich)
			{

				/*
				 * Aus der alten DataDescription eine neue unter dem Parameter
				 * asp.parameterVorgabe erzeugen.
				 */
				DataDescription dataDescriptionNeu = new DataDescription(
						_dataDescription.getAttributeGroup(), _verbindung.getDataModel()
								.getAspect(_aspVorgabe));

				_dataDescription = new DataDescription(_atg, _asp);

				/*
				 * SystemObjekt bestimmen
				 */
				String pidSysObj = bestimmePidOberflaechenObjekt();

				if (pidSysObj != null)
				{
					_systemObject = _verbindung.getDataModel().getObject(pidSysObj);

					/*
					 * Versenden
					 */
					ResultData datenTelegramm = new ResultData(_systemObject,
							dataDescriptionNeu, new Date().getTime(), data);

					LinkedList<ResultData> liste = new LinkedList<ResultData>();
					liste.add(datenTelegramm);

					if (!_sendenOhneUrlasserDlg)
					{
						UrlasserDialog uld = new UrlasserDialog(PlatformUI.getWorkbench()
								.getDisplay().getActiveShell());
						@SuppressWarnings("unused")
						boolean anzeigen = (uld.anzeigen(liste)).getEingabeOk();
					} else
					{
						BenutzerAngemeldet ba = BenutzerAngemeldet.getInstanz();
						UrlasserInfo ui = new UrlasserInfo(ba.getName(), ba.getPasswort(),
								"Senden ohne UrlasserDialog", ba.getName());
						UrlasserObjekt uo = new UrlasserObjekt(ui);
						uo.baueDaVVerbindungAuf();

						LinkedList<ResultData> sendeDaten = uo.sendeDaten(liste);
						if (sendeDaten.size() > 0)
						{
							String meldung = getClass().getName() + "::aktionOk " + "Von "
									+ liste.size() + " Datenstzen konnten " + sendeDaten.size()
									+ " Datenstze nicht versendet werden.";

							String meldungLog = meldung + " Datenstze:\n";
							for (ResultData resultData : sendeDaten)
							{
								meldung = meldung + "\n=> " + resultData;
							}

							Activator.getDefault().getLog().log(
									new Status(Status.WARNING, Activator.PLUGIN_ID, meldungLog));

							MessageBox mb = new MessageBox(PlatformUI.getWorkbench()
									.getDisplay().getActiveShell(), SWT.OK | SWT.ICON_ERROR);
							mb.setText("Datenstze nicht versendbar");
							mb.setMessage(meldung);
							mb.open();
						}
					}

				}

			}
		}

		return erfolgreich;
	}

	/**
	 * Liefert den Eintrag fr einen Benutzer.
	 * 
	 * @param benutzerName
	 *          Name des Benutzers.
	 * @return Den Eintrag fr einen Benutzer oder <code>null</code>, falls nicht
	 *         verfgbar.
	 */
	public String getEintrag(String benutzerName)
	{
		String re = null;

		/*
		 * Daten Holen
		 */
		Data data = empfangeDatenBenutzereinstellungenNetzweit();

		if (data != null)
		{
			Data eintrag = sucheBenutzerspezifischenEintrag(benutzerName, data);
			if (eintrag != null)
			{
				// TextArray textArray = eintrag.getTextArray(_itemVoreinstellungen);
				Array textArray = eintrag.getArray(_itemVoreinstellungen);
				if (textArray.getLength() > 0)
				{
					StringBuffer buf = new StringBuffer();

					for (int i = 0; i < textArray.getLength(); i++)
					{
						buf.append(textArray.getTextValue(i).getText());
						buf.append("\n");
					}

					re = buf.toString();
				}
			}
		}

		return re;
	}

	/**
	 * Fgt einen neuen Eintrag fr einen Benutzer hinzu, so dass
	 * Benutzerspezifische Einstellungen fr den Benutzer gespeichert werden
	 * knnen.
	 * 
	 * atg.benutzerEinstellungenOberflcheNetzweit
	 * 
	 * @param data
	 *          Daten, an die der Benutzer angehngt werden soll (in Attributliste
	 *          Benutzereinstellungen)
	 * @param benutzerName
	 *          Benutzername
	 * @return Liefert <code>true</code>, wenn der Benutzereintrag den bergebenen
	 *         Daten angefgt werden kann, sonnst <code>false</code>.
	 */
	private boolean anlegenBenutzerspezifischenEintrag(Data data,
			String benutzerName)
	{
		boolean erfolgreich = false;

		SystemObject soBenutzer = bestimmeSOEinesBenutzers(benutzerName);
		System.out.println("SystemObjekt Benutzer: " + soBenutzer);

		if (soBenutzer != null)
		{
			/*
			 * Lnge des Arrays um 1 erhhen
			 */
			data.getArray(_itemBenutzereinstellungen).setLength(
					data.getArray(_itemBenutzereinstellungen).getLength() + 1);

			int pos = data.getArray(_itemBenutzereinstellungen).getLength() - 1;

			Data benutzerItem = data.getArray(_itemBenutzereinstellungen)
					.getItem(pos);

			/*
			 * TextArray mit einer leeren XML-Struktur initialisieren
			 */
			TextArray textArrayVoreinstellungen = benutzerItem
					.getTextArray(_itemVoreinstellungen);

			EinstellungenSpeicher es = new EinstellungenSpeicher();
			String[] split = es.ausgabeXmlFormatiert(true).split("\n");
			textArrayVoreinstellungen.setLength(split.length);
			for (int i = 0; i < split.length; i++)
			{
				textArrayVoreinstellungen.getTextValue(i).setText(split[i]);
			}

			/*
			 * Benutzerreferenz setzen
			 */
			benutzerItem.getReferenceValue(_itemBenutzer).setSystemObject(soBenutzer);

			erfolgreich = true;

		}

		return erfolgreich;
	}

	/**
	 * Liefert das ITEM der Benutzereinstellungen zum angegebenen Benutzer
	 * 
	 * atg.benutzerEinstellungenOberflcheNetzweit
	 * 
	 * @param benutzer
	 *          Name des Benutzers
	 * @param data
	 *          Daten, aus denen der Benutzer herausgeholt werden soll.
	 * @return Liefert das "ITEM" als {@link Data}, welches zu dem jeweiligen
	 *         Benutzer gehrt oder <code>null</code>, falls dieser Benutzer nicht
	 *         existiert.
	 */
	private Data sucheBenutzerspezifischenEintrag(String benutzer, Data data)
	{
		Data re = null;

		if (data != null)
		{
			SystemObject soBenutzer = bestimmeSOEinesBenutzers(benutzer);
			if (soBenutzer != null)
			{
				Array array = data.getArray(_itemBenutzereinstellungen);
				int laenge = array.getLength();
				for (int i = 0; i < laenge; i++)
				{
					Data item = array.getItem(i);

					ReferenceValue referenceValue = item.getReferenceValue(_itemBenutzer);
					// System.out.println("REF: " + referenceValue.toString());

					if (referenceValue.getSystemObject() != null)
					{
						if (referenceValue.getSystemObject().equals(soBenutzer))
						{
							re = item;
							break;
						}
					} else
					{
						// System.out.println("UNDEF");
					}
				}
			}
		}

		return re;
	}

	/**
	 * Setzt die Benutzersperzifischen Einstellungen in das Item der
	 * Benutzereinstellungen, die von der Funktion
	 * {@link EinstellungenBenutzerSpezifisch#sucheBenutzerspezifischenEintrag(String, Data)}
	 * geliefert wurde.
	 * 
	 * atg.benutzerEinstellungenOberflcheNetzweit -> Benutzereinstellungen
	 * 
	 * @param data
	 *          "Item" (aus der Attributliste Benutzereinstellungen), in dem die
	 *          Benutzerspezifischen Einstellungen gesetzt werden sollen.
	 * @param xmlStruktur
	 *          Zu setzende XML-Struktur.
	 */
	private void setBenutzersepzifischeEinstellungen(Data data, String xmlStruktur)
	{
		// TextArray textArray = data.getTextArray(_itemVoreinstellungen);
		// System.out.println("Data: " + data.toString());
		Array textArray = data.getArray(_itemVoreinstellungen);
		String[] split = xmlStruktur.split("\\n");
		textArray.setLength(split.length);

		for (int i = 0; i < split.length; i++)
		{
			textArray.getTextValue(i).setText(split[i]);
		}
	}

	/**
	 * Abmeldung
	 */
	private void abmeldenSenden()
	{
		_verbindung.unsubscribeSender(this, _systemObject, _dataDescription);

	}

	/**
	 * Bestimmt die PID des zu einem Benutzer zugehrigen Objektes vom Typ
	 * <code>typ.benutzer</code>.
	 * 
	 * @param benutzer
	 *          Name des Benutzer-Objektes
	 * @return {@link SystemObject} des Benutzers oder <code>null</code>, wenn
	 *         kein entsprechender Benutzer existiert.
	 */
	private SystemObject bestimmeSOEinesBenutzers(String benutzer)
	{
		SystemObject reSo = null;

		List<SystemObject> bestimmeObjekte = bestimmeObjekte("typ.benutzer");
		Iterator<SystemObject> it = bestimmeObjekte.iterator();
		while (it.hasNext())
		{
			SystemObject so = it.next();
			if (so.getName().equals(benutzer))
			{
				reSo = so;
				// System.out.println("Benutzer gefunden: " + so.getName() + "("
				// + so.getPid() + ")");
				break;
			}
		}

		return reSo;
	}

	/**
	 * Bestimmt eine Liste von zu Benutzern (typ.benutzer) zugehrigen
	 * {@link SystemObject}s.
	 * 
	 * @return Liste von zu Benutzern (typ.benutzer) zugehrigen
	 *         {@link SystemObject}s
	 */
	public List<SystemObject> bestimmeAlleBenutzerSO()
	{
		List<SystemObject> bestimmeObjekte = bestimmeObjekte("typ.benutzer");
		return bestimmeObjekte;
	}

	/**
	 * Bestimmt die PID des zu einem Benutzer zugehrigen Objektes vom Typ
	 * <code>typ.benutzer</code>.
	 * 
	 * @param benutzer
	 *          Name des Benutzer-Objektes
	 * @return PID des Benutzerobjektes
	 */
	@SuppressWarnings("unused")
	private String bestimmePidEinesBenutzers(String benutzer)
	{
		String rePid = null;

		List<SystemObject> bestimmeObjekte = bestimmeObjekte("typ.benutzer");
		Iterator<SystemObject> it = bestimmeObjekte.iterator();
		while (it.hasNext())
		{
			SystemObject so = it.next();
			if (so.getName().equals(benutzer))
			{
				rePid = so.getPid();
				// System.out.println("Benutzer gefunden: " + so.getName() + "("
				// + so.getPid() + ")");
				break;
			}
		}

		return rePid;
	}

	/**
	 * Methode zum Bestimmen der Objekte, die zu einer bestimmten PID gehren
	 * (z.B. typ.de)
	 * 
	 * @param objPid
	 *          Pid der Objekttypen
	 * @return Liste mit den Systemobjekten
	 */
	private List<SystemObject> bestimmeObjekte(String objPid)
	{
		// System.out.println("bestimmeObjekte --> " + objPid);

		List<SystemObject> objekte = new ArrayList<SystemObject>();

		// Systemobjekt erzeugen

		SystemObjectType typeSysObj = null;

		typeSysObj = _verbindung.getDataModel().getType(objPid);

		if (typeSysObj == null)
			return objekte;

		if (typeSysObj.isConfigurating())
		{
			typeSysObj = (ConfigurationObjectType) typeSysObj;
			// System.out.println("isConfigurating");
		} else
		{
			typeSysObj = (DynamicObjectType) typeSysObj;
			// System.out.println("Dynamic");
		}

		// Elemente dieses Systemobjekts einlesen

		List<SystemObject> typobjekte = typeSysObj.getElements();

		Iterator<SystemObject> iterator_Typen = typobjekte.iterator();

		while (iterator_Typen.hasNext())
		{
			SystemObject sysObj = (SystemObject) iterator_Typen.next();

			objekte.add(sysObj);

		}

		return objekte;
	}

	@Override
	public void dataRequest(SystemObject object, DataDescription dataDescription,
			byte state)
	{
		if (state == 0)
		{
			synchronized (_sendeSteuerung)
			{
				_sendeSteuerung.notify();
			}
		}
	}

	@Override
	public boolean isRequestSupported(SystemObject object,
			DataDescription dataDescription)
	{
		return true;
	}

	@SuppressWarnings("unused")
	private class Sender implements Runnable
	{
		/**
		 * Thread-Flag
		 */
		private boolean _laufeThread = false;

		public Sender()
		{
		}

		public void startThread()
		{
			if (!_laufeThread)
			{
				Thread th = new Thread(this);
				th.setName("Sender");
				_laufeThread = true;
				th.start();
			}
		}

		public void stopThread()
		{
			if (_laufeThread)
			{
				_laufeThread = false;
			}
		}

		@Override
		public void run()
		{
			while (_laufeThread)
			{

			}
		}

	}

	/*
	 * Empfang
	 */

	/**
	 * Ruft die netzweiten, benutzerspezifischen Einstellungen vom DaV ab und
	 * liefert diese als modifiableCopy eines {@link Data}-Objektes zurck.
	 */
	private Data empfangeDatenBenutzereinstellungenNetzweit()
	{
		Data returnData = null;

		if (DaVVerbindung.getInstanz().istDaVVerbindungAktiv())
		{
			if ((_verbindung != null) && (_dataModell != null))
			{
				String pidSysObj = bestimmePidOberflaechenObjekt();

				if (pidSysObj != null)
				{
					/*
					 * Eigentliche Anmeldung zum Empfang
					 */
					_systemObject = _dataModell.getObject(pidSysObj);

					_atg = _dataModell.getAttributeGroup(_atgBezeichnung);
					_asp = _dataModell.getAspect(_aspSoll);
					_dataDescription = new DataDescription(_atg, _asp);

					ResultData resultData = _verbindung.getData(_systemObject,
							_dataDescription, 10000);
					if (resultData.hasData())
					{
						returnData = resultData.getData().createModifiableCopy();
					}

				}
			}
		}

		return returnData;
	}

	/**
	 * Einfgen von {@link Data}s in die Queue
	 * 
	 * @param data
	 *          Einzufgendes {@link Data}
	 * @throws InterruptedException
	 *           Wird geworfen, wenn eine Unterbrechung eintritt.
	 */
	public void rein(Data data) throws InterruptedException
	{
		lock.lock();
		try
		{
			while (produkte.size() == MAXQUEUE)
				nichtVoll.await();

			produkte.add(data);

			nichtLeer.signalAll();
		} finally
		{
			lock.unlock();
		}
	}

	/**
	 * Rausholen von {@link Data}s aus der Queue
	 * 
	 * @return Jeweils das erste {@link Data} in der Queue
	 * @throws InterruptedException
	 *           Wird geworfen, wenn eine Unterbrechung eintritt.
	 */
	public Data raus() throws InterruptedException
	{
		lock.lock();
		try
		{
			while (produkte.size() == 0)
				nichtLeer.await();

			Data elem = produkte.poll();

			nichtVoll.signalAll();
			return elem;
		} finally
		{
			lock.unlock();
		}
	}

	@SuppressWarnings("unused")
	private class Warten implements Runnable
	{
		/**
		 * Thread-Flag
		 */
		private boolean _laufeThread = false;

		public Warten()
		{
			startThread();
		}

		public void startThread()
		{
			if (!_laufeThread)
			{
				Thread th = new Thread(this);
				th.setName("Wartethread");
				_laufeThread = true;
				th.start();
			}
		}

		public void stopThread()
		{
			if (_laufeThread)
			{
				_laufeThread = false;
			}
		}

		@Override
		public void run()
		{
			while (_laufeThread)
			{
				try
				{
					Thread.sleep(3000);
					synchronized (_warten)
					{
						_warten.notify();
					}
					stopThread();
				} catch (InterruptedException e)
				{
					e.printStackTrace();
				}
			}
		}

	}

	/**
	 * Baut die Verbindung zum DaV auf.
	 */
	@SuppressWarnings("unused")
	private void baueDaVVerbindungAuf()
	{
		try
		{
			_verbindung = new ClientDavConnection();

			// IP setzen
			_verbindung.getClientDavParameters().setDavCommunicationAddress(_ip);

			// Port setzen
			_verbindung.getClientDavParameters().setDavCommunicationSubAddress(_port);

			// Pfad zum Zwischenspeichern setzen
			_verbindung.getClientDavParameters().setConfigurationPath("C:\\temp");

			try
			{
				_verbindung.connect();
				System.out
						.println("Verbindungsaufbau: Verbindung erfolgreich aufgebaut.");

				try
				{
					_verbindung.login(_benutzer, _passwort);
					System.out
							.println("Verbindungsaufbau: Benutzer erfolgreich eingeloggt.");

				} catch (InconsistentLoginException e)
				{
					_verbindung = null;
				}

			} catch (CommunicationError e)
			{
				_verbindung = null;
			} catch (ConnectionException e)
			{
				_verbindung = null;
			}

		} catch (MissingParameterException e)
		{
			_verbindung = null;
		}
	}

	/**
	 * Schliet die Verbindung zum DaV
	 */
	@SuppressWarnings("unused")
	private void schliesseDaVVerbindung()
	{
		if (_verbindung != null)
		{
			_verbindung.disconnect(false, "Kein Fehler.");
		}
	}

	/**
	 * Setzt die DaV-Verbindung.
	 * 
	 * @param verbindung
	 *          DaV-Verbindung
	 */
	public void setDaVVerbindung(ClientDavConnection verbindung)
	{
		this._verbindung = verbindung;
		if (_verbindung != null)
		{
			_dataModell = _verbindung.getDataModel();
		}
	}

	/**
	 * Bestimmt die PID des Oberflchenobjektes an Hand des
	 * Kommandozeilenparameters <code>-oberflaechenObjekt</code>. Wird dieser
	 * nicht gefunden wird der AOE verwendet.
	 * 
	 * @return Pid des Oberflchenobjektes oder <code>null</code>, wenn diese Pid
	 *         nicht bestimmt werden konnte.
	 */
	protected String bestimmePidOberflaechenObjekt()
	{
		String[] kommandozeilenArgumente = Platform.getCommandLineArgs();

		String pidSysObj = null;

		/*
		 * Suchen des bentigten Parameters -oberflaechenObjekt
		 */
		for (int i = 0; i < kommandozeilenArgumente.length; i++)
		{
			if (kommandozeilenArgumente[i].contains("-oberflaechenObjekt"))
			{
				String[] split = kommandozeilenArgumente[i].split("=");
				if (split.length == 2)
				{
					pidSysObj = split[1];
				}

				/*
				 * Prfen ob ein Objekt zu dieser PID existiert. Ist dies nicht der
				 * Fall, wird pidSysObj auf Null gesetzt und der AOE wird verwendet.
				 */
				_systemObject = _dataModell.getObject(pidSysObj);

				if (_systemObject == null)
				{
					final String meldung = "Der Parameter -oberflaechenObjekt ist fehlerhaft. "
							+ "Es existiert kein OberflchenObjekt mit der PID "
							+ pidSysObj
							+ ". Bestimme und verwende die PID der AutarkenOrganisationsEinheit "
							+ "zur Bestimmung der benutzerspezifischen, netzweiten Einstellungen (Klasse:"
							+ this.getClass().getName() + ").";

					Activator.getDefault().getLog().log(
							new Status(Status.WARNING, Activator.PLUGIN_ID, Status.OK,
									meldung, null));

					if (!_meldungFalschesOberflaechenObjektSchonGezeigt)
					{
						Runnable run = new Runnable()
						{

							@Override
							public void run()
							{
								MessageBox mb = new MessageBox(PlatformUI.getWorkbench()
										.getDisplay().getActiveShell(), SWT.OK | SWT.ICON_WARNING);
								mb.setText("Parameter -oberflaechenObjekt fehlerhaft");
								mb.setMessage(meldung);
								mb.open();
							}

						};

						PlatformUI.getWorkbench().getDisplay().syncExec(run);

						_meldungFalschesOberflaechenObjektSchonGezeigt = true;
					}

					pidSysObj = null;
				}

				break;
			}
		}

		/*
		 * Wenn dieser nicht gefunden wurde, AOE verwenden!
		 */
		if (pidSysObj == null)
		{
			pidSysObj = _verbindung.getLocalConfigurationAuthority().getPid();
			// System.out.println("Objekt schreiben: " + pidSysObj);

		}

		return pidSysObj;
	}

	/**
	 * Legt fest, ob mit oder ohne {@link UrlasserDialog} gesendet werden soll.
	 * 
	 * @param sendenOhneUrlasserDialog
	 *          Wenn <code>true</code>, dann wird mit {@link UrlasserDialog}
	 *          gesendet, sonst nicht.
	 */
	public void setSendenOhneUrlasserDialog(boolean sendenOhneUrlasserDialog)
	{
		System.out.println("===> " + sendenOhneUrlasserDialog);
		this._sendenOhneUrlasserDlg = sendenOhneUrlasserDialog;
	}

	/**
	 * Prft, ob mit oder ohne {@link UrlasserDialog} gesendet wird.
	 * 
	 * @return Wenn <code>true</code>, dann wird mit {@link UrlasserDialog}
	 *         gesendet, sonst nicht.
	 */
	public boolean isSendenOhneUrlasserDialog()
	{
		return _sendenOhneUrlasserDlg;
	}

}
