/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Monitor;
import org.eclipse.swt.widgets.Shell;

import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Eingabedialog fr Zeitfilter. <br>
 * 
 * In der Klasse KalenderZeitFilter werden Eingabefelder fr Zeitfilter zur
 * Verfgung gestellt.<br>
 * Eingabefelder: Zeitfilter Monatstag (1-31)<br>
 * Zeitfilter Monat (Januar,...)<br>
 * Zeitfilter Wochentag (Montag,...)<br>
 * Zeitfilter Zeitspanne(Zeitgruppen (Anfangs- und Endzeitpunkt)<br>
 * Bei den Zeitgruppen gibt es neben der Tastatureingabe noch die Mglichkeit
 * der Eingabe ber eine Auswahlbox und die Mglichkeit weitere Zeitgruppen
 * hinzuzufgen. <br>
 * Die Klasse erfllt folgende Technische Anforderungen des Segments Bedienung
 * und Visualisierung (BUV)<br> - TBuV-43 und 46
 *
 * 
 * @author Dambach-Werke GmbH
 * @author HofmannJ
 * @version 1.0
 */
public class KalenderZeitFilterDialog extends Dialog implements SelectionListener
{
	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	/**
	 * Shell, in der dieser Dialog dargestellt wird.
	 */
	private Shell _shell = null;

	private Group _gruppeMonat = null;

	private Group _gruppeTag = null;

	private Group _gruppeWochentag = null;

	private Group _gruppeZeitgruppen = null;

	private ScrolledComposite _verschiebbaresCompositeZeitgruppen = null;

	private Composite _compositeZeitGruppenInhalt = null;

	private Composite _compositeUnten = null;

	private Button[] _monatCheckboxButtons = null;

	private Button[] _tagCheckboxButtons = null;

	private Button[] _wochentagCheckboxButtons = null;

	/**
	 * Datenstruktur, die in diesem Dialog angezeigt/manipuliert werden kann
	 */
	private KalenderZeitFilterDatenStruktur _DatenStruktur = null;

	/**
	 * Liste der Zeitgruppenzeilen, wird fr die Verwaltung der Darstellung der
	 * Zeitgruppenzeilen bentigt.
	 */
	private List<ZeitGruppenZeile> _listeZeitGruppenZeilen;

	/**
	 * Flag, signalisiert ob eine Initialisierung der Datenstrukturen durchgefhrt
	 * wurde oder nicht. Ist keine Initialisierung durchgefhrt worden, darf sich
	 * der Dialog nicht ffnen.
	 */
	private boolean _InitialisierungDurchgefuehrt = false;

	/**
	 * Konstruktor fr Dialoga. Shell & Style mssen angegben werden.
	 * 
	 * @param parent
	 *          Parent-Shell
	 * @param style
	 *          Style
	 */
	public KalenderZeitFilterDialog(Shell parent, int style)
	{
		super(parent, style);
	}

	/**
	 * ffnet den Dialog und gibt die erstellte/manipulierte
	 * KalenderZeitFilterDatenStruktur zurck.
	 * 
	 * @return Erstellte/Manipulierte KalenderZeitFilterDatenStruktur
	 */
	public KalenderZeitFilterDatenStruktur oeffne(
			KalenderZeitFilterDatenStruktur daten)
	{
		if (daten != null)
		{
			init(daten);
		}

		/*
		 * Initialisieren mit leerer Datenstruktur.
		 */
		if (!_InitialisierungDurchgefuehrt)
		{	
			init();
		}

		Shell parent = getParent();
		// _shell = new Shell(parent, SWT.TITLE | SWT.BORDER
		// | SWT.APPLICATION_MODAL);
		_shell = new Shell(parent, SWT.TITLE | SWT.BORDER | SWT.RESIZE);
		_shell.setText("Zeitfilter");

		erzeugeGui();
		leseDatenstruktur();

		_shell.pack();
		_shell.open();
		_shell.setSize(720, 500);

		Display display = parent.getDisplay();

		// Dialog zentrieren
		Monitor primary = display.getPrimaryMonitor();
		Rectangle bounds = primary.getBounds();
		Rectangle rect = _shell.getBounds();
		int x = bounds.x + (bounds.width - rect.width) / 2;
		int y = bounds.y + (bounds.height - rect.height) / 2;
		_shell.setLocation(x, y);

		while (!_shell.isDisposed())
			if (!display.readAndDispatch())
				display.sleep();

		
		if(_DatenStruktur != null)
		{
			System.out.println(_DatenStruktur.erzeugeFilterString());
		}
		else
		{
			System.out.println(">>null<<");
		}
		
		return _DatenStruktur;
	}

	/**
	 * Fhrt eine Initialisierung ohne KalenderZeitFilterDatenStruktur durch.
	 * Diese Datenstruktur wird frisch erstellt.
	 * 
	 */
	private void init()
	{
		_DatenStruktur = new KalenderZeitFilterDatenStruktur();
		_listeZeitGruppenZeilen = new LinkedList<ZeitGruppenZeile>();
		_InitialisierungDurchgefuehrt = true;
	}

	/**
	 * Fhrt eine Initialisierung mit einer vorgegebenen
	 * KalenderZeitFilterDatenStruktur durch.
	 * 
	 * @param daten
	 *          Vorgegebene KalenderZeitFilterDatenStruktur
	 */
	private void init(KalenderZeitFilterDatenStruktur daten)
	{
		_DatenStruktur = daten;
		_listeZeitGruppenZeilen = new LinkedList<ZeitGruppenZeile>();
		_InitialisierungDurchgefuehrt = true;
	}

	/**
	 * Erzeugt die Benutzerschnittstelle.
	 * 
	 */
	private void erzeugeGui()
	{

		// Gruppe fr die Monate
		_gruppeMonat = new Group(_shell, SWT.NONE);
		_gruppeMonat.setText("Monat");

		// Gruppe fr die Tage
		_gruppeTag = new Group(_shell, SWT.NONE);
		_gruppeTag.setText("Tag");

		// Gruppe fr die Wochentage
		_gruppeWochentag = new Group(_shell, SWT.NONE);
		_gruppeWochentag.setText("Wochentag");

		// Gruppe fr die Zeitgruppen
		_gruppeZeitgruppen = new Group(_shell, SWT.NONE);
		_gruppeZeitgruppen.setText("Zeitgruppen");

		// ScrolledComposite fr die Zeitgruppen
		_verschiebbaresCompositeZeitgruppen = new ScrolledComposite(
				_gruppeZeitgruppen, SWT.H_SCROLL | SWT.V_SCROLL);
		
		_verschiebbaresCompositeZeitgruppen.getVerticalBar().setIncrement(20);
		
		_compositeZeitGruppenInhalt = new Composite(
				_verschiebbaresCompositeZeitgruppen, SWT.NONE);
		_compositeZeitGruppenInhalt.setLayout(new FillLayout());

		_compositeUnten = new Composite(_shell, SWT.NONE);

		fuelleGruppeMonatTagWochentTag();
		fuelleGruppeTag();
		fuelleGruppeWochenTag();
		fuelleCompositeUnten();

		// Layout groupMonat
		GridLayout glGroupMonat = new GridLayout();
		glGroupMonat.numColumns = 12;
		glGroupMonat.makeColumnsEqualWidth = true;
		_gruppeMonat.setLayout(glGroupMonat);

		// Layout groupTag
		GridLayout glGroupTag = new GridLayout();
		glGroupTag.numColumns = 16;
		glGroupTag.makeColumnsEqualWidth = true;
		_gruppeTag.setLayout(glGroupTag);

		// Layout groupWochenTag
		GridLayout glGroupWochentag = new GridLayout();
		glGroupWochentag.numColumns = 7;
		glGroupWochentag.makeColumnsEqualWidth = true;
		_gruppeWochentag.setLayout(glGroupWochentag);

		// Layout groupZeitgruppen

		_gruppeZeitgruppen.setLayout(new FillLayout());

		// Layout compositeZeitgruppen
		GridLayout glGroupZeitgruppen = new GridLayout();
		glGroupZeitgruppen.numColumns = 1;
		glGroupZeitgruppen.makeColumnsEqualWidth = false;

		_verschiebbaresCompositeZeitgruppen.setContent(_compositeZeitGruppenInhalt);
		_verschiebbaresCompositeZeitgruppen.setMinSize(600, 4000);
		_verschiebbaresCompositeZeitgruppen.setExpandHorizontal(true);
		_verschiebbaresCompositeZeitgruppen.setExpandVertical(true);

		_compositeZeitGruppenInhalt.setLayout(glGroupZeitgruppen);

		// GridDate CompositeUnten
		GridData gridDataCompositeUnten = new GridData();
		gridDataCompositeUnten.horizontalAlignment = GridData.END;
		gridDataCompositeUnten.grabExcessHorizontalSpace = true;
		gridDataCompositeUnten.verticalAlignment = GridData.CENTER;

		// Layout compositeUnten
		GridLayout glCompositeUnten = new GridLayout();
		glCompositeUnten.numColumns = 3;
		glCompositeUnten.makeColumnsEqualWidth = true;

		_compositeUnten.setLayout(glCompositeUnten);
		_compositeUnten.setLayoutData(gridDataCompositeUnten);

		GridData gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = false;
		gridData.verticalAlignment = GridData.BEGINNING;
		gridData.horizontalAlignment = GridData.FILL;

		GridData gridDataMehrPlatz = new GridData();
		gridDataMehrPlatz.grabExcessHorizontalSpace = true;
		gridDataMehrPlatz.grabExcessVerticalSpace = true;
		gridDataMehrPlatz.verticalAlignment = GridData.FILL;
		gridDataMehrPlatz.horizontalAlignment = GridData.FILL;

		_gruppeMonat.setLayoutData(gridData);
		_gruppeTag.setLayoutData(gridDataMehrPlatz);
		_gruppeWochentag.setLayoutData(gridData);
		_gruppeZeitgruppen.setLayoutData(gridDataMehrPlatz);

		// Zentrierte Elemente
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.CENTER;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.verticalAlignment = GridData.CENTER;

		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 1;
		_shell.setLayout(gridLayout);

	}

	/**
	 * Fllt die Gruppe Monate
	 * 
	 */
	private void fuelleGruppeMonatTagWochentTag()
	{
		Label label1 = new Label(_gruppeMonat, SWT.NONE);
		label1.setText("Januar");
		Label label2 = new Label(_gruppeMonat, SWT.NONE);
		label2.setText("Februar");
		Label label3 = new Label(_gruppeMonat, SWT.NONE);
		label3.setText("Mrz");
		Label label4 = new Label(_gruppeMonat, SWT.NONE);
		label4.setText("April");
		Label label5 = new Label(_gruppeMonat, SWT.NONE);
		label5.setText("Mai");
		Label label6 = new Label(_gruppeMonat, SWT.NONE);
		label6.setText("Juni");
		Label label7 = new Label(_gruppeMonat, SWT.NONE);
		label7.setText("Juli");
		Label label8 = new Label(_gruppeMonat, SWT.NONE);
		label8.setText("August");
		Label label9 = new Label(_gruppeMonat, SWT.NONE);
		label9.setText("September");
		Label label10 = new Label(_gruppeMonat, SWT.NONE);
		label10.setText("Oktober");
		Label label11 = new Label(_gruppeMonat, SWT.NONE);
		label11.setText("November");
		Label label12 = new Label(_gruppeMonat, SWT.NONE);
		label12.setText("Dezember");

		_monatCheckboxButtons = new Button[12];

		for (int i = 0; i < _monatCheckboxButtons.length; i++)
		{
			Button button = new Button(_gruppeMonat, SWT.CHECK);
			_monatCheckboxButtons[i] = button;
			button.addSelectionListener(this);
		}

	}

	/**
	 * Fllt die Gruppe Tag
	 * 
	 */
	private void fuelleGruppeTag()
	{
		_tagCheckboxButtons = new Button[31];
		for (int i = 0; i < _tagCheckboxButtons.length; i++)
		{
			Label tagLabel = new Label(_gruppeTag, SWT.NONE);
			tagLabel.setText((i + 1) + ": ");
			Button button = new Button(_gruppeTag, SWT.CHECK);
			_tagCheckboxButtons[i] = button;
			button.addSelectionListener(this);
		}
	}

	/**
	 * Fllt die Gruppe Wochentag
	 * 
	 */
	private void fuelleGruppeWochenTag()
	{
		_wochentagCheckboxButtons = new Button[7];
		String[] wochentagsNamen = { "Monatag", "Dienstag", "Mittwoch",
				"Donnerstag", "Freitag", "Samstag", "Sonntag" };
		for (String tagName : wochentagsNamen)
		{
			Label tagNameLabel = new Label(_gruppeWochentag, SWT.NONE);
			tagNameLabel.setText(tagName);
		}
		for (int i = 0; i < _wochentagCheckboxButtons.length; i++)
		{
			Button button = new Button(_gruppeWochentag, SWT.CHECK);
			_wochentagCheckboxButtons[i] = button;
			button.addSelectionListener(this);
		}
	}

	

	/**
	 * Fllt das CompositeUnten
	 * 
	 */
	private void fuelleCompositeUnten()
	{
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.CENTER;

		Button buttonOk = new Button(_compositeUnten, SWT.PUSH);
		buttonOk.setText("OK");
		buttonOk.setLayoutData(gridData);
		buttonOk.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_debug.fine("Button OK wurde gedrckt.");
				
				_shell.dispose();
			}			
		});

		Button buttonAbbrechen = new Button(_compositeUnten, SWT.PUSH);
		buttonAbbrechen.setText("Abbrechen");
		buttonAbbrechen.setLayoutData(gridData);
		buttonAbbrechen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_debug.fine("Button Abbrechen wurde gedrckt!");

				_shell.dispose();
			}			
		});

		Button buttonRuecksetzen = new Button(_compositeUnten, SWT.PUSH);
		buttonRuecksetzen.setText("Rcksetzen");
		buttonRuecksetzen.setLayoutData(gridData);
		buttonRuecksetzen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_debug.fine("Button Rcksetzen wurde gedrckt!");

				Label testLabel = new Label(_compositeZeitGruppenInhalt, SWT.BORDER);
				testLabel.setText("Nachweis");
			}			
		});

	}

	/**
	 * Findet die Position eines Controls im <code>Monate</code> Array.
	 * 
	 * @param monKontrolle
	 *          Das Widget, dessen zugehriges Monat gefunden werden soll
	 * @return Index des gefundenen Elements
	 */
	private int findeMonatCheckbox(Button bu)
	{
		for (int i = 0; i < _monatCheckboxButtons.length; i++)
		{
			if (_monatCheckboxButtons[i] == bu)
			{
				return i;
			}
		}
		return -1;
	}

	private int findeTagCheckbox(Button bu)
	{
		for (int i = 0; i < _tagCheckboxButtons.length; i++)
		{
			if (_tagCheckboxButtons[i] == bu)
			{
				return i;
			}
		}
		return -1;
	}

	private int findeWochentagCheckbox(Button bu)
	{
		for (int i = 0; i < _wochentagCheckboxButtons.length; i++)
		{
			if (_wochentagCheckboxButtons[i] == bu)
			{
				return i;
			}
		}
		return -1;
	}

	/**
	 * Liest die Datenstruktur ein
	 * 
	 */
	private void leseDatenstruktur()
	{
		// Monate setzen
		int Monate[] = _DatenStruktur.getSelektierteMonate();
		for (int i : Monate)
		{
			_monatCheckboxButtons[i].setSelection(true);
		}

		// Tage setzen
		int Tage[] = _DatenStruktur.getSelektierteTage();
		for (int i : Tage)
		{
			_tagCheckboxButtons[i].setSelection(true);
		}

		// Wochentage setzen
		int WochenTage[] = _DatenStruktur.getSelektierteWochentage();
		for (int i : WochenTage)
		{
			_wochentagCheckboxButtons[i].setSelection(true);
		}

		// Zeitgruppen setzen
		List<KalenderZeitFilterDatenStrukturZeitGruppe> zeitGruppen = _DatenStruktur.getListeZeitGruppen();
		// Wenn die ZeitGruppenListe nicht leer ist, dann auslesen und darstellen
		if (zeitGruppen.size() > 0)
		{
			Iterator<KalenderZeitFilterDatenStrukturZeitGruppe> it = zeitGruppen.iterator();
			while (it.hasNext())
			{
				KalenderZeitFilterDatenStrukturZeitGruppe zg = it.next();
				generiereZeitgruppeMitVorgabe(zg);
			}
		} else
		// Sonst nur eine Leerzeile darstellen.
		{
			generiereZeitgruppeLeer();
		}

	}

	/**
	 * Generiert eine leere Zeitgruppe
	 * 
	 */
	private void generiereZeitgruppeLeer()
	{
		GregorianCalendar anfang = new GregorianCalendar(1970, 1, 1, 0, 0, 0);
		long nu = anfang.getTimeInMillis();		
		
		KalenderZeitFilterDatenStrukturZeitGruppe kz = new KalenderZeitFilterDatenStrukturZeitGruppe(
				nu, nu);
		_DatenStruktur.fuegeZeitGruppeHinzu(kz);
		ZeitGruppenZeile zgz = new ZeitGruppenZeile(_compositeZeitGruppenInhalt, kz);
		zgz.generiereComposite();
		
		_listeZeitGruppenZeilen.add(zgz);
		_compositeZeitGruppenInhalt.layout();		
	}
	

	/**
	 * Generiert eine Zeitgruppe mit Zeitvorgabe von Startzeitpunkt und
	 * Endzeitpunkt.
	 * 
	 * @param lStartZeitPunkt
	 *          Startzeitpunkt
	 * @param lEndZeitPunkt
	 *          Endzeitpunkt
	 */
	@SuppressWarnings("unused")
	private void generiereZeitgruppeMitVorgabe(long lStartZeitPunkt,
			long lEndZeitPunkt)
	{
		KalenderZeitFilterDatenStrukturZeitGruppe kz = new KalenderZeitFilterDatenStrukturZeitGruppe(
				lStartZeitPunkt, lEndZeitPunkt);
		_DatenStruktur.fuegeZeitGruppeHinzu(kz);
		ZeitGruppenZeile zgz = new ZeitGruppenZeile(_compositeZeitGruppenInhalt, kz);
		zgz.generiereComposite();
		_listeZeitGruppenZeilen.add(zgz);
		_compositeZeitGruppenInhalt.layout();
	}

	/**
	 * Generiert eine Zeitgruppe mit Zeitvorgabe in Form einer Instanz der Klasse
	 * KalenderZeitFilterDatensStrukturZeitGruppe
	 * 
	 * @param zg
	 *          Vorgabe von Start- und Endzeitpunkt
	 */
	private void generiereZeitgruppeMitVorgabe(
			KalenderZeitFilterDatenStrukturZeitGruppe zg)
	{
		_DatenStruktur.fuegeZeitGruppeHinzu(zg);
		ZeitGruppenZeile zgz = new ZeitGruppenZeile(_compositeZeitGruppenInhalt, zg);
		zgz.generiereComposite();
		_listeZeitGruppenZeilen.add(zgz);
		_compositeZeitGruppenInhalt.layout();
	}

	private void loescheZeitGruppenZeile(ZeitGruppenZeile zgz)
	{
		if (_listeZeitGruppenZeilen.contains(zgz))
		{
			_listeZeitGruppenZeilen.remove(zgz);
			zgz.dispose();
			_compositeZeitGruppenInhalt.layout();
		}
	}

	private void einfuegenZeitGruppenZeileNachPosition(ZeitGruppenZeile zg)
	{
		generiereZeitgruppeLeer();

		LinkedList<KalenderZeitFilterDatenStrukturZeitGruppe> _dsAlt = new LinkedList<KalenderZeitFilterDatenStrukturZeitGruppe>();

		int idx = _listeZeitGruppenZeilen.indexOf(zg) + 1;

		if (idx < _listeZeitGruppenZeilen.size())
		{
			if (idx > -1)
			{
				for (int i = idx; i < (_listeZeitGruppenZeilen.size() - 1); i++)
				{
					_dsAlt.add(_listeZeitGruppenZeilen.get(i)
							.getDatenStrukturZeitGruppe());
				}
			}

			int i = idx + 1;
			Iterator<KalenderZeitFilterDatenStrukturZeitGruppe> it = _dsAlt
					.iterator();
			while (it.hasNext())
			{
				KalenderZeitFilterDatenStrukturZeitGruppe datenStrukturZeitGruppe = it
						.next();
				_listeZeitGruppenZeilen.get(i).setDatenStrukturZeitGruppe(
						datenStrukturZeitGruppe);
				i++;
			}

			
			/*
			 * Neue Zeile mit Nullstellung erzeugen
			 */
			GregorianCalendar gcNullStellung = new GregorianCalendar();
			gcNullStellung.set(Calendar.HOUR_OF_DAY, 0);
			gcNullStellung.set(Calendar.MINUTE, 0);
			gcNullStellung.set(Calendar.SECOND, 0);
			gcNullStellung.set(Calendar.MILLISECOND, 0);
			
			_listeZeitGruppenZeilen.get(idx).setDatenStrukturZeitGruppe(
					new KalenderZeitFilterDatenStrukturZeitGruppe(gcNullStellung.getTimeInMillis(), gcNullStellung.getTimeInMillis()));
		}

	}

	// /**
	// * Fgt einen KaldenderZeitFilterListener zur Liste der
	// * KalenderZeitFilterListener hinzu.
	// * @param listener KalenderZeitFilterListener der hinzugefgt werden soll.
	// */
	// public void addKalenderZeitFilterListener(KalenderZeitFilterListener
	// listener)
	// {
	// _lKalenderZeitFilterListener.add(listener);
	// }

	// /**
	// * Entfernt einen KalenderZeitFilterListener von der Liste der
	// * KalenderZeitFilterListener.
	// * @param listener KalenderZeitFilterListener der entfernt werden soll
	// */
	// public void removeKalenderZeitFilterListener(KalenderZeitFilterListener
	// listener)
	// {
	// _lKalenderZeitFilterListener.remove(listener);
	// }

	// /**
	// * Wird aufgerufen, wenn sich etwas an der Zeit, z.B. nach einem Mausklick
	// * gendert hat. Im Array aller bekannter Listener wird der entsprechende
	// * SWTZeitEvent hinterlegt
	// *
	// * Wird der Aufruf vergessen, dann werden die Listener NICHT durch einen
	// event
	// * benachrichtigt.
	// *
	// */
	// private void fireEvent()
	// {
	//		
	// System.err.println("KalenderZeitFilter::timeChanged() => benachrichtige " +
	// _lKalenderZeitFilterListener.size() +"
	// Listener");
	// if(!_lKalenderZeitFilterListener.isEmpty())
	// {
	// Iterator<KalenderZeitFilterListener> it =
	// _lKalenderZeitFilterListener.iterator();
	// while(it.hasNext())
	// {
	// KalenderZeitFilterListener zfl = it.next();
	// zfl.zeitFilterGeaendert(new KalenderZeitFilterEvent(_DatenStruktur));
	// }
	// }
	// }

	/**
	 * Bildet die Struktur einer ZeitgruppenZeile
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class ZeitGruppenZeile
	{
		private Composite _parent = null;

		private KalenderZeitFilterDatenStrukturZeitGruppe _daten = null;

		private Label _labelVon;

		private Label _labelBis;

		private Label _labelInfo;

		private Button _schaltflaecheHinzufuegen = null;

		private Button _schaltflaecheLoeschen = null;

		/**
		 * Diese Composite enthlt die Darstellung eines Zeitgruppen-Eintrags
		 */
		private Composite _returnComposite;

		public DatumZeit _dzVon;

		public DatumZeit _dzBis;

		/**
		 * Konstruktor
		 * 
		 * @param parent
		 *          Eltern-Composite, in dass die Zeitgruppenzeile eingeblendet
		 *          werden soll.
		 * @param Daten
		 *          Von-Bis.
		 */
		public ZeitGruppenZeile(Composite parent,
				KalenderZeitFilterDatenStrukturZeitGruppe Daten)
		{
			this._parent = parent;
			this._daten = Daten;
		}

//		/**
//		 * Sichert die erfassten Zeiten in daten
//		 * 
//		 */
//		public void sichereErfassteZeiten()
//		{
//			// _daten.setStartzeitpunkt(_kzVon.getZeit());
//			// _daten.setEndzeitpunkt(_kzBis.getZeit());
//
//			// _daten.setStartzeitpunkt(_dzVon.get)
//
//			_daten.setStartzeitpunkt(_dzVon.getDatum().getTime());
//			_daten.setEndzeitpunkt(_dzBis.getDatum().getTime());
//
//		}

		/**
		 * Liefert die {@link KalenderZeitFilterDatenStrukturZeitGruppe} dieser
		 * {@link ZeitGruppenZeile}.
		 * 
		 * @return {@link KalenderZeitFilterDatenStrukturZeitGruppe}
		 */
		public KalenderZeitFilterDatenStrukturZeitGruppe getDatenStrukturZeitGruppe()
		{
			return this._daten;
		}

		/**
		 * Setzt die {@link KalenderZeitFilterDatenStrukturZeitGruppe} dieser
		 * {@link ZeitGruppenZeile}
		 * 
		 * @param datenStrukturZeitGruppe
		 *          {@link KalenderZeitFilterDatenStrukturZeitGruppe}
		 */
		public void setDatenStrukturZeitGruppe(
				KalenderZeitFilterDatenStrukturZeitGruppe datenStrukturZeitGruppe)
		{
			this._daten = datenStrukturZeitGruppe;
			_dzVon.setDatum(new Date(_daten.getStartzeitpunkt()));
			_dzBis.setDatum(new Date(_daten.getEndzeitpunkt()));
		}

		/**
		 * Liefert true zurck, wenn der Startzeitpunkt VOR dem Endzeitpunkt liegt.
		 * Andernfalls wird false geliefert.
		 * 
		 * @return true, wenn der Startzeitpunkt VOR dem Endzeitpunkt liegt.
		 */
		private boolean pruefePlausibilitaet()
		{
			boolean returnWert = true;
			if (_daten.getStartzeitpunkt() > _daten.getEndzeitpunkt())
			{
				returnWert = false;
			}

			return returnWert;
		}

		/**
		 * Dispose-Methode fr das {@link Composite} dieser {@link ZeitGruppenZeile}
		 */
		public void dispose()
		{
			_returnComposite.dispose();
		}

		/**
		 * Lscht diese {@link ZeitGruppenZeile}.
		 */
		private void aktionZeitgruppenzeileLoeschen()
		{
			loescheZeitGruppenZeile(this);
		}

		/**
		 * Hngt eine Zeitgruppenzeile an
		 */
		private void aktionZeitgruppenzeileAnhaengen()
		{
			// generiereZeitgruppeLeer();
			einfuegenZeitGruppenZeileNachPosition(this);
		}

		/**
		 * Generiert das Composuite
		 * 
		 * @return Ein Composite
		 */
		public Composite generiereComposite()
		{
			_returnComposite = new Composite(_parent, SWT.BORDER);

			RowLayout rl = new RowLayout(SWT.HORIZONTAL);
			rl.pack = false;
			_returnComposite.setLayout(rl);

			_labelVon = new Label(_returnComposite, SWT.NONE);
			_labelVon.setText("von: ");

			// _kzVon = new KalenderZeit(_returnComposite, SWT.NONE, true, true,
			// true);
			// _kzVon.setZeit(_daten.get_startzeitpunkt());

			_dzVon = new DatumZeit(_returnComposite, SWT.NONE,
					DatumZeit.Eingabetyp.uhrMS, false, false);
//			_dzVon = new DatumZeit(_returnComposite, SWT.NONE,
//					DatumZeit.Eingabetyp.uhrEinfachMS, false, false);
			_dzVon.setDatum(new Date(_daten.getStartzeitpunkt()));
			_dzVon.setLayout(new GridLayout());

			// CDateTime _cdt = new CDateTime(_returnComposite, CDT.DROP_DOWN |
			// CDT.DATE_MEDIUM | CDT.TIME_MEDIUM |
			// CDT.CLOCK_24_HOUR);

			_labelBis = new Label(_returnComposite, SWT.NONE);
			_labelBis.setText("bis: ");

			// _kzBis = new KalenderZeit(_returnComposite, SWT.NONE, true, true,
			// true);
			// _kzBis.setZeit(_daten.get_endzeitpunkt());

			_dzBis = new DatumZeit(_returnComposite, SWT.NONE,
					DatumZeit.Eingabetyp.uhrMS, false, false);
//			_dzBis = new DatumZeit(_returnComposite, SWT.NONE,
//					DatumZeit.Eingabetyp.uhrEinfachMS, false, false);
			
			_dzBis.setDatum(new Date(_daten.getStartzeitpunkt()));
			_dzBis.setLayout(new GridLayout());

			_labelInfo = new Label(_returnComposite, SWT.NONE);

			_dzVon.hinzufuegenSelektionsZuhoerer(new SelectionListener()
			{

				private long _startzeitpunkt = -1;

				public void widgetDefaultSelected(SelectionEvent e)
				{
					gemeinsam(e);
				}

				public void widgetSelected(SelectionEvent e)
				{
					gemeinsam(e);
				}

				private void gemeinsam(SelectionEvent e)
				{
					if (e.data instanceof Date)
					{
						Date datum = (Date) e.data;
						_startzeitpunkt = datum.getTime();
						_daten.setStartzeitpunkt(_startzeitpunkt);

						if (!pruefePlausibilitaet())
						{
							Color color = new Color(Display.getDefault(), 255, 0, 0);
							_labelInfo.setForeground(color);
							_labelInfo.setText("Startzeitpunkt liegt vor dem Endzeitpunkt!");
							_compositeZeitGruppenInhalt.layout();
						} else
						{
							_labelInfo.setText("");
							_compositeZeitGruppenInhalt.layout();
						}

						// fireEvent();
					}
				}

			});

			_dzBis.hinzufuegenSelektionsZuhoerer(new SelectionListener()
			{

				private long _endzeitpunkt = -1;

				public void widgetDefaultSelected(SelectionEvent e)
				{
					gemeinsam(e);
				}

				public void widgetSelected(SelectionEvent e)
				{
					gemeinsam(e);
				}

				private void gemeinsam(SelectionEvent e)
				{
					if (e.data instanceof Date)
					{
						Date datum = (Date) e.data;
						_endzeitpunkt = datum.getTime();
						_daten.setEndzeitpunkt(_endzeitpunkt);

						if (!pruefePlausibilitaet())
						{
							Color color = new Color(Display.getDefault(), 255, 0, 0);
							_labelInfo.setForeground(color);
							_labelInfo.setText("Startzeitpunkt liegt vor dem Endzeitpunkt!");
							_compositeZeitGruppenInhalt.layout();
						} else
						{
							_labelInfo.setText("");
							_compositeZeitGruppenInhalt.layout();
						}

						// fireEvent();
					}
				}

			});

		

			_schaltflaecheHinzufuegen = new Button(_returnComposite, SWT.NONE);
			_schaltflaecheHinzufuegen.setText("+");
			_schaltflaecheLoeschen = new Button(_returnComposite, SWT.NONE);
			_schaltflaecheLoeschen.setText("-");

			_schaltflaecheHinzufuegen.addSelectionListener(new SelectionAdapter()
			{
				@Override
				public void widgetSelected(SelectionEvent e)
				{
					aktionZeitgruppenzeileAnhaengen();
				}			
			});
						
			_schaltflaecheLoeschen.addSelectionListener(new SelectionAdapter()
			{
				@Override
				public void widgetSelected(SelectionEvent e)
				{
					aktionZeitgruppenzeileLoeschen();
				}			
			});

			GridData gridData = new GridData();
			gridData.horizontalAlignment = GridData.FILL;
			gridData.grabExcessHorizontalSpace = false;
			gridData.grabExcessVerticalSpace = false;
			gridData.verticalAlignment = GridData.CENTER;
			_returnComposite.setLayoutData(gridData);

			GridLayout gridLayout = new GridLayout();
			gridLayout.numColumns = 7;
			_returnComposite.setLayout(gridLayout);

			return _returnComposite;
		}

	}

	@Override
	public void widgetDefaultSelected(SelectionEvent e)
	{		
	}

	@Override
	public void widgetSelected(SelectionEvent e)
	{
		if (e.getSource() instanceof Button)
		{
			System.out.println("HERE");
			// Monat finden
			int monatIndex = findeMonatCheckbox((Button) e.getSource());
			if (monatIndex > -1)
			{
				System.out.println("Monat: " + monatIndex);
				if (((Button) e.getSource()).getSelection())
				{
					_DatenStruktur.selektiereMonat(monatIndex);
				} else
				{
					_DatenStruktur.deSelektiereMonat(monatIndex);
				}

				// fireEvent();
			}

			// Tag finden
			int tagIndex = findeTagCheckbox((Button) e.getSource());
			if (tagIndex > -1)
			{
				_debug.fine("Tag: " + tagIndex);
				if (((Button) e.getSource()).getSelection())
				{
					_DatenStruktur.selektiereTag(tagIndex);
				} else
				{
					_DatenStruktur.deSelektiereTag(tagIndex);
				}
				// fireEvent();
			}

			// Wochentag finden
			int wochentagIndex = findeWochentagCheckbox((Button) e.getSource());
			if (wochentagIndex > -1)
			{
				_debug.fine("Wochentag: " + wochentagIndex);
				if (((Button) e.getSource()).getSelection())
				{
					_DatenStruktur.selektiereWochentag(wochentagIndex);
				} else
				{
					_DatenStruktur.deSelektiereWochentag(wochentagIndex);
				}
				// fireEvent();
			}

		}
		else
		{
			System.out.println("null");
		}

		// verwalteStunden(index);
		//  			
		// waehleStunde(index);
		
	}

}
