/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender.ereignisSystemKalender;

import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.Activator;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;

/**
 * Dialog zur Auswahl von Ereignissen aus dem Ereignis-/Systemkalender.
 * 
 * @author HofmannJ
 * 
 */
public class DialogEreignisSystemkalenderUebernahme extends Dialog
{

	private Shell _shell = null;
	private Group _gruppeQuelle = null;
	private Group _gruppeEintraege = null;
	private Group _gruppeBereiche = null;
	private Combo _comboQuelle = null;
	private Composite _compositeDatum = null;
	private Button _schaltflaecheLinksJahr = null;
	private Button _schaltflaecheLinksMonat = null;
	private Text _textJahr = null;
	private Text _textMonat = null;
	private Button _schaltflaecheRechtsJahr = null;
	private Button _schaltflaecheRechtsMonat = null;
	private Group _gruppeAktion = null;
	private Composite _compositeAktion = null;
	private Button _schalflaecheOk = null;
	private Button _schaltflaecheAbbruch = null;
	private Table _tabelleEintraege = null;
	private TableViewer _tableViewerEintraege = null;
	private ComboViewer _comboViewer = null;
	private Button _schaltflaecheHilfe = null;
	private Label _labelDaVVerbindung;
	private Table _tabelleBereiche;
	private TableViewer _tableViewerBereiche;

	private static final String[] _monate = new String[] { "Januar", "Februar",
			"Mrz", "April", "Mai", "Juni", "Juli", "August", "September", "Oktober",
			"November", "Dezember" };

	/**
	 * Index zur Bestimmung des Monats
	 */
	private int _idxMonat = 0;

	/**
	 * Index zur Bestimmung des Jahres
	 */
	private int _idxJahr = 2008;

	/**
	 * Flag das Anzeigt, ob der Systemkalender-Arbeiter gestartet wurde oder
	 * nicht.
	 */
	private boolean _sysKalArbeiterGestartet = false;

	/**
	 * Arbeiter fr Anfragen an den Systemkalender
	 */
	private SysKalAnfrageArbeiter _sysKalAnfrageArbeiter = null;

	private DaVKommunikation _dav = null;

	/**
	 * Quelle
	 */
	private Quelle _quelle = Quelle.systemkalender;

	/**
	 * Selektierter Bereich
	 */
	@SuppressWarnings("unused")
	private IEintragBereich _bereich = null;

	/**
	 * Liste der selektierten Bereiche
	 */
	private LinkedList<IEintragBereich> _selektierteBereiche = new LinkedList<IEintragBereich>();

	/**
	 * TableColumn fr die Eintragsart im jeweiligen Bereich.
	 */
	private TableColumn _tcEintragsArt;

	/**
	 * Titel fr Systemkalendereintrge.
	 */
	private static final String EINTRAGS_ART_SYSTEMKALENDER = "Systemkalendereintrag";

	/**
	 * Titel fr Ereigniskalendereintrge.
	 */
	private static final String EINTRAGS_ART_EREIGNISKALENDER = "Ereigniskalendereintrag";

	/**
	 * Konstruktor dieses Dialoges.
	 * 
	 * @param parent
	 *          {@link Shell}.
	 */
	public DialogEreignisSystemkalenderUebernahme(Shell parent)
	{
		super(parent, 0);

		_shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL
				| SWT.RESIZE);
		_shell.setText("Ereignis- und Systemkalenderbernahme");
		Image img = Activator.getImageDescriptor(
				"icons/sonstiges/zeit_datum_16x16.png").createImage();
		_shell.setImage(img);

		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;

		_labelDaVVerbindung = new Label(_shell, SWT.NONE);
		_labelDaVVerbindung.setForeground(new Color(PlatformUI.getWorkbench()
				.getDisplay(), 255, 0, 0));
		_labelDaVVerbindung.setText("Keine DaV-Verbindung aufgebaut!");
		_labelDaVVerbindung.setVisible(false);

		erzeugeGruppeQuelle();
		erzeugeGruppeEintraege();
		erzeugeGruppeBereiche();
		erzeugeGruppeAktion();
		_shell.setLayout(new GridLayout());
		_shell.setSize(new Point(400, 500));

		GregorianCalendar gc = new GregorianCalendar();
		_idxJahr = gc.get(Calendar.YEAR);
		_idxMonat = gc.get(Calendar.MONTH);

		/*
		 * Felder fr Jahr und Monat vorbelegen
		 */
		_textJahr.setText("" + _idxJahr);
		_textMonat.setText(_monate[_idxMonat]);

		/*
		 * Bereich bestimmen
		 */
		bestimmeBereich();

		/*
		 * Positionieren des Dialogs bei der Position des Mauszeigers
		 */
		Rectangle bounds = _shell.getBounds();
		Point cursorLocation = Display.getDefault().getCursorLocation();
		Rectangle newBounds = new Rectangle(cursorLocation.x, cursorLocation.y,
				bounds.width, bounds.height);
		_shell.setBounds(newBounds);
	}

	/**
	 * Setzt das Jahr
	 * 
	 * @param jahr
	 *          Jahr
	 */
	public void setJahr(int jahr)
	{
		this._idxJahr = jahr;
		_textJahr.setText("" + _idxJahr);
	}

	/**
	 * Setzt den Monat. 0 = Januar, 1 = Februar, ..., 11 = Dezember.
	 * 
	 * @param monat
	 *          Index des Monat >= 0 und <= 11
	 */
	public void setMonat(int monat)
	{
		if ((monat >= 0) && (monat <= 11))
		{
			this._idxMonat = monat;
			_textMonat.setText(_monate[_idxMonat]);
		}
	}

	/**
	 * ffnet den Dialog. Es werden Ereignis- und Systemkalendereintrge
	 * bercksichtigt.
	 * 
	 * @return Liste der ausgewhlten Eintrgen.
	 */
	public LinkedList<IEintragBereich> oeffnen()
	{
		_comboViewer.setSelection(new StructuredSelection(_comboViewer
				.getElementAt(0)), true);

		darstellen();

		// return _bereich;
		return _selektierteBereiche;
	}

	/**
	 * ffnet den Dialog. Es werden Eintrge entsprechend der angegebenen
	 * {@link Quelle}.
	 * 
	 * @param quelle
	 *          <code>Quelle.ereigniskalender</code> oder
	 *          <code>Quelle.systemkalender</code>.
	 * @return Liste der ausgewhlten Eintrge.
	 */
	public LinkedList<IEintragBereich> oeffnen(Quelle quelle)
	{
		if (quelle != null)
		{
			_comboViewer.setSelection(new StructuredSelection(quelle));
		}

		darstellen();

		// return _bereich;
		return _selektierteBereiche;
	}

	private void darstellen()
	{
		/*
		 * Wenn keine DaV-Verbindung vorhanden ist, wird eine entsprechende Meldung
		 * eingeblendet.
		 */
		if (!DaVVerbindung.getInstanz().istDaVVerbindungAktiv())
		{
			_labelDaVVerbindung.setVisible(true);
		}

		/*
		 * Selektion der Quelle
		 */
		_comboViewer.addSelectionChangedListener(new ISelectionChangedListener()
		{

			@Override
			public void selectionChanged(SelectionChangedEvent event)
			{
				ISelection selection = event.getSelection();
				if (selection instanceof IStructuredSelection)
				{
					IStructuredSelection iss = (IStructuredSelection) selection;

					if (iss.getFirstElement() instanceof Quelle)
					{
						Quelle quelle = (Quelle) iss.getFirstElement();
						// System.out.println("=>" + quelle);
						_quelle = quelle;
						bestimmeBereich();
					}
				}

			}

		});

		_shell.open();

		Display display = PlatformUI.getWorkbench().getDisplay();
		while (!_shell.isDisposed())
		{
			if (!display.readAndDispatch())
				display.sleep();
		}
	}

	/**
	 * Initialisiert gruppeQuelle
	 * 
	 */
	private void erzeugeGruppeQuelle()
	{
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;
		_gruppeQuelle = new Group(_shell, SWT.NONE);
		_gruppeQuelle.setLayout(new GridLayout());
		_gruppeQuelle.setLayoutData(gridData);
		erzeugeComboQuelle();
		_gruppeQuelle.setText("Quelle:");
		erzeugeCompositeDatum();
	}

	/**
	 * Initialisiert gruppeEintraege
	 * 
	 */
	private void erzeugeGruppeEintraege()
	{
		GridData gridData8 = new GridData();
		gridData8.horizontalAlignment = GridData.FILL;
		gridData8.grabExcessHorizontalSpace = true;
		gridData8.grabExcessVerticalSpace = true;
		gridData8.verticalAlignment = GridData.FILL;
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.grabExcessVerticalSpace = true;
		gridData1.verticalAlignment = GridData.FILL;
		_gruppeEintraege = new Group(_shell, SWT.NONE);
		_gruppeEintraege.setLayout(new GridLayout());
		_gruppeEintraege.setLayoutData(gridData1);
		_gruppeEintraege.setText("Eintrge:");
		_tabelleEintraege = new Table(_gruppeEintraege, SWT.FULL_SELECTION);
		_tabelleEintraege.setHeaderVisible(true);
		_tabelleEintraege.setLayoutData(gridData8);
		_tabelleEintraege.setLinesVisible(true);
		_tableViewerEintraege = new TableViewer(_tabelleEintraege);

		_tcEintragsArt = new TableColumn(_tabelleEintraege, SWT.LEFT);
		_tcEintragsArt.setText("Systemkalendereintrag");
		_tcEintragsArt.setWidth(345);

		SysErKalEintragContentProvider cp = new SysErKalEintragContentProvider();
		_tableViewerEintraege.setContentProvider(cp);

		SysErKalEintragLabelProvider lp = new SysErKalEintragLabelProvider();
		_tableViewerEintraege.setLabelProvider(lp);

		_tableViewerEintraege
				.addSelectionChangedListener(new ISelectionChangedListener()
				{

					@Override
					public void selectionChanged(SelectionChangedEvent event)
					{
						ISelection selection = event.getSelection();
						if (selection instanceof IStructuredSelection)
						{
							IStructuredSelection iss = (IStructuredSelection) selection;
							Object firstElement = iss.getFirstElement();
							if (firstElement instanceof IEintragKapsel)
							{
								IEintragKapsel kapsel = (IEintragKapsel) firstElement;
								_tableViewerBereiche.setInput(kapsel);

								/*
								 * Erstes Element selektieren
								 */
								if (_tabelleBereiche.getItemCount() > 0)
								{

									// StructuredSelection ss = new StructuredSelection(
									// _tableViewerBereiche.getElementAt(0));
									// _tableViewerBereiche.setSelection(ss, true);

									_tabelleBereiche.setFocus();
									_tabelleBereiche.selectAll();
								}
							}
						}
					}

				});

	}

	private void erzeugeGruppeBereiche()
	{
		GridData gridDataBereiche = new GridData();
		gridDataBereiche.horizontalAlignment = GridData.FILL;
		gridDataBereiche.grabExcessHorizontalSpace = true;
		gridDataBereiche.grabExcessVerticalSpace = true;
		gridDataBereiche.verticalAlignment = GridData.FILL;

		GridData gridDataTabelleBereiche = new GridData();
		gridDataTabelleBereiche.horizontalAlignment = GridData.FILL;
		gridDataTabelleBereiche.grabExcessHorizontalSpace = true;
		gridDataTabelleBereiche.grabExcessVerticalSpace = true;
		gridDataTabelleBereiche.verticalAlignment = GridData.FILL;

		_gruppeBereiche = new Group(_shell, SWT.NONE);
		_gruppeBereiche.setLayout(new GridLayout());
		_gruppeBereiche.setLayoutData(gridDataBereiche);
		_gruppeBereiche.setText("Bereiche:");
		_tabelleBereiche = new Table(_gruppeBereiche, SWT.MULTI
				| SWT.FULL_SELECTION);
		_tabelleBereiche.setHeaderVisible(true);
		_tabelleBereiche.setLayoutData(gridDataTabelleBereiche);
		_tabelleBereiche.setLinesVisible(true);
		_tableViewerBereiche = new TableViewer(_tabelleBereiche);

		TableColumn tcVon = new TableColumn(_tabelleBereiche, SWT.LEFT);
		tcVon.setText("Von:");
		tcVon.setWidth(175);

		TableColumn tcBis = new TableColumn(_tabelleBereiche, SWT.LEFT);
		tcBis.setText("Bis:");
		tcBis.setWidth(175);

		SysErKalBereichContentProvider cp = new SysErKalBereichContentProvider();
		_tableViewerBereiche.setContentProvider(cp);
		SysErKalBereichLabelProvider lp = new SysErKalBereichLabelProvider();
		_tableViewerBereiche.setLabelProvider(lp);

		_tableViewerBereiche
				.addSelectionChangedListener(new ISelectionChangedListener()
				{

					@Override
					public void selectionChanged(SelectionChangedEvent event)
					{
						ISelection selection = event.getSelection();
						if (selection instanceof IStructuredSelection)
						{
							IStructuredSelection iss = (IStructuredSelection) selection;
							Object firstElement = iss.getFirstElement();
							if (firstElement instanceof IEintragBereich)
							{
								IEintragBereich bereich = (IEintragBereich) firstElement;
								_bereich = bereich;
							}

							/*
							 * Liste Fllen
							 */
							_selektierteBereiche.clear();
							@SuppressWarnings("unchecked")
							Iterator it = iss.iterator();
							while (it.hasNext())
							{
								Object object = it.next();
								if (object instanceof IEintragBereich)
								{
									IEintragBereich bereich = (IEintragBereich) object;
									_selektierteBereiche.add(bereich);
								}
							}
						}
					}

				});

	}

	/**
	 * Erzeugt comboQuelle
	 * 
	 */
	private void erzeugeComboQuelle()
	{
		GridData gridData5 = new GridData();
		gridData5.horizontalAlignment = GridData.FILL;
		gridData5.grabExcessHorizontalSpace = true;
		gridData5.verticalAlignment = GridData.CENTER;
		_comboQuelle = new Combo(_gruppeQuelle, SWT.NONE);
		_comboQuelle.setLayoutData(gridData5);
		_comboViewer = new ComboViewer(_comboQuelle);

		/*
		 * Eintragen der Werte in die Quellenauswahl
		 */
		Quelle[] quellen = new Quelle[] { Quelle.systemkalender,
				Quelle.ereigniskalender };
		_comboViewer.add(quellen);

	}

	protected void aktionUebernehmen()
	{
		ISelection selection = _tableViewerBereiche.getSelection();

		if (selection.isEmpty())
		{
			_bereich = null;
			_selektierteBereiche = null;
		} else if (selection instanceof IStructuredSelection)
		{
			/*
			 * Erstes Element
			 */
			IStructuredSelection iss = (IStructuredSelection) selection;
			Object firstElement = iss.getFirstElement();
			if (firstElement instanceof IEintragBereich)
			{
				IEintragBereich bereich = (IEintragBereich) firstElement;
				_bereich = bereich;
			}

			/*
			 * Liste Fllen
			 */
			_selektierteBereiche.clear();
			@SuppressWarnings("unchecked")
			Iterator it = iss.iterator();
			while (it.hasNext())
			{
				Object object = it.next();
				if (object instanceof IEintragBereich)
				{
					IEintragBereich bereich = (IEintragBereich) object;
					_selektierteBereiche.add(bereich);
				}
			}
			System.out.println("ANZ: " + _selektierteBereiche.size());
		}

		/*
		 * Aktionen zum bernehmen
		 */
		aktionSchliessen();
	}

	/**
	 * Aktionen hinter der Schaltflche Abbrechen.
	 */
	private void aktionAbbrechen()
	{
		_bereich = null;
		_selektierteBereiche = null;
		aktionSchliessen();
	}

	private void aktionSchliessen()
	{
		_shell.setVisible(false);
		_shell.dispose();
	}

	/**
	 * Aktionen hinter der Schaltflche Hilfe
	 */
	protected void aktionHilfe()
	{

		String helpContextId = Activator.PLUGIN_ID + ".ereignisSyskalDialog";
		PlatformUI.getWorkbench().getHelpSystem().displayHelp(helpContextId);
	}

	protected void aktionJahrLinks()
	{
		_idxJahr = _idxJahr - 1;
		_textJahr.setText("" + _idxJahr);
		bestimmeBereich();
	}

	protected void aktionJahrRechts()
	{
		_idxJahr = _idxJahr + 1;
		_textJahr.setText("" + _idxJahr);
		bestimmeBereich();
	}

	protected void aktionMonatLinks()
	{
		if (_idxMonat >= 0)
		{
			_idxMonat = _idxMonat - 1;
		}
		if (_idxMonat < 0)
		{
			_idxMonat = 11;
		}

		_textMonat.setText(_monate[_idxMonat]);
		bestimmeBereich();
	}

	protected void aktionMonatRechts()
	{
		if (_idxMonat <= 11)
		{
			_idxMonat = _idxMonat + 1;
		}
		if (_idxMonat > 11)
		{
			_idxMonat = 0;
		}

		_textMonat.setText(_monate[_idxMonat]);
		bestimmeBereich();
	}

	protected void bestimmeBereich()
	{
		GregorianCalendar gcStart = new GregorianCalendar();
		gcStart.set(_idxJahr, _idxMonat, 1, 0, 0, 0);
		gcStart.set(Calendar.MILLISECOND, 0);

		GregorianCalendar gcEnde = new GregorianCalendar();
		gcEnde.setTime(new Date(gcStart.getTimeInMillis()));
		gcEnde.add(Calendar.MONTH, 1);
		// gcEnde.set(Calendar.DAY_OF_MONTH, 0);
		gcEnde.add(Calendar.MILLISECOND, -1);

		ClientDavInterface veb = DaVVerbindung.getInstanz().getDaVVerbindung();
		if (veb != null)
		{
			if ((_quelle != null) && (_quelle.equals(Quelle.systemkalender)))
			{
				_tcEintragsArt.setText(EINTRAGS_ART_SYSTEMKALENDER);
				
				starteSysKaArbeiter();

				LinkedList<SkeKapsel> list = _sysKalAnfrageArbeiter
						.berechneGueltigVonBisS(gcStart.getTimeInMillis(), gcEnde
								.getTimeInMillis());

				if (list != null)
				{
					Collections.sort(list);
				}

				_tableViewerEintraege.setInput(list);

				/*
				 * Selektion in Tabelle Eintrge setzen
				 */
				if (_tabelleEintraege.getItemCount() > 0)
				{
					StructuredSelection ss = new StructuredSelection(
							_tableViewerEintraege.getElementAt(0));
					_tableViewerEintraege.setSelection(ss, true);

				}
			} else if ((_quelle != null) && (_quelle.equals(Quelle.ereigniskalender)))
			{
				_tcEintragsArt.setText(EINTRAGS_ART_EREIGNISKALENDER);
				
				starteSysKaArbeiter();

				if (_dav == null)
				{
					_dav = new DaVKommunikation();
				}

				LinkedList<EintragEreigniskalender> eintraege = _dav
						.getEreigniskalenderEintraegeAlle();
				if ((eintraege != null) && (eintraege.size() > 0))
				{
					LinkedList<IEintragKapsel> kapseln = new LinkedList<IEintragKapsel>();
					Iterator<EintragEreigniskalender> it = eintraege.iterator();
					while (it.hasNext())
					{
						EintragEreigniskalender ereigniskalender = it.next();
						ErkaKapsel kapsel = new ErkaKapsel(ereigniskalender,
								_sysKalAnfrageArbeiter, gcStart.getTimeInMillis(), gcEnde
										.getTimeInMillis());
						kapseln.add(kapsel);
					}
					_tableViewerEintraege.setInput(kapseln);

					/*
					 * Selektion in Tabelle Eintrge setzen
					 */
					if (_tabelleEintraege.getItemCount() > 0)
					{
						StructuredSelection ss = new StructuredSelection(
								_tableViewerEintraege.getElementAt(0));
						_tableViewerEintraege.setSelection(ss, true);
					}

				}
			}
		}

	}

	public void starteSysKaArbeiter()
	{
		ClientDavInterface veb = DaVVerbindung.getInstanz().getDaVVerbindung();
		String kalenderObj = veb.getLocalConfigurationAuthority().getPid();

		if (_sysKalAnfrageArbeiter == null)
		{
			_sysKalAnfrageArbeiter = new SysKalAnfrageArbeiter(veb, kalenderObj);
		}

		if (!_sysKalArbeiterGestartet)
		{
			if (_sysKalAnfrageArbeiter != null)
			{
				_sysKalAnfrageArbeiter.getSyskalArbeiter()
						.starteSystemKalenderArbeiter();
				_sysKalArbeiterGestartet = true;
			}
		}
	}

	/**
	 * This method initializes compositeDatum
	 * 
	 */
	private void erzeugeCompositeDatum()
	{
		GridData gridData4 = new GridData();
		gridData4.horizontalAlignment = GridData.FILL;
		gridData4.grabExcessHorizontalSpace = true;
		gridData4.verticalAlignment = GridData.CENTER;
		GridData gridData3 = new GridData();
		gridData3.horizontalAlignment = GridData.FILL;
		gridData3.grabExcessHorizontalSpace = true;
		gridData3.verticalAlignment = GridData.CENTER;
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 3;
		GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.verticalAlignment = GridData.CENTER;
		_compositeDatum = new Composite(_gruppeQuelle, SWT.NONE);
		_compositeDatum.setLayoutData(gridData2);
		_compositeDatum.setLayout(gridLayout);
		_schaltflaecheLinksJahr = new Button(_compositeDatum, SWT.NONE);
		_schaltflaecheLinksJahr.setText("<");
		_schaltflaecheLinksJahr.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionJahrLinks();
			}
		});

		_textJahr = new Text(_compositeDatum, SWT.BORDER | SWT.CENTER);
		_textJahr.setEditable(false);
		_textJahr.setLayoutData(gridData3);
		_schaltflaecheRechtsJahr = new Button(_compositeDatum, SWT.NONE);
		_schaltflaecheRechtsJahr.setText(">");
		_schaltflaecheRechtsJahr.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionJahrRechts();
			}
		});

		_schaltflaecheLinksMonat = new Button(_compositeDatum, SWT.NONE);
		_schaltflaecheLinksMonat.setText("<");
		_schaltflaecheLinksMonat.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionMonatLinks();
			}
		});

		_textMonat = new Text(_compositeDatum, SWT.BORDER | SWT.CENTER);
		_textMonat.setEditable(false);
		_textMonat.setLayoutData(gridData4);
		_schaltflaecheRechtsMonat = new Button(_compositeDatum, SWT.NONE);
		_schaltflaecheRechtsMonat.setText(">");
		_schaltflaecheRechtsMonat.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionMonatRechts();
			}
		});
	}

	/**
	 * This method initializes gruppeAktion
	 * 
	 */
	private void erzeugeGruppeAktion()
	{
		GridLayout gridLayout1 = new GridLayout();
		gridLayout1.numColumns = 1;
		GridData gridData6 = new GridData();
		gridData6.horizontalAlignment = GridData.FILL;
		gridData6.grabExcessHorizontalSpace = true;
		gridData6.verticalAlignment = GridData.CENTER;
		_gruppeAktion = new Group(_shell, SWT.NONE);
		_gruppeAktion.setLayoutData(gridData6);
		erzeugeCompositeAktion();
		_gruppeAktion.setLayout(gridLayout1);
		_gruppeAktion.setText("Aktion:");
	}

	/**
	 * This method initializes _compositeAktion
	 * 
	 */
	private void erzeugeCompositeAktion()
	{
		RowLayout rowLayout = new RowLayout();
		rowLayout.pack = false;
		GridData gridData7 = new GridData();
		gridData7.horizontalAlignment = GridData.END;
		gridData7.grabExcessHorizontalSpace = true;
		gridData7.verticalAlignment = GridData.CENTER;
		_compositeAktion = new Composite(_gruppeAktion, SWT.NONE);
		_compositeAktion.setLayoutData(gridData7);
		_compositeAktion.setLayout(rowLayout);
		_schalflaecheOk = new Button(_compositeAktion, SWT.NONE);
		_schalflaecheOk.setText("bernehmen");
		_schalflaecheOk.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionUebernehmen();
			}
		});

		_schaltflaecheAbbruch = new Button(_compositeAktion, SWT.NONE);
		_schaltflaecheAbbruch.setText("Abbrechen");
		_schaltflaecheAbbruch.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionAbbrechen();
			}
		});

		_schaltflaecheHilfe = new Button(_compositeAktion, SWT.NONE);
		_schaltflaecheHilfe.setText("Hilfe");
		_schaltflaecheHilfe.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionHilfe();
			}
		});
	}

	/**
	 * Contentprovider fr Ereignis-/Systemkalendereintrge.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class SysErKalEintragContentProvider implements
			IStructuredContentProvider
	{
		@SuppressWarnings("unchecked")
		@Override
		public Object[] getElements(Object inputElement)
		{
			Object[] re = null;

			if (inputElement instanceof LinkedList)
			{
				LinkedList<IEintragKapsel> kapseln = (LinkedList<IEintragKapsel>) inputElement;
				re = new Object[kapseln.size()];
				int i = 0;
				Iterator<IEintragKapsel> it = kapseln.iterator();
				while (it.hasNext())
				{
					IEintragKapsel skeKapsel = it.next();
					re[i] = skeKapsel;
					i++;
				}
			}

			return re;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}

	}

	/**
	 * Labelprovider fr Ereignis-/Systemkalendereintrge.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class SysErKalEintragLabelProvider implements ITableLabelProvider
	{

		@Override
		public Image getColumnImage(Object element, int columnIndex)
		{
			return null;
		}

		@Override
		public String getColumnText(Object element, int columnIndex)
		{
			String re = "";
			if (element instanceof IEintragKapsel)
			{
				IEintragKapsel skeKapsel = (IEintragKapsel) element;
				re = skeKapsel.getPid();
			}
			return re;
		}

		@Override
		public void addListener(ILabelProviderListener listener)
		{
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		@Override
		public void removeListener(ILabelProviderListener listener)
		{
		}

	}

	/**
	 * Contentprovider fr Ereignis-/Systemkalenderbereiche.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class SysErKalBereichContentProvider implements
			IStructuredContentProvider
	{

		@Override
		public Object[] getElements(Object inputElement)
		{
			Object[] re = null;
			if (inputElement instanceof IEintragKapsel)
			{
				IEintragKapsel skeKapsel = (IEintragKapsel) inputElement;
				re = skeKapsel.getBereiche().toArray();
			}

			return re;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}

	}

	/**
	 * Labelprovider fr Ereignis-/Systemkalendereintrge.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class SysErKalBereichLabelProvider implements ITableLabelProvider
	{

		@Override
		public Image getColumnImage(Object element, int columnIndex)
		{
			return null;
		}

		@Override
		public String getColumnText(Object element, int columnIndex)
		{
			String re = "";
			if (element instanceof IEintragBereich)
			{
				IEintragBereich skeBereich = (IEintragBereich) element;
				if (columnIndex == 0)
				{
					re = skeBereich.getVonLesbar();
				} else if (columnIndex == 1)
				{
					re = skeBereich.getBisLesbar();
				}

			}

			return re;
		}

		@Override
		public void addListener(ILabelProviderListener listener)
		{
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		@Override
		public void removeListener(ILabelProviderListener listener)
		{
		}

	}

	/**
	 * Enumerator zur Auswahl der Quelle (Ereignis- oder Systemkalender)
	 * 
	 * @author HofmannJ
	 * 
	 */
	public enum Quelle
	{
		ereigniskalender("Ereigniskalender", "Eintrge aus dem Ereigniskalender"), systemkalender(
				"Systemkalender", "Eintrge aus dem Systemkalender");

		/**
		 * Bezeichnung des Eintrags
		 */
		private String _bezeichnung;

		/**
		 * Beschreibung des Eintrags
		 */
		private String _beschreibung;

		/**
		 * Konstruktor. Setzt Bezeichnung und Beschreibung des Eintrags
		 * 
		 * @param bezeichnung
		 *          Bezeichnung des Eintrags
		 * @param beschreibung
		 *          Beschreibung des Eintrags
		 */
		private Quelle(String bezeichnung, String beschreibung)
		{
			this._bezeichnung = bezeichnung;
			this._beschreibung = beschreibung;
		}

		/**
		 * Liefert die Bezeichnung.
		 * 
		 * @return Bezeichnung
		 */
		public String getBezeichnung()
		{
			return _bezeichnung;
		}

		/**
		 * Liefert die Beschreibung.
		 * 
		 * @return Beschreibung
		 */
		public String getBeschreibung()
		{
			return _beschreibung;
		}

		public String toString()
		{
			String re = _bezeichnung + " (" + _beschreibung + ")";
			return re;
		}

	}

}
