/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender.ereignisSystemKalender;

import java.util.Iterator;
import java.util.LinkedList;

import de.bsvrz.vew.syskal.syskal.systemkalendereintrag.SystemkalenderEintrag;

/**
 * Kapsel fr Systemkalendereintrge.
 * 
 * @author HofmannJ
 * 
 */
public class SkeKapsel implements Comparable<IEintragKapsel>, IEintragKapsel
{
	/**
	 * Gekapselter Systemkalendereintrag
	 */
	private SystemkalenderEintrag _ske = null;

	/**
	 * Die Zeitbereiche dieses Systemkalendereintrags als Instanzen der Klasse
	 * {@link SkeBereich}.
	 */
	private LinkedList<IEintragBereich> _bereiche = new LinkedList<IEintragBereich>();

	/**
	 * Konstruktor, bernimmt einen {@link SystemkalenderEintrag} fr diese
	 * Kapsel.
	 * 
	 * @param ske
	 *          {@link SystemkalenderEintrag}
	 */
	public SkeKapsel(SystemkalenderEintrag ske)
	{
		this._ske = ske;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.bsvrz.buv.rw.basislib.kalender.ereignissystem.IEintragKapsel#getPid()
	 */
	public String getPid()
	{
		return _ske.getPid();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.bsvrz.buv.rw.basislib.kalender.ereignissystem.IEintragKapsel#addBereich
	 * (java.lang.Long, java.lang.Long)
	 */
	public void addBereich(Long von, Long bis)
	{
		SkeBereich bereich = new SkeBereich(von, bis);
		_bereiche.add(bereich);
	}

	@Override
	public void addBereich(IEintragBereich bereich)
	{
		this._bereiche.add(bereich);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.bsvrz.buv.rw.basislib.kalender.ereignissystem.IEintragKapsel#getBereiche
	 * ()
	 */
	public LinkedList<IEintragBereich> getBereiche()
	{
		return _bereiche;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seede.bsvrz.buv.rw.basislib.kalender.ereignissystem.IEintragKapsel#
	 * getBereicheAnzahl()
	 */
	public int getBereicheAnzahl()
	{
		return _bereiche.size();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.bsvrz.buv.rw.basislib.kalender.ereignissystem.IEintragKapsel#toString()
	 */
	public String toString()
	{
		String re = _ske.getPid();
		return re;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.bsvrz.buv.rw.basislib.kalender.ereignissystem.IEintragKapsel#toStringLong
	 * ()
	 */
	public String toStringLong()
	{
		String re = "";
		if (_ske != null)
		{
			re = re + "===" + _ske.getPid() + "===\n";

			if (_bereiche.size() > 0)
			{
				int counter = 0;

				Iterator<IEintragBereich> it = _bereiche.iterator();
				while (it.hasNext())
				{
					re = re + "Bereich " + counter + ": " + it.next() + "\n";
					counter++;
				}

			} else
			{
				re = re + "Keine Bereiche definiert!\n";
			}

			for (int i = 0; i < _ske.getPid().length() + 6; i++)
			{
				re = re + "=";
			}
			re = re + "\n";
		}

		return re;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.bsvrz.buv.rw.basislib.kalender.ereignissystem.IEintragKapsel#compareTo
	 * (de.bsvrz.buv.rw.basislib.kalender.ereignissystem.SkeKapsel)
	 */
	@Override
	public int compareTo(IEintragKapsel o)
	{
		String pidThis = getPid();
		String pidOther = o.getPid();
		return pidThis.compareTo(pidOther);
	}

}
