/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.login;

import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.preference.PreferenceStore;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Monitor;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.Activator;
import de.bsvrz.buv.rw.rw.dav.BenutzerAngemeldet;
import de.bsvrz.buv.rw.rw.dav.BenutzerOffline;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Log-in-Dialog zur Anmeldung an das System . In der unteren Reihe wird der
 * Domain-Name (bzw. die IP-Adresse) und die Port-Adresse eingegeben, darber
 * Name und Passwort des Benutzers.
 * 
 * <br>
 * Die Klasse erfllt folgende Technische Anforderungen des Segments Bedienung
 * und Visualisierung (BUV)<br>
 * - TBuV-93
 * 
 * @author Dambach-Werke GmbH
 * @author HofmannJ
 * @version $Revision: 1.7 $ / $Date: 2012/08/22 07:10:09 $ / ($Author: HofmannJ
 *          $)
 */
public class LoginDialog extends Dialog
{
	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	/**
	 * Liste der Server-IPs bzw. Domainnamen
	 */
	private List<String> _serverIps = new LinkedList<String>();

	/**
	 * Liste der Server-Ports
	 */
	private List<String> _serverPorts = new LinkedList<String>();

	private boolean _praeferenzenSpeicherGeladen = false;

	/**
	 * Prfix fr eine ServerIP in dem Properties-File
	 */
	private String _aufbauStringServerIpId = "ServerIp";

	/**
	 * Prfix fr einen Server-Port in dem Properties-file
	 */
	private String _aufbauStringServerPortId = "ServerPort";

	/**
	 * Name des Properties-Files, welche die letzten 5 Serveradressen/ports
	 * speichert.
	 */
	private String _praeferenzenSpeicherDateiname = "login.properties";

	private PreferenceStore _praeferenzenSpeicher = null;

	/**
	 * Maximale Anzahl von Eintrgen in der Historie
	 */
	private static final int _maxLoginHistorie = 5;

	/**
	 * Name
	 */
	private String _name;

	/**
	 * Passwort
	 */
	private String _passwort;

	/**
	 * Domne
	 */
	private String _domain;

	/**
	 * Port
	 */
	private int _tcpPort;

	private Label _labelBenutzername = null;

	private Label _labelPasswort = null;

	private Label _labelAdresse = null;

	private Text _textBenutzername = null;

	private Text _textPasswort = null;

	private Combo _comboAdresse = null;

	private Label _labelPort = null;

	private Combo _comboPort = null;

	private Button _buttonOk = null;

	private Button _buttonOfflineBetrieb = null;

	private Button _buttonBeenden = null;

	/**
	 * Breite der Buttons
	 */
	private int _breiteButton = 90;

	/**
	 * Hhe der Buttons
	 */
	private int _hoeheButton = 31;

	/**
	 * Startoffste der Buttons, linke Seite
	 */
	private int _xButtonStartOffset = 114;

	/**
	 * Abstand der Buttons zueinander
	 */
	private int _xButtonAbstand = 20;

	/**
	 * X-Position des OK-Buttons
	 */
	private int _xButtonOk = _xButtonStartOffset;

	/**
	 * Y-Position des OK-Buttons
	 */
	private int _yButtonOk = 100;

	/**
	 * X-Position des OfflineBetriebs-Buttons
	 */
	private int _xButtonOfflineBetrieb = _xButtonOk + _breiteButton
			+ _xButtonAbstand;

	/**
	 * Y-Position des OfflineBetriebs-Buttons
	 */
	private int _yButtonOfflineBetrieb = 100;

	/**
	 * X-Position des Beenden-Buttons
	 */
	private int _xButtonBeenden = _xButtonOfflineBetrieb + _breiteButton
			+ _xButtonAbstand;

	/**
	 * Y-Position des Beenden Buttons
	 */
	private int _yButtonBeenden = 100;

	private boolean _EingabeOk = false;

	private BenutzerAngemeldet _benutzerAngemeldet;

	private Shell _shell;

	private boolean _dynamisch = false;

	/**
	 * Konstruktor zur Initialisierung des Login-Dialogs zur Systemanmeldung. Die
	 * gestalt des Dialogs ist MODAL. ber die Methode <code>anzeigen()</code>
	 * wird der Dialog geffnet.
	 * 
	 * @param parent
	 *          Eltern-Shell.
	 */
	public LoginDialog(Shell parent)
	{
		super(parent);
	}

	/**
	 * Prft, ob das Passwortfeld ausgefllt wurde.
	 * 
	 * @param passwort
	 *          Zu prfender String
	 * @return Liefert <code>true</code>, wenn das Feld ausgefllt wurde, sonst
	 *         <code>false</code>.
	 */
	private boolean verifizierePasswort(String passwort)
	{
		boolean ergebnis = false;

		if ((passwort != null) && (passwort.length() > 0))
		{
			ergebnis = true;
		} else
		{
			String message = "Das im Login-Dialog angegebene Passwort ist zu kurz!";

			MessageBox mb = new MessageBox(getParent(), SWT.OK | SWT.ICON_ERROR);
			mb.setText("Passwortfeld nicht ausgefllt");
			mb.setMessage(message);
			mb.open();
		}

		return ergebnis;
	}

	/**
	 * Prft, ob das Benutzerfeld ausgefllt wurde.
	 * 
	 * @param benutzer
	 *          Zu prfender String.
	 * @return Liefert <code>true</code>, wenn das Feld ausgefllt wurde, sonst
	 *         <code>false</code>.
	 */
	private boolean verifiziereBenutzer(String benutzer)
	{
		boolean ergebnis = false;

		if ((benutzer != null) && (benutzer.length() > 0))
		{
			ergebnis = true;
		} else
		{
			String message = "Der im Login-Dialog angegebene Benutzername ist zu kurz!";

			MessageBox mb = new MessageBox(getParent(), SWT.OK | SWT.ICON_ERROR);
			mb.setText("Benutzerfeld nicht ausgefllt");
			mb.setMessage(message);
			mb.open();
		}

		return ergebnis;
	}

	/**
	 * Prft die Gltigkeit einer IP-Adresse.
	 * 
	 * @param ipAdresse
	 *          Zu prfende IP-Adresse
	 * @return Liefert <code>true</code>, wenn die IP-Adresse gltig ist, sonst
	 *         <code>false</code>.
	 */
	private boolean verifiziereIpAdresse(String ipAdresse)
	{
		boolean ergebnis = false;

		try
		{
			InetAddress.getByName(ipAdresse);
			ergebnis = true;
		} catch (UnknownHostException e)
		{
			String message = "Die im Login-Dialog angegebene IP-Adresse \""
					+ ipAdresse + "\" ist formal nicht korrekt.";

			MessageBox mb = new MessageBox(getParent(), SWT.OK | SWT.ICON_ERROR);
			mb.setText("Ungltige IP");
			mb.setMessage(message);
			mb.open();

		}

		return ergebnis;
	}

	/**
	 * Prft, ob der angegebene Port eine Zahl ist und im Bereich von 0 - 65535
	 * liegt und damit gltig ist.
	 * 
	 * @param port
	 *          Zu prfender Port.
	 * @return Liefert <code>false</code>, wenn der Port keine Zahl ist oder
	 *         ausserhalb der gltigen Bereiches liegt, sonst <code>true</code>.
	 */
	private boolean verifizierePort(String port)
	{
		boolean ergebnis = false;
		try
		{
			int wert = Integer.valueOf(port);
			if ((wert >= 0) && (wert <= 65535))
			{
				ergebnis = true;
			} else
			{
				String message = "Der im Login-Dialog angegebene Port \""
						+ wert
						+ "\" liegt nicht im Bereich von 0 bis 65535 und ist damit ungltig!";

				MessageBox mb = new MessageBox(getParent(), SWT.OK | SWT.ICON_ERROR);
				mb.setText("Port ausserhalb des gltigen Bereichs");
				mb.setMessage(message);
				mb.open();
			}

		} catch (NumberFormatException e)
		{
			String message = "Der im Login-Dialog angegebene Port \"" + port
					+ "\" ist keine Zahl und damit ungltig!";

			MessageBox mb = new MessageBox(getParent(), SWT.OK | SWT.ICON_ERROR);
			mb.setText("Port ist keine Zahl");
			mb.setMessage(message);
			mb.open();
		}

		return ergebnis;
	}

	/**
	 * Fhrt die Aktionen des OK-Buttons aus und versucht das Login durchzufhren.
	 */
	private void aktionOnlineModus()
	{
		// Sicherstellen, dass die Combobox fr den Port auch ein
		// Ergebnis liefert.

		if (_comboPort.getItemCount() > 0)
		{
			if (_comboPort.getText().equals(""))
			{
				if (_comboPort.getItem(0).equals(""))
				{
					_comboPort.add("80");
				} else
				{
					_comboPort.select(0);
				}
			}
		}

		// Prfung auf ausgefllte Textfelder
		_name = _textBenutzername.getText();
		_passwort = _textPasswort.getText();
		_domain = _comboAdresse.getText();

		try
		{
			_tcpPort = Integer.valueOf(_comboPort.getText());
		} catch (NumberFormatException nfe)
		{
			// wird durch nachfolgendes Konstrukt abgefangen
		}

		if ((verifiziereBenutzer(_name)) && (verifizierePasswort(_passwort))
				&& (verifiziereIpAdresse(_domain))
				&& (verifizierePort(_comboPort.getText())))
		{
			_EingabeOk = true;

			_benutzerAngemeldet = BenutzerAngemeldet.getInstanz();
			_benutzerAngemeldet.setAngemeldeterBenutzer(_name, _passwort, _domain,
					_tcpPort);

			hinzufuegenIp(_domain);
			hinzufuegenPort(_comboPort.getText());

			speicherePraeferenzen();

			// getParent().dispose();

			_shell.setVisible(false);
			_shell.dispose();

		}
	}

	/**
	 * Fhrt die Aktion hinter dem Button Offline Betrieb durch und startet das
	 * Rahmenwerk im Offline Modus.
	 */
	private void aktionOfflineModus()
	{
		_EingabeOk = false;

		// getParent().dispose();

		if (_comboPort.getItemCount() > 0)
		{
			if (_comboPort.getText().equals(""))
			{
				if (_comboPort.getItem(0).equals(""))
				{
					_comboPort.add("80");
				} else
				{
					_comboPort.select(0);
				}
			}
		}

		// Prfung auf ausgefllte Textfelder
		_name = _textBenutzername.getText();
		_passwort = _textPasswort.getText();
		_domain = _comboAdresse.getText();

		try
		{
			_tcpPort = Integer.valueOf(_comboPort.getText());
		} catch (NumberFormatException nfe)
		{
			// wird durch nachfolgendes Konstrukt abgefangen
		}

		if (verifiziereBenutzer(_name))
		{
			BenutzerOffline benutzerOffline = BenutzerOffline.getInstance();

			benutzerOffline.setOfflineBenutzer(_name, _passwort, _domain, _tcpPort);

			hinzufuegenIp(_domain);
			hinzufuegenPort(_comboPort.getText());
		}
		_shell.setVisible(false);
		_shell.dispose();
	}

	/**
	 * Fhrt die Aktion hinter dem Button Beenden durch. Das Rahmenwerk wird
	 * beendet, sofern es noch nicht aktiv war. Ansonsten wird nur der Dialog
	 * geschlossen.
	 */
	private void aktionAbbrechen()
	{
		if (!DaVVerbindung.getInstanz().istArbeitsbereichAktiv())
		{
			DaVVerbindung.getInstanz().infoBereichAbschalten();
			System.exit(0);
		} else
		{
			_shell.setVisible(false);
			_shell.dispose();
		}
	}

	/**
	 * ffnet die Shell
	 * 
	 * @return boolean <code>true</code>, wenn die Darstellung erfolgen konnte,
	 *         sonst <code>false</code>.
	 */
	public boolean anzeigen()
	{
		if (!_dynamisch)
		{
			Shell parent = getParent();
			_shell = new Shell(parent, SWT.TITLE | SWT.BORDER | SWT.SYSTEM_MODAL);
			_shell.setText("Login Dialog");

			ladePraeferenzen();

			// ////
			// Zusammenstellen der SWT-Elemente fr den Dialog

			_shell.setSize(new Point(559, 168));
			_shell.setLayout(null);

			_labelBenutzername = new Label(_shell, SWT.NONE);
			_labelBenutzername.setBounds(new Rectangle(10, 13, 91, 13));
			_labelBenutzername.setText("Benutzername:");

			_textBenutzername = new Text(_shell, SWT.BORDER);
			_textBenutzername.setBounds(new Rectangle(150, 10, 391, 21));
			_textBenutzername.addListener(SWT.DefaultSelection, new Listener()
			{
				public void handleEvent(Event e)
				{
					aktionOnlineModus();
				}
			});

			_labelPasswort = new Label(_shell, SWT.NONE);
			_labelPasswort.setBounds(new Rectangle(10, 43, 91, 13));
			_labelPasswort.setText("Passwort:");

			_textPasswort = new Text(_shell, SWT.BORDER | SWT.PASSWORD);
			_textPasswort.setBounds(new Rectangle(150, 40, 391, 21));

			/*
			 * Enter auf dem Passwortfeld lst die Einlog-Aktion aus.
			 */
			_textPasswort.addListener(SWT.DefaultSelection, new Listener()
			{
				public void handleEvent(Event e)
				{
					aktionOnlineModus();
				}
			});

			_labelAdresse = new Label(_shell, SWT.NONE);
			_labelAdresse.setBounds(new Rectangle(10, 73, 131, 13));
			_labelAdresse.setText("Domainname / IP-Adresse:");

			erzeugeAuswahlAdresse(_shell);

			_labelPort = new Label(_shell, SWT.NONE);
			_labelPort.setBounds(new Rectangle(315, 75, 91, 13));
			_labelPort.setText("TCP-Portnummer:");

			erzeugeAuswahlPort(_shell);

			@SuppressWarnings("unused")
			Label filler = new Label(_shell, SWT.NONE);

			/*
			 * Ok
			 */
			_buttonOk = new Button(_shell, SWT.NONE);
			// _buttonOk.setBounds(new Rectangle(180, 100, 90, 31));
			_buttonOk.setBounds(new Rectangle(_xButtonOk, _yButtonOk, _breiteButton,
					_hoeheButton));
			_buttonOk.setText("Online-Modus");

			_buttonOk.addSelectionListener(new SelectionAdapter()
			{
				public void widgetSelected(SelectionEvent e)
				{
					aktionOnlineModus();
				}
			});

			/*
			 * Button OfflineBetrieb
			 */
			_buttonOfflineBetrieb = new Button(_shell, SWT.NONE);
			// _buttonOfflineBetrieb.setBounds(new Rectangle(300, 100, 90, 31));
			_buttonOfflineBetrieb.setBounds(new Rectangle(_xButtonOfflineBetrieb,
					_yButtonOfflineBetrieb, _breiteButton, _hoeheButton));
			_buttonOfflineBetrieb.setText("Offline-Modus");

			_buttonOfflineBetrieb.addSelectionListener(new SelectionAdapter()
			{
				public void widgetSelected(SelectionEvent e)
				{
					aktionOfflineModus();
				}
			});
			
			_buttonOfflineBetrieb.setEnabled(!DaVVerbindung.getInstanz().istArbeitsbereichAktiv());
			

			/*
			 * Button Abbrechen
			 */
			_buttonBeenden = new Button(_shell, SWT.NONE);
			_buttonBeenden.setBounds(new Rectangle(_xButtonBeenden, _yButtonBeenden,
					_breiteButton, _hoeheButton));
			_buttonBeenden.setText("Abbrechen");

			_buttonBeenden.addSelectionListener(new SelectionAdapter()
			{
				public void widgetSelected(SelectionEvent e)
				{
					aktionAbbrechen();
				}
			});

			Control[] tabListe = new Control[] { _textBenutzername, _textPasswort,
					_comboAdresse, _comboPort, _buttonOk, _buttonOfflineBetrieb,
					_buttonBeenden };
			_shell.setTabList(tabListe);

			if (DaVVerbindung.getInstanz().istArbeitsbereichAktiv())
			{
				String helpContextId = Activator.PLUGIN_ID + ".logindialog";
				PlatformUI.getWorkbench().getHelpSystem()
						.setHelp(_shell, helpContextId);
			}

			_shell.open();

			Display display = parent.getDisplay();

			// Dialog zentrieren
			Monitor primary = display.getPrimaryMonitor();
			Rectangle bounds = primary.getBounds();
			Rectangle rect = _shell.getBounds();
			int x = bounds.x + (bounds.width - rect.width) / 2;
			int y = bounds.y + (bounds.height - rect.height) / 2;
			_shell.setLocation(x, y);

			while (!_shell.isDisposed())
				if (!display.readAndDispatch())
					display.sleep();

			return _EingabeOk;
		} else
		{
			// dynamisch
			return anzeigenDynamisch();
		}
	}

	/**
	 * ffnet die Shell mit einem dynamischen Layout.
	 * 
	 * @return boolean <code>true</code>, wenn die Darstellung erfolgen konnte,
	 *         sonst <code>false</code>.
	 */
	public boolean anzeigenDynamisch()
	{
		{
			Shell parent = getParent();
			_shell = new Shell(parent, SWT.TITLE | SWT.BORDER | SWT.SYSTEM_MODAL);
			_shell.setText("Login Dialog");

			GridLayout gridLayoutShell = new GridLayout();
			gridLayoutShell.numColumns = 1;
			_shell.setLayout(gridLayoutShell);
			_shell.setSize(new Point(559, 170));

			ladePraeferenzen();

			// ////
			// Zusammenstellen der SWT-Elemente fr den Dialog

			/*
			 * Composite oben
			 */
			GridLayout gridLayoutCompositeOben = new GridLayout();
			gridLayoutCompositeOben.numColumns = 4;
			GridData gridDataOben = new GridData();
			gridDataOben.horizontalAlignment = GridData.FILL;
			gridDataOben.grabExcessHorizontalSpace = true;
			gridDataOben.verticalAlignment = GridData.CENTER;
			Composite compositeOben = new Composite(_shell, SWT.NONE);
			compositeOben.setLayoutData(gridDataOben);
			compositeOben.setLayout(gridLayoutCompositeOben);

			/*
			 * Label Benutzername
			 */
			_labelBenutzername = new Label(compositeOben, SWT.NONE);
			_labelBenutzername.setText("Benutzername:");

			/*
			 * Text Benutzername
			 */
			GridData gridDataTextBenutzername = new GridData();
			gridDataTextBenutzername.horizontalAlignment = GridData.FILL;
			gridDataTextBenutzername.grabExcessHorizontalSpace = true;
			gridDataTextBenutzername.horizontalSpan = 3;
			gridDataTextBenutzername.verticalAlignment = GridData.CENTER;

			_textBenutzername = new Text(compositeOben, SWT.BORDER);
			_textBenutzername.setLayoutData(gridDataTextBenutzername);
			_textBenutzername.addListener(SWT.DefaultSelection, new Listener()
			{
				public void handleEvent(Event e)
				{
					aktionOnlineModus();
				}
			});

			/*
			 * Label Passwort
			 */
			_labelPasswort = new Label(compositeOben, SWT.NONE);
			_labelPasswort.setText("Passwort:");

			/*
			 * Text Passwort
			 */
			GridData gridDataTextPasswort = new GridData();
			gridDataTextPasswort.horizontalAlignment = GridData.FILL;
			gridDataTextPasswort.grabExcessHorizontalSpace = true;
			gridDataTextPasswort.horizontalSpan = 3;
			gridDataTextPasswort.verticalAlignment = GridData.CENTER;

			_textPasswort = new Text(compositeOben, SWT.BORDER | SWT.PASSWORD);
			_textPasswort.setLayoutData(gridDataTextPasswort);

			/*
			 * Enter auf dem Passwortfeld lst die Einlog-Aktion aus.
			 */
			_textPasswort.addListener(SWT.DefaultSelection, new Listener()
			{
				public void handleEvent(Event e)
				{
					aktionOnlineModus();
				}
			});

			/*
			 * Label Adresse
			 */
			_labelAdresse = new Label(compositeOben, SWT.NONE);
			_labelAdresse.setText("Domainname / IP-Adresse:");

			/*
			 * Combo Adresse
			 */
			GridData gridDataComboAdresse = new GridData();
			gridDataComboAdresse.horizontalAlignment = GridData.FILL;
			gridDataComboAdresse.grabExcessHorizontalSpace = true;
			gridDataComboAdresse.verticalAlignment = GridData.CENTER;
			Combo auswahlAdresse = erzeugeAuswahlAdresse(compositeOben);
			auswahlAdresse.setLayoutData(gridDataComboAdresse);

			/*
			 * Label Port
			 */
			_labelPort = new Label(compositeOben, SWT.NONE);
			_labelPort.setText("TCP-Portnummer:");

			/*
			 * Combo Port
			 */
			GridData gridDataComboPort = new GridData();
			gridDataComboPort.horizontalAlignment = GridData.FILL;
			gridDataComboPort.grabExcessHorizontalSpace = true;
			gridDataComboPort.verticalAlignment = GridData.CENTER;
			Combo auswahlPort = erzeugeAuswahlPort(compositeOben);
			auswahlPort.setLayoutData(gridDataComboPort);

			/*
			 * Composite Unten
			 */
			GridData gridDataUnten = new GridData();
			gridDataUnten.horizontalAlignment = GridData.CENTER;
			gridDataUnten.verticalAlignment = GridData.CENTER;
			RowLayout rowLayoutUnten = new RowLayout();
			rowLayoutUnten.pack = false;
			rowLayoutUnten.marginRight = 3;
			rowLayoutUnten.spacing = 15;
			rowLayoutUnten.marginLeft = 3;
			Composite compositeUnten = new Composite(_shell, SWT.NONE);
			compositeUnten.setLayout(rowLayoutUnten);
			compositeUnten.setLayoutData(gridDataUnten);

			/*
			 * Ok
			 */
			_buttonOk = new Button(compositeUnten, SWT.NONE);
			_buttonOk.setText("Online-Modus");

			_buttonOk.addSelectionListener(new SelectionAdapter()
			{
				public void widgetSelected(SelectionEvent e)
				{
					aktionOnlineModus();
				}
			});

			/*
			 * Button OfflineBetrieb
			 */
			_buttonOfflineBetrieb = new Button(compositeUnten, SWT.NONE);
			_buttonOfflineBetrieb.setText("Offline-Modus");

			_buttonOfflineBetrieb.addSelectionListener(new SelectionAdapter()
			{
				public void widgetSelected(SelectionEvent e)
				{
					aktionOfflineModus();
				}
			});
			_buttonOfflineBetrieb.setEnabled(!DaVVerbindung.getInstanz().istArbeitsbereichAktiv());

			/*
			 * Button Abbrechen
			 */
			_buttonBeenden = new Button(compositeUnten, SWT.NONE);
			_buttonBeenden.setText("Abbrechen");

			_buttonBeenden.addSelectionListener(new SelectionAdapter()
			{
				public void widgetSelected(SelectionEvent e)
				{
					aktionAbbrechen();
				}
			});

			/*
			 * TabListen
			 */
			Control[] tabListeOben = new Control[] { _textBenutzername,
					_textPasswort, _comboAdresse, _comboPort };
			compositeOben.setTabList(tabListeOben);

			Control[] tabListeUnten = new Control[] { _buttonOk,
					_buttonOfflineBetrieb, _buttonBeenden };
			compositeUnten.setTabList(tabListeUnten);

			Control[] tabListeShell = new Control[] { compositeOben, compositeUnten };
			_shell.setTabList(tabListeShell);

			if (DaVVerbindung.getInstanz().istArbeitsbereichAktiv())
			{
				String helpContextId = Activator.PLUGIN_ID + ".logindialog";
				PlatformUI.getWorkbench().getHelpSystem()
						.setHelp(_shell, helpContextId);
			}

			_shell.open();

			Display display = parent.getDisplay();

			// Dialog zentrieren
			Monitor primary = display.getPrimaryMonitor();
			Rectangle bounds = primary.getBounds();
			Rectangle rect = _shell.getBounds();
			int x = bounds.x + (bounds.width - rect.width) / 2;
			int y = bounds.y + (bounds.height - rect.height) / 2;
			_shell.setLocation(x, y);

			while (!_shell.isDisposed())
				if (!display.readAndDispatch())
					display.sleep();

			return _EingabeOk;
		}
	}

	/**
	 * Initialisierung von _comboAdresse
	 * 
	 */
	private Combo erzeugeAuswahlAdresse(Shell shell)
	{
		if (_comboAdresse == null)
		{
			_comboAdresse = new Combo(shell, SWT.NONE);

			_comboAdresse.setBounds(new Rectangle(150, 70, 151, 21));

			_debug.fine("Anz Server: " + _serverIps.size());

			if (_serverIps.size() > 0)
			{
				Iterator<String> it = _serverIps.iterator();
				while (it.hasNext())
				{
					_comboAdresse.add(it.next());
				}
				_comboAdresse.setText(_serverIps.get(0));
			}

			_comboAdresse.addListener(SWT.DefaultSelection, new Listener()
			{
				public void handleEvent(Event e)
				{
					aktionOnlineModus();
				}
			});
		}

		return _comboAdresse;
	}

	/**
	 * Initialisierung von _comboAdresse
	 * 
	 */
	private Combo erzeugeAuswahlAdresse(Composite comp)
	{
		if (_comboAdresse == null)
		{
			_comboAdresse = new Combo(comp, SWT.NONE);

			_debug.fine("Anz Server: " + _serverIps.size());

			if (_serverIps.size() > 0)
			{
				Iterator<String> it = _serverIps.iterator();
				while (it.hasNext())
				{
					_comboAdresse.add(it.next());
				}
				_comboAdresse.setText(_serverIps.get(0));
			}

			_comboAdresse.addListener(SWT.DefaultSelection, new Listener()
			{
				public void handleEvent(Event e)
				{
					aktionOnlineModus();
				}
			});
		}

		return _comboAdresse;
	}

	/**
	 * Initialisierung von _comboPort
	 * 
	 */
	private Combo erzeugeAuswahlPort(Shell shell)
	{
		if (_comboPort == null)
		{
			_comboPort = new Combo(shell, SWT.NONE);

			_comboPort.setBounds(new Rectangle(420, 70, 121, 21));

			_debug.fine("Anz Ports: " + _serverPorts.size());

			if (_serverPorts.size() > 0)
			{
				Iterator<String> it = _serverPorts.iterator();
				while (it.hasNext())
				{
					_comboPort.add(it.next());
				}
				_comboPort.setText(_serverPorts.get(0));
			}

			_comboPort.addListener(SWT.DefaultSelection, new Listener()
			{
				public void handleEvent(Event e)
				{
					aktionOnlineModus();
				}
			});
		}

		return _comboPort;
	}

	/**
	 * Initialisierung von _comboPort
	 * 
	 */
	private Combo erzeugeAuswahlPort(Composite composite)
	{
		if (_comboPort == null)
		{
			_comboPort = new Combo(composite, SWT.NONE);

			_debug.fine("Anz Ports: " + _serverPorts.size());

			if (_serverPorts.size() > 0)
			{
				Iterator<String> it = _serverPorts.iterator();
				while (it.hasNext())
				{
					_comboPort.add(it.next());
				}
				_comboPort.setText(_serverPorts.get(0));
			}

			_comboPort.addListener(SWT.DefaultSelection, new Listener()
			{
				public void handleEvent(Event e)
				{
					aktionOnlineModus();
				}
			});
		}

		return _comboPort;
	}

	/**
	 * Ldt die Prferenzen fr diesen Dialog. Diese beinhalten eine Liste der
	 * gesicherten DaV-Adressen und Ports.
	 */
	private void ladePraeferenzen()
	{
		_praeferenzenSpeicher = new PreferenceStore(_praeferenzenSpeicherDateiname);
		try
		{
			_praeferenzenSpeicher.load();
			_praeferenzenSpeicherGeladen = true;
		} catch (IOException e)
		{
			_debug.fine(getClass().getName()
					+ " <= IOException beim PreferenceStore-Laden: " + e.toString());
		}

		if (_praeferenzenSpeicherGeladen)
		{
			for (int i = 0; i < _maxLoginHistorie; i++)
			{
				String tempId = _aufbauStringServerIpId + i;
				String tempPort = _aufbauStringServerPortId + i;

				// IPs holen
				if (_praeferenzenSpeicher.contains(tempId))
				{
					_serverIps.add(_praeferenzenSpeicher.getString(tempId));
				}

				// Ports holen
				if (_praeferenzenSpeicher.contains(tempPort))
				{
					_serverPorts.add(_praeferenzenSpeicher.getString(tempPort));
				}

			}
		}
	}

	/**
	 * Fgt eine IP-Adresse hinzu.
	 * 
	 * @param ip
	 *          Hinzufgende IP-Adresse.
	 */
	private void hinzufuegenIp(String ip)
	{
		boolean enthalten = false;

		Iterator<String> it = _serverIps.iterator();
		int position = 0;
		while (it.hasNext())
		{
			if (it.next().equalsIgnoreCase(ip))
			{
				enthalten = true;
				break;
			}
			position++;
		}

		if (enthalten)
		{
			_serverIps.remove(position);
			_serverIps.add(0, ip);
		} else
		{
			_serverIps.add(0, ip);
		}
	}

	/**
	 * Fgt einen Port hinzu.
	 * 
	 * @param port
	 *          Hinzuzufgender Port.
	 */
	private void hinzufuegenPort(String port)
	{
		boolean enthalten = false;

		Iterator<String> it = _serverPorts.iterator();
		int position = 0;
		while (it.hasNext())
		{
			if (it.next().equalsIgnoreCase(port))
			{
				enthalten = true;
				break;
			}
			position++;
		}

		if (enthalten)
		{
			_serverPorts.remove(position);
			_serverPorts.add(0, port);
		} else
		{
			_serverPorts.add(0, port);
		}
	}

	/**
	 * Speichert die Prferenzen, dies beinhaltet die Liste der DaV-Adressen sowie
	 * die Liste der zugehrigen Ports.
	 */
	private void speicherePraeferenzen()
	{
		int i = 0;

		Iterator<String> it = _serverIps.iterator();
		while (it.hasNext())
		{
			String tempId = _aufbauStringServerIpId + i;
			_praeferenzenSpeicher.putValue(tempId, it.next());
			i++;
			if (i == _maxLoginHistorie)
			{
				break;
			}
		}

		int j = 0;
		Iterator<String> itPorts = _serverPorts.iterator();
		while (itPorts.hasNext())
		{
			String tempPort = _aufbauStringServerPortId + j;
			_praeferenzenSpeicher.putValue(tempPort, itPorts.next());
			j++;
			if (j == _maxLoginHistorie)
			{
				break;
			}
		}

		try
		{
			_praeferenzenSpeicher.save();
		} catch (IOException e)
		{
			e.printStackTrace();
		}
	}

}
