/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.menu.editoren;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IPersistableElement;
import org.eclipse.ui.part.EditorPart;

import de.bsvrz.buv.rw.basislib.Activator;
import de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten;

public class EditorMenuSymbolLeisterInput implements IEditorInput
{
  /**
   * Struktur der Men- bzw. Symbolleiste
   */
  private MenuSymbolLeisteDaten _msld = null;

  /**
   * Konstruktor, bernimmt eine Instanz der Klasse {@link MenuSymbolLeisteDaten} als Input fr einen {@link EditorPart}.
   * 
   * @param msld
   *          {@link MenuSymbolLeisteDaten} als Input fr einen {@link EditorPart}
   */
  public EditorMenuSymbolLeisterInput(MenuSymbolLeisteDaten msld)
  {
    this._msld = msld;
  }

  @Override
  public boolean exists()
  {
    return false;
  }

  @Override
  public ImageDescriptor getImageDescriptor()
  {
    ImageDescriptor re = null;

    if (_msld != null)
    {
      switch (_msld.getLeistenArt())
      {
        case menue:
          re = Activator.getImageDescriptor("icons/menubar/menu.gif");
          break;
        case symbol:
          re = Activator.getImageDescriptor("icons/menubar/menubar.gif");
          break;
      }
    }

    return re;
  }

  @Override
  public String getName()
  {
    String re = "Undefinierte";

    if (_msld != null)
    {
      switch (_msld.getLeistenArt())
      {
        case menue:
          re = "Menleiste";
          break;
        case symbol:
          re = "Symbolleiste";
          break;
      }
    }

    return re;
  }

  @Override
  public IPersistableElement getPersistable()
  {
    return null;
  }

  @Override
  public String getToolTipText()
  {
    String re = "Undefinierte Struktur";

    if (_msld != null)
    {
      switch (_msld.getLeistenArt())
      {
        case menue:
          re = "Struktur einer Menleiste";
          break;
        case symbol:
          re = "Struktur einer Symbolleiste";
          break;
      }
    }

    return re;
  }

  @SuppressWarnings("unchecked")
	@Override
  public Object getAdapter(Class adapter)
  {
    return null;
  }

  /**
   * Liefert die Struktur der Mens bzw. der Symbolleiste.
   * 
   * @return {@link MenuSymbolLeisteDaten}
   */
  public MenuSymbolLeisteDaten getMSLD()
  {
    return _msld;
  }

}
