/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.objektzusammenstellung;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.Map.Entry;

import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.basislib.Activator;
import de.bsvrz.buv.rw.basislib.berechtigung.DaVKommunikation;
import de.bsvrz.buv.rw.basislib.berechtigung.IOberflaechenBerechtigungZuhoererErweitert;
import de.bsvrz.buv.rw.basislib.berechtigung.ISystemObjekteZuhoerer;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenBerechtigung;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenBerechtigungEreignis;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenFunktion;
import de.bsvrz.buv.rw.basislib.berechtigung.SystemObjekteEreignis;
import de.bsvrz.buv.rw.basislib.datenidentifikation.DatenidentifikationGtm;
import de.bsvrz.buv.rw.basislib.datenidentifikation.GtmListeComposite;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.buv.rw.rw.dav.datenbasis.GtmDatenBasis;
import de.bsvrz.buv.rw.rw.dav.datenbasis.prelader.DaVPreLader;
import de.bsvrz.buv.rw.rw.dav.datenbasis.prelader.IDaVPreLader;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse liefert eine Sicht (Instanz der Klasse {@link ViewPart}), mit
 * deren Hilfe sich Objekte zusammenstellen lassen.<br>
 * <br>
 * <br>
 * <b><u>Lebenszyklusabhniges Laden und Speichern aus dem Memento der
 * View:</u></b><br>
 * War whrend des Lebenszyklus dieser View eine Verbindung zum DaV aktiv, so
 * wir das Flag <code>_davConnectionInLifetime<code> auf <code>true</code>
 * gesetzt. Dies kann in <code>createPartControl</code> sowie in dem dort
 * erzeugten {@link IDaVVerbindungZuhoerer} ausgelst werden.<br>
 * Regeln:
 * <ul>
 * <li>Wenn beim Beenden
 * <code>_davConnectionInLifetime == true<code> galt, wird der Zustand der View gespeichert.
 * </li>
 * <li>Ist beim Starten der View
 * <code>_davConnectionInLifetime == true<code>, wird der Zustand der View wiederhergestellt.
 * </li>
 * <li>Ist beim Starten der View
 * 
 * <code>_davConnectionInLifetime == false<code>, kann der alte Zustand nicht wiederhergestellt werden. Meldet jedoch der Listener <code>_zuhoererDaVVerbindung</code>
 * dass eine Verbindung zum DaV aktiv ist, wird auch
 * <code>_davConnectionInLifetime<code> auf <code>true</code> gesetzt und der
 * Zustand der View wiederhergestellt.</li>
 * </ul>
 * <br>
 * Der Vorteil ist, dass der Zustand der Objektzusammenstellung auch dann
 * gesichert wird, wenn whrend des Betriebes die Verbindung zum DaV
 * unterbrochen wird. <br>
 * <br>
 * Erfllt die Anforderungen von TBuV-98
 * 
 * @author HofmannJ
 * 
 */
public class SichtObjektzusammenstellung extends ViewPart
{
	/**
	 * Im Memento dieser View knnen maximal 9999 Attribute mit Wert abgelegt
	 * werden. Hierfr ist Eclipse 3.3 verantwortlich.<br>
	 * Das Memento selbst vertrgt 10001 Attribute, jedoch geht eins fr die
	 * Anzahl der SystemObjekte und eines fr die SecondaryID dieser View drauf. <br>
	 * Hinweismeldung erfolgt bei der bernahme aus dem
	 * Datenidentifikationsdialog, bei der Speicherung im Memento der View werden
	 * dem entsprechend nur die ersten 9999 bercksichtigt.
	 */
	private static final int _maxSizeXmlMemento = 9999;

	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	/**
	 * ID des {@link ViewPart}s.
	 */
	// public static final String ID =
	// "de.bsvrz.buv.rw.basislib.objektzusammenstellung.SichtObjektzusammenstellung";
	public static final String ID = "de.bsvrz.buv.rw.basislib.SichtObjektzusammenstellung";

	/**
	 * Secondary ID dieser View fr Multipleviews
	 */
	private String _secondaryID = null;

	private Composite _oben = null;

	/**
	 * Gruppe zur Darstellung von Objekten
	 */
	private Group _gruppeObjekte = null;

	/**
	 * Mittlerer Bereich
	 */
	private Composite _compositeMitte = null;

	/**
	 * Gruppe zur Darstellung der Beschreibung
	 */
	private Group _gruppeBeschreibung = null;

	/**
	 * Textfeld zur Darstellung der Beschreibung
	 */
	private Text _textFeldBeschreibung = null;

	/**
	 * CheckBox zum Einschrnken des Objekttyps
	 */
	private Button _checkBoxObjekttypEinschraenken = null;

	/**
	 * Linker Bereich
	 */
	private Composite _compositeObjekteLinks = null;

	/**
	 * Rechter Bereich
	 */
	private Composite _compositeObjekteRechts = null;

	/**
	 * Schaltflche Hinzufgen zum Hinzufgen von Objekten
	 */
	private Button _schaltflaecheHinzufuegen = null;

	/**
	 * Schaltflche Lschen zum Entfernen von selektierten Objekten.
	 */
	private Button _schaltflaecheLoeschen = null;

	/**
	 * Bereich unten Rechts
	 */
	private Composite _compositeObjekteRechtsUnten = null;

	/**
	 * Schaltflche Hoch, bewegt das selektierte Objekt um eine Position nach
	 * oben.
	 */
	private Button _schaltflaecheHoch = null;

	/**
	 * Schaltflche Runter, bewegt das selektierte Objekt um eine Position nach
	 * unten.
	 */
	private Button _schaltflaecheRunter = null;

	/**
	 * Bereich fr die Schaltflaechen
	 */
	private Composite _compositeKontrolle = null;

	/**
	 * Schaltflche OK
	 */
	private Button _schaltflaecheOK = null;

	/**
	 * Schaltflche Abbrechen
	 */
	private Button _schaltflaecheAbbrechen = null;

	/**
	 * Schaltflche Hilfe
	 */
	private Button _schaltflaecheHilfe = null;

	/**
	 * Liste der Oberflchen Elemente die dem Berechtigungskonzept unterliegen
	 * sollen
	 */
	private Map<String, Object> _oberflaechenElemente = new HashMap<String, Object>();

	/**
	 * Der Listener fr das Berechtigungskonzept
	 */
	private IOberflaechenBerechtigungZuhoererErweitert _berechtigungsZuhoererErweitert = null;

	/**
	 * Instanz der Oberflchen Berechtigung
	 */
	private OberflaechenBerechtigung _oberflaechenBerechtigung = OberflaechenBerechtigung
			.getInstanz();

	/**
	 * Verbindung zum DaV. Wird selbststndig besorgt.
	 */
	private ClientDavInterface _connection;

	/**
	 * ber die Check-Box "Objekttypen einschrnken" wird vorgegeben, dass nur
	 * Datenidentifikationen in die Objektzusammenstellung bernommen werden
	 * knnen, die von einem Objekttyp sind, der bereits in der
	 * Objektzusammenstellung vorkommt. Wenn z.B. nur Objekte vom Typ MQ in der
	 * Objektzusammenstellung enthalten sind und die Check-Box aktiviert ist, dann
	 * knnen auch nur weitere Objekte vom Typ MQ hinzugefht werden. Dieser
	 * Umstand muss sich direkt fr die Datenidentifikations-Eingabe auswirken.
	 * D.h. wenn in diesem Fall "Hinzufgen" bettigt wird, muss die
	 * Datenidentifikations-Eingabe bereits auf Objekte vom Typ "MQ" filtern. Drag
	 * und Drop wird nur fr Objekte der entsprechenden Objekttypen zugelassen Die
	 * Check-Box ist optional. Fr die Zusammenstellung eines Streckenprofils ist
	 * die Check-Box nicht erforderlich. In diesen Fllen bestimmt die Anwendung
	 * bereits welche Objekttypen zulssig sind. Checkbox ja/nein
	 * 
	 * Gibt an, ob nach ObjektTyp eingeschrnkt werden soll oder nicht.
	 */
	private boolean _einschraenkenObjektTyp = false;

	/**
	 * Liste der Systemobjekte.
	 */
	private List<SystemObject> _listeDaten = new LinkedList<SystemObject>();

	/**
	 * {@link TableViewer} zur Darstellung der {@link SystemObject}s.
	 */
	private static TableViewer _tabelleSystemObjekte = null;

	/**
	 * Zuhrer auf die DaVVerbindung.
	 */
	private static IDaVVerbindungZuhoerer _zuhoererDaVVerbindung;

	/**
	 * Liste der {@link IObjektzusammenstellungZuhoerer}, die beim Klicken auf OK
	 * darber informiert werden wollen, dass {@link SystemObject}s
	 * zusammengestellt wurden.
	 */
	private static List<IObjektzusammenstellungZuhoerer> _zuhoererObjektzusammenstellung = new LinkedList<IObjektzusammenstellungZuhoerer>();

	/**
	 * DropAdapter fr den TableViewer
	 */
	private ObjektzusammenstellungFallAdapter _objektzusammenstellungFallAdapter;

	/**
	 * DragListener fr den TableViewer
	 */
	private ZugZuhoererSystemObjekt _zugZuhoererSystemObjekt;

	/**
	 * Memento fr die Wiederherstellung
	 */
	private IMemento _memento = null;

	/**
	 * Speichert die IDs der Systemobjekte fr das Memento
	 */
	private LinkedList<Long> _systemObjekteMemento = new LinkedList<Long>();

	/**
	 * Prelader fr Informationen vom DaV
	 */
	private DaVPreLader _preLader = DaVPreLader.getInstanz();

	/**
	 * Listener fr den Prelader. Ist u.a. fr das Ein- und Ausblenden des
	 * Hinzufgenknopfes zustndig.
	 */
	private IDaVPreLader _preLaderListener = null;

	/**
	 * Speicherklasse fr den Objekttransfer
	 */
	ObjektSpeicher _objektSpeicher = ObjektSpeicher.getInstanz();

	@Override
	public void createPartControl(Composite parent)
	{
		_connection = DaVVerbindung.getInstanz().getDaVVerbindung();

		if (_zuhoererDaVVerbindung != null)
		{
			_debug.fine(getClass().getName()
					+ "::createPartControl \t\n=>Entferne alten Listener.\n");

			DaVVerbindung.getInstanz().entferneZuhoerer(_zuhoererDaVVerbindung);
			_zuhoererDaVVerbindung = null;
		}

		_zuhoererDaVVerbindung = new IDaVVerbindungZuhoerer()
		{

			public void verbindungAbgebaut(DaVVerbindungEreignis e)
			{
				_connection = null;
				_debug
						.info(getClass().getName()
								+ "::createPartControl \n\t=> Die Verbindung zum DaV wurde getrennt oder wurde noch nicht aufgebaut. Der Dialog zur Datenidentifikation kann NICHT werden.\n");

				if (_schaltflaecheHinzufuegen != null)
				{
					Runnable run = new Runnable()
					{
						@Override
						public void run()
						{
							if (!_schaltflaecheHinzufuegen.isDisposed())
							{
								_schaltflaecheHinzufuegen.setEnabled(false);
							}
						}
					};

					Display display = PlatformUI.getWorkbench().getDisplay();
					if (!display.isDisposed())
					{
						PlatformUI.getWorkbench().getDisplay().asyncExec(run);
					}
				}
			}

			public void verbindungAufgebaut(DaVVerbindungEreignis e)
			{
				_connection = e.getDaVVerbindung();

				/*
				 * MessageBox box = new MessageBox(new Shell(Display.getDefault()),
				 * SWT.OK | SWT.ICON_INFORMATION);
				 * box.setText("Verbindung zum DaV aufgebaut"); box .setMessage("Es
				 * wurde eine Verbindung zum DaV aufgebaut.\nDer Dialog zur
				 * Datenidentifikation kann angezeigt werden."); box.open();
				 */

				System.out
						.println(getClass().getName()
								+ "::createPartControl \n\t=> Es wurde eine Verbindung zum DaV aufgebaut. Der Dialog zur Datenidentifikation kann angezeigt werden.\n");

				if (_schaltflaecheHinzufuegen != null)
				{
					if (_preLader.istInitialesLadenAbgeschlossen())
					{
						_schaltflaecheHinzufuegen.setEnabled(true);
					}

				}
			}

		};
		DaVVerbindung.getInstanz().hinzufuegenZuhoerer(_zuhoererDaVVerbindung);

		_preLaderListener = new IDaVPreLader()
		{

			@Override
			public void systemObjekteAktualisierung(GtmDatenBasis datenBasis)
			{
				if (_schaltflaecheHinzufuegen != null)
				{
					Runnable run = new Runnable()
					{
						@Override
						public void run()
						{
							_schaltflaecheHinzufuegen.setEnabled(true);
						}
					};

					Display display = PlatformUI.getWorkbench().getDisplay();
					if (!display.isDisposed())
					{
						display.asyncExec(run);
					}

				}

			}

			@Override
			public void systemObjekteInitial(GtmDatenBasis datenBasis)
			{
				if (_schaltflaecheHinzufuegen != null)
				{
					Runnable run = new Runnable()
					{
						@Override
						public void run()
						{
							_schaltflaecheHinzufuegen.setEnabled(true);
						}
					};

					Display display = PlatformUI.getWorkbench().getDisplay();
					if (!display.isDisposed())
					{
						display.asyncExec(run);
					}

				}

			}

		};
		_preLader.hinzufuegenListenerPreLader(_preLaderListener);

		_oben = new Composite(parent, SWT.NONE);
		_oben.setLayout(new GridLayout());
		erzeugeGruppeObjekte();
		erzeugeCompositeMitte();
		erzeugeGruppeBeschreibung();
		erzeugeCompositeKontrolle();

		// KmdParameter kp = _kommandoZeilenParameter
		// .getKommandoZeilenParameter("-berechtigungsKonzept");
		// String s = kp.getWert();
		// if (s.equals("true"))
		// {
		erzeugeOberflaechenBerechtigung();
		// }

		restoreState();
	}

	private void erzeugeOberflaechenBerechtigung()
	{

		_berechtigungsZuhoererErweitert = new IOberflaechenBerechtigungZuhoererErweitert()
		{
			@Override
			public void freigabe(OberflaechenBerechtigungEreignis e)
			{
				_debug.fine("Info Freigabe erhalten: " + e.hatFreigabe());

				List<SystemObject> liste1 = e.getSystemObjekteFreigabe();
				List<SystemObject> liste2 = e.getSystemObjekteSperrung();
				for (SystemObject so : liste1)
				{
					_debug.fine("Objekte Freigabe: " + so);
				}
				for (SystemObject so : liste2)
				{
					_debug.fine("Objekte Sperrung: " + so);
				}

				List<OberflaechenFunktion> liste3 = e.getOberflaechenFunktionFreigabe();
				List<OberflaechenFunktion> liste4 = e.getOberflaechenFunktionSperrung();
				for (OberflaechenFunktion of : liste3)
				{
					_debug.fine("Funktion Freigabe: " + of.getFunktionsName());
					Object object = _oberflaechenElemente.get(of.getFunktionsName());

					if (object instanceof Button)
					{

						final Button button = (Button) object;
						// System.out.println(button);
						// if (!button.isDisposed())
						{

							Display display = Display.getDefault();

							if (!display.isDisposed())
							{
								display.asyncExec(new Runnable()
								{
									public void run()
									{
										if (!button.isDisposed())
										{
											button.setEnabled(true);
										}
									}
								});
							}
						}

					}
				}
				for (OberflaechenFunktion of : liste4)
				{
					_debug.fine("Funktion Sperrung: " + of.getFunktionsName());
				}

			}

			@Override
			public void sperrung(OberflaechenBerechtigungEreignis e)
			{
				_debug.fine("Info Sperrung erhalten: " + e.hatFreigabe());

				List<SystemObject> liste1 = e.getSystemObjekteFreigabe();
				List<SystemObject> liste2 = e.getSystemObjekteSperrung();
				for (SystemObject so : liste1)
				{
					_debug.fine("Objekte Freigabe: " + so);
				}
				for (SystemObject so : liste2)
				{
					_debug.fine("Objekte Sperrung: " + so);
				}

				List<OberflaechenFunktion> liste3 = e.getOberflaechenFunktionFreigabe();
				List<OberflaechenFunktion> liste4 = e.getOberflaechenFunktionSperrung();
				for (OberflaechenFunktion of : liste3)
				{
					_debug.fine("Funktion Freigabe: " + of.getFunktionsName());
				}
				for (OberflaechenFunktion of : liste4)
				{
					_debug.fine("Funktion Sperrung: " + of.getFunktionsName());
					Object object = _oberflaechenElemente.get(of.getFunktionsName());

					if (object instanceof Button)
					{
						final Button button = (Button) object;
						// System.out.println(button);
						// if (!button.isDisposed())
						{
							Display display = Display.getDefault();

							if (!display.isDisposed())
							{
								display.asyncExec(new Runnable()
								{
									public void run()
									{
										if (!button.isDisposed())
										{
											button.setEnabled(false);
										}
									}
								});
							}
						}

					}
				}
			}

			@Override
			public void oberflaechenBerechtigungBereit()
			{
				Display.getDefault().asyncExec(new Runnable()
				{
					public void run()
					{
						IStructuredSelection selection = (IStructuredSelection) _tabelleSystemObjekte
								.getSelection();

						Object firstElement = selection.getFirstElement();
						if (firstElement instanceof SystemObject)
						{
							List<SystemObject> liste = new LinkedList<SystemObject>();
							@SuppressWarnings("unchecked")
							Iterator it = selection.iterator();
							while (it.hasNext())
							{
								SystemObject so = (SystemObject) it.next();
								liste.add(so);
							}
							pruefeBerechtigung(liste);

						}

					}

				});
			}
		};

		de.bsvrz.buv.rw.basislib.berechtigung.DaVKommunikation.getInstanz()
				.hinzufuegenOberflaechenBenutzerZuhoerer(
						_berechtigungsZuhoererErweitert);

		OberflaechenFunktion funktion1 = new OberflaechenFunktion(
				"SichtObjektzusammenstellungOk", _berechtigungsZuhoererErweitert,
				"SichtObjektzusammenstellung", "Ok");
		OberflaechenFunktion funktion2 = new OberflaechenFunktion(
				"SichtObjektzusammenstellungAbbrechen",
				_berechtigungsZuhoererErweitert, "SichtObjektzusammenstellung",
				"Abbrechen");
		OberflaechenFunktion funktion3 = new OberflaechenFunktion(
				"SichtObjektzusammenstellungHilfe", _berechtigungsZuhoererErweitert,
				"SichtObjektzusammenstellung", "Hilfe");
		OberflaechenFunktion funktion4 = new OberflaechenFunktion(
				"SichtObjektzusammenstellungHinzufgen",
				_berechtigungsZuhoererErweitert, "SichtObjektzusammenstellung",
				"Hinzufgen");
		OberflaechenFunktion funktion5 = new OberflaechenFunktion(
				"SichtObjektzusammenstellungLschen", _berechtigungsZuhoererErweitert,
				"SichtObjektzusammenstellung", "Lschen");

		_oberflaechenElemente.put(funktion1.getFunktionsName(), _schaltflaecheOK);
		_oberflaechenElemente.put(funktion2.getFunktionsName(),
				_schaltflaecheAbbrechen);
		_oberflaechenElemente
				.put(funktion3.getFunktionsName(), _schaltflaecheHilfe);
		_oberflaechenElemente.put(funktion4.getFunktionsName(),
				_schaltflaecheHinzufuegen);
		_oberflaechenElemente.put(funktion5.getFunktionsName(),
				_schaltflaecheLoeschen);

		_oberflaechenBerechtigung.hinzufuegenOberflaechenFunktion(funktion1);
		_oberflaechenBerechtigung.hinzufuegenOberflaechenFunktion(funktion2);
		_oberflaechenBerechtigung.hinzufuegenOberflaechenFunktion(funktion3);
		// _oberflaechenBerechtigung.hinzufuegenOberflaechenFunktion(funktion4);
		// _oberflaechenBerechtigung.hinzufuegenOberflaechenFunktion(funktion5);
	}

	@Override
	public void setFocus()
	{
		this.setTitleToolTip("SecondaryID: " + getSecondaryId());
	}

	/**
	 * Setzt die angezeigte Liste der SystemObjekte
	 * 
	 */
	public void setListeDaten(List<SystemObject> systemObjekte)
	{
		this._listeDaten = systemObjekte;
		// _lvListe.setInput(_listeDaten);
		_tabelleSystemObjekte.setInput(_listeDaten);
	}

	/**
	 * Liefert die Liste der dargestellten Systemobjekte
	 * 
	 * @return
	 */
	public List<SystemObject> getListeDaten()
	{
		return _listeDaten;
	}

	/**
	 * Fgt ein Objekt zur Liste hinzu.
	 * 
	 */
	private void hinzufuegenObjekt()
	{
		if (_connection != null)
		{

			LinkedList<SystemObjectType> objektTypen = null;
			if (_einschraenkenObjektTyp)
			{
				DataModel dataModel = _connection.getDataModel();
				objektTypen = new LinkedList<SystemObjectType>();
				String[] vorhandeneObjektTypen = bestimmeVorhandeneObjektTypen();
				for (String pid : vorhandeneObjektTypen)
				{
					objektTypen.add(dataModel.getType(pid));
				}
			}

			LinkedList<SystemObject> systemObjekte = null;

			DatenidentifikationGtm dialog = new DatenidentifikationGtm(PlatformUI
					.getWorkbench().getDisplay().getActiveShell(), null);
			if (objektTypen == null)
			{
				systemObjekte = dialog
						.oeffnen(GtmListeComposite._darstlungSystemObjekteObjektTypenAttributGruppenAspekte);
			} else
			{
				System.out.println("Anzahl ObjektTypen: " + objektTypen.size());

				systemObjekte = dialog
						.oeffnen(
								objektTypen,
								GtmListeComposite._darstlungSystemObjekteObjektTypenAttributGruppenAspekte);
			}

			if (systemObjekte != null)
			{
				_listeDaten.addAll(systemObjekte);
				_tabelleSystemObjekte.setInput(_listeDaten);

				if (_listeDaten != null)
				{
					if (_listeDaten.size() > _maxSizeXmlMemento)
					{
						MessageBox mb = new MessageBox(PlatformUI.getWorkbench()
								.getDisplay().getActiveShell(), SWT.OK | SWT.ICON_WARNING);
						String msg = "";
						msg = msg
								+ "Die Anzahl der Systemobjekte in dieser Sicht ist > 10000!\n";
						msg = msg
								+ "Zur Laufzeit dieser Applikation ist dies kein Problen,\n";
						msg = msg + "smtliche Daten bleiben verfgbar!\n";
						msg = msg
								+ "Bleibt diese Sicht jedoch beim Beenden von Eclipse geffnet,\n";
						msg = msg
								+ "so werden nur die ersten 10000 SystemObjekte zum Wiederaufbau\n";
						msg = msg
								+ "dieser Sicht gespeichert. Mehr kann durch die Eclipse RCP,\n";
						msg = msg
								+ "welche die Basis dieser Anwendung ist, nicht verarbeitet\n";
						msg = msg + "werden.";
						mb.setText("Anzahl Systemobjekte > 10000");
						mb.setMessage(msg);
						mb.open();
					}
				}
			}

		

		} else
		{
			MessageBox box = new MessageBox(new Shell(Display.getDefault()), SWT.OK
					| SWT.ICON_ERROR);
			box.setText("Keine Verbindung zum DaV");
			box
					.setMessage("Es wurde keine Verbindung zum DaV bergeben. \nDer Dialog zur Datenidentifikation wird nicht angezeigt.");
			box.open();
		}

	}

	/**
	 * Diese Methode entfernt die in der Tabelle selektierten Objekte.
	 */
	private void entferneSelektierteObjekte()
	{
		// IStructuredSelection selection = (IStructuredSelection) _lvListe
		// .getSelection();
		IStructuredSelection selection = (IStructuredSelection) _tabelleSystemObjekte
				.getSelection();
		Object[] objects = selection.toArray();
		for (int i = 0; i < objects.length; i++)
		{
			if (objects[i] instanceof SystemObject)
			{
				SystemObject systemObject = (SystemObject) objects[i];
				_listeDaten.remove(systemObject);
			}
		}
		// _lvListe.setInput(_listeDaten);
		_tabelleSystemObjekte.setInput(_listeDaten);

		/**
		 * Schaltflche Lschen deaktivieren.
		 */
		_schaltflaecheLoeschen.setEnabled(false);
	}

	/**
	 * Bestimmt die in der Liste vorhandenen ObjektTypen und gibt deren Namen in
	 * einem Array von Strings zurck. Die wird bentigt wenn der der Haken fr
	 * die Einschrnkung gesetzt wird, dass nur Objekte vom gleichen Typ ber den
	 * Datenidentifiaktions- Dialog aufgenommen werden sollen.
	 */
	private String[] bestimmeVorhandeneObjektTypen()
	{
		// Rckgabe String
		String tempString[] = null;

		TreeMap<String, String> tempMap = new TreeMap<String, String>();
		Iterator<SystemObject> it = _listeDaten.iterator();
		while (it.hasNext())
		{
			SystemObject tempSO = it.next();

			String insertKey = tempSO.getType().getPid();
			String insertValue = tempSO.getType().getName();
			if (!tempMap.containsKey(insertKey))
			{
				tempMap.put(insertKey, insertValue);
			}
		}

		if (tempMap.size() > 0)
		{
			int i = 0;
			tempString = new String[tempMap.size()];
			Iterator<Entry<String, String>> itTempMap = tempMap.entrySet().iterator();
			while (itTempMap.hasNext())
			{
				tempString[i] = itTempMap.next().getKey();
				i++;
			}
		}

		return tempString;
	}

	/**
	 * Erzeugt die Gruppe gruppeObjekte
	 * 
	 */
	private void erzeugeGruppeObjekte()
	{
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		_gruppeObjekte = new Group(_oben, SWT.NONE);
		_gruppeObjekte.setLayoutData(gridData);
		erzeugeCompositeObjekteLinks();
		_gruppeObjekte.setLayout(gridLayout);
		erzeugeCompositeObjekteRechts();
		_gruppeObjekte.setText("Objekte");
	}

	/**
	 * Erzeugt den Bereich compositeMitte
	 * 
	 */
	private void erzeugeCompositeMitte()
	{
		RowLayout rowLayout = new RowLayout();
		rowLayout.fill = false;
		rowLayout.pack = true;
		rowLayout.justify = true;
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.verticalAlignment = GridData.CENTER;
		_compositeMitte = new Composite(_oben, SWT.NONE);
		_compositeMitte.setLayoutData(gridData1);
		_compositeMitte.setLayout(rowLayout);
		_checkBoxObjekttypEinschraenken = new Button(_compositeMitte, SWT.CHECK);
		_checkBoxObjekttypEinschraenken.setText("Objekttyp einschrnken");

		_checkBoxObjekttypEinschraenken.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				Object source = e.getSource();
				if (source instanceof Button)
				{
					Button b = (Button) source;

					if (b.getSelection())
					{
						_einschraenkenObjektTyp = true;
					} else
					{
						_einschraenkenObjektTyp = false;
					}
				}
			}
		});

		// Checkbox ObjekttypEinschraenken abhaken, wen eingeschrnkt werden soll.
		if (_einschraenkenObjektTyp)
		{
			_checkBoxObjekttypEinschraenken.setSelection(true);
		}
		// Sonst nicht.
		else
		{
			_checkBoxObjekttypEinschraenken.setSelection(false);
		}

	}

	/**
	 * Erzeugt die Gruppe gruppeBeschreibung
	 * 
	 */
	private void erzeugeGruppeBeschreibung()
	{
		GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.grabExcessVerticalSpace = true;
		gridData2.verticalAlignment = GridData.FILL;
		_gruppeBeschreibung = new Group(_oben, SWT.NONE);
		_gruppeBeschreibung.setLayout(new FillLayout());
		_gruppeBeschreibung.setLayoutData(gridData2);
		_gruppeBeschreibung.setText("Beschreibung");
		_textFeldBeschreibung = new Text(_gruppeBeschreibung, SWT.MULTI | SWT.WRAP
				| SWT.V_SCROLL);
		_textFeldBeschreibung.setEditable(false);
	}

	/**
	 * Erzeugt den Bereich compositeObjekteLinks
	 * 
	 */
	private void erzeugeCompositeObjekteLinks()
	{
		GridData gridData4 = new GridData();
		gridData4.horizontalAlignment = GridData.FILL;
		gridData4.grabExcessHorizontalSpace = true;
		gridData4.grabExcessVerticalSpace = true;
		gridData4.verticalAlignment = GridData.FILL;
		_compositeObjekteLinks = new Composite(_gruppeObjekte, SWT.NONE);
		_compositeObjekteLinks.setLayout(new FillLayout());
		_compositeObjekteLinks.setLayoutData(gridData4);

		erzeugeTabelle();
	}

	/**
	 * Erzeugt die Tabelle zur Darstellung der Systemobjekte
	 */
	private void erzeugeTabelle()
	{
		_tabelleSystemObjekte = new TableViewer(_compositeObjekteLinks,
				SWT.FULL_SELECTION | SWT.MULTI | SWT.BORDER);

		TableColumn tc1 = new TableColumn(_tabelleSystemObjekte.getTable(),
				SWT.CENTER);
		tc1.setText("PID:");
		tc1.setWidth(250);

		TableColumn tc2 = new TableColumn(_tabelleSystemObjekte.getTable(),
				SWT.LEFT);
		tc2.setText("SystemObjekt:");
		tc2.setWidth(350);

		_tabelleSystemObjekte.getTable().setHeaderVisible(true);
		_tabelleSystemObjekte.getTable().setLinesVisible(true);

		_tabelleSystemObjekte.setContentProvider(new TabelleInhaltsVersorger());
		_tabelleSystemObjekte.setLabelProvider(new TabelleDarstellungsVersorger());

		int ops = DND.DROP_COPY | DND.DROP_MOVE;
		_objektzusammenstellungFallAdapter = new ObjektzusammenstellungFallAdapter(
				_tabelleSystemObjekte, _listeDaten);
		Transfer[] dropTransfers = new Transfer[] { TextTransfer.getInstance() };
		_tabelleSystemObjekte.addDropSupport(ops, dropTransfers,
				_objektzusammenstellungFallAdapter);

		_zugZuhoererSystemObjekt = new ZugZuhoererSystemObjekt(
				_tabelleSystemObjekte, _listeDaten);
		Transfer[] dragTransfers = new Transfer[] { TextTransfer.getInstance() };
		_tabelleSystemObjekte.addDragSupport(ops, dragTransfers,
				_zugZuhoererSystemObjekt);

		_tabelleSystemObjekte
				.addSelectionChangedListener(new ISelectionChangedListener()
				{
					public void selectionChanged(SelectionChangedEvent event)
					{
						IStructuredSelection selection = (IStructuredSelection) event
								.getSelection();

						/*
						 * Wenn mehr als 0 Elemente selektiert sind, wird der Lschen-Button
						 * aktiviert
						 */
						if (selection.size() > 0)
						{
							_schaltflaecheLoeschen.setEnabled(true);
						}

						Object firstElement = selection.getFirstElement();
						if (firstElement instanceof SystemObject)
						{
							SystemObject systemObject = (SystemObject) firstElement;

							// KmdParameter kp = _kommandoZeilenParameter
							// .getKommandoZeilenParameter("-berechtigungsKonzept");
							// String s = kp.getWert();
							// if (s.equals("true"))
							// {

							List<SystemObject> liste = new LinkedList<SystemObject>();
							@SuppressWarnings("unchecked")
							Iterator it = selection.iterator();
							while (it.hasNext())
							{
								SystemObject so = (SystemObject) it.next();
								liste.add(so);
								// SpeicherTransfer.getInstanz().setSystemObjekt(so);
								// if
								// (!ObjektSpeicher.getInstanz().getSystemObjekte().contains(so)){
								// ObjektSpeicher.getInstanz().hinzufuegenSystemObjekt(so);
								// // System.out.println("nicht vorhanden: " + so.getName());
								// }
								// // else
								// // System.out.println("schon vorhanden: " + so.getName());
							}

							// System.out.println("sel: " + selection.size());
							// System.out.println("list: " +
							// ObjektSpeicher.getInstanz().getSystemObjekte().size());

							ObjektSpeicher.getInstanz().setSystemObjekte(liste);
							pruefeBerechtigung(liste);

							// }

							_textFeldBeschreibung.setText(systemObject.getInfo()
									.getShortInfo());
						} else
						{
							_textFeldBeschreibung.setText("");
						}
					}

				});

		// getSite().setSelectionProvider(_tabelleSystemObjekte);

	}

	/**
	 * Erzeugt den Bereich compositeObjekteRechts
	 * 
	 */
	private void erzeugeCompositeObjekteRechts()
	{
		RowLayout rowLayout1 = new RowLayout();
		rowLayout1.type = org.eclipse.swt.SWT.VERTICAL;
		rowLayout1.fill = true;
		rowLayout1.justify = false;
		GridData gridData3 = new GridData();
		gridData3.horizontalAlignment = GridData.BEGINNING;
		gridData3.grabExcessVerticalSpace = true;
		gridData3.verticalAlignment = GridData.FILL;
		_compositeObjekteRechts = new Composite(_gruppeObjekte, SWT.NONE);
		_compositeObjekteRechts.setLayoutData(gridData3);
		_compositeObjekteRechts.setLayout(rowLayout1);

		/*
		 * Hinzufgen
		 */
		_schaltflaecheHinzufuegen = new Button(_compositeObjekteRechts, SWT.NONE);
		_schaltflaecheHinzufuegen.setText("Hinzufgen");
		_schaltflaecheHinzufuegen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				hinzufuegenObjekt();
			}
		});

		if (!_preLader.istInitialesLadenAbgeschlossen())
		{
			_schaltflaecheHinzufuegen.setEnabled(false);
		}

		/*
		 * Ist keine DaV-Verbindung verfgbar, so wird der Knopf zum hinzufgen von
		 * SystemObjekten deaktiviert.
		 */
		DaVVerbindung daVVerbindung = DaVVerbindung.getInstanz();
		if (!daVVerbindung.istDaVVerbindungAktiv())
		{
			_schaltflaecheHinzufuegen.setEnabled(false);
		}

		/*
		 * Lschen
		 */
		_schaltflaecheLoeschen = new Button(_compositeObjekteRechts, SWT.NONE);
		_schaltflaecheLoeschen.setText("Lschen");
		_schaltflaecheLoeschen.setEnabled(false);
		_schaltflaecheLoeschen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				entferneSelektierteObjekte();
			}
		});

		erzeugeCompositeObjekteRechtsUnten();
	}

	/**
	 * Erzeugt den Bereich compositeObjekteRechtsUnten
	 * 
	 */
	private void erzeugeCompositeObjekteRechtsUnten()
	{
		RowLayout rowLayout2 = new RowLayout();
		rowLayout2.fill = false;
		rowLayout2.justify = true;
		_compositeObjekteRechtsUnten = new Composite(_compositeObjekteRechts,
				SWT.NONE);
		_compositeObjekteRechtsUnten.setLayout(rowLayout2);

		/*
		 * Hoch
		 */
		_schaltflaecheHoch = new Button(_compositeObjekteRechtsUnten, SWT.ARROW
				| SWT.UP);
		_schaltflaecheHoch.addSelectionListener(new SelectionAdapter()
		{

			@Override
			public void widgetSelected(SelectionEvent e)
			{

				IStructuredSelection selection = (IStructuredSelection) _tabelleSystemObjekte
						.getSelection();
				Object firstElement = selection.getFirstElement();

				if (firstElement instanceof SystemObject)
				{
					SystemObject systemObject = (SystemObject) firstElement;
					int altePosition = bestimmePositionInListe(_listeDaten, systemObject);
					if (altePosition > 0)
					{
						int neuePosition = altePosition - 1;
						_listeDaten.remove(systemObject);
						_listeDaten.add(neuePosition, systemObject);
						// _lvListe.setInput(_listeDaten);
						_tabelleSystemObjekte.setInput(_listeDaten);
						// _lvListe.getList().setSelection(neuePosition);
					}
				}
			}

		});

		/*
		 * Runter
		 */
		_schaltflaecheRunter = new Button(_compositeObjekteRechtsUnten, SWT.ARROW
				| SWT.DOWN);
		_schaltflaecheRunter.addSelectionListener(new SelectionAdapter()
		{

			@Override
			public void widgetSelected(SelectionEvent e)
			{

				IStructuredSelection selection = (IStructuredSelection) _tabelleSystemObjekte
						.getSelection();
				Object firstElement = selection.getFirstElement();

				if (firstElement instanceof SystemObject)
				{
					SystemObject systemObject = (SystemObject) firstElement;
					int altePosition = bestimmePositionInListe(_listeDaten, systemObject);
					if (altePosition < _listeDaten.size() - 1)
					{
						int neuePosition = altePosition + 1;
						_listeDaten.remove(systemObject);
						_listeDaten.add(neuePosition, systemObject);
						// _lvListe.setInput(_listeDaten);
						_tabelleSystemObjekte.setInput(_listeDaten);
						// _lvListe.getList().setSelection(neuePosition);
					}
				}
			}

		});

	}

	/**
	 * Erzeugt den Bereich compositeKontrolle
	 * 
	 */
	private void erzeugeCompositeKontrolle()
	{
		RowLayout rowLayout3 = new RowLayout();
		rowLayout3.fill = true;
		rowLayout3.type = org.eclipse.swt.SWT.HORIZONTAL;
		rowLayout3.wrap = true;
		rowLayout3.pack = false;
		rowLayout3.justify = true;
		GridData gridData5 = new GridData();
		gridData5.horizontalAlignment = GridData.FILL;
		gridData5.grabExcessHorizontalSpace = true;
		gridData5.verticalAlignment = GridData.CENTER;
		_compositeKontrolle = new Composite(_oben, SWT.NONE);
		_compositeKontrolle.setLayoutData(gridData5);
		_compositeKontrolle.setLayout(rowLayout3);

		/*
		 * Button OK
		 */
		_schaltflaecheOK = new Button(_compositeKontrolle, SWT.NONE);
		_schaltflaecheOK.setText("OK");

		_schaltflaecheOK.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				benachrichtigeObjektzusammenstellungZuhoerer();
				versteckeSicht();
			}
		});

		/*
		 * Button Abbrechen
		 */
		_schaltflaecheAbbrechen = new Button(_compositeKontrolle, SWT.NONE);
		_schaltflaecheAbbrechen.setText("Abbrechen");
		_schaltflaecheAbbrechen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				versteckeSicht();
			}
		});

		/*
		 * Button Hilfe
		 */
		_schaltflaecheHilfe = new Button(_compositeKontrolle, SWT.NONE);
		_schaltflaecheHilfe.setText("Hilfe");
		_schaltflaecheHilfe.addMouseListener(new MouseAdapter()
		{
			public void mouseDown(MouseEvent e)
			{
				/*
				 * Nur bei mittlerer oder rechter Maustaste werden die Objekte erzeugt.
				 */
				if (e.button > 1)
				{

					ArrayList<SystemObject> list = new ArrayList<SystemObject>();

					SystemObject o = _connection.getDataModel().getObject("mq.a10.0000");
					list.add(o);
					o = _connection.getDataModel().getObject("mq.a10.0001");
					list.add(o);
					o = _connection.getDataModel().getObject("mq.a10.0002");
					list.add(o);
					o = _connection.getDataModel().getObject("mq.a10.0003");
					list.add(o);

					_tabelleSystemObjekte.setInput(list);

					new ISystemObjekteZuhoerer()
					{

						@Override
						public void aenderung(SystemObjekteEreignis e)
						{
							if (e.getAenderung())
							{
								_debug.fine("SystemObjektListener hinzu: "
										+ e.getSystemObjektHinzu().getPid());
							} else
							{
								_debug.fine("SystemObjektListener weg: "
										+ e.getSystemObjektWeg().getPid());
							}
						}
					};

				}
			}
		});

		_schaltflaecheHilfe.addSelectionListener(new SelectionAdapter()
		{

			@Override
			public void widgetSelected(SelectionEvent e)
			{
				/*
				 * Hilfeaufruf - nicht entfernene!
				 */
				String hilfeKontextId = Activator.PLUGIN_ID + ".objektzusammenstellung";
				PlatformUI.getWorkbench().getHelpSystem().displayHelp(hilfeKontextId);
			}

		});

	}

	/**
	 * Bestimmt die Position eines Elementes in einer Liste.
	 * 
	 * @param liste
	 *          Liste.
	 * @param object
	 *          Element.
	 * @return Position des Elementes in der Liste (von 0 an!).
	 */
	@SuppressWarnings("unchecked")
	private int bestimmePositionInListe(List liste, Object object)
	{
		int position = -1;

		Iterator it = liste.iterator();
		while (it.hasNext())
		{
			position++;
			if (object.equals(it.next()))
			{
				break;
			}
		}

		return position;
	}

	/**
	 * Fgt einen {@link IObjektzusammenstellungZuhoerer} zur Liste der Zuhrer
	 * hinzu, wenn dieser noch nicht i.d. Liste vorhanden ist.
	 * 
	 * @param zuhoerer
	 *          Zuhrer, der zur Liste der Zuhrer hinzugefgt werden soll.
	 */
	public void hinzufuegenObjektzusammenstellungZuhoerer(
			IObjektzusammenstellungZuhoerer zuhoerer)
	{
		if (!_zuhoererObjektzusammenstellung.contains(zuhoerer))
		{
			_zuhoererObjektzusammenstellung.add(zuhoerer);
		} else
		{
			_debug
					.error(getClass().getName()
							+ "::hinzufuegenObjektzusammenstellungZuhoerer \n\t=> Der bergebene Zuhrer ist bereits in der Liste der Zuhrer vorhanden. Er wird nicht erneut hinzugefgt!");

		}
	}

	/**
	 * Entfernt einen {@link IObjektzusammenstellungZuhoerer} von der Liste der
	 * Zuhrer, der nicht mehr ber die aktuelle Objektzusammenstellung informiert
	 * werden mcht.
	 * 
	 * @param zuhoerer
	 *          Zu entfernender Zuhrer.
	 */
	public void entferneObjektzusammenstellungZuhoerer(
			IObjektzusammenstellungZuhoerer zuhoerer)
	{
		if (_zuhoererObjektzusammenstellung.contains(zuhoerer))
		{
			_zuhoererObjektzusammenstellung.remove(zuhoerer);
		} else
		{
			_debug
					.error(getClass().getName()
							+ "::hinzufuegenObjektzusammenstellungZuhoerer \n\t=>Der angegebenen Zuhrer existiert nicht in der Liste der Zuhrer und kann somit auch nicht entfernt werden. Abbruch.");

		}
	}

	/**
	 * Benachrichtigt alle Zuhrer ber die Fertigstellung einer neuen
	 * Objektzusammenstellung.
	 * 
	 */
	private void benachrichtigeObjektzusammenstellungZuhoerer()
	{
		if (_zuhoererObjektzusammenstellung.size() > 0)
		{
			// SystemObjektliste kopieren
			List<SystemObject> kopie = new LinkedList<SystemObject>();
			Iterator<SystemObject> itKopieren = _listeDaten.iterator();
			while (itKopieren.hasNext())
			{
				kopie.add(itKopieren.next());
			}

			ObjektzusammenstellungEreignis ereignis = new ObjektzusammenstellungEreignis(
					kopie);

			LinkedList<IObjektzusammenstellungZuhoerer> kopieZuhoererObjektZusammenstellung = new LinkedList<IObjektzusammenstellungZuhoerer>();
			for (IObjektzusammenstellungZuhoerer zuhoerer : _zuhoererObjektzusammenstellung)
			{
				kopieZuhoererObjektZusammenstellung.add(zuhoerer);
			}

			Iterator<IObjektzusammenstellungZuhoerer> itZu = kopieZuhoererObjektZusammenstellung
					.iterator();
			while (itZu.hasNext())
			{
				IObjektzusammenstellungZuhoerer zuhoerer = itZu.next();
				zuhoerer.systemObjekteAusgewaehlt(ereignis);
			}

		}
	}

	private void versteckeSicht()
	{
		getViewSite().getWorkbenchWindow().getActivePage().hideView(this);
	}

	/**
	 * {@link IStructuredContentProvider} fr die Tabelle zur Darstellung der
	 * zusammengestellten Objekte.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class TabelleInhaltsVersorger implements IStructuredContentProvider
	{
		@SuppressWarnings("unchecked")
		public Object[] getElements(Object inputElement)
		{
			Object[] systemObjekte = null;

			if (inputElement instanceof List)
			{
				List tempList = (List) inputElement;

				List<SystemObject> tempListeSystemObjekte = new LinkedList<SystemObject>();

				Iterator it = tempList.iterator();
				while (it.hasNext())
				{
					Object tempObject = it.next();
					if (tempObject instanceof SystemObject)
					{
						SystemObject tempSystemObject = (SystemObject) tempObject;
						tempListeSystemObjekte.add(tempSystemObject);
					}
				}

				systemObjekte = new Object[tempListeSystemObjekte.size()];

				int i = -1;
				Iterator<SystemObject> itTempSo = tempListeSystemObjekte.iterator();
				while (itTempSo.hasNext())
				{
					i++;
					systemObjekte[i] = itTempSo.next();
				}

			} else
			{
				_debug.fine(getClass().getName() + "::getElements => KEINE Eingabe!");
				systemObjekte = new Object[0];
			}

			return systemObjekte;
		}

		public void dispose()
		{
		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}

	}

	/**
	 * {@link ITableLabelProvider} fr die Elemente der Tabelle zur
	 * Objektzusammenstellung.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class TabelleDarstellungsVersorger implements ITableLabelProvider
	{

		public Image getColumnImage(Object element, int columnIndex)
		{
			return null;
		}

		public String getColumnText(Object object, int columnIndex)
		{
			String returnString;
			if (object instanceof SystemObject && object != null)
			{
				SystemObject systemObject = (SystemObject) object;

				// ID
				if (columnIndex == 0)
				{
					String tempName = "";
					if (systemObject.getName() != null && systemObject.getName() != "")
						tempName = tempName + systemObject.getName() + " ";
					if (systemObject.getPid() != null && systemObject.getPid() != "")
						tempName = tempName + "(" + systemObject.getPid() + ") ";
					if (systemObject.getId() > 0)
						tempName = tempName + "[" + String.valueOf(systemObject.getId())
								+ "]";

					returnString = tempName;
					// returnString = systemObject.getPid();

				}
				// Bezeichnung
				else if (columnIndex == 1)
				{
					returnString = systemObject.getPidOrNameOrId();
				}
				// Sonst
				else
				{
					returnString = "Fr diese Spalte findet keine Versorgung statt.";
				}

			} else
			{
				returnString = "Nicht untersttztes Element vom Typ "
						+ object.getClass().getName() + ".";
			}

			return returnString;
		}

		public void addListener(ILabelProviderListener listener)
		{
		}

		public void dispose()
		{
		}

		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		public void removeListener(ILabelProviderListener listener)
		{
		}

	}

	@Override
	public void dispose()
	{

		/*
		 * Zuhrer auf die DaV-Verbindung entfernen
		 */
		if (_zuhoererDaVVerbindung != null)
		{
			DaVVerbindung.getInstanz().entferneZuhoerer(_zuhoererDaVVerbindung);
		}

		/*
		 * Zuhrer auf den Pre-Lader entfernen
		 */
		if (_preLaderListener != null)
		{
			_preLader.entfernenListenerPreLader(_preLaderListener);
		}

		/*
		 * Zuhrer auf den Berechtigungs-Zuhoerer entfernen
		 */
		if (_berechtigungsZuhoererErweitert != null)
		{
			DaVKommunikation.getInstanz().entferneOberflaechenBenutzerZuhoerer(
					_berechtigungsZuhoererErweitert);
		}

		ObjektSpeicher.getInstanz().getSystemObjekte().clear();

		super.dispose();
	}

	/**
	 * Prft alle SystemObjekte der Liste ob eine Freigabe oder Sperrung vorliegt
	 * 
	 * @param liste
	 */
	private void pruefeBerechtigung(List<SystemObject> liste)
	{

		for (Map.Entry<String, Object> me : _oberflaechenElemente.entrySet())
		{
			OberflaechenBerechtigung.getInstanz().pruefeBerechtigungElement(liste,
					me.getKey());

		}
	}

	public void init(IViewSite site, IMemento memento) throws PartInitException
	{
		this._memento = memento;
		init(site);
	}

	public void saveState(IMemento memento)
	{
		if (_connection != null)
		{
			TableItem[] items = _tabelleSystemObjekte.getTable().getItems();
			int anzahlSystemObjekte = items.length;

			/*
			 * Sollen mehr SystemObjekte als durch XmlMemento mglich, gespeichert
			 * werden? Nein, wir beschrnken uns dann auf die ersten 10000.
			 */
			if (anzahlSystemObjekte > _maxSizeXmlMemento)
			{
				anzahlSystemObjekte = _maxSizeXmlMemento;
			}
			memento.putInteger("anzahlSystemObjekte", anzahlSystemObjekte);

			for (int i = 0; i < anzahlSystemObjekte; i++)
			{
				memento.putString("id" + i, ""
						+ ((SystemObject) _tabelleSystemObjekte.getElementAt(i)).getId());
			}
		} else
		{
			memento.putInteger("anzahlSystemObjekte", _systemObjekteMemento.size());
			int i = 0;
			Iterator<Long> it = _systemObjekteMemento.iterator();
			while (it.hasNext())
			{
				Long id = it.next();
				memento.putString("id" + i, "" + id);
				i++;
				if (i > _maxSizeXmlMemento)
				{
					break;
				}
			}
		}

		if (_secondaryID != null)
		{
			memento.putString("secondaryID", _secondaryID);
		}

	}

	/**
	 * Stellt aus einem Memento die View mit den aktuellen Daten wieder her. Ist
	 * keine Verbindung zum DaV vorhanden, so ist dies nicht mglich.
	 */
	private void restoreState()
	{
		if (_memento != null)
		{
			LinkedList<SystemObject> _objs = new LinkedList<SystemObject>();

			DataModel dataModel = null;
			if (_connection != null)
			{
				dataModel = _connection.getDataModel();
			}

			Integer anzahl = _memento.getInteger("anzahlSystemObjekte");
			if ((anzahl != null) && (anzahl > 0))
			{

				for (int i = 0; i < anzahl; i++)
				{
					String string = _memento.getString("id" + i);
					if (string != null)
					{
						try
						{
							long id = Long.valueOf(string);

							if (dataModel != null)
							{
								SystemObject object = dataModel.getObject(id);
								_objs.add(object);
							} else
							{
								_systemObjekteMemento.add(new Long(id));
							}
						} catch (NumberFormatException nfe)
						{

						}
					}
				}
			}

			if (_objs.size() > 0)
			{
				setListeDaten(_objs);
			}

			/*
			 * SecondaryID
			 */
			_secondaryID = _memento.getString("secondaryID");
		}
	}

	/**
	 * Liefert die secondary ID dieser View.
	 * 
	 * @return Secondary ID dieser View
	 */
	public String getSecondaryId()
	{
		return _secondaryID;
	}

	/**
	 * Setzt die secondary ID dieser View.
	 * 
	 * @param secId
	 *          Secondary ID dieser View
	 */
	protected void setSecondaryId(String secId)
	{
		this._secondaryID = secId;
	}

	public static TableViewer getTabelleSystemObjekte()
	{
		return _tabelleSystemObjekte;
	}

} // @jve:decl-index=0:visual-constraint="10,10,610,284"
