/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.basislib.ort.hilfsklassen;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import java.util.Map.Entry;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse die Methoden zur Verfgung stellt, die die Objekte ermittelt die fr
 * die Ermittlung der Stauprognose bentigt werden.
 * 
 * @author Dambach Werke GmbH
 */
public class Konfiguration
{

	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	/**
	 * Datenmodell
	 */
	private DataModel m_dataModel;

	/**
	 * * Verbindung zum DaV
	 */
	private ClientDavConnection m_connection;

	/**
	 * Hashmap mit Instanzen der einzelnen Staenknoten (Key = Pid, Value =
	 * Instanz)
	 */
	private HashMap<String, StrassenKnoten> m_strassenKnoten = new HashMap<String, StrassenKnoten>();

	/**
	 * Hashmap mit Instanzen der einzelnen ueren Straensegmente (Key = Pid,
	 * Value = Instanz)
	 */
	private HashMap<String, AeusseresStrassenSegment> m_aeusseresStrassenSegment = new HashMap<String, AeusseresStrassenSegment>();

	/**
	 * Hashmap mit Instanzen der einzelnen inneren Staensegmente (Key = Pid,
	 * Value = Instanz)
	 */
	private HashMap<String, InneresStrassenSegment> m_inneresStrassenSegment = new HashMap<String, InneresStrassenSegment>();

	/**
	 * Hashmap mit Instanzen der einzelnen Messquerschnitte (Key = Pid, Value =
	 * Instanz)
	 */
	private HashMap<String, MessQuerschnitt> m_messQuerschnitt = new HashMap<String, MessQuerschnitt>();

	/**
	 * Zuordnungstabelle MessQuerschnitt zu NBA MessQuerschnitt. (Key = PID des
	 * MessQuerschnitt, Value = die Pid des NbaMessQuerschnitts)
	 */
	private static HashMap<String, String> m_zuordnungMqZuNbaMq = new HashMap<String, String>();

	/**
	 * Methode zum Lesen der einzigen Instanz der Klasse
	 * 
	 * @return einzige Instanz der Klasse
	 */
	public static Konfiguration getInstanz()
	{
		return Inner.INSTANCE;
	}

	/**
	 * Innere Klasse zum Sicherstellen, dass wirklich nur eine Instanz der Klasse
	 * gebildet wird
	 * 
	 * @author Dambach Werke GmbH
	 */
	public static class Inner
	{
		private static Konfiguration INSTANCE = new Konfiguration();
	}

	/**
	 * Konstruktor der Klasse
	 */
	private Konfiguration()
	{
	}

	/**
	 * Methode die das eigentliche Bestimmen der Objekte auslst. Methode bentigt
	 * die Verbindung zum DaV
	 * 
	 * @param connection
	 *          Verbindung zum DaV
	 * @param netz
	 *          zu betrachtendes Netz
	 * @param kbMessQuerschnitte
	 *          Konfigurationsbereich(e) der zu betrachtenden Messquerschnitte
	 */
	public void bestimmeObjekte(ClientDavConnection connection, String netz,
			String kbMessQuerschnitte)
	{

		_debug.fine("Objekte bestimmen fr Netz: " + netz);

		m_connection = connection;

		m_dataModel = m_connection.getDataModel();

		_debug.fine("Bestimme uere Straensegmente");

		bestimmeAeussereStrassenSegmente(netz);

		_debug.fine("Bestimme Straenknoten");

		bestimmeStrassenKnoten();

		_debug.fine("Bestimme innere Straensegmente");

		bestimmeInnereStrassenSegmente();

		_debug.fine("Bestimme Messquerschnitte");

		bestimmeMessQuerschnitte(kbMessQuerschnitte);

		_debug.fine("Bestimme NBA Messquerschnitte");

		bestimmeNbaMessQuerschnitte();

		ausgabeStrassenKnoten();

		ausgabeAeussereStrassenSegmente();

		ausgabeInnereStrassenSegmente();

		_debug.fine("Anzahl uerer Strassensegmente: "
				+ m_aeusseresStrassenSegment.size());
		_debug.fine("Anzahl innerer Strassensegmente: "
				+ m_inneresStrassenSegment.size());
		_debug.fine("Anzahl Staenknoten            : " + m_strassenKnoten.size());
		_debug.fine("Anzahl Messquerschnitt         : " + m_messQuerschnitt.size());
	}

	/**
	 * Methode zum Bestimmen aller Objekte vom Typ "typ.straenKnoten"
	 */
	private void bestimmeStrassenKnoten()
	{
		List<SystemObject> strassenKnoten = bestimmeObjekte("typ.straenKnoten");

		Iterator<SystemObject> it = strassenKnoten.iterator();
		while (it.hasNext())
		{
			SystemObject objekt = it.next();

			m_strassenKnoten.put(objekt.getPid(), new StrassenKnoten(m_connection,
					objekt));
		}
	}

	/**
	 * Methode zum Bestimmen aller Objekte com Typ "typ.ueresStraenSegment" im
	 * bergebenen Netz. Zustzlich werden die dazugehrenden Strassenknoten
	 * angelegt und eine Zuordnung der Strassensegmente zu den Strassenknoten
	 * vorgenommen
	 */
	private void bestimmeAeussereStrassenSegmente(String netz)
	{
		List<SystemObject> aeussereStrassenSegmente = new ArrayList<SystemObject>();

		ConfigurationObject co = (ConfigurationObject) m_dataModel.getObject(netz);

		if (co == null)
			return;

		List<SystemObject> liste = co.getNonMutableSet("NetzBestandTeile")
				.getElements();

		for (int i = 0; i < liste.size(); i++)
		{
			ConfigurationObject coNetz = (ConfigurationObject) liste.get(i);

			if (!coNetz.getType().isOfType("typ.ueresStraenSegment"))
			{
				List<SystemObject> liste2 = coNetz.getNonMutableSet("NetzBestandTeile")
						.getElements();

				for (int j = 0; j < liste2.size(); j++)
				{
					ConfigurationObject coNetz2 = (ConfigurationObject) liste2.get(j);

					if (!coNetz2.getType().getPid().equals("typ.ueresStraenSegment"))
					{
						_debug.fine("Kein ueresStrassenSegment gefunden!");
					} else
					{
						if (!aeussereStrassenSegmente.contains(coNetz2))
							aeussereStrassenSegmente.add(coNetz2);
					}
				}
			}
		}

		// uereStraenSegmente = bestimmeObjekte( "typ.ueresStraenSegment"
		// ); // Test mit allen eren
		// Straensegmenten

		Iterator<SystemObject> it = aeussereStrassenSegmente.iterator();
		while (it.hasNext())
		{
			SystemObject objekt = (SystemObject) it.next();

			AeusseresStrassenSegment strassenSegment = new AeusseresStrassenSegment(
					m_dataModel, objekt);
			strassenSegment.initialisiere();

			m_aeusseresStrassenSegment.put(objekt.getPid(), strassenSegment);

			StrassenKnoten vonKnoten = strassenSegment.getVonKnoten();
			StrassenKnoten nachKnoten = strassenSegment.getNachKnoten();

			if (vonKnoten != null)
				vonKnoten.addAbgehendesAeusseresStrassenSegment(strassenSegment);

			if (nachKnoten != null)
				nachKnoten.addHinfuehrendesAeusseresStrassenSegment(strassenSegment);
		}
	}

	/**
	 * Methode zum Bestimmen aller Objekte com Typ "typ.inneresStraenSegment".
	 * Zustzlich wird eine Zuordnung der Strassensegmente zu den Strassenknoten
	 * vorgenommen
	 */
	private void bestimmeInnereStrassenSegmente()
	{
		List<SystemObject> innereStrassenSegmente = bestimmeObjekte("typ.inneresStraenSegment");

		Iterator<SystemObject> it = innereStrassenSegmente.iterator();
		while (it.hasNext())
		{
			SystemObject objekt = (SystemObject) it.next();

			InneresStrassenSegment strassenSegment = new InneresStrassenSegment(
					m_dataModel, objekt);

			m_inneresStrassenSegment.put(objekt.getPid(), strassenSegment);

			AeusseresStrassenSegment vonStrassenSegment = strassenSegment
					.getVonStrassenSegmemt();
			AeusseresStrassenSegment nachStrassenSegment = strassenSegment
					.getNachStrassenSegment();

			if ((vonStrassenSegment) == null && (nachStrassenSegment == null))
			{
				m_inneresStrassenSegment.remove(objekt.getPid());
			} else
			{
				strassenSegment.initialisiere();

				if (vonStrassenSegment != null)
				{
					strassenSegment.setVorgaengerAufStrasse(vonStrassenSegment);

					StrassenKnoten sk = m_strassenKnoten.get(vonStrassenSegment
							.getNachKnoten().getPid());
					if (sk != null)
						sk.addInneresStrassenSegment(strassenSegment);

					if (vonStrassenSegment.liegtAufSelberStrasse(nachStrassenSegment))
						vonStrassenSegment.addNachfolgerAufStrasse(strassenSegment);
					else
						vonStrassenSegment.addNachfolger(strassenSegment);
				}

				if (nachStrassenSegment != null)
				{
					strassenSegment.setNachfolgerAufStrae(nachStrassenSegment);

					StrassenKnoten sk = m_strassenKnoten.get(nachStrassenSegment
							.getVonKnoten().getPid());
					if (sk != null)
						sk.addInneresStrassenSegment(strassenSegment);

					if (nachStrassenSegment.liegtAufSelberStrasse(vonStrassenSegment))
						nachStrassenSegment.addVorgaengerAufStrasse(strassenSegment);
					else
						nachStrassenSegment.addVorgaenger(strassenSegment);
				}
			}
		}
	}

	/**
	 * Methode liefert einen Straenknoten zu einer Pid, existiert der
	 * Stasenknoten noch nicht und handlet es sich bei dem Objekt das zu der Pid
	 * gehrt um ein Objekt vom Typ 'typ.straenKnoten', so wird ein neuer
	 * StraenKnoten angelegt.
	 * 
	 * @param pid
	 *          Pid des Straenknotens
	 * @return Instanz des Straenknotens, im Fehlerfall null
	 */
	public StrassenKnoten getStrassenKnoten(String pid)
	{
		if (m_strassenKnoten.containsKey(pid))
			return m_strassenKnoten.get(pid);
		else
		{
			ConfigurationObject objekt = (ConfigurationObject) m_dataModel
					.getObject(pid);
			if (objekt != null)
			{
				if (objekt.isOfType("typ.straenKnoten"))
				{
					StrassenKnoten sk = new StrassenKnoten(m_connection, objekt);
					m_strassenKnoten.put(sk.getPid(), sk);
					return sk;
				}
			}
		}

		return null;
	}

	/**
	 * Methode liefert ein ueres Straensegment
	 * 
	 * @param pid
	 *          Pid des Straensegments
	 * @return Instanz des Straensegments, im Fehlerfall null
	 */
	public AeusseresStrassenSegment getAeusseresStrassenSegment(String pid)
	{
		if (m_aeusseresStrassenSegment.containsKey(pid))
			return m_aeusseresStrassenSegment.get(pid);

		return null;
	}

	/**
	 * Methode liefert ein inneres Straensegment
	 * 
	 * @param pid
	 *          Pid des Straensegments
	 * @return Instanz des Straensegments, im Fehlerfall null
	 */
	public InneresStrassenSegment getInneresStrassenSegment(String pid)
	{
		if (m_inneresStrassenSegment.containsKey(pid))
			return m_inneresStrassenSegment.get(pid);
		return null;
	}

	/**
	 * Methode bestimmt die vorhandenen Messquerschnitte und weist den einzelnen
	 * Straensegmente (innere und uere) die Messquerschnitte zu, die auf den
	 * jeweiligen Strassensegmenten liegen (geordnet nach Offset).
	 * 
	 * @param kbMessQuerschnitte
	 *          Konfigurationsbereiche der zu verwendenden Messquerschnitte mit
	 *          ":" getrennt. Leerer String: alle Konfigurationsbereiche
	 */
	private void bestimmeMessQuerschnitte(String kbMessQuerschnitte)
	{
		// List<SystemObject> elemente = bestimmeObjekte(
		// "typ.messQuerschnittAllgemein" );

		List<SystemObject> elemente = objektListeErstellen(m_dataModel,
				"typ.messQuerschnittAllgemein", kbMessQuerschnitte);

		for (int i = 0; i < elemente.size(); i++)
		{
			ConfigurationObject co = (ConfigurationObject) elemente.get(i);

			MessQuerschnitt mq = new MessQuerschnitt(m_connection, co);
			m_messQuerschnitt.put(co.getPid(), mq);

			StrassenSegment linie = mq.getReferenz();

			if (linie != null)
				linie.addMessQuerschnitt(mq);
		}
	}

	/**
	 * Methode ermittelt, ob es sich bei der bergebenen Pid um die Pid eines
	 * inneren Straensegments handelt
	 * 
	 * @param pid
	 *          zu prfende Pid
	 * @return true: Pid gehrt zu einem inneren Straensegment, sonst false
	 */
	private boolean isInneresStassenSegment(String pid)
	{
		return m_inneresStrassenSegment.containsKey(pid);
	}

	/**
	 * Methode ermittelt, ob es sich bei der bergebenen Pid um die Pid eines
	 * ueren Straensegments handelt
	 * 
	 * @param pid
	 *          zu prfende Pid
	 * @return true: Pid gehrt zu einem ueren Straensegment, sonst false
	 */
	private boolean isAeusseresStrassenSegment(String pid)
	{
		return m_aeusseresStrassenSegment.containsKey(pid);
	}

	/**
	 * Methode liefert zu einem Strassensegment die dazugehrenden
	 * Messquerschnitte
	 * 
	 * @param pidStrassenSegment
	 *          Pid des Strassensegments
	 * @return Vector mit Messquerschnitten
	 */
	public Vector<MessQuerschnitt> getMessQuerschnitteVonStrassenSegment(
			String pidStrassenSegment)
	{
		if (isAeusseresStrassenSegment(pidStrassenSegment))
		{
			if (m_aeusseresStrassenSegment.containsKey(pidStrassenSegment))
				return m_aeusseresStrassenSegment.get(pidStrassenSegment)
						.getMessquerschnitte();
		}

		if (isInneresStassenSegment(pidStrassenSegment))
		{
			if (m_inneresStrassenSegment.containsKey(pidStrassenSegment))
				return m_inneresStrassenSegment.get(pidStrassenSegment)
						.getMessquerschnitte();
		}

		return null;
	}

	/**
	 * Methode zum Bestimmen der Objekte, die zu einer bestimmten PID gehren
	 * (z.B. typ.de)
	 * 
	 * @param objPid
	 *          Pid der Objekttypen
	 * @return Liste mit den Systemobjekten
	 */
	private List<SystemObject> bestimmeObjekte(String objPid)
	{
		_debug.fine("bestimmeObjekte --> " + objPid);

		List<SystemObject> objekte = new ArrayList<SystemObject>();

		// Systemobjekt erzeugen

		SystemObjectType typeSysObj = null;

		typeSysObj = m_connection.getDataModel().getType(objPid);

		if (typeSysObj == null)
			return objekte;

		if (typeSysObj.isConfigurating())
			typeSysObj = (ConfigurationObjectType) typeSysObj;
		else
			typeSysObj = (DynamicObjectType) typeSysObj;

		// Elemente dieses Systemobjekts einlesen

		List<SystemObject> typobjekte = typeSysObj.getElements();

		Iterator<SystemObject> iterator_Typen = typobjekte.iterator();

		while (iterator_Typen.hasNext())
		{
			SystemObject sysObj = iterator_Typen.next();

			objekte.add(sysObj);

			// System.out.println(sysObj.getPid());
		}

		return objekte;
	}

	/**
	 * Methode zur Ausgabe der ueren Staensegmente (fr Testzwecke)
	 */
	private void ausgabeAeussereStrassenSegmente()
	{
		Iterator<Entry<String, AeusseresStrassenSegment>> it = m_aeusseresStrassenSegment
				.entrySet().iterator();

		while (it.hasNext())
		{

			Entry<String, AeusseresStrassenSegment> entry = it.next();

			AeusseresStrassenSegment s = entry.getValue();

			// if (!DebugFilter.debugSS( s.getPid() ))
			// continue;

			_debug.fine("--------------------------------------");
			_debug.fine(s.getPidRichtungStrasse());

			String buffer = null;

			long l = 0;

			buffer = "Straenteilsegmente:   ";
			Vector<StrassenTeilSegment> pidsts = s.getStrassenTeilSegmente();
			Iterator<StrassenTeilSegment> itvsts = pidsts.iterator();
			while (itvsts.hasNext())
			{
				StrassenTeilSegment sts = itvsts.next();
				buffer += " " + sts.getPid() + " (" + sts.getOffset() + "/"
						+ sts.getLaenge() + ")";
				l += sts.getLaenge();
			}
			_debug.fine(buffer);

			_debug.fine("Lnge: " + s.getLaenge() + " (" + l + ")");

			buffer = "Vorgnger auf Strae:  ";
			Vector<StrassenSegment> pids = s.getVorgaengerAufStrae();
			Iterator<StrassenSegment> itv = pids.iterator();
			while (itv.hasNext())
			{
				StrassenSegment s1 = itv.next();
				buffer += " " + s1.getPidRichtungStrasse() + "(" + s1.getLaenge() + ")";
			}
			_debug.fine(buffer);

			if (s.getKuerzestenVorgaengerAufStrasse() != null)
				_debug.fine("krzester Vorgnger:   "
						+ s.getKuerzestenVorgaengerAufStrasse().getPid());

			buffer = "sonstige Vorgnger:    ";
			pids = s.getVorgaenger();
			itv = pids.iterator();
			while (itv.hasNext())
				buffer += " " + ((StrassenSegment) itv.next()).getPidRichtungStrasse();
			_debug.fine(buffer);

			buffer = "Nachfolger auf Strae: ";
			pids = s.getNachfolgerAufStrasse();
			itv = pids.iterator();
			while (itv.hasNext())
				buffer += " " + ((StrassenSegment) itv.next()).getPidRichtungStrasse();
			_debug.fine(buffer);

			buffer = "sonstige Nachfolger:   ";
			pids = s.getNachfolger();
			itv = pids.iterator();
			while (itv.hasNext())
				buffer += " " + ((StrassenSegment) itv.next()).getPidRichtungStrasse();
			_debug.fine(buffer);

			buffer = "Einfahrten:            ";
			pids = s.getEinfahrendeStrassenSegmente();
			itv = pids.iterator();
			while (itv.hasNext())
				buffer += " " + ((StrassenSegment) itv.next()).getPidRichtungStrasse();
			_debug.fine(buffer);

			buffer = "Einfahrten (MQ):       ";
			Vector<MessQuerschnitt> v1 = s.getEinfahrendeMessQuerschnitte();
			Iterator<MessQuerschnitt> itVEinfahrendeMQs = v1.iterator();
			while (itVEinfahrendeMQs.hasNext())
				buffer += " " + (itVEinfahrendeMQs.next()).getPid();
			_debug.fine(buffer);

			buffer = "Ausfahrten:            ";
			pids = s.getAusfahrendeStrassenSegmente();
			itv = pids.iterator();
			while (itv.hasNext())
				buffer += " " + ((StrassenSegment) itv.next()).getPidRichtungStrasse();
			_debug.fine(buffer);

			buffer = "Ausfahrten (MQ):       ";
			Vector<MessQuerschnitt> v2 = s.getAusfahrendeMessQuerschnitte();
			Iterator<MessQuerschnitt> itAusfahrendeMQs = v2.iterator();

			while (itAusfahrendeMQs.hasNext())
				buffer += " " + (itAusfahrendeMQs.next()).getPid();
			_debug.fine(buffer);

			buffer = "Messquerschnitte:      ";
			Vector<MessQuerschnitt> v = s.getMessquerschnitte();
			Iterator<MessQuerschnitt> itMQs = v.iterator();
			while (itMQs.hasNext())
			{
				MessQuerschnitt m = itMQs.next();
				buffer += " " + m.getPid() + "(" + m.getOffset() + ") ";
			}
			_debug.fine(buffer);
		}
	}

	/**
	 * Methode zur Ausgabe der inneren Staensegmente (fr Testzwecke)
	 */
	private void ausgabeInnereStrassenSegmente()
	{
		Iterator<Entry<String, InneresStrassenSegment>> it = m_inneresStrassenSegment
				.entrySet().iterator();
		while (it.hasNext())
		{
			Entry<String, InneresStrassenSegment> entry = it.next();

			InneresStrassenSegment s = entry.getValue();

			// if (!DebugFilter.debugSS( s.getPid() ))
			// continue;

			_debug.fine("--------------------------------------");

			String buffer = s.getPid();

			if (s.isEinfahrt())
				buffer += " (Einfahrt)";

			if (s.isAusfahrt())
				buffer += " (Ausfahrt)";

			_debug.fine(buffer);

			long l = 0;

			buffer = "Straenteilsegmente:   ";
			Vector<StrassenTeilSegment> pidsts = s.getStrassenTeilSegmente();
			Iterator<StrassenTeilSegment> itvsts = pidsts.iterator();
			while (itvsts.hasNext())
			{
				StrassenTeilSegment sts = itvsts.next();
				buffer += " " + sts.getPid() + " (" + sts.getOffset() + "/"
						+ sts.getLaenge() + ")";
				l += sts.getLaenge();
			}
			_debug.fine(buffer);

			_debug.fine("Lnge: " + s.getLaenge() + " (" + l + ")");

			buffer = "Vorgnger auf Strae:  ";
			Vector<StrassenSegment> pids = s.getVorgaengerAufStrae();
			Iterator<StrassenSegment> itv = pids.iterator();
			while (itv.hasNext())
				buffer += " " + itv.next().getPid();
			_debug.fine(buffer);

			buffer = "Nachfolger auf Strae: ";
			pids = s.getNachfolgerAufStrasse();
			itv = pids.iterator();
			while (itv.hasNext())
				buffer += " " + itv.next().getPid();
			_debug.fine(buffer);

			buffer = "Messquerschnitte:      ";
			Vector<MessQuerschnitt> vMQs = s.getMessquerschnitte();
			Iterator<MessQuerschnitt> itMQs = vMQs.iterator();
			while (itMQs.hasNext())
			{
				MessQuerschnitt m = itMQs.next();
				buffer += " " + m.getPid() + "(" + m.getOffset() + ") ";
			}
			_debug.fine(buffer);
		}
	}

	/**
	 * Methode zum Ausgeben der StassenKnoten (fr Testzwecke)
	 */
	private void ausgabeStrassenKnoten()
	{
		Iterator<Entry<String, StrassenKnoten>> it = m_strassenKnoten.entrySet()
				.iterator();

		while (it.hasNext())
		{
			Entry<String, StrassenKnoten> entry = it.next();

			StrassenKnoten sk = entry.getValue();

			_debug.fine("--------------------------------------");
			_debug.fine(sk.getPid() + " (" + sk.getTyp() + ")");

			_debug.fine("Interne Strassensegmente:  ");
			List<InneresStrassenSegment> segmente = sk.getInnereStrassenSegmente();

			Iterator<InneresStrassenSegment> itv = segmente.iterator();
			while (itv.hasNext())
			{
				InneresStrassenSegment iss = itv.next();
				_debug.fine(iss.getSystemObject().getPid());
			}
		}
	}

	/**
	 * Methode liefert ein Strassensegment zurck.
	 * 
	 * @param pid
	 *          Pid des Strassensegments
	 * @return Instanz des Strassensegments, im Fehlerfall null
	 */
	public StrassenSegment getStrassenSegment(String pid)
	{
		if (isAeusseresStrassenSegment(pid))
			return m_aeusseresStrassenSegment.get(pid);

		if (isInneresStassenSegment(pid))
			return m_inneresStrassenSegment.get(pid);

		return null;
	}

	/**
	 * Methode ermittelt die NBA-Messquerschnitte
	 */
	private void bestimmeNbaMessQuerschnitte()
	{
		List<SystemObject> elemente = bestimmeObjekte("typ.nbaMessQuerschnitt");

		for (int i = 0; i < elemente.size(); i++)
		{
			ConfigurationObject co = (ConfigurationObject) elemente.get(i);

			Data confData = co.getConfigurationData(m_dataModel
					.getAttributeGroup("atg.nbaMessQuerschnitt"));

			if (confData == null)
				continue;

			String ref = confData.getReferenceValue("MessQuerschnittReal")
					.getValueText();

			if (ref != null)
			{
				// _debug.fine("Mq: " + ref + " nbaMQ: " + co.getPid());

				m_zuordnungMqZuNbaMq.put(ref, co.getPid());
			}
		}
	}

	/**
	 * Methode bestimmt zu einem Messquerschnitt der mit der Pid vom Typ
	 * "typ.messQuerschnitt" die dazugehrende Pid vom Typ
	 * "typ.nbaMessQuerschnitt"
	 * 
	 * @param pidMq
	 *          Pid vom Typ "typ.messQuerschnitt"
	 * @return Pid vom Typ "typ.nbaMessQuerschnitt"
	 */
	public String getNbaMqVonMq(String pidMq)
	{
		if (m_zuordnungMqZuNbaMq.containsKey(pidMq))
			return m_zuordnungMqZuNbaMq.get(pidMq);

		return null;
	}

	/**
	 * Methode bestimmt zu einem Messquerschnitt der mit der Pid vom Typ
	 * "typ.nbaMessQuerschnitt" die dazugehrende Pid vom Typ
	 * "typ.messQuerschnitt"
	 * 
	 * @param pidNbaMq
	 *          Pid vom Typ "typ.nbaMessQuerschnitt"
	 * @return Pid vom Typ "typ.messQuerschnitt"
	 */
	public String getMqVonNbaMq(String pidNbaMq)
	{
		Iterator<Entry<String, String>> it = m_zuordnungMqZuNbaMq.entrySet()
				.iterator();

		while (it.hasNext())
		{
			Entry<String, String> entry = it.next();

			String _pidMq = entry.getKey();
			String _pidNbaMq = entry.getValue();

			if (pidNbaMq.equals(_pidNbaMq))
				return _pidMq;
		}

		return null;
	}

	/**
	 * Methode liefert alle Messquerschnitte (ty.messQuerschnitt)
	 * 
	 * @return Liste mit den Messquerschnitten
	 */
	public List<SystemObject> getMessQuerschnitte()
	{
		List<SystemObject> l = new ArrayList<SystemObject>();

		Iterator<Entry<String, MessQuerschnitt>> it = m_messQuerschnitt.entrySet()
				.iterator();

		while (it.hasNext())
		{
			Entry<String, MessQuerschnitt> entry = it.next();

			MessQuerschnitt mq = entry.getValue();

			l.add(mq.getSystemObjekt());
		}

		return l;
	}

	/**
	 * Methode zur Erstellung einer Liste von Objekten bestimmter
	 * Konfigurationsbereiche. Es knnen die Pids der Konfigurationsbereichen
	 * getrennt durch ':' bergeben werden.
	 * 
	 * @param dataModel
	 *          Datenmodell
	 * @param pidObjektTyp
	 *          Pid des gesuchten ObjektTyps
	 * @param konfigurationsBereiche
	 *          Pids der Konfigurationsbereiche getrennt durch ';'.
	 * @throws IllegalArgumentException
	 */
	public List<SystemObject> objektListeErstellen(DataModel dataModel,
			String pidObjektTyp, String konfigurationsBereiche)
			throws IllegalArgumentException
	{
		List<SystemObject> objekte = new ArrayList<SystemObject>();

		String[] argumente = null;

		List<SystemObject> alleObjekteTyp = null;

		if (dataModel.getType(pidObjektTyp) == null)
		{
			throw new IllegalArgumentException("Gesuchter Objekttyp nicht vorhanden "
					+ pidObjektTyp);
		}

		// keine Argumente bergeben, dann alle Konfigurationsbereiche verwenden

		if (konfigurationsBereiche.length() == 0)
		{

			_debug
					.fine("Kein Argument bergeben, alle Konfigurationsbereiche werden verwendet.");

			SystemObjectType sotKonfigurationsbereiche = dataModel
					.getType("typ.konfigurationsBereich");

			List<SystemObject> listKonfigurationsbereiche = sotKonfigurationsbereiche
					.getElements();

			argumente = new String[listKonfigurationsbereiche.size()];

			for (int i = 0; i < listKonfigurationsbereiche.size(); i++)
			{
				ConfigurationObject konfigurationsbereich = (ConfigurationObject) listKonfigurationsbereiche
						.get(i);

				argumente[i] = konfigurationsbereich.getPid();
			}
		}

		// vorhandene Argumente bernehmen

		else
		{
			if (konfigurationsBereiche.length() != 0)
			{

				_debug.fine("Argument '" + konfigurationsBereiche + " wird verwendet.");

				argumente = konfigurationsBereiche.split(":");
			}
		}

		// Einzelobjekte bestimmen

		if (argumente != null)
		{
			for (int i = 0; i < argumente.length; i++)
			{
				_debug.fine("Konfiguratiosbereich: " + argumente[i]);

				SystemObject so = dataModel.getObject(argumente[i]);

				if (so != null)
				{
					if (so.isOfType(pidObjektTyp))
					{
						_debug.finer("Einzelobjekt gefunden " + so.getPid());

						if (!objekte.contains(so))
						{
							objekte.add(so);

							_debug
									.finest("MessQuerschnitt in Liste eingefgt " + so.getPid());
						} else
						{
							_debug.finest("MessQuerschnitt schon in Liste vorhanden "
									+ so.getPid());
						}
					} else
					{
						if (so.isOfType("typ.konfigurationsBereich"))
						{
							_debug.finer("Konfigurationsbereich gefunden " + so.getPid());

							if (alleObjekteTyp == null)
							{
								alleObjekteTyp = dataModel.getType(pidObjektTyp).getElements();
							}

							for (int j = 0; j < alleObjekteTyp.size(); j++)
							{
								SystemObject so1 = alleObjekteTyp.get(j);

								if (!so1.getConfigurationArea().equals(so))
								{
									continue;
								}

								if (!objekte.contains(so1))
								{
									objekte.add(so1);

									_debug.finest("MessQuerschnitt in Liste eingefgt "
											+ so1.getPid() + " "
											+ so1.getConfigurationArea().getPid());
								} else
								{
									_debug.finest("MessQuerschnitt schon in Liste vorhanden "
											+ so1.getPid());
								}
							}
						} else
						{
							// Fehlermeldung, falscher Typ

							_debug.info("Falscher Typ gefunden " + so.getPid() + " "
									+ so.getType().getPid());
						}
					}
				} else
				{
					// Fehlermeldung, Objekt nicht in Konfiguration

					_debug.info("Objekt nicht in Konfiguration " + argumente[i]);
				}
			}
		}

		return objekte;
	}

	/**
	 * Methode liefert alle ueren Straensegmente
	 * 
	 * @return Vektor mit eren Straensegmenten
	 */
	public Vector<AeusseresStrassenSegment> getAeussereStrassenSegmente()
	{
		Vector<AeusseresStrassenSegment> v = new Vector<AeusseresStrassenSegment>();

		Iterator<Entry<String, AeusseresStrassenSegment>> it = m_aeusseresStrassenSegment
				.entrySet().iterator();
		while (it.hasNext())
		{
			Entry<String, AeusseresStrassenSegment> entry = it.next();

			AeusseresStrassenSegment s = entry.getValue();

			v.add(s);
		}

		return v;
	}

	/**
	 * Methode liefert alle inneren Straensegmente
	 * 
	 * @return Vektor mit inneren Straensegmenten
	 */
	public Vector<InneresStrassenSegment> getInnereStrassenSegmente()
	{
		Vector<InneresStrassenSegment> v = new Vector<InneresStrassenSegment>();

		Iterator<Entry<String, InneresStrassenSegment>> it = m_inneresStrassenSegment
				.entrySet().iterator();
		while (it.hasNext())
		{
			Entry<String, InneresStrassenSegment> entry = it.next();

			InneresStrassenSegment s = entry.getValue();

			v.add(s);
		}

		return v;
	}

	/**
	 * Methode liefert alle Straenteilsegmente
	 * 
	 * @return Straensegmente
	 */
	public Vector<StrassenTeilSegment> getStrassenTeilSegmente()
	{
		Vector<StrassenTeilSegment> v = new Vector<StrassenTeilSegment>();

		// uere Straensegmente

		Iterator<AeusseresStrassenSegment> it = getAeussereStrassenSegmente()
				.iterator();
		while (it.hasNext())
		{
			AeusseresStrassenSegment s = it.next();

			Iterator<StrassenTeilSegment> itSts = s.getStrassenTeilSegmente()
					.iterator();
			while (itSts.hasNext())
			{
				v.add(itSts.next());
			}
		}

		// innere Straensegmente

		Iterator<InneresStrassenSegment> itI = getInnereStrassenSegmente()
				.iterator();
		while (itI.hasNext())
		{
			InneresStrassenSegment s = itI.next();

			Iterator<StrassenTeilSegment> itSts = s.getStrassenTeilSegmente()
					.iterator();
			while (itSts.hasNext())
			{
				v.add(itSts.next());
			}
		}

		return v;
	}
}
