/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.ort.hilfsklassen;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map.Entry;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;

public class KonfigurationNeu
{

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Datenmodell
   */
  private DataModel m_dataModel;

  /**
   * * Verbindung zum DaV
   */
  private ClientDavConnection m_connection;

  /**
   * {@link HashMap} mit den Strae als Instanz der Klasse {@link Strasse} (Key = Pid, Value = Instanz)
   */
  private HashMap<String, Strasse> m_strassen = new HashMap<String, Strasse>();

  /**
   * Hashmap mit Instanzen der einzelnen Staenknoten (Key = Pid, Value = Instanz)
   */
  private HashMap<String, StrassenKnoten> m_strassenKnoten = new HashMap<String, StrassenKnoten>();

  /**
   * Hashmap mit Instanzen der einzelnen ueren Straensegmente (Key = Pid, Value = Instanz)
   */
  private HashMap<String, AeusseresStrassenSegment> m_aeusseresStrassenSegment = new HashMap<String, AeusseresStrassenSegment>();

  /**
   * Hashmap mit Instanzen der einzelnen inneren Staensegmente (Key = Pid, Value = Instanz)
   */
  private HashMap<String, InneresStrassenSegment> m_inneresStrassenSegment = new HashMap<String, InneresStrassenSegment>();

  /**
   * Hashmap mit Instanzen der einzelnen, gefilterten Staenknoten (Key = Pid, Value = Instanz)
   */
  private HashMap<String, StrassenKnoten> m_strassenKnotenGefiltert = new HashMap<String, StrassenKnoten>();

  /**
   * Hashmap mit Instanzen der einzelnen <b>gefilterten</b> ueren Straensegmente (Key = Pid, Value = Instanz)
   */
  private HashMap<String, AeusseresStrassenSegment> m_aeusseresStrassenSegmentGefiltert = new HashMap<String, AeusseresStrassenSegment>();

  /**
   * Hashmap mit Instanzen der einzelnen <b>gefilterten</b> inneren Staensegmente (Key = Pid, Value = Instanz)
   */
  private HashMap<String, InneresStrassenSegment> m_inneresStrassenSegmentGefiltert = new HashMap<String, InneresStrassenSegment>();

  /**
   * Privater Konstruktor
   */
  private KonfigurationNeu()
  {
  }

  private static class Inner
  {
    private static KonfigurationNeu _instanz = new KonfigurationNeu();
  }

  public static KonfigurationNeu getInstanz()
  {
    return Inner._instanz;
  }

  /**
   * Bestimmt die bentigten Objekte zu einer Strae.
   * 
   * @param connection
   *          Verbindung zum DaV
   */
  public void bestimmeObjekte(ClientDavConnection connection)
  {
    m_connection = connection;

    m_dataModel = m_connection.getDataModel();

    bestimmeStrassen();
  }

  /**
   * Methode zum Bestimmen aller Objekte vom Typ "typ.strae";
   */
  private void bestimmeStrassen()
  {
    List<SystemObject> strassen = bestimmeObjekte("typ.strae");

    m_dataModel.getConfigurationData(strassen, m_dataModel.getAttributeGroup("atg.strae"));
   
    Iterator<SystemObject> it = strassen.iterator();
    while (it.hasNext())
    {
      SystemObject so = it.next();
      // System.out.println("PID: " + so.getPid());
      m_strassen.put(so.getPid(), new Strasse(so, m_dataModel));
    }
  }

  /**
   * Fhrt die Filterung nach der selektierten Strae durch.
   * 
   * @param pidSelektierteStrasse
   */
  public void filtereNachStrasse(String pidSelektierteStrasse)
  {
    m_strassenKnoten.clear();
    bestimmeStrassenKnoten();
    m_aeusseresStrassenSegmentGefiltert.clear();
    bestimmeAeussereStrassenSegmenteGefiltert(pidSelektierteStrasse);
    m_inneresStrassenSegmentGefiltert.clear();
    bestimmeInnereStrassenSegmenteGefiltert(pidSelektierteStrasse);

    bestimmeStrassenKnotenGefiltert(pidSelektierteStrasse);

    // /*
    // * Lschen berflssiger StrassenKnoten
    // */
    // loescheUeberfluessigeStrassenKnoten();
    //
    // /*
    // * Entfernen der entsprechenden Strassenknoten aus den ueren
    // * Straensegmenten
    // */
    // loescheNichtVorhandeneStrassenKnotenAusAeusserenStrassenSegmenten();
  }

  /**
   * Methode zum Bestimmen aller ueren Straensegmente, die zu einer bestimmten Strae gehren.
   * 
   * @param pidSelektierteStrasse
   *          Pid der Strae, zu der die ueren Straensegmente bestimmt werden sollen.
   */
  private void bestimmeAeussereStrassenSegmenteGefiltert(String pidSelektierteStrasse)
  {
    // LinkedList<SystemObject> aeussereStrassenSegmente = new
    // LinkedList<SystemObject>();

    List<SystemObject> aeussereStrassenSegmente = bestimmeObjekte("typ.ueresStraenSegment");

    //PERFORMANCEVERBESSERUNGEN ANSEHEN
    m_dataModel.getConfigurationData(aeussereStrassenSegmente, m_dataModel.getAttributeGroup("atg.straenSegment"));
    m_dataModel.getConfigurationData(aeussereStrassenSegmente, m_dataModel.getAttributeGroup("atg.ueresStraenSegment"));

    Iterator<SystemObject> it = aeussereStrassenSegmente.iterator();
    while (it.hasNext())
    {
      SystemObject so = it.next();
      AttributeGroup attributeGroup = m_dataModel.getAttributeGroup("atg.straenSegment");
      Data data = so.getConfigurationData(attributeGroup);
      String valueText = data.getTextValue("gehrtZuStrae").getValueText();

      // System.out.println("VAL: " + valueText);
      // System.out.println("PID: " + pidSelektierteStrasse);

      AeusseresStrassenSegment aeusseresStrassenSegment = new AeusseresStrassenSegment(m_dataModel, so);
      // m_aeusseresStrassenSegment.put(so.getPid(),
      // new AeusseresStrassenSegment(m_dataModel, so));
      m_aeusseresStrassenSegment.put(aeusseresStrassenSegment.getPid(), aeusseresStrassenSegment);

      StrassenKnoten vonKnoten = aeusseresStrassenSegment.getVonKnoten();
      StrassenKnoten nachKnoten = aeusseresStrassenSegment.getNachKnoten();

      if (vonKnoten != null)
        vonKnoten.addAbgehendesAeusseresStrassenSegment(aeusseresStrassenSegment);

      if (nachKnoten != null)
        nachKnoten.addHinfuehrendesAeusseresStrassenSegment(aeusseresStrassenSegment);

      /*
       * Gefiltertes ueresStraenSegment
       */
      if (valueText.equals(pidSelektierteStrasse))
      {
        m_aeusseresStrassenSegmentGefiltert.put(aeusseresStrassenSegment.getPid(), aeusseresStrassenSegment);

        /*
         * Bestimmung des STS
         */
        aeusseresStrassenSegment.initialisiere();

      }

    }

    // Iterator<SystemObject> itASS = aeussereStrassenSegmente.iterator();
    // while (itASS.hasNext())
    // {
    // SystemObject objekt = itASS.next();
    //
    // AeusseresStrassenSegment strassenSegment = new AeusseresStrassenSegment(
    // m_dataModel, objekt);
    // strassenSegment.initialisiere();
    //
    // m_aeusseresStrassenSegment.put(objekt.getPid(), strassenSegment);
    //
    // StrassenKnoten vonKnoten = strassenSegment.getVonKnoten();
    // StrassenKnoten nachKnoten = strassenSegment.getNachKnoten();
    //
    // if (vonKnoten != null)
    // vonKnoten.addAbgehendesAeusseresStrassenSegment(strassenSegment);
    //
    // if (nachKnoten != null)
    // nachKnoten.addHinfuehrendesAeusseresStrassenSegment(strassenSegment);
    // }
  }

  /**
   * Methode zum Bestimmen aller Objekte com Typ "typ.inneresStraenSegment". Zustzlich wird eine Zuordnung der
   * Strassensegmente zu den Strassenknoten vorgenommen
   */
  private void bestimmeInnereStrassenSegmenteGefiltert(String pidSelektierteStrasse)
  {
    List<SystemObject> innereStrassenSegmente = bestimmeObjekte("typ.inneresStraenSegment");

    m_dataModel.getConfigurationData(innereStrassenSegmente, m_dataModel.getAttributeGroup("atg.inneresStraenSegment"));
    
    List<SystemObject> innereStrassenSegmenteGefiltert = new LinkedList<SystemObject>();

    /*
     * Nur die innerenStrassensegmente verwenden, die zu einer Strae gehren
     */
    Iterator<SystemObject> itFi = innereStrassenSegmente.iterator();
    while (itFi.hasNext())
    {
      SystemObject so = itFi.next();
      AttributeGroup attributeGroup = m_dataModel.getAttributeGroup("atg.straenSegment");
      Data data = so.getConfigurationData(attributeGroup);
      String valueText = data.getTextValue("gehrtZuStrae").getValueText();
      if (valueText.equals(pidSelektierteStrasse))
      {
        innereStrassenSegmenteGefiltert.add(so);
      }

      InneresStrassenSegment strassenSegment = new InneresStrassenSegment(m_dataModel, so);

      m_inneresStrassenSegment.put(so.getPid(), strassenSegment);
    }

    // Iterator it = innereStrassenSegmente.iterator();
    Iterator<SystemObject> it = innereStrassenSegmenteGefiltert.iterator();
    while (it.hasNext())
    {

      SystemObject objekt = it.next();

      InneresStrassenSegment strassenSegment = new InneresStrassenSegment(m_dataModel, objekt);

      // m_inneresStrassenSegment.put(objekt.getPid(), strassenSegment);

      // Fllen der Gefiltert Liste
      if (innereStrassenSegmenteGefiltert.contains(objekt))
      {
        m_inneresStrassenSegmentGefiltert.put(objekt.getPid(), strassenSegment);
      }

      AeusseresStrassenSegment vonStrassenSegment = strassenSegment.getVonStrassenSegmemt();
      AeusseresStrassenSegment nachStrassenSegment = strassenSegment.getNachStrassenSegment();

      if ((vonStrassenSegment) == null && (nachStrassenSegment == null))
      {
        m_inneresStrassenSegment.remove(objekt.getPid());
      }
      else
      {
        strassenSegment.initialisiere();

        if (vonStrassenSegment != null)
        {
          strassenSegment.setVorgaengerAufStrasse(vonStrassenSegment);

          StrassenKnoten sk = m_strassenKnoten.get(vonStrassenSegment.getNachKnoten().getPid());
          if (sk != null)
            sk.addInneresStrassenSegment(strassenSegment);

          if (vonStrassenSegment.liegtAufSelberStrasse(nachStrassenSegment))
            vonStrassenSegment.addNachfolgerAufStrasse(strassenSegment);
          else
            vonStrassenSegment.addNachfolger(strassenSegment);
        }

        if (nachStrassenSegment != null)
        {
          strassenSegment.setNachfolgerAufStrae(nachStrassenSegment);

          StrassenKnoten sk = m_strassenKnoten.get(nachStrassenSegment.getVonKnoten().getPid());
          if (sk != null)
            sk.addInneresStrassenSegment(strassenSegment);

          if (nachStrassenSegment.liegtAufSelberStrasse(vonStrassenSegment))
            nachStrassenSegment.addVorgaengerAufStrasse(strassenSegment);
          else
            nachStrassenSegment.addVorgaenger(strassenSegment);
        }
      }
    }
  }

  /**
   * Bestimmt die Knoten, die auf einer bestimmten Strae liegen und legt diese in der HashMap
   * {@link KonfigurationNeu#m_strassenKnotenGefiltert} ab.
   * 
   * @param pidSelektierteStrasse
   *          Pid der Strae, zu der die zugehrigen Knoten bestimmt werden sollen.
   */
  private void bestimmeStrassenKnotenGefiltert(String pidSelektierteStrasse)
  {

    /*
     * Ablauf
     * 
     * OK - lschen der alten gefilterten Map - durchlaufen aller Knoten der m_strassenKnoten - prfen ob
     * Abgehende/Hinzufhrende uere Straensegmente auf der Strae liegen. -wenn ja, dann kommt der Knoten in die
     * gefilterte Map, sonst nicht.
     * 
     */
    m_strassenKnotenGefiltert.clear();

    Iterator<Entry<String, StrassenKnoten>> itKnoten = m_strassenKnoten.entrySet().iterator();
    while (itKnoten.hasNext())
    {
      boolean gefunden = false;

      Entry<String, StrassenKnoten> entryKnoten = itKnoten.next();
      List<AeusseresStrassenSegment> abgehendeAeussereStrassenSegmente = entryKnoten.getValue()
          .getAbgehendeAeussereStrassenSegmente();
      List<AeusseresStrassenSegment> hinfuehrendeAeussereStrassenSegmente = entryKnoten.getValue()
          .getHinfuehrendeAeussereStrassenSegmente();
      List<InneresStrassenSegment> innereStrassenSegmente = entryKnoten.getValue().getInnereStrassenSegmente();

      /*
       * Abgehende uereStraenSegmente
       */
      if (!gefunden)
      {
        Iterator<AeusseresStrassenSegment> itAngehendeASS = abgehendeAeussereStrassenSegmente.iterator();
        while (itAngehendeASS.hasNext())
        {
          AeusseresStrassenSegment aeusseresStrassenSegment = itAngehendeASS.next();
          if (m_aeusseresStrassenSegmentGefiltert.containsKey(aeusseresStrassenSegment.getPid()))
          {
            gefunden = true;
            break;
          }
        }
      }

      /*
       * Hinzufhrende uereStraenSegmente
       */
      if (!gefunden)
      {
        Iterator<AeusseresStrassenSegment> itHinzufASS = hinfuehrendeAeussereStrassenSegmente.iterator();
        while (itHinzufASS.hasNext())
        {
          AeusseresStrassenSegment aeusseresStrassenSegment = itHinzufASS.next();
          if (m_aeusseresStrassenSegmentGefiltert.containsKey(aeusseresStrassenSegment.getPid()))
          {
            gefunden = true;
            break;
          }
        }
      }

      /*
       * InnereStraenSegmente
       */
      if (!gefunden)
      {
        Iterator<InneresStrassenSegment> it = innereStrassenSegmente.iterator();
        while (it.hasNext())
        {
          InneresStrassenSegment inneresStrassenSegment = it.next();
          if (m_inneresStrassenSegmentGefiltert.containsKey(inneresStrassenSegment.getPid()))
          {
            gefunden = true;
            break;
          }
        }
      }

      if (gefunden)
      {
        m_strassenKnotenGefiltert.put(entryKnoten.getKey(), entryKnoten.getValue());
      }
    }

  }

  /**
   * Methode zum Bestimmen aller Objekte vom Typ "typ.straenKnoten"
   */
  private void bestimmeStrassenKnoten()
  {
    List<SystemObject> strassenKnoten = bestimmeObjekte("typ.straenKnoten");

    m_dataModel.getConfigurationData(strassenKnoten, m_dataModel.getAttributeGroup("atg.straenKnoten"));
    
    Iterator<SystemObject> it = strassenKnoten.iterator();
    while (it.hasNext())
    {
      SystemObject objekt = it.next();

      m_strassenKnoten.put(objekt.getPid(), new StrassenKnoten(m_connection, objekt));
    }
  }

  /**
   * Methode zum Bestimmen der Objekte, die zu einer bestimmten PID gehren (z.B. typ.de)
   * 
   * @param objPid
   *          Pid der Objekttypen
   * @return Liste mit den Systemobjekten
   */
  private List<SystemObject> bestimmeObjekte(String objPid)
  {
    _debug.fine("bestimmeObjekte --> " + objPid);

    List<SystemObject> objekte = new ArrayList<SystemObject>();

    // Systemobjekt erzeugen

    SystemObjectType typeSysObj = null;

    typeSysObj = m_connection.getDataModel().getType(objPid);

    if (typeSysObj == null)
      return objekte;

    if (typeSysObj.isConfigurating())
      typeSysObj = (ConfigurationObjectType)typeSysObj;
    else
      typeSysObj = (DynamicObjectType)typeSysObj;

    // Elemente dieses Systemobjekts einlesen

    List<SystemObject> typobjekte = typeSysObj.getElements();
    
    Iterator<SystemObject> iterator_Typen = typobjekte.iterator();

    while (iterator_Typen.hasNext())
    {
      SystemObject sysObj = (SystemObject)iterator_Typen.next();

      objekte.add(sysObj);

      // System.out.println(sysObj.getPid());
    }

    return objekte;
  }

  /**
   * Lscht die Straenknoten, in denen kein Verweis auf Innere Straensegmente dieser Strae vorhanden sind.
   */
  @SuppressWarnings("unused")
	private void loescheUeberfluessigeStrassenKnoten()
  {
    LinkedList<String> pidsLoeschbarerStrassenKnoten = new LinkedList<String>();

    _debug.fine("ANZAHL STRASSENKNOTEN: " + m_strassenKnoten.size());
    Iterator<Entry<String, StrassenKnoten>> it = m_strassenKnoten.entrySet().iterator();
    int zaehler = 0;
    while (it.hasNext())
    {
      Entry<String, StrassenKnoten> entry = it.next();
      StrassenKnoten strassenKnoten = entry.getValue();
      if (strassenKnoten.getInnereStrassenSegmente().size() == 0)
      {
        pidsLoeschbarerStrassenKnoten.add(entry.getKey());
        zaehler++;
      }
    }
    _debug.fine("ANZAHL ZU LSCHENDER STRASSENKNOTEN: " + zaehler);

    /*
     * Lschen der berflssigen StrassenKnoten
     */
    Iterator<String> itLoeschen = pidsLoeschbarerStrassenKnoten.iterator();
    while (itLoeschen.hasNext())
    {
      String key = itLoeschen.next();
      if (m_strassenKnoten.containsKey(key))
      {
        m_strassenKnoten.remove(key);
      }
      else
      {
        _debug.fine("SHIT!");
      }
    }

  }

  /**
   * Lscht aus den ueren Straensegmenten die nach dem Filtern nicht mehr vorhandenen von- und nach StraenKnoten.
   */
  @SuppressWarnings("unused")
	private void loescheNichtVorhandeneStrassenKnotenAusAeusserenStrassenSegmenten()
  {
    /*
     * Iterator ber alle ueren Straensegmente
     */
    Iterator<Entry<String, AeusseresStrassenSegment>> it = m_aeusseresStrassenSegment.entrySet().iterator();
    while (it.hasNext())
    {
      Entry<String, AeusseresStrassenSegment> entry = it.next();
      AeusseresStrassenSegment aeusseresStrassenSegment = entry.getValue();

      /*
       * Prfen ob vonKnoten noch in der HashMap der StrassenKnoten vorhanden ist.
       */
      if (aeusseresStrassenSegment.getVonKnoten() != null)
      {
        if (!m_strassenKnoten.containsKey(aeusseresStrassenSegment.getVonKnoten().getPid()))
        {
          aeusseresStrassenSegment.setVonKnoten(null);
          _debug.fine("LSCHE VON-KNOTEN");
        }
      }

      /*
       * Prfen ob nachKnoten noch in der HashMap der StrassenKnoten vorhanden ist
       */
      if (aeusseresStrassenSegment.getNachKnoten() != null)
      {
        if (!m_strassenKnoten.containsKey(aeusseresStrassenSegment.getNachKnoten().getPid()))
        {
          aeusseresStrassenSegment.setNachKnoten(null);
          _debug.fine("LSCHE NACH-KNOTEN");
        }
      }

    }
  }

  /**
   * Methode liefert ein ueres Straensegment
   * 
   * @param pid
   *          Pid des Straensegments
   * @return Instanz des Straensegments, im Fehlerfall null
   */
  public AeusseresStrassenSegment getAeusseresStrassenSegment(String pid)
  {
    if (m_aeusseresStrassenSegment.containsKey(pid))
      return m_aeusseresStrassenSegment.get(pid);

    return null;
  }

  /**
   * Methode liefert ein inneres Straensegment
   * 
   * @param pid
   *          Pid des Straensegments
   * @return Instanz des Straensegments, im Fehlerfall null
   */
  public InneresStrassenSegment getInneresStrassenSegment(String pid)
  {
    if (m_inneresStrassenSegment.containsKey(pid))
      return m_inneresStrassenSegment.get(pid);
    return null;
  }

  /**
   * Methode liefert einen Straenknoten zu einer Pid, existiert der Stasenknoten noch nicht und handlet es sich bei
   * dem Objekt das zu der Pid gehrt um ein Objekt vom Typ 'typ.straenKnoten', so wird ein neuer StraenKnoten
   * angelegt.
   * 
   * @param pid
   *          Pid des Straenknotens
   * @return Instanz des Straenknotens, im Fehlerfall null
   */
  public StrassenKnoten getStrassenKnoten(String pid)
  {
    if (m_strassenKnoten.containsKey(pid))
      return m_strassenKnoten.get(pid);
    else
    {
      ConfigurationObject objekt = (ConfigurationObject)m_dataModel.getObject(pid);
      if (objekt != null)
      {
        if (objekt.isOfType("typ.straenKnoten"))
        {
          StrassenKnoten sk = new StrassenKnoten(m_connection, objekt);
          m_strassenKnoten.put(sk.getPid(), sk);
          return sk;
        }
      }
    }

    return null;
  }

  /**
   * Methode ermittelt, ob es sich bei der bergebenen Pid um die Pid eines inneren Straensegments handelt
   * 
   * @param pid
   *          zu prfende Pid
   * @return true: Pid gehrt zu einem inneren Straensegment, sonst false
   */
  @SuppressWarnings("unused")
	private boolean isInneresStassenSegment(String pid)
  {
    return m_inneresStrassenSegment.containsKey(pid);
  }

  /**
   * Methode ermittelt, ob es sich bei der bergebenen Pid um die Pid eines ueren Straensegments handelt
   * 
   * @param pid
   *          zu prfende Pid
   * @return true: Pid gehrt zu einem ueren Straensegment, sonst false
   */
  @SuppressWarnings("unused")
	private boolean isAeusseresStrassenSegment(String pid)
  {
    return m_aeusseresStrassenSegment.containsKey(pid);
  }

  /**
   * Liefert die HashMap, welche alle {@link Strasse}n beinhaltet.
   * 
   * @return HashMap der {@link Strasse}n.
   */
  public HashMap<String, Strasse> getStrassen()
  {
    return m_strassen;
  }

  public HashMap<String, AeusseresStrassenSegment> getAeussereStrassenSegmente()
  {
    return m_aeusseresStrassenSegment;
  }

  public HashMap<String, AeusseresStrassenSegment> getAeussereStrassenSegmenteGefiltert()
  {
    return m_aeusseresStrassenSegmentGefiltert;
  }

  public HashMap<String, InneresStrassenSegment> getInnereStrassenSegmente()
  {
    return m_inneresStrassenSegment;
  }

  public HashMap<String, InneresStrassenSegment> getInnereStrassenSegmenteGefiltert()
  {
    return m_inneresStrassenSegmentGefiltert;
  }

  public HashMap<String, StrassenKnoten> getStrassenKnoten()
  {
    return m_strassenKnoten;
  }

  /**
   * Liefert die {@link HashMap} der gefilterten StraenKnoten
   * 
   * @return {@link HashMap} der gefilterten StraenKnoten
   */
  public HashMap<String, StrassenKnoten> getStrasseKnotenGefiltert()
  {
    return m_strassenKnotenGefiltert;
  }

  public String toString()
  {
    String re = "";

    re = re + "Anzahl Straen:                             " + m_strassen.size() + "\n";
    re = re + "Anzahl uerer Straensegmente:             " + m_aeusseresStrassenSegment.size() + "\n";

    re = re + "Anzahl uerer Straensegmente (Gefiltert): " + m_aeusseresStrassenSegmentGefiltert.size() + "\n";

    re = re + "Anzahl innerer Straensegmente:             " + m_inneresStrassenSegment.size() + "\n";

    re = re + "Anzahl innerer Straensegmente (Gefiltert): " + m_inneresStrassenSegmentGefiltert.size() + "\n";

    re = re + "Anzahl StraenKnoten:                       " + m_strassenKnoten.size() + "\n";

    re = re + "Anzahl StraenKnoten (Gefiltert):           " + m_strassenKnotenGefiltert.size() + "\n";

    return re;
  }

}
