/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.ort.hilfsklassen;

import java.util.Iterator;
import java.util.LinkedList;

import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.AsbOrtsangabe.AsbKnoten;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.AsbOrtsangabe.AsbKnotenKombiniert;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.AsbOrtsangabe.AsbKnotenVerwaltungKombiniert;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.sys.funclib.debug.Debug;

public class TestAsbOrtsangabe
{

	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	/**
	 * Verbindung zum DaV
	 */
	private ClientDavConnection _verbindung = null;

	/**
	 * IP
	 */
	// private static final String _ip = "192.0.1.117";
	private static final String _ip = "127.0.0.1";

	/**
	 * Port
	 */
	private static final int _port = 8083;

	/**
	 * Login
	 */
	private static final String _benutzer = "Tester";

	/**
	 * Passwort
	 */
	private static final String _passwort = "geheim";

	/**
	 * @param args
	 */
	public static void main(String[] args)
	{
		TestAsbOrtsangabe asb = new TestAsbOrtsangabe();
		asb.baueDaVVerbindungAuf();
		if (asb._verbindung != null)
		{
			// asb.testAlt();
			asb.testKombiniert();
		}

		asb.schliesseDaVVerbindung();

	}

	/**
	 * Kombinierter Test unter Bercksichtung beider Richtungen
	 */
	private void testKombiniert()
	{
		AsbOrtsangabe asb = new AsbOrtsangabe(_verbindung);
		String pidStrasse = "strasse.A.5.tmclc.07129"; // Darmstadt-Heidelberg
		System.out.println("Initialisierung fr: " + pidStrasse + "...");
		asb.initialisiereStrukturNachStrasse(pidStrasse);
		System.out.println("Initialisierung abgeschlossen.");

		LinkedList<AsbKnotenKombiniert> vonAuswahlfeldKombiniert = asb
				.getVonAuswahlfeldKombiniert();
		Iterator<AsbKnotenKombiniert> itVon = vonAuswahlfeldKombiniert.iterator();
		while (itVon.hasNext())
		{
			AsbKnotenKombiniert von = itVon.next();
			
			
			
			String out = "";
			out = out + "===Von-Knoten: " + von + "===\n";
			out = out + "Hat Positive: " + (von.getAsbKnotenPositiv() != null) + "\n";
			out = out + "Hat Negative: " + (von.getAsbKnotenNegativ() != null) + "\n";
			
			
			AsbKnotenVerwaltungKombiniert erreichbareAsbKnotenKombiniertVerwaltung = asb
					.bestimmeErreichbareAsbKnotenKombiniert(von);
			Iterator<AsbKnotenKombiniert> itBis = erreichbareAsbKnotenKombiniertVerwaltung
					.getListeKombinierteKnoten().iterator();
			while (itBis.hasNext())
			{				
				AsbKnotenKombiniert bis = itBis.next();
				
				out = out + "=> " + bis + "\n";

				for (int i = 0; i <= 2000; i = i + 100)
				{
					out = out
							+ "\t-> "
							+ i
							+ " m: "
							+ erreichbareAsbKnotenKombiniertVerwaltung.istInBereich(von, bis,
									i) + "\n";

					out = out
							+ "\t\t->"
							+ erreichbareAsbKnotenKombiniertVerwaltung.istInBereichDetailiert(von,
									bis, i) + "\n";
				}
			}

			for (int i = 0; i < (18 + von.toString().length()); i++)
			{
				out = out + "=";
			}
			out = out + "\n";

			System.out.println(out);
		}
	}

	/**
	 * Test, basierend auf der alten Variante unter Bercksichtigung der
	 * ASB-Knoten in jeweils eine der TMC-Richtungen der Strae.
	 */
	@SuppressWarnings("unused")
	private void testAlt()
	{

		AsbOrtsangabe asb = new AsbOrtsangabe(_verbindung);

		// String pidStrasse = "strasse.A.3.tmclc.07077"; //Kln-Frankfurt
		// String pidStrasse = "strasse.A.45.tmclc.07115"; // Gieen-Hagen
		// asb.initialisiereStrukturNachStrasse(pidStrasse);

		// LinkedList<Strasse> strassen = asb.getStrassen();
		// System.out.println("Anzahl Straen: " + strassen);
		// Iterator<Strasse> it = strassen.iterator();
		// while (it.hasNext())
		// {
		// Strasse strasse = it.next();
		// System.out.println("Strae (" + strasse.getPid() + ")" + strasse);
		// }

		String pidStrasse = "strasse.A.5.tmclc.07129"; // Darmstadt-Heidelberg
		System.out.println("Initialisierung fr: " + pidStrasse + "...");
		asb.initialisiereStrukturNachStrasse(pidStrasse);
		System.out.println("Initialisierung abgeschlossen.");
		System.out.println("TEST ===>");

		// asb.filtereAsbKnoten();

		Iterator<AsbKnoten> it = asb.getVerfuegbareAsbKnoten().iterator();
		while (it.hasNext())
		{
			AsbKnoten knoten = it.next();
			System.out.println(knoten);

			LinkedList<AsbKnoten> knotenErreichbar = null;

			knotenErreichbar = asb.bestimmeErreichbareAsbKnoten(knoten);

			Iterator<AsbKnoten> itErreichbar = knotenErreichbar.iterator();
			while (itErreichbar.hasNext())
			{
				AsbKnoten knoten2 = itErreichbar.next();
				System.out.println("\tErreichbar => " + knoten2);
			}
		}

	}

	/**
	 * Baut die Verbindung zum DaV auf.
	 */
	private void baueDaVVerbindungAuf()
	{
		try
		{
			_verbindung = new ClientDavConnection();

			// IP setzen
			_verbindung.getClientDavParameters().setDavCommunicationAddress(_ip);

			// Port setzen
			_verbindung.getClientDavParameters().setDavCommunicationSubAddress(_port);

			// Pfad zum Zwischenspeichern setzen
			_verbindung.getClientDavParameters().setConfigurationPath("C:\\temp");

			try
			{
				_verbindung.connect();
				System.out
						.println("Verbindungsaufbau: Verbindung erfolgreich aufgebaut.");

				try
				{
					_verbindung.login(_benutzer, _passwort);
					System.out
							.println("Verbindungsaufbau: Benutzer erfolgreich eingeloggt.");
				} catch (InconsistentLoginException e)
				{
					// e.printStackTrace();
					System.err
							.println("Verbindungsaufbau Fehler: Konnte Benutzer nicht einloggen.");
					_verbindung = null;
				}

			} catch (CommunicationError e)
			{
				// e.printStackTrace();
				System.err.println("Verbindungsaufbau Fehler: " + e.toString());
				_verbindung = null;
			} catch (ConnectionException e)
			{
				// e.printStackTrace();
				System.err.println("Verbindungsaufbau Fehler: " + e.toString());
				_verbindung = null;
			}

		} catch (MissingParameterException e)
		{
			// e.printStackTrace();
			System.err.println("Verbindungsaufbau Fehler: " + e.toString());
			_verbindung = null;
		}
	}

	/**
	 * Schliet die Verbindung zum DaV
	 */
	private void schliesseDaVVerbindung()
	{
		if (_verbindung != null)
		{
			_verbindung.disconnect(false, "Kein Fehler.");
			_debug.fine("\nVerbindungsabbau: Verbindung wurde abgebaut.");
		}
	}

}
