/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.seiteneinstellungen.druck.vorschau;

import org.eclipse.jface.action.Action;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.Activator;

/**
 * Dies ist eine Action, die den Dialog zur Druckvorschau ffnet. Ist ein
 * {@link IViewPart} oder {@link IEditorPart} aktiv, der das Interface
 * {@link IDruckvorschau} implementiert, so wird dieser Button aktiv und kann
 * die vom jeweiligen Editor oder View gelieferten Druckauftrge durchfhren.
 * 
 * @author HofmannJ
 * 
 */
public class DruckVorschauAction extends Action implements IPartListener
{
	/**
	 * Druckvorschau Part
	 */
	private IDruckvorschau _druckvorschauPart = null;

	/**
	 * Konstruktor fr eine Druckvorschau {@link Action}.
	 * 
	 * @param id
	 *          ID der Action
	 */
	public DruckVorschauAction(String id)
	{
		super();
		this.setId(id);
		this.setText("Druckvorschau");
		this.setToolTipText("ffnet den Druckvorschaudialog");
		this.setImageDescriptor(Activator
				.getImageDescriptor("icons/sonstiges/print.gif"));
		this.setEnabled(false);
	}

	@Override
	public void run()
	{
		if (_druckvorschauPart != null)
		{
			if (_druckvorschauPart.getDruckAuftrag() != null)
			{
				DruckvorschauDialog druckVorschau = new DruckvorschauDialog(PlatformUI
						.getWorkbench().getDisplay().getActiveShell());

				if (_druckvorschauPart.getDruckAuftrag() != null)
					druckVorschau.oeffne(_druckvorschauPart.getDruckAuftrag(),
							_druckvorschauPart.getTitel());
			} else
			{
				MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
						.getActiveShell(), SWT.ICON_INFORMATION | SWT.OK);
				mb.setText("Keine Druckseite verfgbar");
				mb
						.setMessage("Zur Zeit ist keine Druckseite verfgbar. Kann nicht drucken.");
				mb.open();
			}
		}
	}

	@Override
	public void partActivated(IWorkbenchPart part)
	{
		if (isIDruckVorschau(part))
		{
			setEnabled(true);
		} else
		{
			setEnabled(false);
		}
	}

	@Override
	public void partBroughtToTop(IWorkbenchPart part)
	{		
		if (isIDruckVorschau(part))
		{
			setEnabled(true);
		} else
		{
			setEnabled(false);
		}

	}

	@Override
	public void partClosed(IWorkbenchPart part)
	{
		this.setEnabled(false);
	}

	@Override
	public void partDeactivated(IWorkbenchPart part)
	{
		this.setEnabled(false);

	}

	@Override
	public void partOpened(IWorkbenchPart part)
	{
		if (isIDruckVorschau(part))
		{
			setEnabled(true);
		} else
		{
			setEnabled(false);
		}
	}

	/**
	 * Prft ob der {@link IWorkbenchPart} das Interface {@link IDruckvorschau}
	 * implementiert.
	 * 
	 * @param part
	 *          Zu Prfender {@link IWorkbenchPart}
	 * @return Liefert <code>true</code>, wenn das Interface
	 *         {@link IDruckvorschau} implementiert wird und es sich um einen
	 *         {@link IViewPart} oder einen {@link IEditorPart} handelt, sonst
	 *         <code>false</code>.
	 */
	private boolean isIDruckVorschau(IWorkbenchPart part)
	{
		boolean re = false;
		if ((part instanceof IViewPart) || (part instanceof IEditorPart))
		{
			if (part instanceof IDruckvorschau)
			{
				_druckvorschauPart = (IDruckvorschau) part;

				re = true;

			}
		}

		if (re == false)
		{
			_druckvorschauPart = null;
		}

		return re;
	}

}
