/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.simulationsvariante;

import org.eclipse.jface.action.Action;
import org.eclipse.ui.IStartup;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.Activator;
import de.bsvrz.buv.rw.rw.menu.ElementRegistrierung;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorAktion;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Sorgt fr die Registrierung einer Aktion, mit der sich der Dialog zum
 * Einstellen der Simulationsvariante aufrufen lsst.
 * 
 * @author HofmannJ
 * 
 */
public class SimulationsvarianteStarter implements IStartup
{
	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	private static String idViewSimVar = "de.bsvrz.buv.rw.basislib.simulationsvariante.SichtSimulationsvarianteDarstellenEinstellen";

	public SimulationsvarianteStarter()
	{
		_debug.fine(getClass().getName() + " ==> Konstruiert.");
	}

	public void earlyStartup()
	{
		_debug.fine(getClass().getName() + " ==> Early StartUp");

		Action actionSimVar = new Action()
		{
			public void run()
			{
				try
				{
					PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage()
							.showView(idViewSimVar);
				} catch (PartInitException e)
				{
					_debug.warning(getClass().getName()
							+ "::earlyStartup \n\t=>Die View (id=" + idViewSimVar
							+ ") kann nicht geffnet werden.");
					e.printStackTrace();
				}
			}
		};

		actionSimVar.setText("Simulationsvariante einstellen");
		actionSimVar.setId("de.bsvrz.buv.rw.basislib.simvareinst");
		actionSimVar
				.setToolTipText("Ermglicht das Einstellen der Simulationsvariante und zeigt die aktuelle Simulationsvariante an.");
		actionSimVar.setImageDescriptor(Activator
				.getImageDescriptor("icons/sample.gif"));
		DeskriptorAktion deskriptorAktion = new DeskriptorAktion(
				"de.bsvrz.buv.rw.basislib", actionSimVar, false);
		try
		{
			ElementRegistrierung.getInstanz().registriereDeskriptorAktion(
					deskriptorAktion);
		} catch (IllegalArgumentException iae)
		{
			//Element sind schon Registriert
		}

		_debug.fine(getClass().getName() + " ==> StartUP beendet");
	}

}
