/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.statusleiste.editor;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.ViewerDropAdapter;
import org.eclipse.swt.dnd.TransferData;
import org.jdom.Element;

import de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteElement;
import de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteVerwaltung;
import de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.XmlBezeichnerStatus;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Liefert den {@link ViewerDropAdapter} mit dem es mglich wird, in dem TableViewer des
 * {@link SichtEditorStatusleistenElemente} per Drag & Drop Statusleistenelemente in Ihrer XML-Reprsentation fallen zu
 * lassen.
 * 
 * @version $Revision: 1.1 $ / $Date: 2009/12/02 09:19:34 $ / ($Author: HofmannJ $)
 * 
 * @author HofmannJ
 * 
 */
public class TabellenFallAdapterElementStatusleiste extends ViewerDropAdapter
{
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  private ZugElementSpeicher _zugElementSpeicher;

  protected TabellenFallAdapterElementStatusleiste(TableViewer viewer)
  {
    super(viewer);
    // this._mbd = mbd;
  }

  @Override
  public boolean performDrop(Object data)
  {
    Object target = getCurrentTarget();
    _zugElementSpeicher = ZugElementSpeicher.getInstanz();

    /*
     * Ist das Ziel ein in der Liste vorhandenes Element?
     */
    if (target instanceof Element)
    {
      Element zielElement = (Element)target;
      if (data instanceof String)
      {
        _debug.fine(getClass().getName() + "::performDrop => DATA=STRING: " + (String)data);

        String eingang = (String)data;
        if (eingang.startsWith("[XML]"))
        {
          _debug.fine(getClass().getName() + "::performDrop => XML-Element aus DragElementSpeicher.");

          String id = eingang.substring(5);

          _debug.fine(getClass().getName() + "::performDrop => ID: " + id);

          Element dragElement = _zugElementSpeicher.getZugElement();
          if (dragElement.getAttributeValue(XmlBezeichnerStatus.attributId).equals(id))
          {
            _debug.fine(getClass().getName() + "::performDrop => Passendes Element im DragElementSpeicher.");

            if (zielElement != dragElement)
            {
              _debug.fine(getClass().getName() + "::performDrop => EINFGEN");

              StatusLeisteVerwaltung.getInstanz().verschiebeXmlElement(dragElement, zielElement);
              getViewer().refresh();

            }
            else
            {
              _debug.fine(getClass().getName() + "::performDrop => KANN NICHT AUF MICH SELBST EINFGEN!");
            }
          }
          else
          {
            _debug.fine(getClass().getName() + "::performDrop => Kein passendes DragElement im Speicher.");
          }
        }
        else
        {
          _debug.fine(getClass().getName() + "::performDrop => Kein XML-Element aus DragElementSpeicher");

          String id = eingang;
          try
          {
            StatusLeisteElement statusLeistenElement = StatusLeisteVerwaltung.getInstanz().getStatusLeistenElement(id);
            StatusLeisteVerwaltung.getInstanz().verschiebeXmlElement(statusLeistenElement.konvertiereInJDOMElement(),
                zielElement);
            getViewer().refresh();

          }
          catch (IllegalArgumentException iae)
          {
            System.out.println(getClass().getName() + "::performDrop => Das Statusleistenelement ist nicht existent: "
                + iae.toString());
          }

        }
      }
    }
    /*
     * Wenn nein, dann als neues Element am Ende der Liste einfgen.
     */
    else
    {
      if (data instanceof String)
      {
        String eingang = (String)data;

        if (eingang.startsWith("[XML]"))
        {
          String id = eingang.substring(5);
          Element dragElement = _zugElementSpeicher.getZugElement();
          if (dragElement.getAttributeValue(XmlBezeichnerStatus.attributId).equals(id))
          {
            _debug.fine(getClass().getName() + "::performDrop => Passendes Element im DragElementSpeicher.");

            StatusLeisteVerwaltung.getInstanz().hinzufuegenXmlElement(dragElement);
            getViewer().refresh();

          }
          else
          {
            _debug.fine(getClass().getName() + "::performDrop => Kein passendes DragElement im Speicher.");
          }

        }
        else
        {
          String id = eingang;
          try
          {
            StatusLeisteElement statusLeistenElement = StatusLeisteVerwaltung.getInstanz().getStatusLeistenElement(id);

            StatusLeisteVerwaltung.getInstanz().hinzufuegenXmlElement(statusLeistenElement.konvertiereInJDOMElement());
            getViewer().refresh();

          }
          catch (IllegalArgumentException iae)
          {
            System.out.println(getClass().getName() + "::performDrop => Das Statusleistenelement ist nicht existent: "
                + iae.toString());
          }
        }

      }

    }

    // boolean actionFlag = false;
    //
    // if (_debug)
    // {
    // System.out.println(getClass().getName()
    // + "::performDrop => Eingehende Daten Klasse: " + data.getClass());
    // }
    //
    // // ZielElement
    // Object target = getCurrentTarget();
    //
    // if (target == null)
    // target = (Element) getViewer().getInput();
    //
    // Element zielElement = (Element) target;
    //
    // if (_debug)
    // {
    // System.out.println(getClass().getName()
    // + "::performDrop => Target Name: " + zielElement.getName());
    // }
    //
    // List<Element> elemente = new LinkedList<Element>();
    //
    // if (data instanceof DescriptorElement[])
    // {
    // actionFlag = true;
    //
    // DescriptorElement[] toDrop = (DescriptorElement[]) data;
    //
    // for (int i = 0; i < toDrop.length; i++)
    // {
    // String id = toDrop[i].getActionID();
    // String actionid = toDrop[i].getActionID();
    // String name = toDrop[i].getLabelText();
    //
    // XmlElementItem elementItem = new XmlElementItem(id, actionid, name);
    //
    // Element element = elementItem.getElementItem();
    //
    // elemente.add(element);
    // }
    // }
    //
    // if (data instanceof DescriptorAction[])
    // {
    // actionFlag = true;
    //
    // DescriptorAction[] toDrop = (DescriptorAction[]) data;
    // for (int i = 0; i < toDrop.length; i++)
    // {
    // String id = toDrop[i].getActionID();
    // String actionid = toDrop[i].getActionID();
    // String name = toDrop[i].getLabelText();
    //
    // XmlElementItem elementItem = new XmlElementItem(id, actionid, name);
    //
    // Element element = elementItem.getElementItem();
    // elemente.add(element);
    // }
    // }
    //
    // if (data instanceof String)
    // {
    // actionFlag = false;
    //
    // Element dragElement = DragElementSpeicher.getInstanz().getDragElement();
    // if (dragElement != null)
    // {
    // /*
    // * Wenn das Element im DragElementSpeicher von der ID mit der ID aus dem
    // * TextTransfer bereinstimmt, dann handelt es sich um das richtige
    // * Element.
    // */
    // if (dragElement.getAttributeValue(XmlBezeichnerMenu.attId).equals(
    // (String) data))
    // {
    // // Bestimmen der Position
    // int loc = getCurrentLocation();
    //
    // // Item
    // if (dragElement.getName().equals(XmlBezeichnerMenu.elementItem))
    // {
    // Element cloneElement = (Element) dragElement.clone();
    // if (zielElement.getName().equals(XmlBezeichnerMenu.elementItem))
    // {
    // int position = _mbd
    // .bestimmeElementPositionInElternElement(zielElement);
    // if (loc == LOCATION_BEFORE)
    // {
    // System.out.println("VOR");
    // _mbd.verschiebeItem(dragElement
    // .getAttributeValue(XmlBezeichnerMenu.attId), zielElement
    // .getParentElement().getAttributeValue(XmlBezeichnerMenu.attId),
    // position);
    //
    // } else if (loc == LOCATION_AFTER)
    // {
    // _mbd.verschiebeItem(dragElement
    // .getAttributeValue(XmlBezeichnerMenu.attId), zielElement
    // .getParentElement().getAttributeValue(XmlBezeichnerMenu.attId),
    // position++);
    //
    // } else
    // {
    // // Nichts
    // }
    // }
    // if (zielElement.getName().equals(XmlBezeichnerMenu.elementMenu))
    // {
    // dragElement.detach();
    // zielElement.addContent(cloneElement);
    // }
    //
    // }
    // // Separator
    // else if
    // (dragElement.getName().equals(XmlBezeichnerMenu.elementSeparator))
    // {
    // Element cloneElement = (Element) dragElement.clone();
    // dragElement.detach();
    // zielElement.addContent(cloneElement);
    // }
    // // Men
    // else if (dragElement.getName().equals(XmlBezeichnerMenu.elementMenu))
    // {
    // if (zielElement.getName().equals(XmlBezeichnerMenu.elementSubMenu))
    // {
    // if (zielElement.getChildren().size() < 1)
    // {
    // Element cloneElement = (Element) dragElement.clone();
    // dragElement.detach();
    // zielElement.addContent(cloneElement);
    // }
    // }
    // }
    // // Submen
    // else if (dragElement.getName().equals(XmlBezeichnerMenu.elementSubMenu))
    // {
    // Element cloneElement = (Element) dragElement.clone();
    // if (zielElement.getName().equals(XmlBezeichnerMenu.elementItem)
    // || zielElement.getName().equals(XmlBezeichnerMenu.elementSubMenu)
    // || zielElement.getName().equals(XmlBezeichnerMenu.elementSeparator))
    // {
    // int position = _mbd
    // .bestimmeElementPositionInElternElement(zielElement);
    // if (loc == LOCATION_BEFORE)
    // {
    // System.out.println("VOR");
    // _mbd.verschiebeSubMenu(dragElement
    // .getAttributeValue(XmlBezeichnerMenu.attId), zielElement
    // .getParentElement().getAttributeValue(XmlBezeichnerMenu.attId),
    // position);
    //
    // } else if (loc == LOCATION_AFTER)
    // {
    // _mbd.verschiebeSubMenu(dragElement
    // .getAttributeValue(XmlBezeichnerMenu.attId), zielElement
    // .getParentElement().getAttributeValue(XmlBezeichnerMenu.attId),
    // position++);
    //
    // } else
    // {
    // // Nichts
    // }
    // }
    // if (zielElement.getName().equals(XmlBezeichnerMenu.elementMenu))
    // {
    // dragElement.detach();
    // zielElement.addContent(cloneElement);
    // }
    //
    // // ////Code zum allgemeinen anfgen an beliebige Elemente
    // // Element cloneElement = (Element) dragElement.clone();
    // // dragElement.detach();
    // // zielElement.addContent(cloneElement);
    // }
    //
    // TreeViewer viewer = (TreeViewer) getViewer();
    // viewer.refresh();
    // viewer.expandAll();
    //
    // }
    //
    // }
    //
    // }
    //
    // if (actionFlag)
    // {
    // if (_debug)
    // {
    // System.out.println(getClass().getName()
    // + "::performDrop => Listsize einzufgender Elemente: "
    // + elemente.size());
    // }
    //
    // TreeViewer viewer = (TreeViewer) getViewer();
    // Iterator<Element> it = elemente.iterator();
    // int j = 0;
    //
    // while (it.hasNext())
    // {
    // j++;
    //
    // Element addElement = it.next();
    //
    // boolean ziel = false;
    // if (zielElement.getName().equals(XmlBezeichnerMenu.elementItem))
    // {
    // ziel = true;
    // } else if
    // (zielElement.getName().equals(XmlBezeichnerMenu.elementSeparator))
    // {
    // ziel = true;
    // }
    //
    // if (target != null && ziel)
    // {
    // int loc = getCurrentLocation();
    //
    // int position = _mbd
    // .bestimmeElementPositionInElternElement(zielElement);
    //
    // zielElement = zielElement.getParentElement();
    //
    // if (loc == LOCATION_BEFORE)
    // {
    // } else if (loc == LOCATION_AFTER)
    // {
    // position++;
    // } else
    // {
    // }
    //
    // _mbd.setzeElementBei(zielElement, addElement, position);
    //
    // } else
    // {
    // zielElement.addContent(addElement);
    //
    // }
    //
    // // viewer.add(target, addElement);
    //
    // viewer.refresh();
    // viewer.expandAll();
    //
    // }
    // } else
    // {
    // System.out.println("Texttransfer");
    // }

    return true;

  }

  @Override
  public boolean validateDrop(Object target, int operation, TransferData transferType)
  {

    if (target instanceof Element)
    {
      return true;
      // if (_debug)
      // {
      // Element em = (Element) target;
      // System.out.println(getClass().getName()
      // + "::validateDrop => Target ist Gltig: " + em.getName());
      // }
      //
      // if
      // (TransferDescriptorElement.getInstanz().isSupportedType(transferType)
      // || (TransferDescriptorAction.getInstanz()
      // .isSupportedType(transferType)))
      // {
      // Element element = (Element) target;
      // boolean elementTyp = false;
      //
      // if (element.getName().equals(XmlBezeichnerMenu.elementMenu))
      // {
      // elementTyp = true;
      // } else if (element.getName().equals(XmlBezeichnerMenu.elementItem))
      // {
      // elementTyp = true;
      // } else if
      // (element.getName().equals(XmlBezeichnerMenu.elementSeparator))
      // {
      // elementTyp = true;
      // }
      //
      // // if (element.getName().equals(XmlBezeichner.elementMenu)
      // // || element.getName().equals(XmlBezeichner.elementItem))
      // if (elementTyp)
      // {
      // return true;
      // } else
      // {
      // return false;
      // }
      // } else if (TextTransfer.getInstance().isSupportedType(transferType))
      // {
      // return true;
      // } else
      // {
      // return false;
      // }

    }
    else
    {
      return true;
    }

  }
}
