/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.tabellen;

import org.jdom.Attribute;
import org.jdom.Element;

/**
 * Klasse zur Beschreibung eines regulren Ausdrucks.
 * 
 * @author HofmannJ
 * 
 */
public class ElementRegEx
{
	/**
	 * ID des Elementes
	 */
	private int _id = -1;

	/**
	 * Bezeichnung
	 */
	private String _bezeichnung = null;

	/**
	 * Regulrer Ausdruck
	 */
	private String _regEx = null;

	/**
	 * Das Element selbst
	 */
	private Element _xmlElement = null;

	/**
	 * Konstuktor, initialisiert ein {@link ElementRegEx} mit ID und Bezeichnung
	 * 
	 * @param id
	 *          ID des Elementes
	 * @param bezeichnung
	 *          Bezeichnung
	 * @param regEx
	 *          Regulrer Ausdruck
	 */
	public ElementRegEx(int id, String bezeichnung, String regEx)
	{
		initialisiere(id, bezeichnung, regEx);
	}

	/**
	 * Konstruktor, initialisiert ein {@link ElementRegEx} an Hand des JDOM-{@link Element}es
	 * 
	 * @param element
	 */
	public ElementRegEx(Element element)
	{
		initialisiere(element);
	}

	/**
	 * Fhrt die Initialisierung durch.
	 * 
	 * @param id
	 *          ID zur Initialisierung
	 * @param bezeichnung
	 *          Bezeichnung zur Initialisierung
	 * @param regEx
	 *          Regulrer Ausdruck
	 */
	public void initialisiere(int id, String bezeichnung, String regEx)
	{
		this._id = id;
		this._bezeichnung = bezeichnung;
		_xmlElement = new Element(XmlFilterVerwaltung._xmlElementAusdruck);
		Attribute attId = new Attribute(XmlFilterVerwaltung._xmlAttId, "" + id);
		_xmlElement.setAttribute(attId);
		Attribute attBez = new Attribute(XmlFilterVerwaltung._xmlAttBez,
				bezeichnung);
		_xmlElement.setAttribute(attBez);
		_regEx = regEx;
		_xmlElement.setText(regEx);
	}

	/**
	 * Initialisiert das Element an Hand eines JDOM-{@link Element}es
	 * 
	 * @param element
	 *          JDOM-{@link Element}
	 */
	public void initialisiere(Element element)
	{
		if (element.getName().equals(XmlFilterVerwaltung._xmlElementAusdruck))
		{
			_xmlElement = element;

			/*
			 * ID Holen
			 */
			try
			{
				this._id = Integer.valueOf(_xmlElement
						.getAttributeValue(XmlFilterVerwaltung._xmlAttId));
			} catch (NumberFormatException e)
			{
				e.printStackTrace();
			}

			/*
			 * Bezeichnung Holen
			 */
			this._bezeichnung = _xmlElement
					.getAttributeValue(XmlFilterVerwaltung._xmlAttBez);

			/*
			 * RegEx holen
			 */
			this._regEx = _xmlElement.getText();

		}

	}

	public Element getElement()
	{
		initialisiere(_id, _bezeichnung, _regEx);
		return _xmlElement;
	}

	/**
	 * Liefert die ID dieses Elementes
	 * 
	 * @return ID
	 */
	public int getId()
	{
		return _id;
	}

	/**
	 * Setzt die ID dieses Elementes
	 * 
	 * @param id
	 *          ID
	 */
	public void setId(int id)
	{
		this._id = id;
	}

	/**
	 * Liefert die Bezeichnung dieses Elementes.
	 * 
	 * @return Bezeichnung
	 */
	public String getBezeichnung()
	{
		return _bezeichnung;
	}

	/**
	 * Setzt die Bezeichnung dieses Elementes.
	 * 
	 * @param bezeichnung
	 *          Zu setzende Bezeichnung
	 */
	public void setBezeichnung(String bezeichnung)
	{
		this._bezeichnung = bezeichnung;
	}

	/**
	 * Liefert den regulren Ausdruck.
	 * 
	 * @return regulrer Ausdruck
	 */
	public String getRegEx()
	{
		return _regEx;
	}

	/**
	 * Setzt den regulren Ausdruck.
	 * 
	 * @param regEx
	 *          regulrer Ausdruck
	 */
	public void setRegEx(String regEx)
	{
		this._regEx = regEx;
	}

	public ElementRegEx clone()
	{
		ElementRegEx re = new ElementRegEx(_id, _bezeichnung, _regEx);
		return re;
	}

	public String toString()
	{
		String re = "";
		re = re + "  RegEx ID: " + _id + " Bezeichnung: " + _bezeichnung
				+ " RegEx: " + _regEx;
		return re;
	}

}