/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.tabellen;

import java.util.Iterator;
import java.util.LinkedList;

/**
 * Diese Klasse realisiert Spalten in Form von Elementen fr die grafische
 * Sortierung der Spalten.
 * 
 * @author HofmannJ
 * 
 */
public class SpaltenElement
{
	/**
	 * Speichert die Bezeichnung der Spalte
	 */
	private String _spaltenBezeichnung = null;

	/**
	 * Sortierung - aufsteigend oder absteigen
	 */
	private boolean _aufsteigend = true;

	public SpaltenElement(String spaltenBezeichnung)
	{
		_spaltenBezeichnung = spaltenBezeichnung;
	}

	public String toString()
	{
		return _spaltenBezeichnung;
	}

	/**
	 * Prft, ob diese Spalte aufsteigend oder absteigend sortiert werden soll.
	 * 
	 * @return Liefert <code>true</code>, wenn die Sortierung aufsteigend
	 *         erfolgen soll, sonst <code>false</code>.
	 */
	public boolean istAufsteigend()
	{
		return _aufsteigend;
	}

	/**
	 * Setzt die Art der Sortierung dieser Spalte.
	 * @param aufsteigend <code>true</code> ist aufsteigend, <code>false</code> ist absteigend.
	 */
	public void setAufsteigend(boolean aufsteigend)
	{
		this._aufsteigend = aufsteigend;
	}

	/**
	 * Wandelt eine Liste von {@link SpaltenElement}en in ein Array von
	 * {@link SpaltenElement}en.
	 * 
	 * @param sortierung
	 *          Liste von {@link SpaltenElement}en.
	 * @return Array von {@link SpaltenElement}en.
	 */
	public static SpaltenElement[] spaltenElementListe2Array(
			LinkedList<SpaltenElement> sortierung)
	{
		SpaltenElement[] re = new SpaltenElement[sortierung.size()];
		int i = 0;
		Iterator<SpaltenElement> it = sortierung.iterator();
		while (it.hasNext())
		{
			SpaltenElement wert = it.next();
			re[i] = wert;
			i++;
		}

		return re;
	}

}