package de.bsvrz.buv.rw.rw.ui.actions;

import org.eclipse.core.commands.Command;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.commands.ICommandService;

import de.bsvrz.buv.rw.rw.ui.internal.Activator;

/**
 * Action zur Installation neuer Features im Rahmenwerk.
 * 
 * @author HofmannJ
 * 
 */
public class ActionInstall implements IWorkbenchWindowActionDelegate
{
	/**
	 * ID des Commands zur Installation neuer Features.
	 */
	private static final String COMMAND_ID = "org.eclipse.equinox.p2.ui.sdk.install";

	@Override
	public void dispose()
	{
	}

	@Override
	public void init(IWorkbenchWindow window)
	{
	}

	@Override
	public void run(IAction action)
	{
		ILog log = Activator.getDefault().getLog();

		Object service = PlatformUI.getWorkbench()
				.getService(ICommandService.class);
		if (service != null)
		{
			ICommandService s = (ICommandService) service;
			Command command = s.getCommand(COMMAND_ID);
			if (command != null)
			{
				IHandler cmdHandler = command.getHandler();
				if (cmdHandler != null)
				{
					try
					{
						cmdHandler.execute(null);
					} catch (ExecutionException e)
					{
						MessageBox mb = new MessageBox(PlatformUI.getWorkbench()
								.getDisplay().getActiveShell(), SWT.OK | SWT.ICON_WARNING);
						String msg = "Das Kommando " + COMMAND_ID
								+ " konnte nicht ausgefhrt werden (ExecutionException)";
						mb.setText("Fehler beim Ausfhren des Commands " + COMMAND_ID);
						log.log(new Status(Status.WARNING, Activator.PLUGIN_ID, msg, e));
						mb.setMessage(msg);
						mb.open();

					}
				} else
				{
					MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
							.getActiveShell(), SWT.OK | SWT.ICON_WARNING);
					mb.setText("Command " + COMMAND_ID + " nicht ausfhrbar!");
					String msg = "Zum Command " + COMMAND_ID
							+ " ist kein Handler verfgbar. Ausfhrung nicht mglich.";
					mb.setMessage(msg);

					log.log(new Status(Status.WARNING, Activator.PLUGIN_ID, msg));
					mb.open();
				}
			} else
			{
				MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
						.getActiveShell(), SWT.OK | SWT.ICON_WARNING);
				mb.setText("Command " + COMMAND_ID + " nicht verfgbar!");
				String msg = "Command "
						+ COMMAND_ID
						+ " ist in der Plattform nicht verfgbar. Evtl. ist das Plugin org.eclipse.equinox.p2.ui.sdk nicht installiert worden.";
				log.log(new Status(Status.WARNING, Activator.PLUGIN_ID, msg));
				mb.setMessage(msg);
			}

		}
	}

	@Override
	public void selectionChanged(IAction action, ISelection selection)
	{
	}

}
