/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.ui.internal;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.ICoolBarManager;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.action.ToolBarContributionItem;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.actions.ContributionItemFactory;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;
import org.eclipse.ui.application.ActionBarAdvisor;
import org.eclipse.ui.application.IActionBarConfigurer;
import org.eclipse.ui.ide.IDEActionFactory;
import org.eclipse.ui.internal.dialogs.ShowViewDialog;
import org.eclipse.ui.internal.ide.IDEInternalWorkbenchImages;
import org.eclipse.ui.internal.ide.IDEWorkbenchMessages;
import org.eclipse.ui.internal.ide.IDEWorkbenchPlugin;
import org.eclipse.ui.keys.IBindingService;
import org.eclipse.ui.views.IViewDescriptor;

import de.bsvrz.buv.rw.rw.FensterSpeicher;
import de.bsvrz.buv.rw.rw.menu.ElementRegistrierung;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorAktion;
import de.bsvrz.buv.rw.rw.statusleiste.StatusLeisteZugriff;
import de.bsvrz.buv.rw.rw.ui.bildschirmfangen.BildschirmInhaltFangen;
import de.bsvrz.buv.rw.rw.ui.internal.actions.AktionAbmelden;
import de.bsvrz.buv.rw.rw.ui.internal.actions.AktionAnmelden;
import de.bsvrz.buv.rw.rw.ui.internal.actions.AktionKonfigurationsdateiAuswaehlen;

@SuppressWarnings("restriction")
public class ApplicationActionBarAdvisor extends ActionBarAdvisor
{
	private final IWorkbenchWindow window;

	// generic actions
	private IWorkbenchAction closeAction;

	private IWorkbenchAction closeOthersAction;

	private IWorkbenchAction closeAllSavedAction;

	private IWorkbenchAction saveAction;

	private IWorkbenchAction newWindowAction;

	private IWorkbenchAction newEditorAction;

	private IWorkbenchAction showHelpAction;

	private IWorkbenchAction searchHelpAction;

	private IWorkbenchAction dynamicHelpAction;

	private IWorkbenchAction aboutAction;

	private IWorkbenchAction openPreferencesAction;

	private IWorkbenchAction saveAsAction;

	private IWorkbenchAction hideShowEditorAction;

	private IWorkbenchAction savePerspectiveAction;

	private IWorkbenchAction resetPerspectiveAction;

	private IWorkbenchAction editActionSetAction;

	private IWorkbenchAction closePerspAction;

	private IWorkbenchAction lockToolBarAction;

	private IWorkbenchAction closeAllPerspsAction;

	private IWorkbenchAction showViewMenuAction;

	private IWorkbenchAction showPartPaneMenuAction;

	private IWorkbenchAction nextPartAction;

	private IWorkbenchAction prevPartAction;

	private IWorkbenchAction nextEditorAction;

	private IWorkbenchAction prevEditorAction;

	private IWorkbenchAction nextPerspectiveAction;

	private IWorkbenchAction prevPerspectiveAction;

	private IWorkbenchAction activateEditorAction;

	private IWorkbenchAction maximizePartAction;

	private IWorkbenchAction minimizePartAction;

	private IWorkbenchAction switchToEditorAction;

	private IWorkbenchAction openEditorDropDownAction;

	private IWorkbenchAction backwardHistoryAction;

	private IWorkbenchAction forwardHistoryAction;

	// generic retarget actions
	private IWorkbenchAction undoAction;

	private IWorkbenchAction redoAction;

	private IWorkbenchAction cutAction;

	private IWorkbenchAction copyAction;

	private IWorkbenchAction pasteAction;

	private IWorkbenchAction deleteAction;

	private IWorkbenchAction selectAllAction;

	private IWorkbenchAction printAction;

	private IWorkbenchAction quitAction;

	private IWorkbenchAction goIntoAction;

	private IWorkbenchAction backAction;

	private IWorkbenchAction forwardAction;

	private IWorkbenchAction upAction;

	private IWorkbenchAction nextAction;

	private IWorkbenchAction previousAction;

	// IDE-specific actions
	private IWorkbenchAction projectPropertyDialogAction;

	private IWorkbenchAction newWizardDropDownAction;

	private IWorkbenchAction importResourcesAction;

	private IWorkbenchAction exportResourcesAction;

	private IWorkbenchAction buildAllAction; // Incremental workspace build

	private IWorkbenchAction cleanAction;

	private IWorkbenchAction toggleAutoBuildAction;

	private IWorkbenchAction quickStartAction;

	private IWorkbenchAction tipsAndTricksAction;

	private IWorkbenchAction introAction;

	private IWorkbenchAction buildProjectAction;

	private IWorkbenchAction openProjectAction;

	private IWorkbenchAction closeProjectAction;

	/**
	 * Anmelde-Aktion
	 */
	final static public AktionAnmelden _aktionAnmelden = new AktionAnmelden();

	/**
	 * Abmelde-Aktion
	 */
	final static public AktionAbmelden _aktionAbmelden = new AktionAbmelden();

	/**
	 * Konfigurationsdateiauswahl-Aktion
	 */
	final static public AktionKonfigurationsdateiAuswaehlen _aktionKonfigurationsDateiAuswaehlen = new AktionKonfigurationsdateiAuswaehlen();

	/**
	 * Aktion zum Beenden des Rahmenwerkes
	 */
	private Action _exitAction = null;

	public ApplicationActionBarAdvisor(IActionBarConfigurer configurer)
	{
		super(configurer);
		window = configurer.getWindowConfigurer().getWindow();
	}

	protected void makeActions(final IWorkbenchWindow window)
	{
		FensterSpeicher.getInstanz().setWindow(window);

		/*
		 * Key Bindings
		 */
		Action actionKeyAssist = new Action()
		{
			public void run()
			{
				IWorkbench workbench = PlatformUI.getWorkbench();
				IBindingService bindingService = (IBindingService) workbench
						.getAdapter(IBindingService.class);
				bindingService.openKeyAssistDialog();
			}
		};
		actionKeyAssist.setId("DambachKeyAssist");
		actionKeyAssist.setText("&Tastenzuordnungen");
		actionKeyAssist
				.setToolTipText("ffnet den Dialog zum Bearbeiten von Tastenkombinationen");

		/*
		 * Aktion Anmelden
		 */
		AktionAnmelden aktionAnmelden = new AktionAnmelden();
		DeskriptorAktion daAktionAnmelden = new DeskriptorAktion(
				Activator.PLUGIN_ID, aktionAnmelden, false);
		try
		{
			ElementRegistrierung.getInstanz().registriereDeskriptorAktion(
					daAktionAnmelden);
		} catch (IllegalArgumentException iae)
		{
			iae.printStackTrace();
		}

		/*
		 * Aktion abmelden registrieren.
		 */
		AktionAbmelden aktionAbmelden = new AktionAbmelden();
		DeskriptorAktion daAktionAbmelden = new DeskriptorAktion(
				Activator.PLUGIN_ID, aktionAbmelden, false);
		try
		{
			ElementRegistrierung.getInstanz().registriereDeskriptorAktion(
					daAktionAbmelden);
		} catch (IllegalArgumentException iae)
		{
			iae.printStackTrace();
		}

		// ElementRegistrierung.getInstanz().registriereDeskriptorAktion(
		// new DeskriptorAktion(Activator.PLUGIN_ID, actionKeyAssist, false));

		newWizardDropDownAction = IDEActionFactory.NEW_WIZARD_DROP_DOWN
				.create(window);
		newWizardDropDownAction.setText("&Neue Wizards");
		newWizardDropDownAction
				.setToolTipText("ffnet eine Drop-Down Aktion mit den verfgbaren Assistenten.");
		register(newWizardDropDownAction);
		// ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
		// new DeskriptorAktion(Activator.PLUGIN_ID, newWizardDropDownAction,
		// true));

		importResourcesAction = ActionFactory.IMPORT.create(window);
		importResourcesAction.setText("&Import");
		importResourcesAction.setToolTipText("Importieren");
		register(importResourcesAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", importResourcesAction,
						true));

		exportResourcesAction = ActionFactory.EXPORT.create(window);
		exportResourcesAction.setText("Exp&ort");
		exportResourcesAction.setToolTipText("Exportieren");
		register(exportResourcesAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion(Activator.PLUGIN_ID, exportResourcesAction, true));

		buildAllAction = IDEActionFactory.BUILD.create(window);
		buildAllAction.setText("&Alles bauen");
		buildAllAction.setToolTipText("Alle bauen");
		register(buildAllAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", buildAllAction, true));

		cleanAction = IDEActionFactory.BUILD_CLEAN.create(window);
		cleanAction.setText("Aufrume&n");
		cleanAction.setToolTipText("Aufrumen");
		register(cleanAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", cleanAction, true));

		toggleAutoBuildAction = IDEActionFactory.BUILD_AUTOMATICALLY.create(window);
		toggleAutoBuildAction.setText("Auto&matisches Bauen");
		toggleAutoBuildAction.setToolTipText("Automatisches Bauen");
		register(toggleAutoBuildAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", toggleAutoBuildAction,
						true));

		saveAction = ActionFactory.SAVE.create(window);
		saveAction.setText("&Speichern");
		saveAction.setToolTipText("Speichern");
		register(saveAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", saveAction, true));

		saveAsAction = ActionFactory.SAVE_AS.create(window);
		saveAsAction.setText("Speichern &unter");
		saveAsAction.setToolTipText("Speichern unter");
		register(saveAsAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", saveAsAction, true));

		newWindowAction = ActionFactory.OPEN_NEW_WINDOW.create(getWindow());
		newWindowAction.setText("&Neues Fenster");
		newWindowAction.setToolTipText("Neues Fenster");
		newWindowAction.setText(IDEWorkbenchMessages.Workbench_openNewWindow);
		register(newWindowAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", newWindowAction, true));

		newEditorAction = ActionFactory.NEW_EDITOR.create(window);
		newEditorAction.setText("Neuer &Editor");
		newEditorAction.setToolTipText("Neuer Editor");
		register(newEditorAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", newEditorAction, true));

		undoAction = ActionFactory.UNDO.create(window);
		undoAction.setText("R&ckgngig");
		undoAction.setToolTipText("Rckgngig machen");
		register(undoAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", undoAction, true));

		redoAction = ActionFactory.REDO.create(window);
		redoAction.setText("Wiede&rholen");
		redoAction.setToolTipText("Wiederholen");
		register(redoAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", redoAction, true));

		cutAction = ActionFactory.CUT.create(window);
		cutAction.setText("Auss&chneiden");
		cutAction.setToolTipText("Ausschneiden");
		register(cutAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", cutAction, true));

		copyAction = ActionFactory.COPY.create(window);
		copyAction.setText("&Kopie");
		copyAction.setToolTipText("Kopieren");
		register(copyAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", copyAction, true));

		pasteAction = ActionFactory.PASTE.create(window);
		pasteAction.setText("&Einfgen");
		pasteAction.setToolTipText("Einfgen");
		register(pasteAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", pasteAction, true));

		printAction = ActionFactory.PRINT.create(window);
		printAction.setText("&Drucken");
		printAction.setToolTipText("Drucken");
		register(printAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", printAction, true));

		selectAllAction = ActionFactory.SELECT_ALL.create(window);
		selectAllAction.setText("&Alle selektieren");
		selectAllAction.setToolTipText("Alle selektieren");
		register(selectAllAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", selectAllAction, true));

		closeAction = ActionFactory.CLOSE.create(window);
		closeAction.setText("S&chliessen");
		closeAction.setToolTipText("Schliessen");
		register(closeAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", closeAction, true));

		closeOthersAction = ActionFactory.CLOSE_OTHERS.create(window);
		closeOthersAction.setText("Andere sc&hliessen");
		closeOthersAction.setToolTipText("Andere schliessen");
		register(closeOthersAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", closeOthersAction, true));

		closeAllSavedAction = ActionFactory.CLOSE_ALL_SAVED.create(window);
		closeAllSavedAction.setText("Alle gespe&icherten schliessen");
		closeAllSavedAction.setToolTipText("Alle gespeicherten schliessen");
		register(closeAllSavedAction);
		ElementRegistrierung.getInstanz()
				.registriereDeskriptorAktionGlobal(
						new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", closeAllSavedAction,
								true));

		showHelpAction = ActionFactory.HELP_CONTENTS.create(window);
		showHelpAction.setText("&Hilfe");
		showHelpAction.setToolTipText("Hilfe anzeigen");
		register(showHelpAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", showHelpAction, true));

		searchHelpAction = ActionFactory.HELP_SEARCH.create(window);
		searchHelpAction.setText("Suchen in der Hi&lfe");
		searchHelpAction.setToolTipText("In der Hilfe suchen");
		register(searchHelpAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", searchHelpAction, true));

		dynamicHelpAction = ActionFactory.DYNAMIC_HELP.create(window);
		dynamicHelpAction.setText("&Dynamische Hilfe");
		dynamicHelpAction.setToolTipText("Dynamische Hilfe");
		register(dynamicHelpAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", dynamicHelpAction, true));

		aboutAction = ActionFactory.ABOUT.create(window);
		aboutAction.setImageDescriptor(IDEInternalWorkbenchImages
				.getImageDescriptor(IDEInternalWorkbenchImages.IMG_OBJS_DEFAULT_PROD));
		aboutAction.setText("ber");
		aboutAction.setToolTipText("Liefert ber-Informationen");
		register(aboutAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", aboutAction, true));

		openPreferencesAction = ActionFactory.PREFERENCES.create(window);
		openPreferencesAction.setText("Prferenzen ffnen");
		openPreferencesAction.setToolTipText("ffnet die Prferenzen");
		register(openPreferencesAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion(Activator.PLUGIN_ID, openPreferencesAction, true));

		deleteAction = ActionFactory.DELETE.create(window);
		deleteAction.setText("L&schen");
		deleteAction.setToolTipText("Lschen");
		register(deleteAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", deleteAction, true));

		// Actions for invisible accelerators
		showViewMenuAction = ActionFactory.SHOW_VIEW_MENU.create(window);
		showViewMenuAction.setText("Sichten &Men");
		showViewMenuAction
				.setToolTipText("ffnet das Men zur Auswahl von Sichten.");
		register(showViewMenuAction);

		showPartPaneMenuAction = ActionFactory.SHOW_PART_PANE_MENU.create(window);
		register(showPartPaneMenuAction);

		nextEditorAction = ActionFactory.NEXT_EDITOR.create(window);
		nextEditorAction.setText("Nchster &Editor");
		register(nextEditorAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", nextEditorAction, true));

		prevEditorAction = ActionFactory.PREVIOUS_EDITOR.create(window);
		prevEditorAction.setText("Vorhergehen&der Editor");
		register(prevEditorAction);
		ActionFactory.linkCycleActionPair(nextEditorAction, prevEditorAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", prevEditorAction, true));

		nextPartAction = ActionFactory.NEXT_PART.create(window);
		nextPartAction.setText("Nchster &Teil");
		register(nextPartAction);
		prevPartAction = ActionFactory.PREVIOUS_PART.create(window);
		register(prevPartAction);
		ActionFactory.linkCycleActionPair(nextPartAction, prevPartAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", nextPartAction, true));

		nextPerspectiveAction = ActionFactory.NEXT_PERSPECTIVE.create(window);
		nextPerspectiveAction.setText("&Nchste Perspektive");
		nextPerspectiveAction.setToolTipText("ffnet die nchste Perspektive.");
		register(nextPerspectiveAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion(Activator.PLUGIN_ID, nextPerspectiveAction, true));

		prevPerspectiveAction = ActionFactory.PREVIOUS_PERSPECTIVE.create(window);
		prevPerspectiveAction.setText("&Vorhergehende Perspektive");
		prevPerspectiveAction
				.setToolTipText("ffnet die vorhergehende Perspektive");
		register(prevPerspectiveAction);
		ActionFactory.linkCycleActionPair(nextPerspectiveAction,
				prevPerspectiveAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion("de.bsvrz.buv.rw.rw.ui", prevPerspectiveAction,
						true));

		activateEditorAction = ActionFactory.ACTIVATE_EDITOR.create(window);
		register(activateEditorAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion(Activator.PLUGIN_ID, activateEditorAction, true));

		maximizePartAction = ActionFactory.MAXIMIZE.create(window);
		maximizePartAction.setText("M&aximieren");
		register(maximizePartAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion(Activator.PLUGIN_ID, maximizePartAction, true));

		minimizePartAction = ActionFactory.MINIMIZE.create(window);
		minimizePartAction.setText("M&inimieren");
		register(minimizePartAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion(Activator.PLUGIN_ID, minimizePartAction, true));

		switchToEditorAction = ActionFactory.SHOW_OPEN_EDITORS.create(window);
		switchToEditorAction.setText("Geffnete Editoren anzeigen");
		register(switchToEditorAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion(Activator.PLUGIN_ID, switchToEditorAction, true));

		openEditorDropDownAction = ActionFactory.SHOW_WORKBOOK_EDITORS
				.create(window);
		openEditorDropDownAction.setText("Editor Schnellu&mschaltung");
		register(openEditorDropDownAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion(Activator.PLUGIN_ID, openEditorDropDownAction,
						true));

		hideShowEditorAction = ActionFactory.SHOW_EDITOR.create(window);
		hideShowEditorAction.setText("Editoren &verstecken");
		register(hideShowEditorAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion(Activator.PLUGIN_ID, hideShowEditorAction, true));

		savePerspectiveAction = ActionFactory.SAVE_PERSPECTIVE.create(window);
		savePerspectiveAction.setText("Perspektive s&peichern");
		savePerspectiveAction.setToolTipText("Speichert die aktuelle Perspektive");
		register(savePerspectiveAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion(Activator.PLUGIN_ID, savePerspectiveAction, true));

		editActionSetAction = ActionFactory.EDIT_ACTION_SETS.create(window);
		register(editActionSetAction);

		lockToolBarAction = ActionFactory.LOCK_TOOL_BAR.create(window);
		register(lockToolBarAction);

		resetPerspectiveAction = ActionFactory.RESET_PERSPECTIVE.create(window);
		resetPerspectiveAction.setText("Perspektive zurckset&zen");
		resetPerspectiveAction.setToolTipText("Setzt die Perspektive zurck");
		register(resetPerspectiveAction);
		ElementRegistrierung.getInstanz()
				.registriereDeskriptorAktionGlobal(
						new DeskriptorAktion(Activator.PLUGIN_ID, resetPerspectiveAction,
								true));

		closePerspAction = ActionFactory.CLOSE_PERSPECTIVE.create(window);
		closePerspAction.setText("Perspektive schl&iessen");
		closePerspAction.setText("Schliesst die Perspektive");
		register(closePerspAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion(Activator.PLUGIN_ID, closePerspAction, true));

		closeAllPerspsAction = ActionFactory.CLOSE_ALL_PERSPECTIVES.create(window);
		closeAllPerspsAction.setText("Alle Perspekti&ven schliessen");
		closeAllPerspsAction.setToolTipText("Schliesst alle Perspektiven");
		register(closeAllPerspsAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion(Activator.PLUGIN_ID, closeAllPerspsAction, true));

		forwardHistoryAction = ActionFactory.FORWARD_HISTORY.create(window);
		register(forwardHistoryAction);

		backwardHistoryAction = ActionFactory.BACKWARD_HISTORY.create(window);
		register(backwardHistoryAction);

		quitAction = ActionFactory.QUIT.create(window);
		quitAction.setText("&Beenden");
		register(quitAction);
		// ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(new
		// DeskriptorAktion(Activator.PLUGIN_ID, quitAction, true));

		/*
		 * try { ResourcesPlugin.getWorkspace().save(true, null); } catch
		 * (CoreException e1) { e1.printStackTrace(); }
		 */

		_exitAction = new Action()
		{
			public void run()
			{

				quitAction.run(); // Wird vom BeendenDialog bernommen

			}
		};
		_exitAction.setText("&Beenden");
		_exitAction.setToolTipText("Beendet das Rahmenwerk");
		_exitAction.setId("exitAction");
		register(_exitAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion(Activator.PLUGIN_ID, _exitAction, true));

		goIntoAction = ActionFactory.GO_INTO.create(window);
		register(goIntoAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion(Activator.PLUGIN_ID, goIntoAction, true));

		backAction = ActionFactory.BACK.create(window);
		backAction.setText("Zur&ck");
		register(backAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion(Activator.PLUGIN_ID, backAction, true));

		forwardAction = ActionFactory.FORWARD.create(window);
		forwardAction.setText("Vor&wrts");
		register(forwardAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion(Activator.PLUGIN_ID, forwardAction, true));

		upAction = ActionFactory.UP.create(window);
		upAction.setText("Hoc&h");
		register(upAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion(Activator.PLUGIN_ID, upAction, true));

		nextAction = ActionFactory.NEXT.create(window);
		nextAction.setText("N&chter/s");
		nextAction.setImageDescriptor(IDEInternalWorkbenchImages
				.getImageDescriptor(IDEInternalWorkbenchImages.IMG_ETOOL_NEXT_NAV));
		register(nextAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion(Activator.PLUGIN_ID, nextAction, true));

		previousAction = ActionFactory.PREVIOUS.create(window);
		previousAction.setText("Vorher&gehendes");
		previousAction.setImageDescriptor(IDEInternalWorkbenchImages
				.getImageDescriptor(IDEInternalWorkbenchImages.IMG_ETOOL_PREVIOUS_NAV));
		register(previousAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion(Activator.PLUGIN_ID, previousAction, true));

		buildProjectAction = IDEActionFactory.BUILD_PROJECT.create(window);
		buildProjectAction.setText("&Baue Projekt");
		register(buildProjectAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion(Activator.PLUGIN_ID, buildProjectAction, true));

		openProjectAction = IDEActionFactory.OPEN_PROJECT.create(window);
		openProjectAction.setText("ffne Projekt");
		openProjectAction.setToolTipText("&ffnet ein Projekt.");
		register(openProjectAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion(Activator.PLUGIN_ID, openProjectAction, true));

		closeProjectAction = IDEActionFactory.CLOSE_PROJECT.create(window);
		closeProjectAction.setText("S&chliee Projekt");
		closeProjectAction.setToolTipText("Schliet das Projekt");
		register(closeProjectAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion(Activator.PLUGIN_ID, closeProjectAction, true));

		projectPropertyDialogAction = IDEActionFactory.OPEN_PROJECT_PROPERTIES
				.create(window);
		projectPropertyDialogAction.setText("Projekt Einstellun&gen");
		register(projectPropertyDialogAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion(Activator.PLUGIN_ID, projectPropertyDialogAction,
						true));

		if (window.getWorkbench().getIntroManager().hasIntro())
		{
			introAction = ActionFactory.INTRO.create(window);
			introAction.setText("Einf&hrung");
			introAction.setToolTipText("ffnet die Einfhrung");
			register(introAction);
			ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
					new DeskriptorAktion(Activator.PLUGIN_ID, introAction, true));
		}

		// searchComboItem = ContributionItemFactory.HELP_SEARCH.create(window);

		/*
		 * Sonstiges
		 */
		// Hardcopy
		Action aktionScreenShot = new Action()
		{
			public void run()
			{
				@SuppressWarnings("unused")
				BildschirmInhaltFangen bif = new BildschirmInhaltFangen();
			}
		};
		aktionScreenShot.setText("Hardcopy");
		aktionScreenShot.setToolTipText("Erzeugt eine Hardcopy des Bildschirms");
		aktionScreenShot.setId("action.de.bsvrz.buv.rw.rw.ui.hardcopy");
		DeskriptorAktion des = new DeskriptorAktion(Activator.PLUGIN_ID,
				aktionScreenShot, false);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktion(des);

		/*
		 * Perspektiven Auswahl
		 */
		IWorkbenchAction openPerspectiveAction = ActionFactory.OPEN_PERSPECTIVE_DIALOG
				.create(window);
		openPerspectiveAction.setText("Perspektive &ffnen");
		openPerspectiveAction
				.setToolTipText("ffnet eine Liste der verfgbaren Perspektiven");
		register(openPerspectiveAction);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktionGlobal(
				new DeskriptorAktion(Activator.PLUGIN_ID, openPerspectiveAction, true));

		/*
		 * View Auswahl
		 */
		Action aktionViewsOeffnen = new Action()
		{
			public void run()
			{
				ShowViewDialog svd = new ShowViewDialog(PlatformUI.getWorkbench()
						.getActiveWorkbenchWindow(), PlatformUI.getWorkbench()
						.getViewRegistry());

				int open = svd.open();

				if (open == Window.OK)
				{
					IViewDescriptor[] selection = svd.getSelection();
					if ((selection != null) && (selection.length > 0))
					{
						IWorkbenchWindow activeWorkbenchWindow = PlatformUI.getWorkbench()
								.getActiveWorkbenchWindow();

						for (IViewDescriptor iViewDescriptor : selection)
						{
							if (iViewDescriptor != null)
							{
								try
								{
									activeWorkbenchWindow.getActivePage().showView(
											iViewDescriptor.getId());
									IViewReference viewReference = activeWorkbenchWindow
											.getActivePage().findViewReference(
													iViewDescriptor.getId());
									IViewPart view = viewReference.getView(true);
									activeWorkbenchWindow.getActivePage().activate(view);
								} catch (PartInitException e)
								{
									e.printStackTrace();
								}
							}
						}

					}
				}

			}
		};
		aktionViewsOeffnen.setText("Sichten ffnen JH");
		aktionViewsOeffnen
				.setToolTipText("ffnet eine Liste der verfgbaren Sichten.");
		aktionViewsOeffnen.setId(Activator.PLUGIN_ID + ".sichtenOeffnen");
		DeskriptorAktion da = new DeskriptorAktion(Activator.PLUGIN_ID,
				aktionViewsOeffnen, false);
		ElementRegistrierung.getInstanz().registriereDeskriptorAktion(da);

	}

	protected void fillMenuBar(IMenuManager menuBar)
	{
		menuBar.add(createFileMenu());
		menuBar.add(createEditMenu());
		// menuBar.add(createDarstellungMenu());
		// menuBar.add(createBeeinflussungMenu());
		// menuBar.add(createSimulationMenu());
		// menuBar.add(createBetriebMenu());
		// menuBar.add(createParamterMenu());
		// menuBar.add(createProtokolleMenu());
		// menuBar.add(createSystemMenu());
		menuBar.add(createObjekttypenMenu());
		menuBar.add(createWindowMenu());
		// menuBar.add(createHelpMenu());

		/*
		 * 2.test Testhilfemen
		 */
		// MenuManager helpMenu = new MenuManager("&Help",
		// IWorkbenchActionConstants.M_HELP);
		MenuManager helpMenu = new MenuManager("&Hilfe",
				IWorkbenchActionConstants.M_HELP);
		menuBar.add(helpMenu);
		helpMenu.add(aboutAction);
		helpMenu.add(showHelpAction); // NEW
		helpMenu.add(searchHelpAction); // NEW
		helpMenu.add(dynamicHelpAction); // NEW

		/*
		 * Rausziehen des Menumanagers
		 */
		// de.bsvrz.buv.rw.basislib.alt.menu.zugriff.HauptMenuSymbolZugriff m =
		// de.bsvrz.buv.rw.basislib.alt.menu.zugriff.HauptMenuSymbolZugriff.getInstance();
		// m.setMenuManager(menuBar);
		ElementRegistrierung er = ElementRegistrierung.getInstanz();
		er.setMenuManager(menuBar);

	}

	protected void fillCoolBar(ICoolBarManager coolBar)
	{
		IToolBarManager toolbar = new ToolBarManager(SWT.FLAT | SWT.RIGHT);
		coolBar.add(new ToolBarContributionItem(toolbar, "main"));
		toolbar.add(saveAction);
		toolbar.add(copyAction);

		/*
		 * if (testAction.getStyle() == SWT.DROP_DOWN) {
		 * testAction.setMenuCreator(creator); } else { System.err.println("FEHLER -
		 * Action ist nicht vom Stil SWT.DROP_DOWN"); }
		 */

		IToolBarManager toolbar2 = new ToolBarManager(SWT.FLAT | SWT.RIGHT);
		coolBar.add(new ToolBarContributionItem(toolbar2, "main2"));
		toolbar2.add(copyAction);
		// toolbar2.add(testAction);

		ElementRegistrierung er = ElementRegistrierung.getInstanz();
		er.setStatusLeisteManager(coolBar);

		// toolbar.add(openViewAction);
		// toolbar.add(messagePopupAction);
	}

	/**
	 * Gibt die Statusleiste an {@link StatusLeisteZugriff} weiter.
	 */
	protected void fillStatusLine(final IStatusLineManager statusLine)
	{
		StatusLeisteZugriff.getInstanz().setStatusZeilenManager(statusLine);
	}

	/**
	 * Creates and returns the Window menu.
	 */
	private MenuManager createWindowMenu()
	{
		// MenuManager menu = new MenuManager(
		// IDEWorkbenchMessages.Workbench_window,
		// IWorkbenchActionConstants.M_WINDOW);

		MenuManager menu = new MenuManager("Fenster",
				IWorkbenchActionConstants.M_WINDOW);

		menu.add(newWindowAction);
		menu.add(newEditorAction);
		menu.add(new Separator());
		addPerspectiveActions(menu);
		menu.add(new Separator());
		addKeyboardShortcuts(menu);
		menu.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
		menu.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS + "end")); //$NON-NLS-1$
		menu.add(openPreferencesAction);

		menu.add(ContributionItemFactory.OPEN_WINDOWS.create(getWindow()));
		return menu;
	}

	/**
	 * Adds the keyboard navigation submenu to the specified menu.
	 */
	private void addKeyboardShortcuts(MenuManager menu)
	{
		MenuManager subMenu = new MenuManager(
				IDEWorkbenchMessages.Workbench_shortcuts, "shortcuts"); //$NON-NLS-1$
		menu.add(subMenu);
		subMenu.add(showPartPaneMenuAction);
		subMenu.add(showViewMenuAction);
		subMenu.add(new Separator());
		subMenu.add(maximizePartAction);
		subMenu.add(minimizePartAction);
		subMenu.add(new Separator());
		subMenu.add(activateEditorAction);
		subMenu.add(nextEditorAction);
		subMenu.add(prevEditorAction);
		subMenu.add(switchToEditorAction);
		subMenu.add(openEditorDropDownAction);
		subMenu.add(new Separator());
		subMenu.add(nextPartAction);
		subMenu.add(prevPartAction);
		subMenu.add(new Separator());
		subMenu.add(nextPerspectiveAction);
		subMenu.add(prevPerspectiveAction);
	}

	/**
	 * Adds the perspective actions to the specified menu.
	 */
	private void addPerspectiveActions(MenuManager menu)
	{
		{
			String openText = IDEWorkbenchMessages.Workbench_openPerspective;
			MenuManager changePerspMenuMgr = new MenuManager(openText,
					"openPerspective"); //$NON-NLS-1$
			IContributionItem changePerspMenuItem = ContributionItemFactory.PERSPECTIVES_SHORTLIST
					.create(getWindow());
			changePerspMenuMgr.add(changePerspMenuItem);
			menu.add(changePerspMenuMgr);
		}
		{
			MenuManager showViewMenuMgr = new MenuManager(
					IDEWorkbenchMessages.Workbench_showView, "showView"); //$NON-NLS-1$
			IContributionItem showViewMenu = ContributionItemFactory.VIEWS_SHORTLIST
					.create(getWindow());
			showViewMenuMgr.add(showViewMenu);
			menu.add(showViewMenuMgr);
		}
		menu.add(new Separator());
		menu.add(editActionSetAction);
		menu.add(savePerspectiveAction);
		menu.add(resetPerspectiveAction);
		menu.add(closePerspAction);
		menu.add(closeAllPerspsAction);
	}

	private IContributionItem createObjekttypenMenu()
	{
		MenuManager menu = new MenuManager("Objektt&ypen",
				IRWMenuConstants.MENU_OBJEKTTYPEN);
		menu.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		return menu;
	}

	@SuppressWarnings("unused")
	private IContributionItem createSystemMenu()
	{
		MenuManager menu = new MenuManager("&System", IRWMenuConstants.MENU_SYSTEM);
		menu.add(new GroupMarker(IRWMenuConstants.BENUTZERVERWALTUNG));
		menu.add(new GroupMarker(IRWMenuConstants.VERKEHRSRECHNER));
		menu.add(new GroupMarker(IRWMenuConstants.SYSTEMKALENDER));
		menu.add(new GroupMarker(IRWMenuConstants.EREIGNISKALENDER));
		menu.add(new GroupMarker(IRWMenuConstants.OPTIONEN));
		menu.add(openPreferencesAction);
		menu.add(new GroupMarker(IRWMenuConstants.DATENFLUSSUEBERWACHUNG));
		return menu;
	}

	@SuppressWarnings("unused")
	private IContributionItem createParamterMenu()
	{
		MenuManager menu = new MenuManager("Param&eter",
				IRWMenuConstants.MENU_PARAMETER);
		menu.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		return menu;
	}

	@SuppressWarnings("unused")
	private IContributionItem createProtokolleMenu()
	{
		MenuManager menu = new MenuManager("&Protokolle",
				IRWMenuConstants.MENU_PROTOKOLLE);
		menu.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		return menu;
	}

	@SuppressWarnings("unused")
	private IContributionItem createBetriebMenu()
	{
		MenuManager menu = new MenuManager("Be&trieb",
				IRWMenuConstants.MENU_BETRIEB);
		menu.add(new GroupMarker(IRWMenuConstants.BETRIEBSMELDUNGEN));
		menu.add(new GroupMarker(IRWMenuConstants.FEHLERMELDUNGEN));
		menu.add(new GroupMarker(IRWMenuConstants.LAMPENTAUSCH));
		menu.add(new GroupMarker(IRWMenuConstants.ARCHIVSYSTEM));
		menu.add(new GroupMarker(IRWMenuConstants.WETTERMELDUNGEN));
		return menu;
	}

	@SuppressWarnings("unused")
	private IContributionItem createSimulationMenu()
	{
		MenuManager menu = new MenuManager("&Simulation",
				IRWMenuConstants.MENU_SIMULATION);
		menu.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		return menu;
	}

	@SuppressWarnings("unused")
	private IContributionItem createBeeinflussungMenu()
	{
		MenuManager menu = new MenuManager("&Beeinflussung",
				IRWMenuConstants.MENU_BEEINFLUSSUNG);
		menu.add(new GroupMarker(IRWMenuConstants.RDS_MELDUNGSEINGABE));
		menu.add(new GroupMarker(IRWMenuConstants.BAUSTELLEN_UNFALLEINGABE));
		menu.add(new GroupMarker(IRWMenuConstants.SONDERPROGRAMME));
		menu.add(new GroupMarker(IRWMenuConstants.MANUELLE_SCHALTUNGEN));
		menu.add(new GroupMarker(IRWMenuConstants.BETRIEBSART));
		return menu;
	}

	@SuppressWarnings("unused")
	private IContributionItem createDarstellungMenu()
	{
		MenuManager menu = new MenuManager("D&arstellung",
				IRWMenuConstants.MENU_DARSTELLUNG);
		menu.add(new GroupMarker(IRWMenuConstants.NETZANSICHT));
		menu.add(new GroupMarker(IRWMenuConstants.SBA));
		menu.add(new GroupMarker(IRWMenuConstants.NBA));
		menu.add(new GroupMarker(IRWMenuConstants.STRECKENPROFIL));
		menu.add(new GroupMarker(IRWMenuConstants.ANLAGENSTATUS));
		menu.add(new GroupMarker(IRWMenuConstants.RDS_TMC));
		return menu;
	}

	/**
	 * Creates and returns the File menu.
	 */
	private MenuManager createFileMenu()
	{
		// MenuManager menu = new MenuManager(IDEWorkbenchMessages.Workbench_file,
		// IWorkbenchActionConstants.M_FILE);
		MenuManager menu = new MenuManager("&Datei",
				IWorkbenchActionConstants.M_FILE);
		menu.add(new GroupMarker(IWorkbenchActionConstants.FILE_START));
		menu.add(new GroupMarker(IWorkbenchActionConstants.NEW_EXT));
		menu.add(new Separator());

		menu.add(closeAction);
		// menu.add(closeAllSavedAction);
		menu.add(new GroupMarker(IWorkbenchActionConstants.CLOSE_EXT));
		menu.add(new Separator());
		menu.add(saveAction);
		menu.add(saveAsAction);
		menu.add(new Separator());
		menu.add(new GroupMarker(IWorkbenchActionConstants.SAVE_EXT));
		menu.add(new Separator());
		menu.add(exportResourcesAction);
		menu.add(new GroupMarker(IWorkbenchActionConstants.IMPORT_EXT));
		menu.add(new Separator());
		menu.add(new GroupMarker("printsettings.ext"));
		menu.add(printAction);
		menu.add(new GroupMarker(IWorkbenchActionConstants.PRINT_EXT));
		menu.add(new Separator());
		menu.add(new GroupMarker(IWorkbenchActionConstants.OPEN_EXT));
		menu.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));

		menu.add(new GroupMarker(IWorkbenchActionConstants.MRU));
		menu.add(_aktionAnmelden);
		menu.add(_aktionAbmelden);
		menu.add(new Separator());
		// menu.add(quitAction);
		menu.add(_exitAction);
		menu.add(new GroupMarker(IWorkbenchActionConstants.FILE_END));
		return menu;
	}

	/**
	 * Creates and returns the Edit menu.
	 */
	private MenuManager createEditMenu()
	{
		// MenuManager menu = new MenuManager(IDEWorkbenchMessages.Workbench_edit,
		// IWorkbenchActionConstants.M_EDIT);
		MenuManager menu = new MenuManager("&Bearbeiten",
				IWorkbenchActionConstants.M_EDIT);
		menu.add(new GroupMarker(IWorkbenchActionConstants.EDIT_START));

		menu.add(undoAction);
		menu.add(redoAction);
		menu.add(new GroupMarker(IWorkbenchActionConstants.UNDO_EXT));
		menu.add(new Separator());

		menu.add(cutAction);
		menu.add(copyAction);
		menu.add(pasteAction);
		menu.add(new GroupMarker(IWorkbenchActionConstants.CUT_EXT));
		menu.add(new Separator());

		menu.add(deleteAction);
		menu.add(selectAllAction);

		menu.add(new Separator());
		menu.add(new GroupMarker(IWorkbenchActionConstants.FIND_EXT));
		menu.add(new Separator());

		menu.add(new GroupMarker(IWorkbenchActionConstants.ADD_EXT));

		menu.add(new GroupMarker(IWorkbenchActionConstants.EDIT_END));
		menu.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
		return menu;
	}

	/**
	 * Adds a <code>GroupMarker</code> or <code>Separator</code> to a menu. The
	 * test for whether a separator should be added is done by checking for the
	 * existence of a preference matching the string useSeparator.MENUID.GROUPID
	 * that is set to <code>true</code>.
	 * 
	 * @param menu
	 *          the menu to add to
	 * @param groupId
	 *          the group id for the added separator or group marker
	 */
	private void addSeparatorOrGroupMarker(MenuManager menu, String groupId)
	{
		String prefId = "useSeparator." + menu.getId() + "." + groupId; //$NON-NLS-1$ //$NON-NLS-2$
		boolean addExtraSeparators = IDEWorkbenchPlugin.getDefault()
				.getPreferenceStore().getBoolean(prefId);
		if (addExtraSeparators)
		{
			menu.add(new Separator(groupId));
		} else
		{
			menu.add(new GroupMarker(groupId));
		}
	}

	/**
	 * Creates and returns the Help menu.
	 */
	@SuppressWarnings( { "unused" })
	private MenuManager createHelpMenu()
	{
		MenuManager menu = new MenuManager(IDEWorkbenchMessages.Workbench_help,
				IWorkbenchActionConstants.M_HELP);

		addSeparatorOrGroupMarker(menu, "group.intro"); //$NON-NLS-1$
		// See if a welcome or intro page is specified
		if (introAction != null)
		{
			menu.add(introAction);
		} else if (quickStartAction != null)
		{
			menu.add(quickStartAction);
		}
		menu.add(new GroupMarker("group.intro.ext")); //$NON-NLS-1$
		addSeparatorOrGroupMarker(menu, "group.main"); //$NON-NLS-1$
		menu.add(showHelpAction);
		menu.add(searchHelpAction);
		menu.add(dynamicHelpAction);
		addSeparatorOrGroupMarker(menu, "group.assist"); //$NON-NLS-1$
		// See if a tips and tricks page is specified
		if (tipsAndTricksAction != null)
		{
			menu.add(tipsAndTricksAction);
		}
		// HELP_START should really be the first item, but it was after
		// quickStartAction and tipsAndTricksAction in 2.1.
		menu.add(new GroupMarker(IWorkbenchActionConstants.HELP_START));
		menu.add(new GroupMarker("group.main.ext")); //$NON-NLS-1$
		addSeparatorOrGroupMarker(menu, "group.tutorials"); //$NON-NLS-1$
		addSeparatorOrGroupMarker(menu, "group.tools"); //$NON-NLS-1$
		addSeparatorOrGroupMarker(menu, "group.updates"); //$NON-NLS-1$
		menu.add(new GroupMarker(IWorkbenchActionConstants.HELP_END));
		addSeparatorOrGroupMarker(menu, IWorkbenchActionConstants.MB_ADDITIONS);
		// about should always be at the bottom
		menu.add(new Separator("group.about")); //$NON-NLS-1$
		menu.add(aboutAction);
		menu.add(new GroupMarker("group.about.ext")); //$NON-NLS-1$

		return menu;
	}

	/**
	 * Returns the window to which this action builder is contributing.
	 */
	private IWorkbenchWindow getWindow()
	{
		return window;
	}

}
