/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.ui.internal.ende;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.rw.Activator;

/**
 * Liefert einen Splashscreen, der beim Beenden des Rahmenwerkes dargestellt
 * wird und nach dem Beenden verschwindet.
 * 
 * @author HofmannJ
 * 
 */
public class BeendenDlg implements Runnable
{
	/**
	 * Shell des Splash-Screens
	 */
	private Shell _splash = null;

	/**
	 * Display fr den Splash-Screen
	 */
	private Display _display = null;

	/**
	 * Laufe-Flag fr den Thread
	 */
	private boolean _laufe = false;

	/**
	 * Bild, welches am Ende des Rahmenwerkes dargestellt werden soll.
	 */
	private Image _img = null;

	/**
	 * Konstruktor. Erzeugt eine neue Instanz des End-Splashscreens und zeigt
	 * diesen an.
	 */
	public BeendenDlg()
	{
		_img = Activator.getImageDescriptor("icons/splash/splash_ende.png")
				.createImage();

		oeffnen();
	}


	/**
	 * ffnet den Splashscreen.
	 * 
	 */
	private void oeffnen()
	{
//Auskommentiert, damit der Endsplashscreen auch unter Linux kommt		
//		_display = new Display();
//		erzeugeShell();
//		Rectangle splashRect = _splash.getBounds();
//		Rectangle displayRect = _display.getBounds();
//		int x = (displayRect.width - splashRect.width) / 2;
//		int y = (displayRect.height - splashRect.height) / 2;
//		_splash.setLocation(x, y);
//
//		_splash.open();
//
//		while (!_splash.isDisposed())
//		{
//			if (!_display.readAndDispatch())
//				_display.sleep();
//		}
//
//		_display.dispose();
		
		_display = PlatformUI.getWorkbench().getDisplay();
		erzeugeShell();
		Rectangle splashRect = _splash.getBounds();
		Rectangle displayRect = _display.getBounds();
		int x = (displayRect.width - splashRect.width) / 2;
		int y = (displayRect.height - splashRect.height) / 2;
		_splash.setLocation(x, y);

		_splash.open();

//		while (!_splash.isDisposed())
//		{
//			if (!_display.readAndDispatch())
//				_display.sleep();
//		}
//
//		_display.dispose();
	}

	/**
	 * Initialisierung der Splash-Shell
	 */
	private void erzeugeShell()
	{
		/*
		 * Alternativen: entweder SWT.ON_TOP oder SWT.MODELESS
		 */
		_splash = new Shell(_display, SWT.TITLE | SWT.RESIZE);

		if (_img != null)
		{
			_splash.setSize(_img.getBounds().width + 10, _img.getBounds().height + 30);
		}
		else
		{
			_splash.setSize(new Point(700, 219));
		}

		_splash.setLayout(new GridLayout());

		if (_img != null)
		{
			_splash.setBackgroundImage(_img);
		}
	}

	@Override
	public void run()
	{
		while (_laufe)
		{
			oeffnen();
		}
	}

}
