/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.menu.xml.datenstruktur;

import static org.junit.Assert.assertEquals;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringReader;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 * @author HofmannJ
 * 
 */
public class MenuSymbolLeisteDatenTest
{
	private static final String _pfadTestDatei = "/C:/testdatei.txt";

	private static String _xmlStruktur = null;

	/**
	 * Speichert die zu prfende Struktur
	 */
	private static MenuSymbolLeisteDaten _struktur = null;

	/**
	 * Text-XML-Struktur als JDOM-{@link Element}.
	 */
	private static Element _elementWurzel;

	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception
	{
		_xmlStruktur = "";
		_xmlStruktur = _xmlStruktur
				+ "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n";
		_xmlStruktur = _xmlStruktur + "<menuleiste>\n";
		_xmlStruktur = _xmlStruktur
				+ "\t<menu id=\"hauptmenue.1\" art=\"Menleiste\">\n";
		_xmlStruktur = _xmlStruktur
				+ "\t\t<submenu id=\"sub.1\" name=\"&amp;Datei\" art=\"Menleiste\">\n";
		_xmlStruktur = _xmlStruktur
				+ "\t\t\t<menu id=\"sub.1.menu.1\" art=\"Menleiste\">\n";
		_xmlStruktur = _xmlStruktur
				+ "\t\t\t\t<item id=\"exitAction\" actionId=\"exitAction\" name=\"&amp;Beenden\" art=\"Menleiste\" />\n";
		_xmlStruktur = _xmlStruktur + "\t\t\t</menu>\n";
		_xmlStruktur = _xmlStruktur + "\t\t</submenu>\n";
		_xmlStruktur = _xmlStruktur
				+ "\t\t<submenu id=\"sub.2\" name=\"&amp;Hilfe\">\n";
		_xmlStruktur = _xmlStruktur
				+ "\t\t\t<menu id=\"sub.2.menu.1\" art=\"Menleiste\">\n";
		_xmlStruktur = _xmlStruktur
				+ "\t\t\t\t<item id=\"helpContents\" actionId=\"helpContents\" name=\"&amp;Hilfe\" art=\"Menleiste\" />\n";
		_xmlStruktur = _xmlStruktur
				+ "\t\t\t\t<item id=\"about\" actionId=\"about\" name=\"ber\" />\n";
		_xmlStruktur = _xmlStruktur + "\t\t\t</menu>\n";
		_xmlStruktur = _xmlStruktur + "\t\t</submenu>\n";
		_xmlStruktur = _xmlStruktur + "\t</menu>\n";
		_xmlStruktur = _xmlStruktur + "</menuleiste>\n\n";

		System.out.println("XML fr Test:\n" + _xmlStruktur);

		/*
		 * Erzeugen eine JDOM-Struktur zum Testen
		 */
		StringReader reader = new StringReader(_xmlStruktur);
		SAXBuilder builder = new SAXBuilder(false);
		Document doc = builder.build(reader);
		_elementWurzel = doc.getRootElement();

	}

	/**
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception
	{
	}

	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception
	{
		_struktur = new MenuSymbolLeisteDaten();

	}

	/**
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception
	{
		/*
		 * Lschen einer evtl. angelegten Testdatei, sofern diese existiert
		 */
		File file = new File(_pfadTestDatei);
		if (file.exists())
		{
			if (file.isFile())
			{
				file.delete();
			}
		}

	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten#pushAktuellerZustandUndoStack()}
	 * .
	 */
	@SuppressWarnings("unchecked")
	@Test
	public void testPushAktuellerZustandUndoStack()
	{
		/*
		 * Struktur einlesen
		 */
		_struktur.einlesenXmlAusString(_xmlStruktur);

		/*
		 * Aufbau merken und auf den UndoStack legen
		 */
		String alterAufbau = _struktur.ausgabeXmlFormatiert();
		_struktur.pushAktuellerZustandUndoStack();

		/*
		 * Aufbauen einer Liste von zu lschenden Elementen
		 */
		LinkedList<Element> loeschListe = new LinkedList<Element>();
		Iterator itLoeschelementSuche = _struktur.getWurzel().getChildren()
				.iterator();
		while (itLoeschelementSuche.hasNext())
		{
			Object object = itLoeschelementSuche.next();
			if (object instanceof Element)
			{
				Element loeschElement = (Element) object;
				loeschListe.add(loeschElement);
			}
		}

		/*
		 * Lschen der in der Lschliste gespeicherten Elemente
		 */
		Iterator<Element> it = loeschListe.iterator();
		while (it.hasNext())
		{
			Element loeschElement = it.next();
			loeschElement.detach();
		}

		// /*
		// * Neuen Aufbau merken
		// */
		// String neuerAufbau = _struktur.ausgabeXmlFormatiert();

		/*
		 * Alten Aufbau vom Undo-Stack holen
		 */
		MenuSymbolLeisteDaten pop = _struktur.getUndoStack().pop(_struktur);
		String nachUndo = pop.ausgabeXmlFormatiert();

		/*
		 * Vergleich zwischen altem Aufbau und Undo-stacl
		 */
		assertEquals(alterAufbau, nachUndo);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten#einlesenXmlAusDatei(java.lang.String)}
	 * .
	 */
	@Test
	public void testEinlesenXmlAusDatei()
	{
		_struktur.einlesenXmlAusString(_xmlStruktur);
		LinkedList<Element> listeErwartet = rekursivElementKindListeErstellen(_struktur
				.getWurzel());
		System.out.println("SIZE: " + listeErwartet.size());

		/*
		 * Schreiben der Struktur in eine Datei
		 */
		try
		{
			FileWriter fw = new FileWriter(_pfadTestDatei);
			BufferedWriter bw = new BufferedWriter(fw);

			bw.write(_xmlStruktur);

			bw.close();
			fw.close();
		} catch (IOException e)
		{
			e.printStackTrace();
		}

		Element elementWurzel = null;
		/*
		 * Einlesen in eine JDOM-Struktur
		 */
		File file = new File(_pfadTestDatei);
		if (file.exists())
		{
			try
			{
				SAXBuilder builder = new SAXBuilder(false);
				Document doc = builder.build(file);
				elementWurzel = doc.getRootElement();
			} catch (JDOMException e)
			{
				e.printStackTrace();
			} catch (IOException e)
			{
				e.printStackTrace();
			}

		} else
		{
			throw new IllegalArgumentException("Die Datei " + _pfadTestDatei
					+ " existiert nicht!");
		}

		LinkedList<Element> listeAktuell = rekursivElementKindListeErstellen(elementWurzel);

		assertEquals(true, vergleicheElementListen(listeErwartet, listeAktuell));
	}

	/**
	 * Erstellt eine Liste aller Elemente einer JDOM-Struktur.
	 * 
	 * @param startElement
	 * @return
	 */
	private LinkedList<Element> rekursivElementKindListeErstellen(
			Element startElement)
	{
		LinkedList<Element> re = new LinkedList<Element>();

		re.add(startElement);

		@SuppressWarnings("unchecked")
		Iterator it = startElement.getChildren().iterator();
		while (it.hasNext())
		{
			Object obj = it.next();
			if (obj instanceof Element)
			{
				Element child = (Element) obj;
				re.addAll(rekursivElementKindListeErstellen(child));
			}
		}

		return re;
	}

	/**
	 * Vergleicht zwei Listen vom Typ {@link Element} auf Gleichheit.
	 * 
	 * @param listeA
	 *          Liste mit den Erwartungswerten
	 * @param listeB
	 *          Liste mit den Ist-Werten
	 * @return Liefert <code>true</code>, wenn die Element bereinstimmen, sonst
	 *         <code>false</code>.
	 */
	private boolean vergleicheElementListen(LinkedList<Element> listeA,
			LinkedList<Element> listeB)
	{
		boolean re = true;

		if (listeA.size() == listeB.size())
		{
			for (int i = 0; i < listeA.size(); i++)
			{
				if (!vergleicheElemente(listeA.get(i), listeB.get(i)))
				{
					re = false;
					break;
				}
			}
		} else
		{
			re = false;
		}

		return re;
	}

	/**
	 * Vergleicht zwei Elemente vom Typ {@link Element} auf Gleichheit. Dabei
	 * werden die Attribute bercksichtigt.
	 * 
	 * @param elementA
	 *          Erstes Element mit dem Erwartungswert
	 * @param elementB
	 *          Zweites Element mit dem aktuellen Zustand
	 * @return Liefert <code>true</code>, wenn die Elemente gleich sind sonst
	 *         <code>false</code>.
	 */
	private boolean vergleicheElemente(Element elementA, Element elementB)
	{
		boolean re = true;

		@SuppressWarnings("unchecked")
		List attributesA = elementA.getAttributes();
		@SuppressWarnings("unchecked")
		List attributesB = elementB.getAttributes();

		if (attributesA.size() == attributesB.size())
		{
			boolean attributeGleich = true;
			for (int i = 0; i < attributesA.size(); i++)
			{
				if ((attributesA.get(i) instanceof Attribute)
						&& (attributesB.get(i) instanceof Attribute))
				{
					Attribute attA = (Attribute) attributesA.get(i);
					Attribute attB = (Attribute) attributesB.get(i);

					if (!vergleicheAttribute(attA, attB))
					{
						attributeGleich = false;
						break;
					}
				}
			}
			re = attributeGleich;

		} else
		{
			re = false;
		}

		return re;
	}

	/**
	 * Vergleicht zwei Attribute.
	 * 
	 * @param attributA
	 *          Erstes Attribut mit dem Erwartungswert.
	 * @param attributB
	 *          Zweites Attribut mit dem aktuellen Zustand.
	 * @return Liefert <code>true</code>, wenn die Attribute gleich sind, sonst
	 *         <code>false</code>.
	 */
	private boolean vergleicheAttribute(Attribute attributA, Attribute attributB)
	{
		boolean re = false;

		if (attributA.getName().equals(attributB.getName()))
		{
			if (attributA.getValue().equals(attributB.getValue()))
			{
				re = true;
			}
		}

		return re;
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten#einlesenXmlAusString(java.lang.String)}
	 * .
	 */
	@Test
	public void testEinlesenXmlAusString()
	{
		_struktur.einlesenXmlAusString(_xmlStruktur);
		Element elementWurzel = _struktur.getWurzel();

		LinkedList<Element> listeErwartet = rekursivElementKindListeErstellen(_elementWurzel);
		LinkedList<Element> listeAktuell = rekursivElementKindListeErstellen(elementWurzel);

		assertEquals(true, vergleicheElementListen(listeErwartet, listeAktuell));
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten#schreibeString(java.lang.String, java.lang.String, boolean)}
	 * .
	 */
	@Test
	public void testSchreibeString()
	{
		String testString = "";
		testString = testString + "Dies ist ein Teststring,";
		testString = testString + "der in eine Datei geschrieben werden soll.";
		testString = testString + "Anschlieend wird er aus der Datei gelesen.";

		/*
		 * Schreiben des Strings
		 */
		_struktur.schreibeString(_pfadTestDatei, testString, true);

		/*
		 * Einlesen des Strings
		 */
		String line = null;
		StringBuffer sb = new StringBuffer();
		try
		{
			BufferedReader br = new BufferedReader(new FileReader(_pfadTestDatei));
			try
			{
				while ((line = br.readLine()) != null)
				{
					sb.append(line);
				}
			} catch (IOException e)
			{
				e.printStackTrace();
			}
		} catch (FileNotFoundException e)
		{
			e.printStackTrace();
		}

		String vergleich = sb.toString();

		assertEquals(testString, vergleich);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten#hinzufuegenZuhoerer(de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.IZuhoererMenuSymbolLeisteDatenAenderung)}
	 * .
	 */
	@Test
	public void testHinzufuegenZuhoerer()
	{
		Zuhoerer zuhoerer = new Zuhoerer();
		_struktur.hinzufuegenZuhoerer(zuhoerer);

		_struktur.einlesenXmlAusString(_xmlStruktur);

		_struktur.entferneZuhoerer(zuhoerer);

		assertEquals(true, zuhoerer.istListenerAufgerufenWorden());
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten#entferneZuhoerer(de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.IZuhoererMenuSymbolLeisteDatenAenderung)}
	 * .
	 */
	@Test
	public void testEntferneZuhoerer()
	{
		Zuhoerer zuhoerer = new Zuhoerer();
		_struktur.hinzufuegenZuhoerer(zuhoerer);
		_struktur.entferneZuhoerer(zuhoerer);
		_struktur.einlesenXmlAusString(_xmlStruktur);
		assertEquals(true, !zuhoerer.istListenerAufgerufenWorden());
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten#bestimmeElementPositionInElternElement(org.jdom.Element)}
	 * .
	 */
	@Test
	public void testBestimmeElementPositionInElternElement()
	{
		/*
		 * Teststruktur erzeugen
		 */
		Element elternElement = new Element("ElternElement");
		Element kindElement1 = new Element("KindElement1");
		Element kindElement2 = new Element("KindElement2");
		Element kindElement3 = new Element("KindElement3");
		elternElement.addContent(kindElement1);
		elternElement.addContent(kindElement2);
		elternElement.addContent(kindElement3);

		/*
		 * Werte bestimmen
		 */
		int posKind1 = _struktur
				.bestimmeElementPositionInElternElement(kindElement1);
		int posKind2 = _struktur
				.bestimmeElementPositionInElternElement(kindElement2);
		int posKind3 = _struktur
				.bestimmeElementPositionInElternElement(kindElement3);

		assertEquals(true, (posKind1 == 0) && (posKind2 == 1) && (posKind3 == 2));
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten#bestimmeAnzahlKinderImElement(org.jdom.Element)}
	 * .
	 */
	@Test
	public void testBestimmeAnzahlKinderImElement()
	{
		Element elternElement = new Element("ElternElement");
		Element kindElement1 = new Element("KindElement1");
		Element kindElement2 = new Element("KindElement2");
		elternElement.addContent(kindElement1);
		elternElement.addContent(kindElement2);

		assertEquals(2, _struktur.bestimmeAnzahlKinderImElement(elternElement));
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten#loescheElement(org.jdom.Element)}
	 * .
	 */
	@Test
	public void testLoescheElement()
	{
		Element elternElement = new Element("ElternElement");
		Element kindElement1 = new Element("KindElement1");
		Element kindElement2 = new Element("KindElement2");
		Element subKindElement1 = new Element("SubKindElement1");

		kindElement2.addContent(subKindElement1);
		elternElement.addContent(kindElement1);
		elternElement.addContent(kindElement2);

		LinkedList<Element> listeVorLoeschen = rekursivElementKindListeErstellen(elternElement);
		boolean vorLoeschenVorhanden = false;
		if ((listeVorLoeschen.contains(kindElement2))
				&& (listeVorLoeschen.contains(subKindElement1)))
		{
			vorLoeschenVorhanden = true;
		}

		boolean nachLoeschenNichtMehrVorhanden = false;

		_struktur.loescheElement(kindElement2);

		LinkedList<Element> listeNachLoeschen = rekursivElementKindListeErstellen(elternElement);
		if ((!listeNachLoeschen.contains(kindElement2))
				&& (!listeNachLoeschen.contains(subKindElement1)))
		{
			nachLoeschenNichtMehrVorhanden = true;
		}

		assertEquals(true, vorLoeschenVorhanden && nachLoeschenNichtMehrVorhanden);
	}

	/**
	 * Prft die Plausibilittsprfung beim Lschen
	 */
	@Test
	public void testPlausibilitaetLoeschen()
	{
		/*
		 * Prfstruktur <menuleiste> <menu id="hauptmenue.1" art="Menleiste">
		 * </menu> </menuleiste>
		 */

		Element eleMenuleiste = new Element("menuleiste");
		Element eleMenu = new Element("menu");
		Attribute attId = new Attribute("id", "hauptmenue.1");
		Attribute attArt = new Attribute("art", "Menleiste");
		eleMenu.setAttribute(attId);
		eleMenu.setAttribute(attArt);
		eleMenuleiste.addContent(eleMenu);

		boolean loeschenErlaubtBeiMenue = MenuSymbolLeisteDaten
				.pruefeObElementLoeschenErlaubt(eleMenu);

		/*
		 * Prfstruktur Symbolleiste <menuleiste> <menu id="hauptmenue.1"
		 * art="Symbolleiste"> </menu> </menuleiste>
		 */
		Element eleSymbolleiste = new Element("menuleiste");
		Element eleMenuSymbol = new Element("menu");
		Attribute attIdSymbol = new Attribute("id", "hauptmenue.1");
		Attribute attArtSymbol = new Attribute("art", "Symbolleiste");
		eleMenuSymbol.setAttribute(attIdSymbol);
		eleMenuSymbol.setAttribute(attArtSymbol);
		eleSymbolleiste.addContent(eleMenuSymbol);
		
		boolean loeschenErlaubtBeiSymbolleiste = MenuSymbolLeisteDaten
				.pruefeObElementLoeschenErlaubt(eleMenuSymbol);
		

		/*
		 * Beide Prfungen vereinen
		 */
		boolean loeschenErlaubt = true;
		if( (loeschenErlaubtBeiMenue == false) && (loeschenErlaubtBeiSymbolleiste == false))
		{
			loeschenErlaubt = false;
		}
		

		assertEquals(false, loeschenErlaubt);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten#verschiebeItem(org.jdom.Element, org.jdom.Element, int)}
	 * .
	 */
	@Test
	public void testVerschiebeItemElementElementInt()
	{
		String erwarteteId = "2.1";
		String aktuelleId = null;

		Element elternElement = new Element("ElternElement");
		Element kindElement1 = new Element(XmlBezeichnerMenuSymbol.elementMenu);
		kindElement1.setAttribute(new Attribute("id", "1"));
		Element kindElement2 = new Element(XmlBezeichnerMenuSymbol.elementMenu);
		kindElement2.setAttribute(new Attribute("id", "2"));
		Element subKindElement11 = new Element(XmlBezeichnerMenuSymbol.elementItem);
		subKindElement11.setAttribute(new Attribute("id", "1.1"));
		Element subKindElement12 = new Element(XmlBezeichnerMenuSymbol.elementItem);
		subKindElement12.setAttribute(new Attribute("id", "1.2"));
		Element subKindElement13 = new Element(XmlBezeichnerMenuSymbol.elementItem);
		subKindElement12.setAttribute(new Attribute("id", "1.2"));
		Element subKindElement21 = new Element(XmlBezeichnerMenuSymbol.elementItem);
		subKindElement21.setAttribute(new Attribute("id", erwarteteId));

		kindElement1.addContent(subKindElement11);
		kindElement1.addContent(subKindElement12);
		kindElement1.addContent(subKindElement13);

		kindElement2.addContent(subKindElement21);

		elternElement.addContent(kindElement1);
		elternElement.addContent(kindElement2);

		_struktur.verschiebeItem(subKindElement21, kindElement1, 1);

		// LinkedList<Element> liste =
		// rekursivElementKindListeErstellen(kindElement1);

		@SuppressWarnings("unchecked")
		List children = kindElement1.getChildren();
		Object object = children.get(1);
		if (object instanceof Element)
		{
			Element vergleichsElement = (Element) object;
			aktuelleId = vergleichsElement.getAttributeValue("id");
		}

		assertEquals(erwarteteId, aktuelleId);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten#verschiebeItem(org.jdom.Element, org.jdom.Element)}
	 * .
	 */
	@Test
	public void testVerschiebeItemElementElement()
	{
		String erwarteteId = "2.1";
		String aktuelleId = null;

		Element elternElement = new Element("ElternElement");
		Element kindElement1 = new Element(XmlBezeichnerMenuSymbol.elementMenu);
		kindElement1.setAttribute(new Attribute("id", "1"));
		Element kindElement2 = new Element(XmlBezeichnerMenuSymbol.elementMenu);
		kindElement2.setAttribute(new Attribute("id", "2"));
		Element subKindElement11 = new Element(XmlBezeichnerMenuSymbol.elementItem);
		subKindElement11.setAttribute(new Attribute("id", "1.1"));
		Element subKindElement12 = new Element(XmlBezeichnerMenuSymbol.elementItem);
		subKindElement12.setAttribute(new Attribute("id", "1.2"));
		Element subKindElement13 = new Element(XmlBezeichnerMenuSymbol.elementItem);
		subKindElement12.setAttribute(new Attribute("id", "1.2"));
		Element subKindElement21 = new Element(XmlBezeichnerMenuSymbol.elementItem);
		subKindElement21.setAttribute(new Attribute("id", erwarteteId));

		kindElement1.addContent(subKindElement11);
		kindElement1.addContent(subKindElement12);
		kindElement1.addContent(subKindElement13);

		kindElement2.addContent(subKindElement21);

		elternElement.addContent(kindElement1);
		elternElement.addContent(kindElement2);

		_struktur.verschiebeItem(subKindElement21, kindElement1);
		Object object = kindElement1.getChildren().get(3);

		if (object instanceof Element)
		{
			Element vergleichsElement = (Element) object;
			aktuelleId = vergleichsElement.getAttributeValue("id");
		}

		assertEquals(erwarteteId, aktuelleId);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten#verschiebeMenu(org.jdom.Element, org.jdom.Element)}
	 * .
	 */
	@Test
	public void testVerschiebeMenu()
	{
		String erwartetID = "1";
		String aktuellID = null;

		Element elternMenu = new Element(XmlBezeichnerMenuSymbol.elementMenu);

		Element subMenu1 = new Element(XmlBezeichnerMenuSymbol.elementSubMenu);
		subMenu1.setAttribute(new Attribute("id", "1"));

		Element subMenu2 = new Element((XmlBezeichnerMenuSymbol.elementSubMenu));
		subMenu2.setAttribute(new Attribute("id", "2"));

		Element menu1 = new Element(XmlBezeichnerMenuSymbol.elementMenu);
		menu1.setAttribute(new Attribute("id", erwartetID));
		Element menu2 = new Element(XmlBezeichnerMenuSymbol.elementMenu);
		menu2.setAttribute(new Attribute("id", "2"));
		Element subKindElement11 = new Element(XmlBezeichnerMenuSymbol.elementItem);
		subKindElement11.setAttribute(new Attribute("id", "1.1"));
		Element subKindElement12 = new Element(XmlBezeichnerMenuSymbol.elementItem);
		subKindElement12.setAttribute(new Attribute("id", "1.2"));
		Element subKindElement13 = new Element(XmlBezeichnerMenuSymbol.elementItem);
		subKindElement12.setAttribute(new Attribute("id", "1.2"));
		Element subKindElement21 = new Element(XmlBezeichnerMenuSymbol.elementItem);
		subKindElement21.setAttribute(new Attribute("id", "2.1"));

		menu1.addContent(subKindElement11);
		menu1.addContent(subKindElement12);
		menu1.addContent(subKindElement13);

		menu2.addContent(subKindElement21);

		subMenu1.addContent(menu1);
		subMenu1.addContent(menu2);

		elternMenu.addContent(subMenu1);
		elternMenu.addContent(subMenu2);

		_struktur.verschiebeMenu(menu1, subMenu2);

		if (subMenu2.getChildren().size() > 0)
		{
			Object object = subMenu2.getChildren().get(0);
			if (object instanceof Element)
			{
				Element vergleichElement = (Element) object;
				if (vergleichElement.getName().equals(
						XmlBezeichnerMenuSymbol.elementMenu))
				{
					aktuellID = vergleichElement.getAttributeValue("id");
				}
			}
		}

		assertEquals(erwartetID, aktuellID);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten#verschiebeSubMenu(org.jdom.Element, org.jdom.Element, int)}
	 * .
	 */
	@SuppressWarnings("unchecked")
	@Test
	public void testVerschiebeSubMenuElementElementInt()
	{
		String erwartetId = "1.2";
		String aktuellId = null;

		/*
		 * Struktur erzeugen
		 */
		Element elementMenu1 = new Element(XmlBezeichnerMenuSymbol.elementMenu);
		elementMenu1
				.setAttribute(new Attribute(XmlBezeichnerMenuSymbol.attId, "1"));

		Element elementItem1_1 = new Element(XmlBezeichnerMenuSymbol.elementItem);
		elementItem1_1.setAttribute(new Attribute(XmlBezeichnerMenuSymbol.attId,
				"1.1"));
		Element elementItem1_2 = new Element(XmlBezeichnerMenuSymbol.elementItem);
		elementItem1_2.setAttribute(new Attribute(XmlBezeichnerMenuSymbol.attId,
				"1.2"));

		Element elementSubMenu1_1 = new Element(
				XmlBezeichnerMenuSymbol.elementSubMenu);
		elementSubMenu1_1.setAttribute(new Attribute(XmlBezeichnerMenuSymbol.attId,
				"1.1"));
		Element elementSubMenu1_2 = new Element(
				XmlBezeichnerMenuSymbol.elementSubMenu);
		elementSubMenu1_2.setAttribute(new Attribute(XmlBezeichnerMenuSymbol.attId,
				erwartetId));

		elementMenu1.addContent(elementItem1_1);
		elementMenu1.addContent(elementSubMenu1_1);
		elementMenu1.addContent(elementItem1_2);
		elementMenu1.addContent(elementSubMenu1_2);

		/*
		 * Verschiebung durchfhren
		 */
		_struktur.verschiebeSubMenu(elementSubMenu1_2, elementMenu1, 0);

		/*
		 * ElementID an Position 0 suchen
		 */
		List liste = elementMenu1.getChildren();
		if (liste.size() > 0)
		{
			Object object = liste.get(0);
			if (object instanceof Element)
			{
				Element pruefElement = (Element) object;
				aktuellId = pruefElement
						.getAttributeValue(XmlBezeichnerMenuSymbol.attId);
			}
		}

		assertEquals(erwartetId, aktuellId);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten#verschiebeSubMenu(org.jdom.Element, org.jdom.Element)}
	 * .
	 */
	@Test
	public void testVerschiebeSubMenuElementElement()
	{
		String erwartetId = "1.2";
		String aktuellId = null;

		/*
		 * Struktur erzeugen
		 */
		Element elementMenu1 = new Element(XmlBezeichnerMenuSymbol.elementMenu);
		elementMenu1
				.setAttribute(new Attribute(XmlBezeichnerMenuSymbol.attId, "1"));

		Element elementItem1_1 = new Element(XmlBezeichnerMenuSymbol.elementItem);
		elementItem1_1.setAttribute(new Attribute(XmlBezeichnerMenuSymbol.attId,
				"1.1"));
		Element elementItem1_2 = new Element(XmlBezeichnerMenuSymbol.elementItem);
		elementItem1_2.setAttribute(new Attribute(XmlBezeichnerMenuSymbol.attId,
				"1.2"));

		Element elementSubMenu1_1 = new Element(
				XmlBezeichnerMenuSymbol.elementSubMenu);
		elementSubMenu1_1.setAttribute(new Attribute(XmlBezeichnerMenuSymbol.attId,
				"1.1"));
		Element elementSubMenu1_2 = new Element(
				XmlBezeichnerMenuSymbol.elementSubMenu);
		elementSubMenu1_2.setAttribute(new Attribute(XmlBezeichnerMenuSymbol.attId,
				erwartetId));

		Element elementMenu1_1_1 = new Element(XmlBezeichnerMenuSymbol.elementMenu);
		elementMenu1_1_1.setAttribute(new Attribute(XmlBezeichnerMenuSymbol.attId,
				"1.1.1"));

		elementMenu1.addContent(elementSubMenu1_1);
		elementMenu1.addContent(elementSubMenu1_2);

		elementSubMenu1_1.addContent(elementMenu1_1_1);

		elementMenu1_1_1.addContent(elementItem1_1);
		elementMenu1_1_1.addContent(elementItem1_2);

		/*
		 * Verschieben des Submens 1.2, welches im Hauptmen hngt, ins Men 1.1.1,
		 * welches im Submen 1.2 hngt
		 */

		_struktur.verschiebeSubMenu(elementSubMenu1_2, elementMenu1_1_1);

		/*
		 * Prfen
		 */
		if (elementMenu1_1_1.getParent().equals(elementSubMenu1_1))
		{
			if (elementSubMenu1_1.getParent().equals(elementMenu1))
			{
				@SuppressWarnings("unchecked")
				List children = elementMenu1_1_1
						.getChildren(XmlBezeichnerMenuSymbol.elementSubMenu);
				if (children.size() == 1)
				{
					Object object = children.get(0);
					if (object instanceof Element)
					{
						Element pruefElement = (Element) object;
						if (pruefElement.getName().equals(
								XmlBezeichnerMenuSymbol.elementSubMenu))
						{
							aktuellId = pruefElement
									.getAttributeValue(XmlBezeichnerMenuSymbol.attId);
						}
					}
				}
			}
		}

		assertEquals(erwartetId, aktuellId);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten#istXmlElementKindVon(org.jdom.Element, org.jdom.Element)}
	 * .
	 */
	@Test
	public void testIstXmlElementKindVon()
	{
		Element elternElement = new Element("ElternElement");
		Element kindElement = new Element("KindElement");
		elternElement.addContent(kindElement);

		assertEquals(true, _struktur.istXmlElementKindVon(kindElement,
				elternElement));

	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten#hinzufuegenSeparatorAnMenu(org.jdom.Element, org.jdom.Element)}
	 * .
	 */
	@Test
	public void testHinzufuegenSeparatorAnMenu()
	{
		/*
		 * Struktur erzeugen
		 */
		Element elternElement = new Element(XmlBezeichnerMenuSymbol.elementMenu);
		elternElement
				.setAttribute(new Attribute(XmlBezeichnerMenuSymbol.attId, "1"));

		Element elementItem1 = new Element(XmlBezeichnerMenuSymbol.elementItem);
		Element elementItem2 = new Element(XmlBezeichnerMenuSymbol.elementItem);

		elternElement.addContent(elementItem1);
		elternElement.addContent(elementItem2);

		/*
		 * Separator erzeugen
		 */
		Element separator = new Element(XmlBezeichnerMenuSymbol.elementSeparator);

		_struktur.hinzufuegenSeparatorAnMenu(separator, elternElement);

		Element pruefElement = (Element) elternElement.getChildren().get(2);
		String pruefString = pruefElement.getName();

		assertEquals(XmlBezeichnerMenuSymbol.elementSeparator, pruefString);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten#hinzufuegenSeparatorInMenu(org.jdom.Element, org.jdom.Element)}
	 * .
	 */
	@Test
	public void testHinzufuegenSeparatorInMenuElementElement()
	{
		/*
		 * Struktur erzeugen
		 */
		Element elternElement = new Element(XmlBezeichnerMenuSymbol.elementMenu);
		elternElement
				.setAttribute(new Attribute(XmlBezeichnerMenuSymbol.attId, "1"));

		Element elementItem1 = new Element(XmlBezeichnerMenuSymbol.elementItem);
		elementItem1.setAttribute(new Attribute(XmlBezeichnerMenuSymbol.attId,
				"1.1"));

		Element elementItem2 = new Element(XmlBezeichnerMenuSymbol.elementItem);
		elementItem1.setAttribute(new Attribute(XmlBezeichnerMenuSymbol.attId,
				"1.2"));

		elternElement.addContent(elementItem1);
		elternElement.addContent(elementItem2);

		Element elementSeparator = new Element(
				XmlBezeichnerMenuSymbol.elementSeparator);

		/*
		 * Separator einfgen
		 */
		_struktur.hinzufuegenSeparatorInMenu(elementSeparator, elementItem2);

		@SuppressWarnings("unchecked")
		List kindElemente = elternElement.getChildren();
		Object object = kindElemente.get(1);

		String pruefString = null;
		if (object instanceof Element)
		{
			Element pruefElement = (Element) object;
			pruefString = pruefElement.getName();
		}

		assertEquals(XmlBezeichnerMenuSymbol.elementSeparator, pruefString);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten#hinzufuegenSeparatorInMenu(org.jdom.Element, org.jdom.Element, int)}
	 * .
	 */
	@SuppressWarnings("unchecked")
	@Test
	public void testHinzufuegenSeparatorInMenuElementElementInt()
	{
		/*
		 * Struktur erzeugen
		 */
		Element elternElement = new Element(XmlBezeichnerMenuSymbol.elementMenu);
		elternElement
				.setAttribute(new Attribute(XmlBezeichnerMenuSymbol.attId, "1"));

		Element elementItem1 = new Element(XmlBezeichnerMenuSymbol.elementItem);
		elementItem1.setAttribute(new Attribute(XmlBezeichnerMenuSymbol.attId,
				"1.1"));

		Element elementItem2 = new Element(XmlBezeichnerMenuSymbol.elementItem);
		elementItem1.setAttribute(new Attribute(XmlBezeichnerMenuSymbol.attId,
				"1.2"));

		elternElement.addContent(elementItem1);
		elternElement.addContent(elementItem2);

		Element elementSeparator = new Element(
				XmlBezeichnerMenuSymbol.elementSeparator);

		/*
		 * Separator einfgen
		 */
		_struktur.hinzufuegenSeparatorInMenu(elementSeparator, elternElement, 0);

		String pruefString = null;

		List kindElemente = elternElement.getChildren();
		if (kindElemente.size() > 2)
		{
			Object object = kindElemente.get(0);
			if (object instanceof Element)
			{
				Element pruefElement = (Element) object;
				pruefString = pruefElement.getName();
			}
		}

		assertEquals(XmlBezeichnerMenuSymbol.elementSeparator, pruefString);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten#setzeElementBei(org.jdom.Element, org.jdom.Element, int)}
	 * .
	 */
	@Test
	public void testSetzeElementBei()
	{
		/*
		 * Struktur erzeugen
		 */
		Element elternElement = new Element("ElternElement");
		Element kindElement1 = new Element("KindElement1");
		Element kindElement2 = new Element("KindElement2");
		Element kindElement3 = new Element("KindElement3");

		elternElement.addContent(kindElement1);
		elternElement.addContent(kindElement2);
		elternElement.addContent(kindElement3);

		/*
		 * Flag zur Anzeige, dass die Prfungen dieses Tests erfolgreich sind.
		 */
		boolean erfolgreich = true;

		/*
		 * Pos 0
		 */
		Element einfuegeElement1 = new Element("EingefgtesElement1");
		_struktur.setzeElementBei(elternElement, einfuegeElement1, 0);

		Element pruefElement1 = (Element) elternElement.getChildren().get(0);
		if (!pruefElement1.getName().equals("EingefgtesElement1"))
		{
			erfolgreich = false;
		}

		einfuegeElement1.detach();

		/*
		 * Pos 1
		 */
		Element einfuegeElement2 = new Element("EingefgtesElement2");
		_struktur.setzeElementBei(elternElement, einfuegeElement2, 1);

		Element pruefElement2 = (Element) elternElement.getChildren().get(1);
		if (!pruefElement2.getName().equals("EingefgtesElement2"))
		{
			erfolgreich = false;
		}

		einfuegeElement2.detach();

		/*
		 * Pos 2
		 */
		Element einfuegeElement3 = new Element("EingefgtesElement3");
		_struktur.setzeElementBei(elternElement, einfuegeElement3, 2);

		Element pruefElement3 = (Element) elternElement.getChildren().get(2);
		if (!pruefElement3.getName().equals("EingefgtesElement3"))
		{
			erfolgreich = false;
		}

		einfuegeElement3.detach();

		/*
		 * Pos 3
		 */
		Element einfuegeElement4 = new Element("EingefgtesElement4");
		_struktur.setzeElementBei(elternElement, einfuegeElement4, 3);

		Element pruefElement4 = (Element) elternElement.getChildren().get(3);
		if (!pruefElement4.getName().equals("EingefgtesElement4"))
		{
			erfolgreich = false;
		}

		einfuegeElement4.detach();

		assertEquals(true, erfolgreich);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten#pruefePositionInElement(org.jdom.Element, int)}
	 * .
	 */
	@Test
	public void testPruefePositionInElement()
	{
		/*
		 * Struktur erzeugen
		 */
		Element elternElement = new Element("ElternElement");
		Element kindElement1 = new Element("KindElement1");
		Element kindElement2 = new Element("KindElement2");
		Element kindElement3 = new Element("KindElement3");

		elternElement.addContent(kindElement1);
		elternElement.addContent(kindElement2);
		elternElement.addContent(kindElement3);

		/*
		 * Prfen der Positionen 0, 1 und 2
		 */
		boolean pos0 = _struktur.pruefePositionInElement(elternElement, 0);
		boolean pos1 = _struktur.pruefePositionInElement(elternElement, 1);
		boolean pos2 = _struktur.pruefePositionInElement(elternElement, 2);

		assertEquals(true, pos0 && pos1 && pos2);
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten#sucheMenu(java.lang.String)}
	 * .
	 */
	@Test
	public void testSucheMenu()
	{
		_struktur.einlesenXmlAusString(_xmlStruktur);

		Element menu = _struktur.sucheMenu("sub.1.menu.1");

		assertEquals("sub.1.menu.1", menu
				.getAttributeValue(XmlBezeichnerMenuSymbol.attId));
	}

	/**
	 * Test method for
	 * {@link de.bsvrz.buv.rw.rw.menu.xml.datenstruktur.MenuSymbolLeisteDaten#sucheSubMenu(java.lang.String)}
	 * .
	 */
	@Test
	public void testSucheSubMenu()
	{
		_struktur.einlesenXmlAusString(_xmlStruktur);

		Element subMenu = _struktur.sucheSubMenu("sub.2");

		assertEquals("sub.2", subMenu
				.getAttributeValue(XmlBezeichnerMenuSymbol.attId));
	}

	@Test
	public void testBestimmeFreieId()
	{
		Element elementMenu1 = new Element(XmlBezeichnerMenuSymbol.elementMenu);
		elementMenu1
				.setAttribute(new Attribute(XmlBezeichnerMenuSymbol.attId, "1"));

		Element elementSubmenu1_1 = new Element(
				XmlBezeichnerMenuSymbol.elementSubMenu);
		elementSubmenu1_1.setAttribute(new Attribute(XmlBezeichnerMenuSymbol.attId,
				"1"));

		Element elementMenu1_1_1 = new Element(XmlBezeichnerMenuSymbol.elementMenu);
		elementMenu1_1_1.setAttribute(new Attribute(XmlBezeichnerMenuSymbol.attId,
				"2"));

		Element elementSubmenu1_1_1_1 = new Element(
				XmlBezeichnerMenuSymbol.elementSubMenu);
		elementSubmenu1_1_1_1.setAttribute(new Attribute(
				XmlBezeichnerMenuSymbol.attId, "2"));

		Element elementMenu1_1_1_1_1 = new Element(
				XmlBezeichnerMenuSymbol.elementMenu);
		elementMenu1_1_1_1_1.setAttribute(new Attribute(
				XmlBezeichnerMenuSymbol.attId, "3"));

		Element elementSubmenu1_1_1_1_1_1 = new Element(
				XmlBezeichnerMenuSymbol.elementSubMenu);
		elementSubmenu1_1_1_1_1_1.setAttribute(new Attribute(
				XmlBezeichnerMenuSymbol.attId, "0"));

		elementMenu1.addContent(elementSubmenu1_1);
		elementSubmenu1_1.addContent(elementMenu1_1_1);
		elementMenu1_1_1.addContent(elementSubmenu1_1_1_1);

		boolean korrekt = true;

		/*
		 * Erwartet: 0
		 */
		int bestimmteFreieId = _struktur.bestimmeFreieId(elementMenu1,
				XmlBezeichnerMenuSymbol.elementSubMenu, XmlBezeichnerMenuSymbol.attId);

		if (!(bestimmteFreieId == 0))
		{
			korrekt = false;
		}

		/*
		 * Erwartet: 3
		 */

		elementSubmenu1_1_1_1.addContent(elementMenu1_1_1_1_1);
		elementMenu1_1_1_1_1.addContent(elementSubmenu1_1_1_1_1_1);

		bestimmteFreieId = _struktur.bestimmeFreieId(elementMenu1,
				XmlBezeichnerMenuSymbol.elementSubMenu, XmlBezeichnerMenuSymbol.attId);

		if (!(bestimmteFreieId == 3))
		{
			korrekt = false;
		}

		assertEquals(true, korrekt);
	}

	/**
	 * Stub fr den Test des Listeners
	 * {@link IZuhoererMenuSymbolLeisteDatenAenderung}
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class Zuhoerer implements IZuhoererMenuSymbolLeisteDatenAenderung
	{
		private boolean _listenerWurdeAufgerufen = false;

		@Override
		public void ausfuehren(EreignisMenuSymbolLeisteDatenGeandert e)
		{
			_listenerWurdeAufgerufen = true;
		}

		public boolean istListenerAufgerufenWorden()
		{
			return _listenerWurdeAufgerufen;
		}

	}

}
