/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw;

import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IWorkbenchWindow;

/**
 * Speicher fr das {@link IWorkbenchWindow} der RCP-Anwendung. Ansonsten knnen
 * keine Globalen Aktionen mit {@link IPartListener} auerhalb des
 * ApplicationActionBarAdvisors erzeugt werden.
 * 
 * @author HofmannJ
 * 
 */
public class FensterSpeicher
{	
	/**
	 * Speichert das Window-Objekt
	 */
	private IWorkbenchWindow _window = null;

	/**
	 * Privater Konstruktor
	 */
	private FensterSpeicher()
	{
	}

	/**
	 * Speichert die Singleton-Instanz der Klasse {@link FensterSpeicher}.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class Innen
	{
		private static FensterSpeicher INSTANZ = new FensterSpeicher();
	}

	/**
	 * Liefert die Singleton-Instanz der Klasse {@link FensterSpeicher}.
	 * 
	 * @return Singleton-Instanz der Klasse {@link FensterSpeicher}
	 */
	public static FensterSpeicher getInstanz()
	{
		return Innen.INSTANZ;
	}

	/**
	 * Setzt das {@link IWorkbenchWindow}.
	 * 
	 * @param window
	 *          Zu setzendes {@link IWorkbenchWindow}
	 */
	public void setWindow(IWorkbenchWindow window)
	{
		this._window = window;
	}

	/**
	 * Liefert das {@link IWorkbenchWindow}.
	 * 
	 * @return {@link IWorkbenchWindow}
	 */
	public IWorkbenchWindow getWindow()
	{		
		return _window;
	}

}
