/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.rw.interfaces.darstellungsobjekte;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Interface fr den Zugriff auf DarstellungsObjekte.<br>
 * 
 * @version $Revision: 1.1 $ / $Date: 2009/12/02 09:21:02 $ / ($Author: HofmannJ $)
 * 
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 * 
 */
public interface IDarstellungsObjekt
{
  /**
   * Setzen des SystemObjektes fr das das Darstellungsobjekt Informationen darstellen soll.<br>
   * 
   * @param objekt
   *          SystemObjekt fr das das Darstellungsobjekt Informationen darstellen soll.
   */
  public void setSystemObjekt(SystemObject objekt);

  /**
   * Holen der SystemObjektTypen, fr die das DarstellungsObjekt Informationen anzeigen kann.<br>
   * 
   * @return Feld mit SystemObjektTypen.
   */
  public SystemObjectType[] getUnterstuetzteSystemObjektTypen();

  /**
   * Aufforderung zum Neuzeichnen des DarstellungsObjektes an der bergebenen Position mit der bergebenen Ausrichtung.<br>
   * 
   * @param x
   *          X-Position in Pixel, an der das DarstellungsObjekt angezeigt werden soll.
   * @param y
   *          Y-Position in Pixel, an der das DarstellungsObjekt angezeigt werden soll.
   * @param ausrichtung
   *          Ausrichtung fr das Neuzeichnen des DarstellungsObjektes.
   */
  public void neuzeichnen(int x, int y, DarstellungsObjektAusrichtung ausrichtung);

  /**
   * Holen der DarstellungsObjektTypZoomInfo fr das aktuelle SystemObjekt mit der bergebenen ZoomStufe.<br>
   * Sollte kein SystemObjekt mit {@link #setSystemObjekt(SystemObject)} gesetzt worden sein, wird <code>null</code>
   * zurckgegeben.<br>
   * 
   * @param zoomStufe
   *          ZoomStufe in Prozent, fr die die DarstellungsObjektTypZoomInfo bestimmt werden soll.
   * @return DarstellungsObjektTypZoomInfo oder <code>null</code>, wenn kein SystemObjekt gesetzt.
   */
  public IDarstellungsObjektTypZoomInfo getDarstellungsObjektTypZoomInfo(int zoomStufe);
}
