/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.kommandozeilenparameter;

import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.rw.Activator;

public class KommandoZeilenParameterView extends ViewPart
{

	public static final String ID = "de.bsvrz.buv.rw.rw.kommandozeilenparameter.KommandoZeilenParameterView"; 
	private Composite _top = null;
	private Group _gruppeUebersicht = null;
	private Table _tabelleKmdPar = null;
	private TableViewer _tableViewerKmdPar = null;
	private Image _imgPara = null;

	@Override
	public void createPartControl(Composite parent)
	{
		_imgPara = Activator.getImageDescriptor("icons/allgemein/environment_16x16.gif").createImage();
		_top = new Composite(parent, SWT.NONE);
		_top.setLayout(new GridLayout());
		erzeugeGruppeUebersicht();
		registriereActions();
		aktionAktualisieren();
	}
	
	private void aktionAktualisieren()
	{
		Kommandozeilenparameter instanz = Kommandozeilenparameter.getInstanz();
		
		LinkedList<KmdParameter> kommandoZeilenParameter = instanz.getKommandoZeilenParameter();

		_tableViewerKmdPar.setInput(kommandoZeilenParameter);
	}

	private void registriereActions()
	{
		IMenuManager menuManager = this.getViewSite().getActionBars()
				.getMenuManager();
		IToolBarManager toolBarManager = this.getViewSite().getActionBars()
				.getToolBarManager();

		Action aktionAktualisieren = new Action()
		{
			public void run()
			{
				aktionAktualisieren();
			}
		};
		aktionAktualisieren.setText("Aktualisieren");
		aktionAktualisieren.setImageDescriptor(Activator
				.getImageDescriptor("icons/allgemein/refresh.gif"));
		menuManager.add(aktionAktualisieren);
		toolBarManager.add(aktionAktualisieren);

	}

	@Override
	public void setFocus()
	{
	}

	/**
	 * Erzeugt gruppeUebersicht
	 * 
	 */
	private void erzeugeGruppeUebersicht()
	{
		GridData gridDataTabelle = new GridData();
		gridDataTabelle.horizontalAlignment = GridData.FILL;
		gridDataTabelle.grabExcessHorizontalSpace = true;
		gridDataTabelle.grabExcessVerticalSpace = true;
		gridDataTabelle.verticalAlignment = GridData.FILL;
		GridData gridDataGruppeUebersicht = new GridData();
		gridDataGruppeUebersicht.horizontalAlignment = GridData.FILL;
		gridDataGruppeUebersicht.grabExcessHorizontalSpace = true;
		gridDataGruppeUebersicht.grabExcessVerticalSpace = true;
		gridDataGruppeUebersicht.verticalAlignment = GridData.FILL;
		_gruppeUebersicht = new Group(_top, SWT.NONE);
		_gruppeUebersicht.setLayout(new GridLayout());
		_gruppeUebersicht.setLayoutData(gridDataGruppeUebersicht);
		_gruppeUebersicht.setText("Kommandozeilenparameter:");
		_tabelleKmdPar = new Table(_gruppeUebersicht, SWT.BORDER | SWT.FULL_SELECTION);
		_tabelleKmdPar.setHeaderVisible(true);
		_tabelleKmdPar.setLayoutData(gridDataTabelle);
		_tabelleKmdPar.setLinesVisible(true);
		_tableViewerKmdPar = new TableViewer(_tabelleKmdPar);
		
		TableColumn tcKey = new TableColumn(_tabelleKmdPar, SWT.LEFT);
		tcKey.setText("Schlssel:");
		tcKey.setWidth(200);

		TableColumn tcValue = new TableColumn(_tabelleKmdPar, SWT.LEFT);
		tcValue.setText("Wert:");
		tcValue.setWidth(500);
		
		_tableViewerKmdPar.setContentProvider(new ContentProvider());
		_tableViewerKmdPar.setLabelProvider(new LabelProvider());
	}
	
	private class ContentProvider implements IStructuredContentProvider
	{		
		@SuppressWarnings("unchecked")
		@Override
		public Object[] getElements(Object inputElement)
		{
			Object[] re = null;
			
			if(inputElement instanceof LinkedList)
			{
				LinkedList<KmdParameter> parameter = (LinkedList<KmdParameter>) inputElement;
				re = new Object[parameter.size()];
				Iterator<KmdParameter> it = parameter.iterator();
				int i = 0;
				while(it.hasNext())
				{
					KmdParameter parWert = it.next();
					re[i] = parWert;
					i++;
				}
			}			
			
			return re;
		}

		@Override
		public void dispose()
		{			
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{			
		}
		
	}
	
	private class LabelProvider implements ITableLabelProvider
	{

		@Override
		public Image getColumnImage(Object element, int columnIndex)
		{			
			Image re = null;
			if(columnIndex == 0)
			{
				re = _imgPara;
			}			
			return re;
		}

		@Override
		public String getColumnText(Object element, int columnIndex)
		{
			String re = "unbekannt";
			
			if(element instanceof KmdParameter)
			{
				KmdParameter kmdParameter = (KmdParameter) element;
				
				if(columnIndex == 0)
				{
					re = "  " + kmdParameter.getSchluessel();
				}
				else if(columnIndex == 1)
				{				
					re = kmdParameter.getWert();
				
				}
				
			}
			
			return re;
		}

		@Override
		public void addListener(ILabelProviderListener listener)
		{			
		}

		@Override
		public void dispose()
		{			
		}

		@Override
		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		@Override
		public void removeListener(ILabelProviderListener listener)
		{			
		}
		
	}

}
