/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.offlinekonfiguration;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Monitor;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

/**
 * Liefert den Dialog zur Auswahl der Konfigurationsdatei.
 * 
 * @author HofmannJ
 * 
 */
public class KonfigurationsDateiAuswahlDialog extends Dialog
{
  private Group _gruppeKonfigurationsdatei = null;

  private Label _labelDatei = null;

  private Text _textDatei = null;

  private Button _buttonDatei = null;

  private Composite _kontrolle = null;

  private Button _buttonUebernehmen = null;

  private Button _buttonAbbrechen = null;

  /**
   * Shell fr die Darstellung des Dialogs.
   */
  private Shell _shell = null;

  /**
   * Flag: Zeigt an, ob eine Konfigurationsdatei ausgewhlt wurde oder nicht.
   */
  private boolean _konfigurationsdateiAusgewaehlt = false;

  /**
   * Konfigurationsdatei
   */
  private KonfigurationsDatei _konfigurationsDatei = null;

  /**
   * Pfad zur Konfigurationsdatei
   */
  private String _dateiPfad = null;

  public KonfigurationsDateiAuswahlDialog(Shell elternShell)
  {
    super(elternShell);
  }

  /**
   * ffnet den Dialog zur Auswahl der Konfigurationsdatei.
   * 
   * @return Liefert <code>true</code> zurck, wenn eine Konfigurationsdatei ausgewhlt wurde, ansonsten wird
   *         <code>false</code> zurck geliefert.
   */
  public boolean anzeigen()
  {
    Shell parent = getParent();
    _shell = new Shell(parent, SWT.TITLE | SWT.BORDER | SWT.APPLICATION_MODAL | SWT.CLOSE);

    _shell.setText("Konfigurationsdateiauswahl Dialog");

    _shell.setSize(new Point(500, 130));

    GridLayout gridLayout = new GridLayout();
    _shell.setLayout(gridLayout);
    erzeugeGruppeKonfigurationsdatei();
    erzeugeKontrolle();

    /*
     * ffnen der Shell
     */
    _shell.open();

    /*
     * Zentrieren der Darstellung des Dialogs
     */
    Display display = parent.getDisplay();
    Monitor primary = display.getPrimaryMonitor();
    Rectangle bounds = primary.getBounds();
    Rectangle rect = _shell.getBounds();
    int x = bounds.x + (bounds.width - rect.width) / 2;
    int y = bounds.y + (bounds.height - rect.height) / 2;
    _shell.setLocation(x, y);

    while (!_shell.isDisposed())
      if (!display.readAndDispatch())
        display.sleep();

    return _konfigurationsdateiAusgewaehlt;
  }

  /**
   * This method initializes groupKonfigurationsdatei
   * 
   */
  private void erzeugeGruppeKonfigurationsdatei()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout1 = new GridLayout();
    gridLayout1.numColumns = 3;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.CENTER;
    _gruppeKonfigurationsdatei = new Group(_shell, SWT.NONE);
    _gruppeKonfigurationsdatei.setLayoutData(gridData);
    _gruppeKonfigurationsdatei.setLayout(gridLayout1);
    _gruppeKonfigurationsdatei.setText("Konfigurationsdatei auswhlen:");
    _labelDatei = new Label(_gruppeKonfigurationsdatei, SWT.NONE);
    _labelDatei.setText("Konfigurationsdateipfad:");
    _textDatei = new Text(_gruppeKonfigurationsdatei, SWT.BORDER);
    _textDatei.setEditable(false);
    _textDatei.setLayoutData(gridData1);
    _buttonDatei = new Button(_gruppeKonfigurationsdatei, SWT.NONE);
    _buttonDatei.setText("Datei auswhlen");
    _buttonDatei.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionKonfigurationsdateiAuswaehlen();
			}			
		});
  }

  /**
   * Liefert einen Dateiauswahldialog und setzt die Konfigdatei
   * 
   */
  private void aktionKonfigurationsdateiAuswaehlen()
  {
    FileDialog dateiAuswahlDialog = new FileDialog(_shell, SWT.OPEN);
    dateiAuswahlDialog.setFilterNames(new String[] { "Alle Dateien (*.*)" });
    dateiAuswahlDialog.setFilterExtensions(new String[] { "*.*" });
    dateiAuswahlDialog.setText("Auswahl Konfigurationsdatei");
    String pfad = dateiAuswahlDialog.open();
    if (pfad != null)
    {
      _dateiPfad = pfad;
      _textDatei.setText(_dateiPfad);
    }
  }

  /**
   * This method initializes compositeKontrolle
   * 
   */
  private void erzeugeKontrolle()
  {
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 2;
    gridLayout.makeColumnsEqualWidth = true;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.END;
    gridData2.grabExcessHorizontalSpace = false;
    gridData2.verticalAlignment = GridData.CENTER;
    _kontrolle = new Composite(_shell, SWT.NONE);
    _kontrolle.setLayoutData(gridData2);
    _kontrolle.setLayout(gridLayout);
    _buttonUebernehmen = new Button(_kontrolle, SWT.NONE);
    _buttonUebernehmen.setText("bernehmen");
    _buttonUebernehmen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionUebernehmen();
			}			
		});
    
    
    _buttonAbbrechen = new Button(_kontrolle, SWT.NONE);
    _buttonAbbrechen.setText("Abbrechen");
    _buttonAbbrechen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionSchliessen();
			}			
		});

    GridData gridDataButtons = new GridData();
    gridDataButtons.horizontalAlignment = GridData.FILL;
    gridDataButtons.verticalAlignment = GridData.CENTER;
    _buttonAbbrechen.setLayoutData(gridDataButtons);
    _buttonUebernehmen.setLayoutData(gridDataButtons);
  }

  /**
   * bernehmen der ausgewhlten Datei.
   * 
   */
  private void aktionUebernehmen()
  {
    if (_dateiPfad != null)
    {
      _konfigurationsdateiAusgewaehlt = true;

      /*
       * bernahme des Pfades zur Konfigurationsdatei.
       */
      _konfigurationsDatei = KonfigurationsDatei.getInstanz();
      _konfigurationsDatei.setPfadZurKonfigurationsDatei(_dateiPfad);
      // KonfigurationsDatei.getInstanz()
      // .setPfadZurKonfigurationsDatei(_dateiPfad);

      aktionSchliessen();
    }
  }

  /**
   * Schliet den Dialogs
   * 
   */
  private void aktionSchliessen()
  {
    _shell.dispose();
  }

}
