/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.rw.zustaende;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 * Klasse fr den Austausch von Zustnden zwischen den Plugin's.<br>
 * Diese Klasse ist Thread-Safe. <br>
 * Realisierung der Technischen Anforderungen BuV TBuV-10, TBuV-75 bis TBuV-78.<br>
 * 
 * @version $Revision: 1.1 $ / $Date: 2009/12/02 09:21:05 $ / ($Author: HofmannJ $)
 * 
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 * 
 */
public class Zustaende
{
  /**
   * Speicher fr die einzige Instanz der Klasse.
   */
  private static Zustaende _instanz = new Zustaende();
  
  

  /**
   * Speicher fr die Zustnde.
   */
  private Map<String, Object> _map;

  /**
   * Default-Konstruktor, der nicht auerhalb dieser Klasse aufgerufen werden kann.<br>
   */
  private Zustaende()
  {
    _map = Collections.synchronizedMap(new HashMap<String, Object>());
    
    hinzufuegenZustand("Zustand1", "Ok");
    hinzufuegenZustand("Zustand2", "Teilstrung");
    hinzufuegenZustand("Zustand3", "Gestrt");
    hinzufuegenZustand("Zustand4", "nicht definiert");
  }

  /**
   * Statische Methode, liefert die einzige Instanz dieser Klasse zurck.<br>
   * ber diese Instanz erfolgt der Zugriff auf die Methoden.<br>
   * 
   * @return Instanz der Klasse.
   */
  public static Zustaende getInstanz()
  {
    return _instanz;
  }

  /**
   * Zustand des bergebenen Schlssel holen.<br>
   * <br>
   * Realisierung der Technischen Anforderungen BuV TBuV-78.<br>
   * 
   * @param schluessel
   *          Schlssel, unter dem der Zustand abgelegt ist.
   * @return Zustand.
   */
  public Object getZustand(String schluessel)
  {
    return _map.get(schluessel);
  }

  /**
   * Alle Zustnde holen.<br>
   * <br>
   * Realisierung der Technischen Anforderungen BuV TBuV-77.<br>
   * 
   * @return Alle Zustnde in einer nicht nderbaren HashMap.
   */
  public Map<String, Object> getZustaende()
  {
//    return (Map<String, Object>)Collections.unmodifiableMap(_map);
    return _map;
  }

  /**
   * Zustand unter neuem Schlssel hinzufgen.<br>
   * Ist schon ein Zustand unter dem bergebenen Schlssel vorhanden, so wird der bergebene Zustand nicht eingetragen
   * (hinzugefgt).<br>
   * Um einen Zustand unter einem schon vorhandenen Schlssel zu ndern, siehe {@link #aendereZustand(String, Object)}.<br>
   * <br>
   * Realisierung der Technischen Anforderungen BuV TBuV-75.<br>
   * 
   * @param schluessel
   *          Schlssel, unter dem der Zustand abgelegt werden soll.
   * @param zustand
   *          Zustand, der abgelegt werden soll.
   * @return <code>true</code> wenn Zustand hinzugefgt werden konnte, sonst <code>false</code>
   */
  public boolean hinzufuegenZustand(String schluessel, Object zustand)
  {
    boolean ergebnis = false;

    if (!_map.containsKey(schluessel))
    {
      _map.put(schluessel, zustand);

      ergebnis = true;
    }

    return ergebnis;
  }

  /**
   * ndern eines Zustandes unter vorhandenem Schlssel.<br>
   * Ist noch kein Zustand unter dem bergebenen Schlssel vorhanden, so wird der bergebene Zustand nicht eingetragen
   * (gendert).<br>
   * Fr das Hinzufgen eines Zustandes unter einem neuen Schlssel siehe {@link #hinzufuegenZustand(String, Object)}.<br>
   * <br>
   * Realisierung der Technischen Anforderungen BuV TBuV-76.<br>
   * 
   * @param schluessel
   *          Schlssel, unter dem der Zustand abgelegt werden soll.
   * @param zustand
   *          Zustand, der abgelegt werden soll.
   * @return <code>true</code> wenn Zustand gendert werden konnte, sonst <code>false</code>
   */
  public boolean aendereZustand(String schluessel, Object zustand)
  {
    boolean ergebnis = false;

    if (_map.containsKey(schluessel))
    {
      _map.put(schluessel, zustand);

      ergebnis = true;
    }

    return ergebnis;
  }

  public void entferneZustand(String id)
  {
    if (_map.containsKey(id))
    {
      _map.remove(id);
    }
  }
}
