/*
 * Copyright 2007 by Kappich Systemberatung, Aachen
 * Copyright 2004 by Kappich+Kni Systemberatung, Aachen
 * 
 * This file is part of de.bsvrz.kex.tls.osi2osi3.
 * 
 * de.bsvrz.kex.tls.osi2osi3 is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.kex.tls.osi2osi3 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.kex.tls.osi2osi3; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package de.bsvrz.kex.tls.osi2osi3.osi2.tc57primary;

import de.bsvrz.sys.funclib.hexdump.HexDumper;

/**
 * Klasse zum Zugriff auf den Inhalt eines Primary-Telegramms.
 *
 * @author Kappich Systemberatung
 * @version $Revision: 5114 $
 */
public class PrimaryFrame extends Tc57Frame {

	public static final int RES0 = 0;

	public static final int RES1 = 1;

	public static final int D = 3;

	public static final int TD2 = 7;

	public static final int RQD1 = 10;

	public static final int RQD2 = 11;

	public static final int RQD3 = 8;

	public static final int RQS = 9;

	public static final int RQT = 15;

	public static final int DNR = 4;

	public PrimaryFrame(boolean frameCountBit, boolean frameCountBitValid, int function, int address, byte[] data) {
		super(true, frameCountBit, frameCountBitValid, function, address, data);
	}

	public boolean getFrameCountBit() {
		return getControlBit5();
	}

	public boolean isFrameCountBitValid() {
		return getControlBit4();
	}

	public String toString() {
		StringBuffer result = new StringBuffer();
		result.append("PrimaryFrame ");
		result.append(getFrameCountBit() ? "FCB:1" : "fcb:0").append(", ");
		result.append(isFrameCountBitValid() ? "FCV:1" : "fcv:0");
		result.append(", function:").append(getFunction()).append(":").append(getFunctionName());
		result.append(", address:").append(getAddress()).append(", ");
		byte[] data = getData();
		if(data == null) {
			result.append("no data");
		}
		else {
			result.append(data.length).append(" data bytes:");
			result.append(System.getProperty("line.separator"));
			result.append(HexDumper.toString(data));
		}
		return result.toString();
	}

	private String getFunctionName() {
		switch(getFunction()) {
			case RES0:
				return "RES0";
			case RES1:
				return "RES1";
			case D:
				return "D";
			case TD2:
				return "TD2";
			case RQD1:
				return "RQD1";
			case RQD2:
				return "RQD2";
			case RQD3:
				return "RQD3";
			case RQS:
				return "RQS";
			case RQT:
				return "RQT";
			case DNR:
				return "DNR";
			default:
				return "???";
		}
	}
}

