/*
 * Copyright 2005 by Kappich+Kni Systemberatung Aachen (K2S)
 * 
 * This file is part of de.bsvrz.puk.config.
 * 
 * de.bsvrz.puk.config is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.puk.config is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.puk.config; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package de.bsvrz.puk.config.configFile.fileaccess;


/**
 * Dieses Interface stellt fr Konfigurationsobjekte Informationen zur Verfgung.
 *
 * @author Kappich+Kni Systemberatung Aachen (K2S)
 * @author Achim Wullenkord (AW)
 * @author Stephan Homeyer (sth)
 * @version $Revision: 5074 $ / $Date: 2007-09-02 14:19:12 +0200 (Sun, 02 Sep 2007) $ / ($Author: rs $)
 */
public interface ConfigurationObjectInfo extends SystemObjectInformationInterface {
	/**
	 * Diese Methode gibt an, mit welcher Version des Konfigurationsbereichs das Konfigurationsobjekt gltig werden
	 * soll/sollte.
	 *
	 * @return s.o
	 */
	short getFirstValidVersion();

	/**
	 * Diese Methode gibt an, mit welcher Version des Konfigurationsbereichs das Konfigurationsobjekt ungltig werden
	 * soll/sollte. Der Wert "0" zeigt an, dass dieser Wert noch unbekannt ist und das das Objekt auf eine unbestimmte Zeit
	 * gltig bleibt.
	 *
	 * @return s.o
	 */
	short getFirstInvalidVersion();

	/**
	 * Das Konfigurationsobjekt wird mit der nchsten Version des Konfigurationsbereichs ungltig. Ist das Objekt
	 * in der aktuellen Version noch nicht gltig so wird es nicht auf ungltig gesetzt, sondern gelscht.
	 *
	 * @see #revalidate
	 */
	void invalidate();

	/**
	 * Solange der Konfigurationsbereich noch nicht in eine neue Version berfhrt wurde, kann ein Konfigurationsobjekt,
	 * welches auf ungltig gesetzt wurde, mit dieser Methode wieder auf gltig gesetzt werden.
	 *
	 * @see #invalidate
	 */
	void revalidate();

	/**
	 * Diese Methode gibt die Ids aller Mengen zurck, die an dem Konfigurationsobjekt gespeichert sind.
	 *
	 * @return s.o.
	 */
	long[] getObjectSetIds();

	/**
	 * Diese Methode fgt eine leere Menge an ein Konfigurationsobjekt hinzu.
	 *
	 * @param setId Id der Menge, die zu dem Konfigurationsobjekt hinzugefgt werden soll
	 * @throws IllegalStateException Eine Menge mit der Id existiert bereits an diesem Objekt
	 */
	void addObjectSetId(long setId) throws IllegalStateException;

	/**
	 * Diese Methode gibt zu einer Menge, die zu diesem Konfigurationsobjekt gehrt, alle Ids der jeweiligen Objekte zurck, die sich in
	 * der Menge befinden.
	 *
	 * @param setId Menge, die zu dem Konigurationsobjekt gehrt
	 * @return Objekte, die in der Menge gespeichert sind, die mit der <code>setId</code> identifiziert wurde
	 * @throws IllegalArgumentException Es gibt zu der Id keine Menge
	 */
	long[] getObjectSetObjects(long setId) throws IllegalArgumentException;

	/**
	 * Diese Methode fgt ein Objekt zu einer bestehenden Menge hinzu. Die Menge wird ber die Id identifiziert.
	 *
	 * @param setId    Menge, zu der das Objekt hinzugefgt werden soll
	 * @param objectId Objekt, das zu der Menge hinzugefgt werden soll
	 * @throws IllegalArgumentException Die Menge existiert nicht an dem Konfigurationsobjekt
	 * @throws IllegalStateException Ein Objekt mit der Id existiert bereits in der Menge
	 */
	void addObjectSetObject(long setId, long objectId) throws IllegalArgumentException, IllegalStateException;
}
